/*  $Header$
 *
 *  Copyright (C) 2003 Intersil Americas Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include "version_info.h"

#ifdef NETLINK_SUPPORT
#include "prismoids.h"

#define PRISM_OPEN  prismoid_open
#define PRISM_CLOSE prismoid_close
#define PRISM_GET(name, oid, data, len) get_prismoid(name, oid, data, len)
#define PRISM_SET(name, oid, data, len) set_prismoid(name, oid, data, len)

#else
#include "prismioctl.h"

#define PRISM_OPEN(void)  prismioctl_open(void)
#define PRISM_CLOSE(void) prismioctl_close(void)
#define PRISM_GET(name, oid, data, len) get_prismioctl(name, oid, data, len)
#define PRISM_SET(name, oid, data, len) set_prismioctl(name, oid, data, len)

#endif  /* NETLINK_SUPPORT */

#define DATA_SZ         1024
#define MAX_BSS         16

enum
{
    TI_MAC = 1,
    TI_LONG,
    TI_STRING,
    TI_SSID,
    TI_VDCF,
    TI_KEY,
    TI_STAKEY,
    TI_MLME,
    TI_FREQUENCIES,
    TI_MT,
    TI_STA,
    TI_BSS,
    TI_BSSLIST,
    TI_SS,
    TI_STATS,
    TI_SSMCKEY,
    TI_STASC,
    TI_BINARY,
    TI_SEQ,
    TI_DOUBLE
};


struct data_types {
    char string[17];
    unsigned int id;
} mvc_data_types[] = {
    { "mac", TI_MAC },
    { "long", TI_LONG },
    { "string", TI_STRING },
    { "ssid", TI_SSID },
    { "vdcf", TI_VDCF },
    { "key", TI_KEY },
    { "stakey", TI_STAKEY },
    { "mlme", TI_MLME },
    { "frequencies", TI_FREQUENCIES },
    { "sta", TI_STA },
    { "bss", TI_BSS },
    { "bsslist", TI_BSSLIST },
    { "ss", TI_SS },    
    { "stats", TI_STATS },    
    { "ssmckey", TI_SSMCKEY },    
    { "stasc", TI_STASC },    
    { "binary", TI_BINARY },
    { "seq", TI_SEQ }, 
    { "double", TI_DOUBLE }
};

int main(int argc, char *argv[])
{
    int i, j;
    int tid = -1;
    char ifname[17];
    unsigned int oid = 0;
    unsigned int size = 64;
    unsigned char type[17];
    struct data_types *dtp;

    unsigned char mac[6];
    long longval;
    double doubleval;
    char string[64];
    struct obj_ssid ssid;
    struct obj_vdcf vdcf;
    struct obj_key key;
    struct obj_stakey stakey;
    struct obj_mlme mlme;
    struct obj_frequencies frequencies;
    struct obj_sta sta;
    struct obj_bss bss, *lbss;
    struct obj_bsslist *bsslist;
    struct obj_serviceset ss;
    struct obj_stastats stats;
    struct obj_ssmckey ssmckey;
    struct obj_stasc stasc;
    struct obj_sequential seq;
    unsigned char data[DATA_SZ];

    int ret = 0;

    if (argc < 3)
    {
	printf("getoid version %s\n", VERSIONID );
#ifdef NETLINK_SUPPORT
	printf("prismoids library version %s\n", prismoids_version() );
#else
	printf("prismioctl library version %s\n", prismioctl_version() );
#endif
    	printf("Usage: getoid <device> <oid> [<type>|<size>]\n");
        return -1;
    }

    if ( sscanf ( argv[1], "%s", ifname ) != 1 ) {
    	printf("Usage: getoid <DEVICE> <oid> [<type>|<size>]\n");
        return -1;
    }

    if ( sscanf ( argv[2], "%x", &oid ) != 1 ) {
    	printf("Usage: getoid <device> <OID> [<type>|<size>]\n");
        return -1;
    }

    if (argc > 3) {
        if ( sscanf ( argv[3], "%u", &size ) != 1 ) {
            if ( sscanf ( argv[3], "%s", type ) == 1 ) {
                for ( i=0, dtp=mvc_data_types; i < ((sizeof mvc_data_types) / sizeof(struct data_types)); i++, dtp++ ) {
                    if ( strcmp( dtp->string, type ) == 0 ) {
                        tid = dtp->id;
                        break;
                    }
                }
                if ( tid == -1 ) {
                    printf("[%s] not supported\n", type );
                    return -1;
                }
            }
        }
    }

    if ( size > DATA_SZ ) {
        size = DATA_SZ;
    }

    if( ret = PRISM_OPEN(), ret < 0 ){
	printf("Error: Could not open control channel %d\n", ret );
	return -1;
    }   

    switch ( tid ) {

    case TI_MAC :

        ret = PRISM_GET(ifname, oid, (void*)mac, 6 );
        if ( ret == 0 ) {
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", mac[i] );
            }
            printf( "%2.2X\n", mac[5] );
        }

        break;

    case TI_LONG :

        ret = PRISM_GET(ifname, oid, (void*)&longval, sizeof(long) );
        printf( "%ld (0x%lx)\n", longval, longval );

        break;

    case TI_SSID :

        ret = PRISM_GET(ifname, oid, (void*)&ssid, sizeof(struct obj_ssid) );
        if ( ret == 0 ) {
            printf( "%d ", ssid.length );
            for ( i = 0; i < ssid.length; i++ ) {
                printf( "%c", ssid.octets[i] );
            }
            printf( "\n" );
        }

        break;

    case TI_STRING :

        size = 64;

        if (argc > 4) {
            if ( sscanf ( argv[4], "%u", &size ) != 1 ) {
                size = 64;
            }
            if ( size > 64 ) {
                size = 64;
            }
        }

        ret = PRISM_GET(ifname, oid, (void*)string, size );
        if ( ret == 0 ) {
            printf( "%s\n", string );
        }

        break;

    case TI_VDCF :

        ret = PRISM_GET(ifname, oid, (void*)&vdcf, sizeof(struct obj_vdcf));
        if ( ret == 0 ) {
            printf( "cwmin   %hd\n", vdcf.cwmin );
            printf( "cwmax   %hd\n", vdcf.cwmax );
            printf( "pfact   %hx\n", vdcf.txop );
            printf( "aifs    %hx\n", vdcf.aifs  );
        }

        break;

    case TI_KEY :

        ret = PRISM_GET(ifname, oid, (void*)&key, sizeof(struct obj_key));
        if ( ret == 0 ) {
            printf( "type     %x\n", key.type );
            printf( "length  %hd\n", key.length );
			key.key[key.length] = '\0';
            printf( "key      %s\n", key.key );
			printf( "         " );
			for( i=0; i<key.length; i++ )
				printf("[%02x]", key.key[i] & 0xff );
			printf( "\n" );
        }

        break;

    case TI_STAKEY :

        ret = PRISM_GET(ifname, oid, (void*)&stakey, sizeof(struct obj_stakey));
        if ( ret == 0 ) {
            printf( "address  " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)stakey.address[i] );
            }
            printf( "%2.2X\n", (unsigned char)stakey.address[5] );
            printf( "keyid    %x\n", stakey.keyid );
            printf( "type     %x\n", stakey.type );
            printf( "length   %hd\n", stakey.length );
            printf( "key      %s\n", stakey.key );
        }

        break;

    case TI_MLME :

        ret = PRISM_GET(ifname, oid, (void*)&mlme, sizeof(struct obj_mlme));
        if ( ret == 0 ) {
            printf( "address  " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)mlme.address[i] );
            }
            printf( "%2.2X\n", (unsigned char)mlme.address[5] );
            printf( "id       %x\n", mlme.id );
            printf( "state    %x\n", mlme.state );
            printf( "code     %x\n", mlme.code );
            printf( "size     %x\n", mlme.size );
        }

        break;

    case TI_FREQUENCIES :

        /* ToDo: scan array */

        ret = PRISM_GET(ifname, oid, (void*)&frequencies, sizeof(struct obj_frequencies));
        if ( ret == 0 ) {
            printf( "nr      %hd\n", frequencies.nr );
            printf( "mhz[0]  %hd\n", frequencies.mhz[0] );
        }

        break;

	case TI_STA:

        if (argc > 4)
        {
			// read the supplied MAC address for searching a client
	        if ( sscanf ( argv[4], "%x:%x:%x:%x:%x:%x",
        		(unsigned int *) &(sta.address[0]), (unsigned int *) &(sta.address[1]),
				(unsigned int *) &(sta.address[2]), (unsigned int *) &(sta.address[3]),
				(unsigned int *) &(sta.address[4]), (unsigned int *) &(sta.address[5]) ) != 6 ) {
	            printf("Usage: setoid <device> <oid> sta <MAC_ADDRESS>\n");
    	        break;
			}
        }

        ret = PRISM_GET(ifname, oid, (void*)&sta, sizeof(struct obj_sta));

        if ( ret == 0 ) {
            printf( "address " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%02X:", (unsigned char)sta.address[i] );
            }
            printf( "%02X\n", (unsigned char)sta.address[5] );
            printf( "ext     %04x\t", sta.ext );
            printf( "state   %x\n", sta.state );
            printf( "age     %x\t", sta.age );
            printf( "quality %x\n", sta.quality );
            printf( "rssi    %x\n", sta.rssi );
        }

		break;

    case TI_BSS:

        ret = PRISM_GET(ifname, oid, (void*)&bss, sizeof(struct obj_bss));
        if ( ret == 0 ) {
            printf( "address  " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)bss.address[i] );
            }
            printf( "%2.2X\n", (unsigned char)bss.address[5] );
            printf( "ext      %04x\t", bss.ext );
            printf( "state    0x%x\n", bss.state );
            printf( "age      0x%x\n", bss.age );
            printf( "quality  0x%x\n", bss.quality );
            printf( "rssi     0x%x\n", bss.rssi );
            printf( "ssid     %s\n", bss.ssid );
            printf( "channel  %i\n", bss.channel );
            printf( "beaconp  0x%x\n", bss.beacon_period );
            printf( "dtimp    0x%x\n", bss.dtim_period );
            printf( "capinfo  %i\n", bss.capinfo );
            printf( "rates    0x%x\n", bss.rates );
            printf( "brates   0x%x\n", bss.basic_rates );
        }

        break;


    case TI_BSSLIST:

        ret = PRISM_GET(ifname, oid, (void*)&data, 4+1+MAX_BSS*sizeof(struct obj_bss) );
        if ( ret == 0 ) {
            bsslist = (struct obj_bsslist *) data;
            lbss = (struct obj_bss *) bsslist->bsslist;

            printf( "Nr\tMAC Address\t\tf[MHz]\tSSID\n" );
            for ( j = 0; j < bsslist->nr; j++, lbss++ ) {
                printf( "%2.2i\t", j+1 );
                for ( i = 0; i < 5; i++ ) {
                    printf( "%2.2X:", (unsigned char)lbss->address[i] );
                }
                printf( "%2.2X\t", (unsigned char)lbss->address[5] );
                printf( "%2.2i\t",  lbss->channel );
                printf( "%s\n", &lbss->ssid[1] );
            }

        }

        break;

    case TI_SS:
        ret = PRISM_GET(ifname, oid, (void*)&ss, sizeof(struct obj_serviceset));
        if ( ret == 0 ) {
            printf( "bssid        " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)ss.bssid[i] );
            }
            printf( "%2.2X\n", (unsigned char)ss.bssid[5] );
            printf( "ext           %d\n", ss.ext );
            printf( "ssid length   %d\n", ss.ssid.length );
            printf( "ssid octets   %s\n", ss.ssid.octets );
            printf( "policies      %u\n", ss.policies );
            printf( "authenable    %u\n", ss.authenable );
            printf( "privinvoked   %u\n", ss.privinvoked );
            printf( "exunencrypted %u\n", ss.exunencrypted );
            printf( "wpa unicast   %u\n", ss.wpa.unicast );
            printf( "wpa multicast %u\n", ss.wpa.multicast );
            printf( "wpa authtype  %u\n", ss.wpa.authtype );
            printf( "wpa capabilities %u\n", ss.wpa.capabilities );
            printf( "rsn unicast   %u\n", ss.rsn.unicast );
            printf( "rsn multicast %u\n", ss.rsn.multicast );
            printf( "rsn authtype  %u\n", ss.rsn.authtype );
            printf( "rsn capabilities %u\n", ss.rsn.capabilities );
            printf( "exclusive     %u\n", ss.exclusive );
            printf( "total         %u\n", ss.total );
            printf( "associated    %u\n", ss.associated );
            printf( "mc_msdu_tx    %u\n", ss.mc_msdu_tx );
            printf( "mc_msdu_rx    %u\n", ss.mc_msdu_rx );
            printf( "mc_octets_tx  %u\n", ss.mc_msdu_tx );
            printf( "mc_octets_tx  %u\n", ss.mc_msdu_rx );
        }
	
	break;

    case TI_STATS:
        ret = PRISM_GET(ifname, oid, (void*)&stats, sizeof(struct obj_stastats));
        if ( ret == 0 ) {
            printf( "mac       " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)stats.address[i] );
            }
            printf( "%2.2X\n", (unsigned char)stats.address[5] );
            printf( "ext       %d\n", stats.ext );
            printf( "age       %d\n", stats.age );
            printf( "msdu_tx   %u\n", stats.msdu_tx );
            printf( "msdu_rx   %u\n", stats.msdu_rx );
            printf( "octets_tx %u\n", stats.msdu_tx );
            printf( "octets_tx %u\n", stats.msdu_rx );
        }
	break;

    case TI_SSMCKEY:
    
        if( argc < 6 )
        {
            printf( "Usage: getoid <device> <oid> ssmckey <keyid> <ssnr>\n" );
            break;
        }
        
        sscanf ( argv[4], "%c", &(ssmckey.keyid) );
        ssmckey.keyid -= '0';
        sscanf ( argv[5], "%c", &(ssmckey.ssnr) );
        ssmckey.ssnr -= '0';
            
        ret = PRISM_GET(ifname, oid, (void*)&ssmckey, sizeof(struct obj_ssmckey));
        if ( ret == 0 ) {
            printf( "mac       " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)ssmckey.address[i] );
            }
            printf( "%2.2X\n", (unsigned char)ssmckey.address[5] );
            printf( "ext      %d\n", ssmckey.ext );
            printf( "keyid    %c\n", ssmckey.keyid );
            printf( "ssnr     %c\n", ssmckey.ssnr );
            printf( "type     %d\n", ssmckey.type );
            printf( "length   %d\n", ssmckey.length );
            printf( "key      %s\n", ssmckey.key );
        }
        break;

    case TI_STASC:
    
        if( argc < 7 )
        {
            printf( "Usage: getoid <device> <oid> stasc <keyid> <ssnr> <tx_sc>\n" );
            break;
        }
        
        sscanf ( argv[4], "%c", (unsigned char*) &(stasc.keyid) );
        stasc.keyid -= '0'; 
        sscanf ( argv[5], "%c", (unsigned char*) &(stasc.ssnr) );
        stasc.ssnr -= '0';
        sscanf ( argv[6], "%c", (unsigned char*) &(stasc.tx_sc) );
        stasc.tx_sc -= '0';
            
        ret = PRISM_GET(ifname, oid, (void*)&stasc, sizeof(struct obj_stasc));
        if ( ret == 0 ) {
            printf( "mac       " );
            for ( i = 0; i < 5; i++ ) {
                printf( "%2.2X:", (unsigned char)stasc.address[i] );
            }
            printf( "%2.2X\n", (unsigned char)stasc.address[5] );
            printf( "ext      %d\n", stasc.ext );
            printf( "keyid    %c\n", stasc.keyid );
            printf( "tx_sc    %c\n", stasc.tx_sc );
            printf( "ssnr     %c\n", stasc.ssnr );
            printf( "sc_high  %u\n", stasc.sc_high );
            printf( "sc_low   %d\n", stasc.sc_low );
        }
        break;

    case TI_SEQ:
        if( argc < 4 )
	{
	    printf( "Usage: getoid <device> <oid> seq\n" );
	    break;
	}

	ret = PRISM_GET(ifname, oid, (void*)&seq, sizeof(struct obj_sequential));
	if( ret == 0 )
	{
	    printf( "Sequential BSS:\n" );
	    printf( "  freq      %d\n", seq.frequency );
	    seq.ssid[1 + seq.ssid[0]] = 0;
	    printf( "  ssid      %d %s\n", seq.ssid[0], &seq.ssid[1] );
	    printf( "  timeslice %d\n", seq.timeslice );
	    printf( "  window    %d\n", seq.mcast_window ); 
	}
	break;
	    

    case TI_BINARY:
	if( argc < 4 )
	{
	    printf( "Usage: getoid <device> <oid> binary <length><\n" );
	    break;
	}

	size = strtoul(argv[4], NULL, 0);
        ret = PRISM_GET(ifname, oid, (void*)data, size );
	
        if ( ret == 0 ) {
            for ( i = 0; i < size; i++ ) {
                printf( "%2.2X", data[i] );
                if ( ((i+1) % 16) == 0 ) printf("\n");
		else if ( ((i+1) % 8) == 0 ) printf("-");
		else printf(" ");
            }
            if ( (i % 16) != 0 ) printf("\n");
	}
	break;
	
    case TI_DOUBLE:
        ret = PRISM_GET(ifname, oid, (void*)&doubleval, sizeof(double) );
        printf( "%lu:%lu (0x%lx:0x%lx)\n", (unsigned long)(doubleval/0x100000000),
	    (unsigned long) doubleval, (unsigned long)(doubleval/0x100000000),
	    (unsigned long) doubleval );

        break;

    default:
        ret = PRISM_GET(ifname, oid, (void*)data, size );
        if ( ret == 0 ) {
            for ( i = 0; i < size; i++ ) {
                printf( "%2.2X", data[i] );
                if ( ((i+1) % 4) == 0 ) printf(" ");
                if ( ((i+1) % 32) == 0 ) printf("\n");
            }
            printf("\n");
        }

        break;
    }

    PRISM_CLOSE();

    return ret;
}

