/*  $Header$
 *
 *  Copyright (C) 2003 Intersil Americas Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/socket.h>
#include <net/if.h>
#include "version_info.h"

/* ioctl interface */
#define SOFTMACIOCTL                    SIOCDEVPRIVATE

struct s_softmac_ioctl {
    unsigned int cmd;
    unsigned int oid;
    unsigned int len;
    char *data;
};

static struct ifreq ifr;

int main(int argc, char *argv[])
{
    int fd;
    int netfd;

    if (argc < 2 || argc > 3) {
        fprintf(stderr, "term version %s\n", VERSIONID );
        fprintf(stderr, "usage: %s dev [interface]\n", argv[0]);
        exit(1);
    }

    fd = open(argv[1], O_RDWR);
    if(fd < 0) {
        fprintf(stderr, "Failed opening %s\n", argv[1]);
        exit(1);
    }

    /* Setup our control structures. */
    memset(&ifr, 0, sizeof(ifr));
    if(argc == 3) {
        strncpy(ifr.ifr_name, argv[2], IFNAMSIZ);
    } else {
        fprintf(stderr, "Using eth1 as default\n");
        strncpy(ifr.ifr_name, "eth1", IFNAMSIZ);
    }

    /* Open control socket. */
    netfd = socket(AF_INET, SOCK_DGRAM, 0);
    if (netfd < 0) {
        perror("Cannot open socket");
        exit(1);
    }

    do_terminal(fd, netfd);

    close(fd);
    close(netfd);
 
    exit(0);
}

static int do_cmd(int fd, int cmd)
{
    struct s_softmac_ioctl smioctl;
    int err;

    smioctl.cmd = cmd;
    smioctl.oid = 0;
    smioctl.len = 0;
    smioctl.data = NULL;

    ifr.ifr_data = (caddr_t)&smioctl;
    err = ioctl(fd, SOFTMACIOCTL, &ifr);
    if (err < 0) {
        fprintf(stderr, "SoftMAC Command %d on interface %s failed (%d)",
                cmd, ifr.ifr_name, err);
        perror(" ");
        return -1;
    }
    return 0;
}

int do_terminal(int upci_fd, int net_fd)
{
    char buff[1024];
    struct termios otty, ntty;
    void (*old_sigs[32])(int);
    fd_set mask, rmask;
    register int i;

    printf("[ TERMINAL.\n  Use CTRL-] to get back\n  CTRL-C to reset the card\n  CTRL-U for uploading new firmware ]\n");

    /* Set up the keyboard. */
    if (ioctl(0, TCGETS, &otty)  < 0) {
        perror("term ioctl TCGETS");
        return( -1 );
    }
    ntty = otty;
    ntty.c_lflag &= ~(ECHO  | ICANON | ISIG );
    ntty.c_iflag &= ~(ICRNL | IXON | IXOFF);
    ntty.c_oflag &= ~(ONLCR | OPOST);
    if (ioctl(0, TCSETS, &ntty)  < 0) {
        perror("term ioctl TCSETS");
        return( -1 );
    }
  
    /* Set up the correct descriptor masks for select(2). */
    FD_ZERO(&mask);
    FD_SET(STDIN_FILENO, &mask);
    FD_SET(upci_fd, &mask);

    /* Disable all signals, saving them for later. */
    for (i = 0; i < 32; i++) old_sigs[i] = signal(i + 1, SIG_IGN);

    /* Go into an endless terminal loop. */
    while(1) {
        rmask = mask;
        i = select(32, &rmask, (fd_set *)NULL, (fd_set *)NULL,
                        (struct timeval *)NULL);

        if (i <= 0)
            break;

        if (FD_ISSET(STDIN_FILENO, &rmask)) {
            i = read(STDIN_FILENO, buff, 1);
            if (i > 0) {
                if (buff[0] == (']' & 037))
                    break;
                if (buff[0] == 003) {
                    printf("\r\n[ Resetting ]\n");
                    do_cmd(net_fd, 2);
                }
                if (buff[0] == ('u' & 037)) {
                    printf("\r\n[ Uploading ]\n");
                    do_cmd(net_fd, 1);
                }

                (void) write(upci_fd, buff, i);
            }
        }

        if (FD_ISSET(upci_fd, &rmask)) {
            i = read(upci_fd, buff, 1024);
            if (i > 0)
                (void) write(1, buff, i);
        };
    }

    if (ioctl(0, TCSETS, &otty)  < 0) {
        perror("term ioctl TCSETS");
        return( -1 );
    }

    /* Restore all signals. */
    for (i = 0; i < 32; i++) 
        signal(i + 1, old_sigs[i]);

    printf("\n[ Exiting ]\n");
    return( 0 );
}
