/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "log.h"
#include "acl.h"
#include "client.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "misc.h"
#include "config.h"

struct acl_t *acllist = NULL;

int acl_default_policy = UNAUTHORIZED;

/*
 * Check a just associated client if ACL needs to be checked.
 * Some modes such as dot1x and WPA leaves access control checking
 * to the radius server.
 *
 *	does1X	doesWPA		OS	WEP	dot1X	WPA	WPA-PSK		CHECK
 *	0	0		X						Y
 *	0	0			X					Y
 *	1	0				X				N
 *	1	1					X			N
 *	0	1						X		YwoPORT
 *	1	0			X	X				POST
 *	1	1/0			X		X			POST
 *	1	1/0			X	X	X		`	POST
 *	0	1/0			X			X		Y/YwoPORT
 *	1	1/0			X	X		X		POST/YwoPORT
 *	1	1/0				X	X			N
 *	1	1/0				X		X		N/YwoPORT
 *
 *	Means:
 *	- Y Check ACL if thats configured else set default policy for client.
 *	- N Don't check ACL at all.
 *	- POST Check ACL when request identity times out.
 *	- YwoPORT Check ACL but don't set the portdirectly.
 *
 * So here's the problem:
 * 	We want to do ACL checking if nessecary as soon as possible.
 *	We don't check clients which will be checked by Radius servers.
 *	We don't want to allow clients wich are not checked by Radius and are
 *	not in the current ACL table.
 *
 * Look at the table and find out what to check in this function. The funciton will
 * only be called after clients ASSOCIATION frame. So concentrate on the Y, YwoPORT and N
 * fields. The POST will be done at other locations.
 *
 * IF client->doesWPA && config.enableWPAPSK
 * 	do YwoPORT
 * ESLE IF client->does1X == 0
 * 	do Y
 * ELSE
 * 	do N or POST
 */
int acl_list_check(struct client_t *client, int set_state)
{
	int client_policy = acl_list_check_client(client);
	log(LG_INF, "Client %s is %s", hwaddr2string(client->hwaddr),
		(client_policy == AUTHORIZED) ? "AUTHORIZED" : "UNAUTHORIZED");
	if (set_state)
	{
	    blob_portstatus_set(client, client_policy);
    	}
	return(client_policy);

}

int acl_list_check_client(struct client_t *client)
{
	struct acl_t *acl;

	acl = acllist;

	if (client->cconfig->enableACL)
	{
	    while(acl) {
		    if(memcmp(acl->hwaddr, client->hwaddr, 6) == 0) {

			    if(acl->ps == AUTHORIZED) return(AUTHORIZED);
			    else return(UNAUTHORIZED);

		    }

		    acl = acl->next;
	    }
	    /* Client not in ACL apply default policy to client */
	    return(acl_default_policy);
    	}
	else {
	    /* ACL is not enabled */
	    return(AUTHORIZED);
	}
	
}

/*
 * Add a acl_t struct to the end of the linked list
 */

int acl_list_add(char *hwaddr, int index, int ps)
{
	struct acl_t *new;
	struct acl_t *acl;



	new = malloc(sizeof(struct acl_t));
	assert(new);

	memcpy(new->hwaddr, hwaddr, 6);

	new->ps    = ps;
	new->index = index;
	new->next  = NULL;

	if(acllist == NULL) {

		acllist = new;

	} else {

		acl = acllist;

		while(acl->next) {
			acl = acl->next;
		}

		acl->next = new;
	}


	return(0);
}


/*
 * Sets the portstatus for the row with the given index
 */

int acl_list_setps(int index, int ps)
{
	struct acl_t *acl;

	acl = acllist;

	while(acl) {
		if(acl->index == index) {
			acl->ps = ps;
			return(0);
		}
		acl = acl->next;
	}

	return(-1);
}


int acl_list_dump(void)
{
	struct acl_t *acl;

	acl = acllist;

	log(LG_DBG, "ACL default policy : %s",
		(acl_default_policy == AUTHORIZED) ? "AUTH" : "UNAUTH");


	while(acl) {
		log(LG_DBG, "ACL %d %s %s",
			acl->index,
			hwaddr2string(acl->hwaddr),
			(acl->ps == AUTHORIZED) ? "AUTH" : "UNAUTH");

		acl = acl->next;
	}

	return(0);
}


/*
 * Read a simple ACL configfile (if the mtime changed),
 * and add the records to the list.
 */

int acl_read_file(char *fname)
{
	FILE *f;
	char buf[128];
	struct acl_t *tmp;

	unsigned char *part[5];
	int parts;

	unsigned char hwaddr[6];
	int ps;
	int index;
	int i, t;

	f = fopen(fname, "r");

	if(f == NULL) return(-1);

	while(fgets(buf, sizeof(buf), f)) {

	 	if (skip_white((char *)buf) == 0) continue; // Skip white and #

		parts = split(buf, ' ', part, 5);

		if(parts == 4) {
			if(strcmp(part[0], "Acl") == 0) {

				index = atoi(part[1]);

				for(i=0; i<6; i++) {
					sscanf(part[2]+i*3, "%2x", &t);
					hwaddr[i] = t;
			        }

				acl_list_add(hwaddr, index, UNAUTHORIZED);

				ps = atoi(part[3]) & 1;

				acl_list_setps(index, ps);

			}
		}
	}

	fclose(f);

	// The first entry in the ACL is for mac address ff:ff:ff:ff:ff:ff, this controls
	// the default policy. Set our default policy to the ps of this row, and remove it
	// from the list.

	tmp = acllist;
	acl_default_policy = tmp->ps;
	acllist = tmp->next;
	free(tmp);

	acl_list_dump();

	return(0);
}
