/*
 * IMPORTANT--READ CAREFULLY:
 *
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document")
 * are governed by a legal Agreement ("Agreement") between you (either an
 * individual or a single entity) and Intersil Americas Inc ("Intersil")
 * and/or its licensee (both hereinafter called Licensor).
 *
 * By reproducing, modifying, creating derivative works, accepting,
 * installing, viewing, accessing, transforming or otherwise using this
 * Document, you agree to be bound by the terms of the Agreement. If you
 * do not agree to the terms of the Agreement, Licensor is unwilling to
 * license rights to this Document to you.  In such event, you may not
 * use, reproduce, modify, create derivative works, install, view, access,
 * transport, transform or circulate this Document, and you should promptly
 * contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION: This Document
 * contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 *
 * Unauthorized use, reproduction, modification, creation of derivative
 * works or circulation is strictly prohibited.
 *
 * Intersil is a registered trademark of Intersil Americas Inc.
 *
 * Copyright (c) 2003 Intersil Americas Inc. All Rights Reserved.
 */


/*
 * AES routines. These routines are based on the code produced by
 *
 * Dr. B. R. Gladman (brian.gladman@btinternet.com). 25th January 2000.
 *
 * They were downloaded from:
 * http://fp.gladman.plus.com/cryptography_technology/aes2/index.htm
 *
 * Be aware: These routines were dedicated for speed testing, however,
 * they match perfectly the CKIP usage of AES. In both cases the AES
 * engine is to be used with a 16 byte key, 16 byte input, and 16 byte
 * output. This made these (optimized) routines ideal for CKIP usage.
 * Do not use (without altering) for any other purpose. The code has
 * been modified slightly to be more compliant to souce code style and
 * cplusplus parts have been removed.
 */



typedef signed char 		int8_t;
typedef short int 		int16_t;
typedef int 			int32_t;


#include "aes.h"


typedef uint8_t u1byte;		/* an 8 bit unsigned character type */
typedef uint16_t u2byte;	/* a 16 bit unsigned integer type */
typedef uint32_t u4byte;	/* a 32 bit unsigned integer type */

typedef int8_t s1byte;		/* an 8 bit signed character type */
typedef int16_t s2byte;		/* a 16 bit signed integer type */
typedef int32_t s4byte;		/* a 32 bit signed integer type */

static u4byte k_len;
static u4byte e_key[64];
static u4byte d_key[64];

enum dir_flag mode;		/* in C the mode flag is declared in aes_aux.c */

/* 3. Basic macros for speeding up generic operations */

/* Circular rotate of 32 bit values */

#define rotr(x,n)   (((x) >> ((int_t)((n) & 0x1f))) | ((x) << ((int_t)((32 - ((n) & 0x1f))))))
#define rotl(x,n)   (((x) << ((int_t)((n) & 0x1f))) | ((x) >> ((int_t)((32 - ((n) & 0x1f))))))

/* Invert byte order in a 32 bit variable */

#define bswap(x)    ((rotl(x, 8) & 0x00ff00ff) | (rotr(x, 8) & 0xff00ff00))

/* Put or get a 32 bit word (v) in machine order from a byte address in (x) */

#ifdef  CONF_BIGENDIAN

#define u4byte_in(x)        bswap(*(u4byte*)(x))
#define u4byte_out(x, v)    (*(u4byte*)(x) = bswap(v))

#else

#define u4byte_in(x)        (*(u4byte*)(x))
#define u4byte_out(x, v)    (*(u4byte*)(x) = (v))

#endif


#define LARGE_TABLES


static u1byte pow_tab[256];
static u1byte log_tab[256];
static u1byte sbx_tab[256];
static u1byte isb_tab[256];
static u4byte rco_tab[10];
static u4byte ft_tab[4][256];
static u4byte it_tab[4][256];

#ifdef  LARGE_TABLES
static u4byte fl_tab[4][256];
static u4byte il_tab[4][256];
#endif

static u4byte tab_gen = 0;

static u1byte f_mult(u1byte a, u1byte b)
{
	u1byte aa = log_tab[a], cc = aa + log_tab[b];

	return pow_tab[cc + (cc < aa ? 1 : 0)];

}

/* Extract byte from a 32 bit quantity (little endian notation) */

#define byte(x,n)   ((u1byte)((x) >> (8 * (n))))

#define ff_mult(a,b)    (a && b ? f_mult(a, b) : 0)

#define f_rn(bo, bi, n, k)                          \
    bo[n] =  ft_tab[0][byte(bi[n],0)] ^             \
             ft_tab[1][byte(bi[(n + 1) & 3],1)] ^   \
             ft_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             ft_tab[3][byte(bi[(n + 3) & 3],3)] ^ *(k + n)

#define i_rn(bo, bi, n, k)                          \
    bo[n] =  it_tab[0][byte(bi[n],0)] ^             \
             it_tab[1][byte(bi[(n + 3) & 3],1)] ^   \
             it_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             it_tab[3][byte(bi[(n + 1) & 3],3)] ^ *(k + n)

#ifdef LARGE_TABLES

#define ls_box(x)                \
    ( fl_tab[0][byte(x, 0)] ^    \
      fl_tab[1][byte(x, 1)] ^    \
      fl_tab[2][byte(x, 2)] ^    \
      fl_tab[3][byte(x, 3)] )

#define f_rl(bo, bi, n, k)                          \
    bo[n] =  fl_tab[0][byte(bi[n],0)] ^             \
             fl_tab[1][byte(bi[(n + 1) & 3],1)] ^   \
             fl_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             fl_tab[3][byte(bi[(n + 3) & 3],3)] ^ *(k + n)

#define i_rl(bo, bi, n, k)                          \
    bo[n] =  il_tab[0][byte(bi[n],0)] ^             \
             il_tab[1][byte(bi[(n + 3) & 3],1)] ^   \
             il_tab[2][byte(bi[(n + 2) & 3],2)] ^   \
             il_tab[3][byte(bi[(n + 1) & 3],3)] ^ *(k + n)

#else

#define ls_box(x)                            \
    ((u4byte)sbx_tab[byte(x, 0)] <<  0) ^    \
    ((u4byte)sbx_tab[byte(x, 1)] <<  8) ^    \
    ((u4byte)sbx_tab[byte(x, 2)] << 16) ^    \
    ((u4byte)sbx_tab[byte(x, 3)] << 24)

#define f_rl(bo, bi, n, k)                                      \
    bo[n] = (u4byte)sbx_tab[byte(bi[n],0)] ^                    \
        rotl(((u4byte)sbx_tab[byte(bi[(n + 1) & 3],1)]),  8) ^  \
        rotl(((u4byte)sbx_tab[byte(bi[(n + 2) & 3],2)]), 16) ^  \
        rotl(((u4byte)sbx_tab[byte(bi[(n + 3) & 3],3)]), 24) ^ *(k + n)

#define i_rl(bo, bi, n, k)                                      \
    bo[n] = (u4byte)isb_tab[byte(bi[n],0)] ^                    \
        rotl(((u4byte)isb_tab[byte(bi[(n + 3) & 3],1)]),  8) ^  \
        rotl(((u4byte)isb_tab[byte(bi[(n + 2) & 3],2)]), 16) ^  \
        rotl(((u4byte)isb_tab[byte(bi[(n + 1) & 3],3)]), 24) ^ *(k + n)

#endif

static void gen_tabs(void)
{
	u4byte i, t;
	u1byte p, q;

	/* log and power tables for GF(2**8) finite field with  */
	/* 0x011b as modular polynomial - the simplest prmitive */
	/* root is 0x03, used here to generate the tables       */

	for (i = 0, p = 1; i < 256; ++i) {
		pow_tab[i] = (u1byte) p;
		log_tab[p] = (u1byte) i;

		p ^= (p << 1) ^ (p & 0x80 ? 0x01b : 0);
	}

	log_tab[1] = 0;

	for (i = 0, p = 1; i < 10; ++i) {
		rco_tab[i] = p;

		p = (p << 1) ^ (p & 0x80 ? 0x01b : 0);
	}

	for (i = 0; i < 256; ++i) {
		p = (i ? pow_tab[255 - log_tab[i]] : 0);
		q = ((p >> 7) | (p << 1)) ^ ((p >> 6) | (p << 2));
		p ^= 0x63 ^ q ^ ((q >> 6) | (q << 2));
		sbx_tab[i] = p;
		isb_tab[p] = (u1byte) i;
	}

	for (i = 0; i < 256; ++i) {
		p = sbx_tab[i];

#ifdef  LARGE_TABLES

		t = p;
		fl_tab[0][i] = t;
		fl_tab[1][i] = rotl(t, 8);
		fl_tab[2][i] = rotl(t, 16);
		fl_tab[3][i] = rotl(t, 24);
#endif
		t = ((u4byte) ff_mult(2, p)) |
		    ((u4byte) p << 8) |
		    ((u4byte) p << 16) | ((u4byte) ff_mult(3, p) << 24);

		ft_tab[0][i] = t;
		ft_tab[1][i] = rotl(t, 8);
		ft_tab[2][i] = rotl(t, 16);
		ft_tab[3][i] = rotl(t, 24);

		p = isb_tab[i];

#ifdef  LARGE_TABLES

		t = p;
		il_tab[0][i] = t;
		il_tab[1][i] = rotl(t, 8);
		il_tab[2][i] = rotl(t, 16);
		il_tab[3][i] = rotl(t, 24);
#endif
		t = ((u4byte) ff_mult(14, p)) |
		    ((u4byte) ff_mult(9, p) << 8) |
		    ((u4byte) ff_mult(13, p) << 16) |
		    ((u4byte) ff_mult(11, p) << 24);

		it_tab[0][i] = t;
		it_tab[1][i] = rotl(t, 8);
		it_tab[2][i] = rotl(t, 16);
		it_tab[3][i] = rotl(t, 24);
	}

	tab_gen = 1;
}

#define star_x(x) (((x) & 0x7f7f7f7f) << 1) ^ ((((x) & 0x80808080) >> 7) * 0x1b)

#define imix_col(y,x)       \
    u   = star_x(x);        \
    v   = star_x(u);        \
    w   = star_x(v);        \
    t   = w ^ (x);          \
   (y)  = u ^ v ^ w;        \
   (y) ^= rotr(u ^ t,  8) ^ \
          rotr(v ^ t, 16) ^ \
          rotr(t,24)


/* initialise the key schedule from the user supplied key */

#define loop4(i)                                    \
{   t = rotr(t,  8); t = ls_box(t) ^ rco_tab[i];    \
    t ^= e_key[4 * i];     e_key[4 * i + 4] = t;    \
    t ^= e_key[4 * i + 1]; e_key[4 * i + 5] = t;    \
    t ^= e_key[4 * i + 2]; e_key[4 * i + 6] = t;    \
    t ^= e_key[4 * i + 3]; e_key[4 * i + 7] = t;    \
}

void aes_set_key(const u1byte in_key[], const u4byte key_len,
		 const enum dir_flag f)
{
	u4byte i, t, u, v, w;

//	dump_bindata("aes_set_key", in_key, key_len);

	if (!tab_gen)

		gen_tabs();

	mode = f;

	e_key[0] = u4byte_in(in_key);
	e_key[1] = u4byte_in(in_key + 4);
	e_key[2] = u4byte_in(in_key + 8);
	e_key[3] = u4byte_in(in_key + 12);

	t = e_key[3];
	for (i = 0; i < 10; ++i)
		loop4(i);

	if (mode != enc) {
		d_key[0] = e_key[0];
		d_key[1] = e_key[1];
		d_key[2] = e_key[2];
		d_key[3] = e_key[3];

		for (i = 4; i < 4 * k_len + 24; ++i) {
			imix_col(d_key[i], e_key[i]);
		}
	}

	return;
}

/* encrypt a block of text */

#define f_nround(bo, bi, k) \
    f_rn(bo, bi, 0, k);     \
    f_rn(bo, bi, 1, k);     \
    f_rn(bo, bi, 2, k);     \
    f_rn(bo, bi, 3, k);     \
    k += 4

#define f_lround(bo, bi, k) \
    f_rl(bo, bi, 0, k);     \
    f_rl(bo, bi, 1, k);     \
    f_rl(bo, bi, 2, k);     \
    f_rl(bo, bi, 3, k)

void aes_encrypt(const u1byte in_blk[16], u1byte out_blk[16])
{
	u4byte b0[4], b1[4], *kp;

	b0[0] = u4byte_in(in_blk) ^ e_key[0];
	b0[1] = u4byte_in(in_blk + 4) ^ e_key[1];
	b0[2] = u4byte_in(in_blk + 8) ^ e_key[2];
	b0[3] = u4byte_in(in_blk + 12) ^ e_key[3];

	kp = e_key + 4;

	f_nround(b1, b0, kp);
	f_nround(b0, b1, kp);
	f_nround(b1, b0, kp);
	f_nround(b0, b1, kp);
	f_nround(b1, b0, kp);
	f_nround(b0, b1, kp);
	f_nround(b1, b0, kp);
	f_nround(b0, b1, kp);
	f_nround(b1, b0, kp);
	f_lround(b0, b1, kp);

	u4byte_out(out_blk, b0[0]);
	u4byte_out(out_blk + 4, b0[1]);
	u4byte_out(out_blk + 8, b0[2]);
	u4byte_out(out_blk + 12, b0[3]);
}

/* decrypt a block of text */

#define i_nround(bo, bi, k) \
    i_rn(bo, bi, 0, k);     \
    i_rn(bo, bi, 1, k);     \
    i_rn(bo, bi, 2, k);     \
    i_rn(bo, bi, 3, k);     \
    k -= 4

#define i_lround(bo, bi, k) \
    i_rl(bo, bi, 0, k);     \
    i_rl(bo, bi, 1, k);     \
    i_rl(bo, bi, 2, k);     \
    i_rl(bo, bi, 3, k)

void aes_decrypt(const u1byte in_blk[16], u1byte out_blk[16])
{
	u4byte b0[4], b1[4], *kp;

	b0[0] = u4byte_in(in_blk) ^ e_key[4 * k_len + 24];
	b0[1] = u4byte_in(in_blk + 4) ^ e_key[4 * k_len + 25];
	b0[2] = u4byte_in(in_blk + 8) ^ e_key[4 * k_len + 26];
	b0[3] = u4byte_in(in_blk + 12) ^ e_key[4 * k_len + 27];

	kp = d_key + 4 * (k_len + 5);

	i_nround(b1, b0, kp);
	i_nround(b0, b1, kp);
	i_nround(b1, b0, kp);
	i_nround(b0, b1, kp);
	i_nround(b1, b0, kp);
	i_nround(b0, b1, kp);
	i_nround(b1, b0, kp);
	i_nround(b0, b1, kp);
	i_nround(b1, b0, kp);
	i_lround(b0, b1, kp);

	u4byte_out(out_blk, b0[0]);
	u4byte_out(out_blk + 4, b0[1]);
	u4byte_out(out_blk + 8, b0[2]);
	u4byte_out(out_blk + 12, b0[3]);
}
