/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * Generic functions for communicating with the blob through prismeth IOCTLS
 */

#include <string.h>
#include <stdio.h>
#include <net/if.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <signal.h>
#include <net/if.h>
#include <errno.h>

#undef EOVERFLOW

#include "../../kernel/drivers/net/prismeth.h"
#include "blob.h"
#include "bloboid.h"
#include "linux_mvc_oids.h"

#include "log.h"
#include "blobfuncs.h"
#include "client.h"
#include "misc.h"
#include "config.h"
#include "state_pae.h"
#include "state_ssn.h"

extern char ifname[];
extern int eapolfd;

extern struct config_t config;

char mlmeexbuf[256];
struct obj_mlmeex* mlmeex = (struct obj_mlmex *)mlmeexbuf;
struct obj_sta sta;

volatile int micfailure = 0;

/*
 * Set default-policy global portstats of all ports/clients
 */

int blob_portstatus_set_global(enum portstatus_t state)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;
	char stat;

	switch(state) {
		case AUTHORIZED:
			log(LG_DBG, "state AUTHORIZED");
			stat = 0;
			break;
		case UNAUTHORIZED:
			log(LG_DBG, "state UNAUTHORIZED");
			stat = 1;
			break;
	}

	ioc.oid 	= DOT11_OID_DOT1XENABLE;
	ioc.cmd 	= OPSET;
	ioc.data 	= &stat;
	ioc.len 	= sizeof(stat);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_ERR, "Can't do DOT11_OID_DOT1XENABLE : %d", r);
		return(-1);
	}

	return(0);
}


/*
 * Set portstatus of specific client to authorized or unauthorized
 */

int blob_portstatus_set(struct client_t *client, enum portstatus_t state)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;

	switch(state) {
		case AUTHORIZED:
			log(LG_DBG, "state AUTHORIZED");
			ioc.oid = DOT11_OID_EAPAUTHSTA;
			break;
		case UNAUTHORIZED:
			log(LG_DBG, "state UNAUTHORIZED");
			ioc.oid = DOT11_OID_EAPUNAUTHSTA;
			break;
	}

	ioc.cmd 	= OPSET;
	ioc.data 	= client->hwaddr;
	ioc.len 	= 6;
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if(ioctl(eapolfd, PRISMIOCTL, &ifr) < 0) {
		log(LG_ERR, "Can't set state to %d", state);
		return(-1);
	}


	client->portStatus = state;

	return(0);
}

/*
 * Get information about this client from the blob. Returns the age of
 * the client, or -1 if client is not known in the blob.
 */

int blob_get_client_age(struct client_t *client)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;
	struct obj_sta sta;

	memcpy(sta.address, client->hwaddr, 6);

	ioc.oid 	= DOT11_OID_CLIENTFIND;
	ioc.cmd 	= OPGET;
	ioc.data 	= (void *)&sta;
	ioc.len 	= sizeof(sta);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_DBG, "DOT11_OID_CLIENTFIND returned %d", r);
		return(-1);
	}

	return(sta.age);
}



/*
 * Get information about this client from the blob. Returns the 'state'
 * value of the client obj_sta, or -1 if client is not known in the blob.
 */

int blob_get_client_state(struct client_t *client)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;
	struct obj_sta sta;

	memcpy(sta.address, client->hwaddr, 6);

	ioc.oid 	= DOT11_OID_CLIENTFIND;
	ioc.cmd 	= OPGET;
	ioc.data 	= (void *)&sta;
	ioc.len 	= sizeof(sta);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_DBG, "DOT11_OID_CLIENTFIND returned %d", r);
		return(-1);
	}

	return(sta.state);
}

/*
 * Deauthenticate all clients
 */

int blob_deauthenticate_all(void)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;
	struct obj_mlme mlme;

	mlme.code = 1;
	mlme.id   = 0;
	memcpy(mlme.address, "\xff\xff\xff\xff\xff\xff", 6);

	ioc.oid 	= DOT11_OID_DEAUTHENTICATE;
	ioc.cmd 	= OPSET;
	ioc.data 	= (void *)&mlme;
	ioc.len 	= sizeof(mlme);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_WRN, "DOT11_OID_DEAUTHENTICATE returned %d", r);
		return(-1);
	}

	log(LG_INF, "All clients deauthenticated");

	return(0);
}



/*
 * Deauthenticate the given client hwaddr
 */
int blob_deauthenticate_client(struct client_t *client)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;
	struct obj_mlme mlme;

	memcpy(mlme.address, client->hwaddr, 6);
	mlme.code = 1;
	mlme.id   = -1;

	ioc.oid 	= DOT11_OID_DEAUTHENTICATE;
	ioc.cmd 	= OPSET;
	ioc.data 	= (void *)&mlme;
	ioc.len 	= sizeof(mlme);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_WRN, "DOT11_OID_DEAUTHENTICATE returned %d", r);
		return(-1);
	}

	log(LG_INF, "Client %s deauthenticated", hwaddr2string(client->hwaddr));

	return(0);
}

/*
 * SIGUSR1 signal handler, called when new station associates
 */

void blob_assoc_sighandler(int sig)
{
	handle_new_assoc();
	signal(SIGUSR1, blob_assoc_sighandler);
}

/*
 * SIGUSR2 signal handler, called when a micfailure is detected for a STA.
 * broadcast key's cannot initiate mic failures cause clients do not send
 * using the broadcast key.
 */

void blob_micfailure_sighandler(int sig)
{
	log(LG_DBG, "Received SIGUSR2, mic failure");
	micfailure = 1;
	signal(SIGUSR2, blob_micfailure_sighandler);
}




/*
 * Subscribe to a blob-trap for newly associating clients.
 */

int blob_subscribe_assoc_trap(void)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;

	ioc.oid 	= DOT11_OID_ASSOCIATEEX;
	ioc.cmd 	= OPTRAPADD;
	ioc.data 	= (void *)SIGUSR1;
	ioc.len 	= sizeof(ioc.data);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_FTL, "Can't subscribe to ASSOCIATEEX trap");
	}

	signal(SIGUSR1, blob_assoc_sighandler);

       	log(LG_DBG, "Subscribed to blob ASSOCIATE hander, SIGUSR1");

        return(0);
}

/*
 * Subscribe to a blob-trap for mic failures.
 */

int blob_subscribe_micfailure_trap(void)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;

	ioc.oid 	= DOT11_OID_MICFAILURE;
	ioc.cmd 	= OPTRAPADD;
	ioc.data 	= (void *)SIGUSR2;
	ioc.len 	= sizeof(ioc.data);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_FTL, "Can't subscribe to MICFAILURE trap");
	}

	signal(SIGUSR2, blob_micfailure_sighandler);

       	log(LG_DBG, "Subscribed to blob MICFAILURE hander, SIGUSR2");

        return(0);
}




/*
 * Get the last MLME structure from the prismeth driver
 */

int blob_get_mlmeex(struct obj_mlmeex *mlmeex)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;

	ioc.oid 	= DOT11_OID_ASSOCIATE;
	ioc.cmd 	= OPGETMLME;
	ioc.data 	= (void *)mlmeex;
	ioc.len 	= sizeof(ioc.data);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_FTL, "Can't do OPGETMLME");
	}
        return(0);
}


/*
 * Get the last STA structure from the prismeth driver
 */

int blob_get_sta(struct obj_sta *sta)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;

	ioc.oid 	= DOT11_OID_ASSOCIATE;
	ioc.cmd 	= OPGETSTA;
	ioc.data 	= (void *)sta;
	ioc.len 	= sizeof(ioc.data);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_FTL, "Can't do OPGETSTA");
	}

       	log(LG_DBG, "done OPGETSTA");

        return(0);
}



/*
 * Get the privacy_invoked setting from the blob
 */

int blob_get_privacy_invoked(void)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	long pi = 0;
	int r;

	ioc.oid 	= DOT11_OID_PRIVACYINVOKED;
	ioc.cmd 	= OPGET;
	ioc.data 	= (void *)&pi;
	ioc.len 	= sizeof(pi);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_FTL, "Can't do get DOT11_OID_PRIVACYINVOKED");
		return r;
	}
	return pi;
}



/*
 * Get the number of sent broadcast packets
 */

int blob_get_mcast_packets(void)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	long pkts = 0;
	int r;

	ioc.oid 	= GEN_OID_MSDUTXMULTICAST;
	ioc.cmd 	= OPGET;
	ioc.data 	= (void *)&pkts;
	ioc.len 	= sizeof(pkts);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		printf("Can't do GEN_OID_NSDUTXMULTICAST : %s\n", strerror(errno));
		return -1;
	}
	return pkts;
}

/*
 * Set the privacy_invoked setting in the blob
 */

int blob_set_privacy_invoked(long value)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	long pi = 0;
	int r;

	ioc.oid 	= DOT11_OID_PRIVACYINVOKED;
	ioc.cmd 	= OPSET;
	ioc.data 	= (void *)&value;
	ioc.len 	= sizeof(pi);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r = ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_FTL, "Can't do set DOT11_OID_PRIVACYINVOKED");
	}

	if(pi == 1) {
	        return(1);
	} else {
		return(0);
	}
}

/*
 * set per-station key for the given client
 */
int blob_set_persta_key(struct client_t *client, int keyid)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	int r;
	struct obj_stakey stakey;

	log(LG_DBG, "Setting client->tx_session key in key %d length %d", keyid, client->session_key_len);

#ifdef DEBUG_PRINTKEYS
	dump_bindata("Client tx_session_key", client->tx_session_key, client->session_key_len);
#endif

	memcpy(stakey.address, client->hwaddr, 6);
	stakey.keyid = keyid;
	stakey.length = client->session_key_len  /*13*/;
	if (stakey.length == PTK_LEN)
	{
	    stakey.type = DOT11_PRIV_TKIP;
	}
	else
	{
	    stakey.type = DOT11_PRIV_WEP;
        }
	stakey.operation = DOT11_STAKEY_SET_KEY;
	memcpy(stakey.key, client->tx_session_key, stakey.length);

	ioc.oid 	= DOT11_OID_STAKEY;
	ioc.cmd 	= OPSET;
	ioc.data 	= (void *)&stakey;
	ioc.len 	= sizeof(stakey);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_ERR, "Can't set per-sta key : DOT11_OID_STAKEY returned %d", r);
	}

	// Set the keyid to use for this client

	stakey.keyid = (keyid << 4);
	stakey.operation = DOT11_STAKEY_SET_KEYID;

	ioc.oid 	= DOT11_OID_STAKEY;
	ioc.cmd 	= OPSET;
	ioc.data 	= (void *)&stakey;
	ioc.len 	= sizeof(stakey);
	ifr.ifr_data 	= (caddr_t) &ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	r=ioctl(eapolfd, PRISMIOCTL, &ifr);

	if(r<0) {
		log(LG_ERR, "Can't set per-sta keyid : DOT11_OID_STAKEY returned %d", r);
	}

	return(0);
}




/*
 * set multicast/broadcast key
 */

int blob_set_bcast_key(char *keybuf, int keylen, int keyid)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	struct obj_key key;
	unsigned long key_index;
	unsigned long key_index_read;

	log(LG_DBG, "Setting broadcast key %d len=%d", keyid, keylen);

#ifdef DEBUG_PRINTKEYS
	dump_bindata("Broadcast key", keybuf, keylen);
#endif

	memcpy(key.key, keybuf, keylen);
	key.length = keylen;
	key.domain = 0;
	if (key.length == PTK_LEN)
	{
	    key.type  = DOT11_PRIV_TKIP;
	}
	else
	{
	    key.type  = DOT11_PRIV_WEP;
    	}
	ioc.cmd = OPSET;
	ioc.oid = DOT11_OID_DEFKEY1 + keyid;
	ioc.len = sizeof(struct obj_key);
	ioc.data = (unsigned char *)&key;
	ifr.ifr_data = (caddr_t)&ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if (ioctl(eapolfd, PRISMIOCTL, &ifr) < 0)
	{
		log(LG_ERR, "SetKey ioctl to wireless interface driver failed" );
		return -1;
	}


	log(LG_DBG, "Setting key %d as default broadcast key", keyid);

	key_index = keyid;

	ioc.cmd = OPSET;
	ioc.oid = DOT11_OID_DEFKEYID;
	ioc.len = sizeof(key_index);
	ioc.data = (unsigned char *)&key_index;
	ifr.ifr_data = (caddr_t)&ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if (ioctl(eapolfd, PRISMIOCTL, &ifr) < 0)
	{
		log(LG_ERR, "SetKeyid ioctl to wireless interface driver failed" );
		return -1;
	}

	// Read back the keyid to double check the MVC.. Can be removed later on

	ioc.cmd = OPGET;
	ioc.oid = DOT11_OID_DEFKEYID;
	ioc.len = sizeof(key_index);
	ioc.data = (unsigned char *)&key_index_read;
	ifr.ifr_data = (caddr_t)&ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if (ioctl(eapolfd, PRISMIOCTL, &ifr) < 0)
	{
		log(LG_ERR, "SetKeyid ioctl to wireless interface driver failed" );
		return -1;
	}

	if(key_index_read != key_index) {
		log(LG_ERR, "Key index not proberly set in MVC (set %d read %d)",
			key_index, key_index_read);
	}


	return(0);
}


/*
 * Retrieve key number <keyid> from the blob and place it into <keybuf>
 * with max size <keylen>. Return's actual key length.
 */

int blob_get_bcast_key(int keyid, char *keybuf, int keylen)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;
	struct obj_key key;

	log(LG_DBG, "Get bcast key %d from MVC", keyid);

	key.domain = 0;
	ioc.cmd = OPGET;
	ioc.oid = DOT11_OID_DEFKEY1 + keyid;
	ioc.len = sizeof(struct obj_key);
	ioc.data = (unsigned char *)&key;
	ifr.ifr_data = (caddr_t)&ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if (ioctl(eapolfd, PRISMIOCTL, &ifr) < 0)
	{
		log(LG_ERR, "SetKey ioctl to wireless interface driver failed" );
		return -1;
	}

	memcpy(keybuf, key.key, (keylen < key.length) ? keylen : key.length);

	return(key.length);
}


int blob_set_attachment(struct obj_attachment *oa, int len)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;

	ioc.cmd = OPSET;
	ioc.oid = DOT11_OID_ATTACHMENT;
	ioc.len = len;
	ioc.data = (unsigned char *)oa;
	ifr.ifr_data = (caddr_t)&ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if (ioctl(eapolfd, PRISMIOCTL, &ifr) < 0)
	{
		log(LG_ERR, "ATTACHEMENT ioctl to wireless interface driver failed" );
		return -1;
	}

	return(0);
}


/*
 * Retrieve station statistincs from the blob and place it into stastat struct
 */
int blob_get_stastat(struct obj_stastat *stat)
{
	struct prism_ioctl ioc;
	struct ifreq ifr;

	ioc.cmd = OPGET;
	ioc.oid = DOT11_OID_STASTAT;
	ioc.len = sizeof(struct obj_stastat) + stat->size - 1;
	ioc.data = (unsigned char *)stat;
	ifr.ifr_data = (caddr_t)&ioc;
	strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));

	if (ioctl(eapolfd, PRISMIOCTL, &ifr) < 0)
	{
		log(LG_ERR, "STASTAT ioctl to wireless interface driver failed" );
		return -1;
	}

	return(0);
}

// eof blobfuncs.c




