/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * Functions for handling of list of clients
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <net/ethernet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>
#include "prismoids.h"
#include "misc.h"
#include "log.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "linux_mvc_oids.h"
#include "blobfuncs.h"
#endif
#include "client.h"
#include "ie.h"
#include "radius.h"
#include "state_pae.h"
#include "config.h"
#include "acl.h"
#include "state_gtk.h"
#include "state_ssn.h"


#if (PAED_MSSID == 1)

/* the config array pointer */
extern struct config_t *config;
#else

extern struct config_t config[MAX_INTERFACES];
#endif  /* #if (PAED_MSSID == 1) */


extern struct obj_sta sta;		// obj_sta struct for prismeth ioctls
/*Code added for RADIUS Accounting*/
/*******Sukanya Bhattacharya*******/
/********** Code Start ************/
#if (RAC_SUPPORT == RAC_TRUE)
extern rac_status;    
#endif
/********    Code end  ***********/
struct client_t *clientlist = NULL;

#if (PAED_MSSID == 1)
 
void client_add_to_config(struct config_t *configp,
	                  struct client_t *clientp);


void client_delete_from_config(struct client_t *clientp);
#endif
	
struct client_t *client_add(char *hwaddr, struct config_t *config)
{
	struct client_t *client;
	struct client_t *tmp;

	// First check if this client is not already in the
	// list. If it is, return it's pointer.

	client = client_find_by_hwaddr(hwaddr);
	if(client != NULL) {
		log(LG_DBG, "Client %s is already in the list", hwaddr2string(client->hwaddr));
	} else {

		client = malloc(sizeof(struct client_t));

		if(client == NULL) {
			log(LG_FTL, "Can't malloc client client. aborting");
		}

		bzero(client, sizeof(struct client_t));

		if(clientlist == NULL) {
			clientlist = client;
			client->prev = NULL;
			client->next = NULL;
		} else {
			tmp = clientlist;
			clientlist = client;
			tmp->prev = client;
			client->next = tmp;
		}
#if (PAED_MSSID == 1)

                /* This is a new client. Add to the VSS client list */
                client_add_to_config(config, client);
#endif
        }

	memcpy(client->hwaddr, hwaddr, sizeof(client->hwaddr));

	/* set cconfig pointer to the correct config */
	client->cconfig = config;

	// fill in some data
	client->doesSSN = 0;
	client->does1X  = 0;

	memset(client->tx_session_key, 0x00, SESSION_KEY_LEN);
	memset(client->rx_session_key, 0x00, SESSION_KEY_LEN);
	client->session_key_len = 0;
	
	strncpy(client->identity, "-undef-", MAX_EAP_IDENTITY_LEN);
	client->radius_state_len = 0;
	client->eap_pkt = NULL;
	client->eap_pkt_len = 0;
	client->rxResp = 0;

	client->radius_id = 1;
	client->radiusfd = -1;
	client->rac.rac_fd = -1;
	client->rac.rac_id = 1;
	
	client->secure = 0;
	client->idleTime = 0;
	client->toBeRemoved = 0;

	client->intFailWhile = 0;
	client->update_gtk = 0;

	// for sanity checks

	client->magic  = 0x4242;
	client->check1 = 0x55;
	client->check2 = 0x55;
	client->check3 = 0x55;
	client->check4 = 0x55;
	client->check5 = 0x55;

	return(client);
}

int client_init(struct client_t *client, struct client_t *assoc_client)
{

	/* Copy clients config reference */
	client->cconfig = assoc_client->cconfig;

	client->doesSSN = assoc_client->doesSSN;

	// If dot1X is enabled enable .1x for this client
	if(client->cconfig->enable1X)
	{
		client->does1X = 1;
	}
	// If SSN without SSN-PSK is enabled AND client used SSN_IE
	// also enable .1x
	if (client->cconfig->enableSSN && !client->cconfig->enableSSNPSK && assoc_client->doesSSN)
	{
		client->does1X = 1;
	}

	if(client->doesSSN) {
	    	/* Copy all IE releated client information */
	    	memcpy((struct ie_sta *)&client->client_wpa_ie,
	    	       (struct ie_sta *)&assoc_client->client_wpa_ie,
	    	       sizeof(struct ie_sta));
	}

#if (RAC_SUPPORT == RAC_TRUE)

	if (RAC_STATUS_ENABLE == rac_status){
		/* Set RADIUS accounting state to stopped */
		client->rac.acct_state = RAC_STATE_STOPPED;

		/* Set RADIUS accounting event to none */
		client->rac.acct_event = RAC_EVENT_NONE;
	}
#endif


	return(0);
}


int client_malloc_eap_pkt(struct client_t *client)
{
	if (client == NULL)
		log(LG_DBG, "Tried to malloced client->eap_pkt while client = NULL");

	if(client->eap_pkt == NULL) {

		client->eap_pkt = (char *)malloc(1540);
		if(client->eap_pkt == NULL) {
			log(LG_FTL, "Can't malloc for eap_pkt");
		}

		log(LG_DBG, "Malloced client->eap_pkt at %p", client->eap_pkt);
	}

	return(0);
}


int client_free_eap_pkt(struct client_t *client)
{
	if(client->eap_pkt != NULL) {
		memset(client->eap_pkt, 0, 1540);
		free(client->eap_pkt);
	} else {
		log(LG_WRN, "Can't free client->eap_buf, its NULL already");
	}

	client->eap_pkt = NULL;

	log(LG_DBG, "Freed eap_pkt");

	return(0);
}


int client_del(struct client_t *client)
{
	if(client->prev) client->prev->next = client->next;
	if(client->next) client->next->prev = client->prev;

	if(client == clientlist) {
		clientlist = client->next;
	}

	client_free_eap_pkt(client);

	memset(client, 0, sizeof(struct client_t));
	free(client);

	log(LG_INF, "Client removed from list");

	return(0);
}



struct client_t *client_find_by_hwaddr(char *hwaddr)
{
	struct client_t *tmp;

	tmp=clientlist;

	while(tmp != NULL) {
		if(memcmp(hwaddr, tmp->hwaddr, sizeof(tmp->hwaddr)) == 0) {
				return(tmp);
		}
		tmp = tmp->next;
	}

	return NULL;
}




void client_printclient(struct client_t *client)
{

	log(LG_INF, "--------------------------------------------------------");
	log(LG_INF, "identity     : %s", client->identity);
	log(LG_INF, "hwaddr       : %02x-%02x-%02x-%02x-%02x-%02x",
			client->hwaddr[0],client->hwaddr[1],client->hwaddr[2],
			client->hwaddr[3],client->hwaddr[4],client->hwaddr[5]);
	log(LG_INF, "Does 1X      : %d", client->does1X);
	log(LG_INF, "Does SSN     : %d", client->doesSSN);
	if (client->doesSSN)
	{
	    log(LG_INF, "Mcast cipher : %d", client->client_wpa_ie.mcast);
	    log(LG_INF, "Ucast cipher : %d", client->client_wpa_ie.ucast);
	    log(LG_INF, "Authe suite  : %d", client->client_wpa_ie.auth);
	    log(LG_INF, "Groupkey only: %d", client->client_wpa_ie.groupkey_only);
	    log(LG_INF, "NOF Seq count: %d", client->client_wpa_ie.nof_seq_counters);
	}
	
	log(LG_INF, "ucast cypher : %d", client->client_wpa_ie.ucast);

/*
	log(LG_INF, "paestate     : %d", client->paestate);
	log(LG_INF, "bestate      : %d", client->bestate);
	log(LG_INF, "eap_pkt      : %p", client->eap_pkt);
	log(LG_INF, "eap_pkt_len  : %d", client->eap_pkt_len);
	log(LG_INF, "rad_seclen   : %d", client->radius_secret_len);
	log(LG_INF, "rad_statelen : %d", client->radius_state_len);
	log(LG_INF, "currentId    : %d", client->currentId);
	log(LG_INF, "idFromServer : %d", client->idFromServer);
	log(LG_INF, "aWhile       : %d", client->aWhile-time(NULL));
	log(LG_INF, "txWhen       : %d", client->txWhen-time(NULL));
	log(LG_INF, "quietWhile   : %d", client->quietWhile-time(NULL));
	log(LG_INF, "reqCount     : %d", client->reqCount);
	log(LG_INF, "reAuthCount  : %d", client->reAuthCount);
	log(LG_INF, "aReq         : %d", client->aReq);
	log(LG_INF, "rxResp       : %d", client->rxResp);
*/

}



void client_listclients(void)
{
	struct client_t *tmp;

	tmp=clientlist;

	while(tmp != NULL) {
		client_printclient(tmp);

		tmp = tmp->next;
	}
}


int client_chksan(struct client_t *client)
{
	if(client == NULL) {
		log(LG_WRN, "Sanity check : client = NULL");
		return(0);
	}

	if(client->magic  != 0x4242) log(LG_FTL, "Sanity check failed, magic = %02X", client->magic);
	if(client->check1 != 0x55)   log(LG_FTL, "Sanity check failed on check1");
	if(client->check2 != 0x55)   log(LG_FTL, "Sanity check failed on check2");
	if(client->check3 != 0x55)   log(LG_FTL, "Sanity check failed on check3");
	if(client->check4 != 0x55)   log(LG_FTL, "Sanity check failed on check4");
	if(client->check5 != 0x55)   log(LG_FTL, "Sanity check failed on check5");

	return(0);
}


/*
 * If the toBeRemoved flag is on for this client:
 * -Update Group key update counter
 * -Deauthenticate client
 * -Close port status
 * -Delete structure
 * If the config sais so update all group keys.
 * Returns: 1 if client is removed otherwise 0.
 */
int client_toberemoved(struct client_t *client)
{
#if (RAC_SUPPORT == RAC_TRUE)
	if ((RAC_STATUS_DISABLE == rac_status)
         || ((RAC_STATUS_ENABLE == rac_status) 
              && (RAC_STATE_STOPPED == client->rac.acct_state))){
#endif              
	
    if(client->toBeRemoved) {

	    if(client->update_gtk) client->cconfig->cgtk->GKeyDoneStations--;

	    if (blob_get_client_state(client) >= 0)
	    {
		/* Client is still in the MVC */
	        blob_portstatus_set(client, UNAUTHORIZED);
	        blob_deauthenticate_client(client);
	    }

	    if(client->cconfig->strictUpdate) {
		    gtk_setkeys(client->cconfig);
	    	    if(client->update_gtk) client->cconfig->cgtk->GKeyDoneStations--;
	    }

#if (PAED_MSSID == 1)

          log(LG_DBG, "MSSID: In client_toberemoved. About to delete client from VSS client list.");

          /* delete client from the config list before deleting the 
           * client node in the client list
           */
          client_delete_from_config(client);

#endif /*#if (PAED_MSSID == 1)*/

            // DEBUG code
            log(LG_DBG, "MSSID: Deleting client from the global client list.");

	    client_del(client);

            // DEBUG code
            log(LG_DBG, "MSSID: Client deleted from the global client list.");

	    return 1;
    }

#if (RAC_SUPPORT == RAC_TRUE)
  }
#endif
    return 0;
}

#ifdef SMAC
void handle_disassoc(struct obj_mlme *mlmeex)
#else
void handle_disassoc(struct obj_mlmeex *mlmeex)
#endif
{
    struct client_t *client;

    client = clientlist;

    while(client)
    {

	if (memcmp(mlmeex->address, client->hwaddr, 6) == 0)
        {
            log(LG_INF, "client toBeRemoved");

	    client->toBeRemoved = 1;
#if (RAC_SUPPORT == RAC_TRUE)
	    /*
	     *Client has sent Dissociate or De-Authenticate 
	     *Send accounting stop if accounting enabled
	     */    
	    if ((RAC_STATUS_ENABLE == rac_status) &&
		(RAC_STATE_STOPPED != client->rac.acct_state) &&
		(RAC_STATE_STOP_SENT != client->rac.acct_state))
	    {
	          client->rac.acct_event = RAC_EVENT_SERVICE_STOP;
                  client->rac.terminate_cause =RAC_TERMINATE_USR_REQ;
	    } 
#endif
	}

	client = client->next;
    }
}


#ifdef SMAC
void handle_new_assoc(struct config_t *config, struct obj_mlme *mlmeex, int reassoc)
{
    struct client_t *client;
    client_struct assoc_client;
    int i;
    int cipher_match;

    memset(&assoc_client, 0, sizeof(struct client_t));

    assoc_client.cconfig = config;

    // Check if a new station did associate. The newassoc is a volatile
    // set by the trap's sighandler.
    log(LG_INF, "reassoc = %d\n",reassoc);
    if (mlmeex->state == DOT11_STATE_ASSOCING)
    {
#else
#ifdef BLOB_V1
void handle_new_assoc(struct config_t *config, struct obj_mlmeex *mlmeex)
{
    int reassoc = 0;
    struct client_t *client;
    client_struct assoc_client;
    int i;
    int cipher_match;

    memset(&assoc_client, 0, sizeof(struct client_t));

    assoc_client.cconfig = config;

    // Check if a new station did associate. The newassoc is a volatile
    // set by the trap's sighandler.

    if (mlmeex->state == DOT11_STATE_ASSOCING ||
    	mlmeex->state == DOT11_STATE_REASSOCING)
    {
	if (mlmeex->state == DOT11_STATE_REASSOCING)
		reassoc = 1;
#endif
#ifdef BLOB_V2
void handle_new_assoc(struct config_t *config, struct obj_mlmeex *mlmeex, int reassoc)
{
    struct client_t *client;
    client_struct assoc_client;
    int i;
    int cipher_match;

    memset(&assoc_client, 0, sizeof(struct client_t));

    assoc_client.cconfig = config;

    // Check if a new station did associate. The newassoc is a volatile
    // set by the trap's sighandler.
    log(LG_INF, "reassoc = %d\n",reassoc);
    if (mlmeex->state == DOT11_STATE_ASSOCING)
    {
#endif
#endif	/* SMAC */

	if(ie_parse_assocreq(mlmeex, &assoc_client, reassoc) >= 0)
	{

            // DEBUG Code
            log(LG_DBG, "MSSID: Client IE parsed succesfully. Creating new client struct");

	    /* Allocate a zeroed client structure fill in MAC address */
	    if (reassoc == 1)
	    {
	    	blob_get_client_by_aid(assoc_client.cconfig, mlmeex->id, &sta);
	    	client = client_add(sta.address, config);
	    }
	    else
	    	client = client_add(mlmeex->address, config);

	    /* Initialise client structure, also take possible values from IE parser */
	    client_init(client, &assoc_client);

        // DEBUG Code
        log(LG_DBG, "MSSID: Client struct created and initialized");

	    /*
	     * Set the per-sta key length this client will be using
	     */
	     
	    if(client->client_wpa_ie.ucast == UCAST_CIPHER_TKIP)      client->persta_key_length = 32;
	    if(client->client_wpa_ie.ucast == UCAST_CIPHER_AES_VAR_2) client->persta_key_length = 16;

	    /* Check if ACL should be checked */

	    /*
	     * For WPA-PSK clients cleck ACL but leave portstate as it is.
	     * port will be opened when handshake was succesfull.
	     */
	    if (client->doesSSN && client->cconfig->enableSSNPSK)
	    {
		if (acl_list_check(client, 0) == UNAUTHORIZED)
		{

                    // DEBUG Code
                    log(LG_INF, "MSSID: WPAPSK client failed ACL check");

		    mlmeex->code = DOT11_SC_UNSPECIFIED;
                    mlmeex->size = 0;
		    blob_associate_client(mlmeex, assoc_client.cconfig, reassoc);
		    client->toBeRemoved = 1;
		    client_toberemoved(client);
		    return;
		}
	    }
	    /*
	     * Else if only does1X isn't set
	     * Consult ACL or use default settings and control clients
	     * portstate directly.
	     */
	    else if (!client->does1X)
	    {

		if (acl_list_check(client, 1) == UNAUTHORIZED)
		{
                    // DEBUG Code
                    log(LG_INF, "MSSID: 8021X client failed ACL check");

		    mlmeex->code = DOT11_SC_UNSPECIFIED;
                    mlmeex->size = 0;
		    blob_associate_client(mlmeex, assoc_client.cconfig, reassoc );
		    client->toBeRemoved = 1;
		    client_toberemoved(client);
		    return;
		}

#if (RAC_SUPPORT == RAC_TRUE)
		else 
		{
			/*
			 *WEP client passed in ACL checking. Point of service 
			 *start. Need to send accounting start if accounting 
			 *is enabled.
			 */    
			if (RAC_STATUS_ENABLE == rac_status){
				client->rac.acct_event = 
					RAC_EVENT_SERVICE_START;   
				/* Authentication type is local */ 
				client->rac.acct_auth = RAC_AUTH_LOCAL;
				log(LG_DBG, "RAC_EVENT_SERVICE_START");
			}
		}
#endif
	    }

	    if(client->doesSSN) {
	    
		    /*
		     * Check of the client's cipher suite matches one of the suites
		     * we advertize in the beacon
		     */
	     
		    cipher_match = 0;
		    for( i=0; i < client->cconfig->wpa_ie.ucast_count; i++) {
		    	if( client->client_wpa_ie.ucast == client->cconfig->wpa_ie.ucast[i]) {
		    		cipher_match = 1;
		    	}
		    } 
		    
		    if(cipher_match == 0) {
	    		log(LG_WRN, "Client's IE cipher does not match one of our beacon iE cyphers");
	    	
			    mlmeex->code = DOT11_SC_UNSPECIFIED;
             	  	     mlmeex->size = 0;
			    blob_associate_client(mlmeex, assoc_client.cconfig, reassoc );
			    client->toBeRemoved = 1;
			    client_toberemoved(client);
			    return;
	    	}
	    }
	    
	    /* else ACL shoud not be checked cause that will be handled
	     * by the radius server or client will possibly fall back
	     * and be checked then.
	     */

// DEBUG code - Look at this closely later on.
// Looks like PAED DOES need to send success association response

// #if (PAED_MSSID == 1)
//      	   /* Do not send success associate response in case of MSSID */
// #else
	    /* Send succesful authentication to MVC */
	    mlmeex->code = DOT11_SC_SUCCESSFUL;
            mlmeex->size = 0;
	    blob_associate_client(mlmeex, assoc_client.cconfig, reassoc );

// #endif /* #if (PAED_MSSID == 1) */

 	    log(LG_INF, "Accepted association, hwaddr = %s", hwaddr2string(client->hwaddr));
 	    client_printclient(client);

	    // Initialize the needed statemachines
	    if(client->does1X) {
		    log(LG_INF, "Client %s does 1X", hwaddr2string(client->hwaddr));
		    pae_initialize(client);
	    }

	    if(client->doesSSN) {
		    log(LG_INF, "Client %s does WPA", hwaddr2string(client->hwaddr));
		    /* Remove the clients possible TKIP key, cause this results in 
		     * failed handshake 
		     */
		    blob_set_persta_key(client, PERSTA_KEYID );
		    ssn_initialize(client);
	    }

	}
	else
	{
#if (PAED_MSSID == 1)
           /* We SHOULD NOT get a IE parsing error in case of MSSID, as the IE 
      	    * is already validated at the SoftMAC layer. 
            * If we do reach here, log the case. 
            * TODO: Send the disassoc message during testing. But it needs to be removed finally.
            */
            log(LG_WRN, "MSSID: Client IE parsing failure. This should not have 
                   occurred as the IE should've been validated at SoftMAC.");
#endif
	/*
	 * The STA didn't pass IE checking don't add structure to save mem.
	 */
	    mlmeex->code = DOT11_SC_UNSUPPORTED_AUTH_ALG;
            mlmeex->size = 0;
	    blob_associate_client(mlmeex, assoc_client.cconfig, reassoc );
	}
    }
    else{

        // DEBUG Code - Braces for this else not there earlier
        log(LG_DBG, "MSSID: mlme state is not DOT11_STATE_ASSOCING. Current value - %d", 
             mlmeex->state);


	log(LG_DBG, "Association notify, hwaddr = %s", hwaddr2string(mlmeex->address));
    }
}

void handle_mic_failure(void)
{
    struct client_t *client;

    log(LG_ERR, "MIC failure on %s", hwaddr2string(sta.address));

    client = client_find_by_hwaddr(sta.address);
    if(client) {
	    ssn_integrityfailure(client);
	    /* Check if the client should be removed */
	    client_toberemoved(client);

    } else {
	    log(LG_ERR, "Micfailure for unknown client")
    }
}


#if (PAED_MSSID == 1)

/*
 * Add the client node to the client list for this VSS (config structure).
 */
 
void client_add_to_config(struct config_t *configp, 
		          struct client_t *clientp)
{
    client_ptr_node_t *client_nodep;
    client_ptr_node_t *temp;

    client_nodep = (client_ptr_node_t *)malloc(sizeof (client_ptr_node_t));
    if (NULL == client_nodep){
       log(LG_FTL, "MSSID: Unable to allocate memory for adding node to 
                     client list in the config structure");  
       return;
   }  
   /* pointer to the client node */
   client_nodep->client_ptr = clientp;

   /* add to the list */

   log(LG_DBG, "MSSID: Adding client to VSS %s. Client ptr is %p", 
        configp->vss_ifname, clientp);

   // DEBUG code
   if (NULL == configp->client_ptr_list) {
      log(LG_DBG, "MSSID: VSS client list is Empty");
   }
   else {
      log(LG_DBG, "MSSID: VSS client list is NOT Empty");
   } 

   client_nodep->next = configp->client_ptr_list;
   configp->client_ptr_list = client_nodep;

   return;
}

/*
 * Delete the client node from the client list for this VSS (config 
 * structure).
 */
 
void client_delete_from_config(struct client_t *clientp)
{
    struct config_t *configp;
    char *hwaddr;

    client_ptr_node_t *client_nodep;
    client_ptr_node_t *current;
    client_ptr_node_t *prev;

    /* the client's VSS */
    configp = clientp->cconfig;

    hwaddr = clientp->hwaddr;

    current = configp->client_ptr_list;
    prev = configp->client_ptr_list;

    // DEBUG code
    log(LG_DBG, "MSSID: Searching for client with hwaddr %s in VSS %s", 
               hwaddr2string(hwaddr), configp->vss_ifname);

    log(LG_DBG, "MSSID: Current start ptr is %p", current);
    log(LG_DBG, "MSSID: Client ptr is %p", current->client_ptr);

    while(NULL != current){

          // DEBUG Code
          log(LG_DBG, "MSSID: found client with hwaddr %s", 
               hwaddr2string(current->client_ptr->hwaddr));

	  if (memcmp(hwaddr, current->client_ptr->hwaddr, 
                            sizeof(current->client_ptr->hwaddr)) == 0) {

              if (current == configp->client_ptr_list){ 
                  configp->client_ptr_list = NULL;
              }
              else{
                  prev->next = current->next;    
              }
              free(current);
		return;
	  }
        prev = current;
        current = current->next;
   }
   return;
}


#endif /* #if (PAED_MSSID == 1) */

