/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
#ifndef _CLIENT_H_
#define _CLIENT_H_

#include <sys/socket.h>
#include <sys/types.h>

#include "paed.h"
#include "rac.h"

#if (!RAC_UT)

#ifndef SMAC 
#include "linux_mvc_oids.h"
#endif
#include "prismoids.h"
#else /*(!RAC_UT)*/
#include "netlink_func.h"
#endif /*(!RAC_UT)*/

#include "rc4.h"
#include "mssid.h"

#define INTFAIL_GROUP 0
#define INTFAIL_PSK   1

enum paestate_t {
	INITIALIZE,
	DISCONNECTED,
	CONNECTING,
	AUTHENTICATING,
	AUTHENTICATED,
	ABORTING,
	HELD
};

enum bestate_t {
	REQUEST,
	RESPONSE,
	SUCCESS,
	FAIL,
	TIMEOUT,
	IDLE
};

enum ssnstate_t {
	SSN_DISCONNECT,
	SSN_DISCONNECTED,
	SSN_INITIALIZE,
	SSN_AUTHENTICATION,
	SSN_INITPMK,
	SSN_INITPSK,
	SSN_PTKSTART,
	SSN_PTKINITNEGOTIATING,
	SSN_PTKINITDONE,
	SSN_UPDATE_GTK,
	SSN_UPDATEKEYS,
	SSN_INTEGRITYFAILURE,
	SSN_KEYUPDATE
};

#define WPA_IE_MAX_LEN 32

struct ie_sta {
    unsigned char assoc_ie[WPA_IE_MAX_LEN];
    unsigned int assoc_ie_len;
    unsigned int mcast;
    unsigned int ucast;
    unsigned int auth;
    unsigned int groupkey_only;
    unsigned int nof_seq_counters;
};


// the check# vars are used for struct-sanity checks by the
// client_chksanity function to debug overwritten buffers.

typedef struct client_t {
	int magic;

	unsigned char hwaddr[6];
	char identity[MAX_EAP_IDENTITY_LEN+1];
	char check1;

	int doesSSN;
	int does1X;

	enum paestate_t paestate;
	enum bestate_t bestate;
	enum ssnstate_t ssnstate;

	// Key stuff

	unsigned char tx_session_key[SESSION_KEY_LEN];
	unsigned char rx_session_key[SESSION_KEY_LEN];
	int session_key_len;

	// EAP(OL) packet

	int eap_pkt_len;
	char *eap_pkt;
	unsigned long replay_counter_low;
	unsigned long replay_counter_high;
	char check2;

	unsigned long andere_replay_counter_low;
	unsigned long andere_replay_counter_high;

	// radius related information

	struct sockaddr radius_salocal;
	struct sockaddr radius_saremote;
	int radiusfd;
	int radius_id;

	char radius_vector[16];

	int radius_secret_len;
	char radius_secret[MAX_RADIUS_SECRET_LEN];
	char check3;

	int radius_state_len;
	char radius_state[MAX_RADIUS_STATE_LEN];
	char check4;

	/*Code added for RADIUS Accounting*/
	/*******Sukanya Bhattacharya*******/
	/***********Code Start*************/

#if (RAC_SUPPORT == RAC_TRUE)
    	struct rac_t  rac ;
#endif
	/*********** Code End *************/
		
	int idleTime;
	int toBeRemoved;

	 /*Code added for RADIUS Accounting*/
	 /*******Sukanya Bhattacharya*******/
	 /***********Code Start*************/

#if (RAC_SUPPORT == RAC_TRUE)
	
	int  terminationAction; /*  This is an attribute in access accept whose
				    value is either terminate or re authenticate
				    specifying the action  after re_authenticat
				    ion timeout	*/

#endif /*(RAC_SUPPORT == RAC_TRUE)*/
	 /*********** Code End *************/
	
	char secure;        // 1 if PTK is installed 0 if PTK isn't installed

	// These statemachine vars are from the 802.1x 1EEE Draft

	int currentId;
	int idFromServer;
	int aWhile;
	int reqCount;
	int reAuthCount;
	int reAuthWhen;
	int aReq;
	int rxResp;
	int txWhen;
	int quietWhile;
	int portStatus;

	// These are vars from the SSN Draft

	char ANonce[NONCE_LEN];
	char SNonce[NONCE_LEN];

	int EAPOLKeyRecvd;
	int timeOutEvt;
	int timeOutCtr;

	char eapolkey_enckey[16];
	char eapolkey_MICkey[16];

	rc4_key	rc4key;

	char PMK[PMK_LEN];

	int intFailWhile;

	int update_gtk;
	
	int persta_key_length;		/* The length of the per-sta key, depensd on the cipher used */

	// The SSN IE in the assoc.req.
	struct ie_sta client_wpa_ie;

	struct config_t  *cconfig;

#if (PAED_MSSID == 1) 
     int vssid;  /* the vssid for this client */
#endif

	char check5;
	struct client_t *prev;
	struct client_t *next;
} client_struct;

// proto's

struct client_t *client_add(char *hwaddr, struct config_t *config);
int client_del(struct client_t *client);
int client_malloc_eap_pkt(struct client_t *client);
int client_free_eap_pkt(struct client_t *client);
struct client_t *client_find_by_hwaddr(char *hwaddr);
void client_printclient(struct client_t *tmp);
void client_listclients(void);
int client_chksan(struct client_t *client);
int client_toberemoved(struct client_t *client);
#if (!RAC_UT)
#ifdef SMAC
void handle_disassoc(struct obj_mlme *mlmeex);
void handle_new_assoc(struct config_t *config, struct obj_mlme *mlmeex, int reassoc);
#else
void handle_disassoc(struct obj_mlmeex *mlmeex);
#ifdef BLOB_V1
void handle_new_assoc(struct config_t *config, struct obj_mlmeex *mlmeex);
#endif


#if (PAED_MSSID == 1)

void client_add_to_config(config_t *configp, client_t *clientp);
void client_delete_from_config(struct client_t *clientp);

#endif /* #if (PAED_MSSID == 1) */


#ifdef BLOB_V2
void handle_new_assoc(struct config_t *config, struct obj_mlmeex *mlmeex, int reassoc);
#endif
#endif 	/* SMAC */
#endif /*(!RAC_UT)*/

void handle_mic_failure(void);

#endif	/* _CLIENT_H */
