/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * The command interface handling.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "misc.h"
#include "log.h"
#include "config.h"
#include "client.h"
#include "cmd.h"
#include "paed.h"


struct cmd_t {
	char *cmd;
	void (*fn)(FILE *, int, char **);
	char *help;
};



// protos of helper functions

void cmd_help	(FILE *f, int part, char **parts);
void cmd_liststa(FILE *f, int part, char **parts);
void cmd_listcfg(FILE *f, int part, char **parts);
void cmd_exit	(FILE *f, int part, char **parts);



// Define all command helper functions in this struct

struct cmd_t cmdlist[] = {
	{ "liststa",	cmd_liststa,	"List known stations"},
	{ "listcfg", 	cmd_listcfg,	"List current configuration"},
	{ "help",	cmd_help,	"Show help" },
	{ "exit",	cmd_exit,	"Exit command interface" },
	{ NULL,		cmd_help,	NULL }
};

// externals

extern struct client_t *clientlist;		// from client.c

#if (PAED_MSSID == 1)

/* the config array pointer */
extern struct config_t *config;
#else

extern struct config_t config[MAX_INTERFACES];
#endif  /* #if (PAED_MSSID == 1) */


// globals

int doexit;


/*
 * Open listening unix-domain socket for command interface
 */

int cmd_opensocket(char *fname)
{
	int fd;
	struct sockaddr_un unix_addr;
	int r;

	fd = socket(AF_UNIX, SOCK_STREAM, 0);

	if(fd<0) {
		log(LG_FTL, "Error opening socket : %s\n", strerror(errno));
	}

	r = unlink(fname);

	if(r == 0) {
		log(LG_DBG, "Removed existing socket");
	}

	// Fill in unix_addr struct

	memset(&unix_addr, 0, sizeof(unix_addr));
	unix_addr.sun_family=AF_UNIX;
	strncpy(unix_addr.sun_path, fname, sizeof(unix_addr.sun_path));

	r = bind(fd, (struct sockaddr *)&unix_addr, sizeof(unix_addr));

	if(r<0) {
		log(LG_FTL, "Error binding socket : %s\n", strerror(errno));
	}

	r = listen(fd, 5);

	if(r<0) {
		log(LG_FTL, "Error listen on socket : %s", strerror(errno));
	}

	log(LG_DBG, "Opened command socket %s (fd=%d)", fname, fd);

	// Ignore sigpipe to avoid program exit when remote socket closes
	// at write()

	signal(SIGPIPE, SIG_IGN);

	return(fd);
}


/*
 * Do the talking on the command interface
 */

int cmd_talk(int fd)
{
	int cfd;
        struct sockaddr_un unix_addr;
        FILE *f;
        char buf[128];
        char *part[10];
        char *p;
        int parts;
        int len;
        int i;

	len = sizeof(unix_addr);
	cfd = accept(fd, (struct sockaddr *)&unix_addr, &len);

	if(cfd<0) {
		log(LG_ERR, "Error accepting (fd %d): %s", fd, strerror(errno));
		return(-1);
	}

	if ((f = fdopen(cfd, "r+")) == NULL) {
		log(LG_ERR, "Error fdopen : %s", strerror(errno));
	}

	log(LG_INF, "command interface opened");

	fprintf(f, "%s %s command interface\n", PROGNAME, VERSION);

	doexit = 0;

	while(!doexit) {
		p = fgets(buf, sizeof(buf), f);

		if(p == NULL) {
			log(LG_ERR, "fgets sais NULL");
			break;
		}

		if((p = strchr(buf, '\n')) != NULL) *p=0;
		log(LG_DBG, "Recv : %s", buf);

		parts = split(buf, ';', part, 10);

		if(parts >= 1) {
			i=0;

			while(cmdlist[i].cmd != NULL) {

				if(strcasecmp(cmdlist[i].cmd, part[0]) == 0) {

					cmdlist[i].fn(f, parts, part+1);
					break;
				}

				i++;
			}

			// if no valid cmd found, give errormsg

			if(cmdlist[i].cmd == NULL) {

				fprintf(f, "Unknown command, try 'help'\n\n");

			}

		}

	}


	fclose(f);
//	close(cfd);

	log(LG_INF, "command interface closed");

	return(0);
}


/////////////////////////////////////////////////////////////////////////
////////////// command helper functions /////////////////////////////////
/////////////////////////////////////////////////////////////////////////

void cmd_help(FILE *f, int parts, char **part)
{
	int i=0;

	while(cmdlist[i].cmd != NULL) {
		fprintf(f, "%-16s %s\n", cmdlist[i].cmd, cmdlist[i].help);
		i++;
	}
	fprintf(f, "\n");
}


void cmd_liststa(FILE *f, int parts, char **part)
{
	struct client_t *client = clientlist;

	char *ps[7] = { "INITIALIZE", "DISCONNECTED", "CONNECTING", "AUTHENTICATING", "AUTHENTICATED", "ABORTING", "HELD"};
	char *bs[6] = { "REQUEST", "RESPONSE", "SUCCESS", "FAIL", "TIMEOUT", "IDLE" };

	fprintf(f, "hwaddr;identity;idleTime;portStatus;paeState;beState;\n");

	while(client) {

		fprintf(f, "%s;%s;%d;%s;%s;%s;\n",
			hwaddr2string(client->hwaddr),
			client->identity,
			client->idleTime,
			(client->portStatus == AUTHORIZED) ? "AUTHORIZED" : "UNAUTHORIZED",
			ps[client->paestate],
			bs[client->bestate]);

		client = client->next;
	}

	fprintf(f, "\n");
}


void cmd_listcfg(FILE *f, int parts, char **part)
{

	fprintf(f, "suppTimeout;%d;\n", config.suppTimeout);
	fprintf(f, "serverTimeout;%d;\n", config.serverTimeout);
	fprintf(f, "txPeriod;%d;\n", config.txPeriod);
	fprintf(f, "reAuthPeriod;%d;\n", config.reAuthPeriod);
	fprintf(f, "reAuthMax;%d;\n", config.reAuthMax);
	fprintf(f, "quietPeriod;%d;\n", config.quietPeriod);
	fprintf(f, "idleTimeout;%d;\n", config.idleTimeout);
	fprintf(f, "\n");

}

void cmd_exit(FILE *f, int parts, char **part)
{

	fprintf(f, "Bye now\n");
	fflush(f);

	doexit = 1;
}

// eot
