/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <time.h>
#include <unistd.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#ifdef LINUX
#include "linux.h"
#else
#include <aplog.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <unistd.h>
#include <ctype.h>

// DEBUG: New include
#include "paed.h"

#include "state_gtk.h"
#include "keystuff.h"
#include "log.h"
#include "misc.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif

#include "config.h"

#if (PAED_MSSID == 1)
#include "mssid.h"
#endif


static int config_read_config_interface (void);
static int config_read_simple(char *fname, struct config_t *config);

#if (PAED_MSSID == 1)

void config_sethexkey(char *asckey, struct config_t *config, char *ifname, int keyid);
#else

static void config_sethexkey(char *asckey, struct config_t *config, char *ifname, int keyid);
#endif /* #if (PAED_MSSID == 1)*/


static void config_setssid(char *value,  struct config_t *config);
static void config_setssnhexkey(char *asckey, struct config_t *config);
static void config_setssnpasswordkey(char *asckey, struct config_t *config);
static int config_read_radius (char *fname);

/*Code added for RADIUS Accounting*/
/*******Sukanya Bhattacharya*******/
/********** Code Start ************/

#if (RAC_SUPPORT == RAC_TRUE)
static int config_add_radiusserver(char *hostname, int port, char *secret,
                                   int secret_len, int acct_port);
#else
static int config_add_radiusserver(char *hostname, int port, char *secret,
                                   int secret_len);
#endif

/*********** Code End *************/


/*static int config_add_radiusserver(char *hostname, int port, char *secret, int secretlen);*/
static void config_log_config (struct config_t *config);

#if (PAED_MSSID == 1)
static int config_get_vss_interface_count (void);

static int config_check_vss_exists(char *iface, int vssid);

#endif



#if (PAED_MSSID == 1)
/* The config array. Stores VSS definitions for all created VSSs
 * The array is dynamically allocated when PAED initializes
 */
struct config_t *config;

// DEBUG code
//struct config_t config[MAX_INTERFACES * MSSID_MAX_VSS_COUNT];

/* 2 dimensional Array of pointers to VSS definitions in the config array,
 * indexed on physical interface Id and VSSID. If a VSS is created on a
 * physical wireless interface i.e. the config array contains an entry
 * for a VSS, the pointer to that entry is stored in location
 * <physical_interface_id>,<vssid> in this array. All remaining locations in
 * the array are initialized to NULL.
 */

struct config_t *config_index[MAX_INTERFACES][MSSID_MAX_VSS_COUNT];


/* Number of Virtual Service Sets */
int vss_count;

struct gtkmachine_t gtk[MAX_INTERFACES *  MSSID_MAX_VSS_COUNT];

/* Table for mapping index of wireless ethernet interface rows in the config_index
  * array to devid value for the interface
  */
int phy_iface_devid[MAX_INTERFACES];

#else

struct config_t config[MAX_INTERFACES];
struct gtkmachine_t gtk[MAX_INTERFACES];

#endif /*  #if (PAED_MSSID == 1)  */


struct radius_t radius;

/*Code added for RADIUS Accounting*/
/*******Sukanya Bhattacharya*******/
/********** Code Start ************/

#if (RAC_SUPPORT == RAC_TRUE)
int rac_status =0 ;
int rac_interim_interval=15 ;
char rac_nas_id[32];
#endif

/*********** Code End *************/


/*
 * This function is called for initialisation of the config structures for all interfaces.
 * After all config files are parsed the general initialistation will be set and printed.
 */
int config_read(void)
{
        int i;
        int radius_config_read = FALSE;
        int nof_interfaces=0;

        int k;

#if (PAED_MSSID == 1)

        /* initialize the VSSID-config entry  lookup table -
         * config_index
         */
        memset(config_index, 0, sizeof(config_index));

#endif /* #if (PAED_MSSID == 1) */


         // set default values
        nof_interfaces = config_read_config_interface();

        log(LG_INF, "MSSID: Number of interfaces found = %d", nof_interfaces);
        for (k=0; k<nof_interfaces ; k++){

             log(LG_INF, "MSSID: For entry no %d, ifname = %s, vss_ifname = %s",
                 k, config[k].ifname, config[k].vss_ifname);
        }


        if (nof_interfaces == 0)
        {
            log(LG_FTL, "Found no WLAN interfaces to service");
        }

        for (i = 0; i < nof_interfaces; i++)
        {
            // Convert SNMP booleans to usable values.

            config[i].enableOS       &= 1;
            config[i].enableWEP      &= 1;
            config[i].enableACL      &= 1;
            config[i].enable1X       &= 1;
            config[i].enableSSN      &= 1;
            config[i].enableSSNPSK   &= 1;
            config[i].strictUpdate   &= 1;
            config[i].WEPexunencrypt &= 1;

            // DEBUG Code
            log(LG_DBG, "MSSID: Original Value of EnableOS flag = %d", config[i].enableOS);


            /* Since Privacy Invoked is mapped on enableOS this value should be inverted */
            if (config[i].enableOS == 1) config[i].enableOS = 0;
            else config[i].enableOS = 1;

            // DEBUG Code
            log(LG_DBG, "MSSID: Final Value of EnableOS flag = %d", config[i].enableOS);


            config[i].ssnRekeyPackets *= 1000;   // X * 1000 packets.
            config[i].dot1XRekeyPackets *= 1000;

            /* When WEP is on this length is the max length to use in 1x also.*/
            if (config[i].enableWEP)
            {
                config[i].Dot1XWEPKeySize = config[i].WEPKeySize;
            }
            /* When enableSSNPSK also enableSSN should be on
             * otherwise enableSSNPSK must be disabled
             */
            if (config[i].enableSSNPSK && !config[i].enableSSN)
                    config[i].enableSSNPSK = 0;

            // READ: Privacy_invoked is not set cause use security check box was off on
            //       the weppages!
            if (config[i].enableOS == 1)
            {
               // DEBUG Code
               log(LG_DBG, "MSSID: Resetting All Security");

                // Make sure no security is set.
                config[i].enableWEP    = 0;
                config[i].enable1X     = 0;
                config[i].enableSSN    = 0;
                config[i].enableSSNPSK = 0;

                // Disable rekeying stuff.
                config[i].ssnRekeying   = 1;
                config[i].dot1XRekeying = 1;
            }

            /* Disable strict updating when WEP or OS is on or when WPA isn't on */
            if (config[i].enableWEP || config[i].enableOS || !config[i].enableSSN)
                config[i].strictUpdate = 0;

            // Print out current config values when debugging
            log(LG_DBG, "Config of interface %d", i);


            config_log_config(&config[i]);
            
#if (RAC_SUPPORT == RAC_TRUE)
	     // Read radius servers configfile if needed
            // check whether the radius server configuration data has been read
            if( !radius_config_read )
            {
                config_read_radius(PATH_RADIUS_CONF);
                radius_config_read = TRUE;
            }

            if((config[i].enable1X) || (config[i].enableSSN && !config[i].enableSSNPSK) )
            {
                // reset the enable 1X and SSN modes when no radius server is found
                if( radius.nofRadiusServers == 0 )
                {
                    // DEBUG Code
                    log(LG_DBG, "MSSID: No Radius Server! Resetting enable1X and SSN");

                    // no radius server is found, reset config state and log
                    config[i].enable1X = 0;
                    config[i].enableSSN = 0;
                    log(LG_ERR, "No radius server configured, disabling dot1X and WPA");
                }
            }
#else // else .. (RAC_SUPPORT == RAC_TRUE)
	     // Read radius servers configfile if needed
            if((config[i].enable1X) || (config[i].enableSSN && !config[i].enableSSNPSK) )
            {
                // check whether the radius server configuration data has been read
                if( !radius_config_read )
                {
                    config_read_radius(PATH_RADIUS_CONF);
                    radius_config_read = TRUE;
                }

                // reset the enable 1X and SSN modes when no radius server is found
                if( radius.nofRadiusServers == 0 )
                {
                    // DEBUG Code
                    log(LG_DBG, "MSSID: No Radius Server! Resetting enable1X and SSN");

                    // no radius server is found, reset config state and log
                    config[i].enable1X = 0;
                    config[i].enableSSN = 0;
                    log(LG_ERR, "No radius server configured, disabling dot1X and WPA");
                }

            }
#endif // endif .. else .. (RAC_SUPPORT == RAC_TRUE)            
            
        }

        return(nof_interfaces);
}




// #if 0 // DEBUG: COMMENTED OUT
/*
 * This function will get a interfaces list from the kernel and call config_read_simple
 * to get the configuration for that device.
 * INPUT/OUTPUT = None.
 */
static int config_read_config_interface (void)
{
        int sd;
        FILE *f;
        int nof_interfaces = 0;
        char buf[128];
        char *part[2];
        char *iface;
        struct ifreq ifr_for_index;

#if (PAED_MSSID == 1)

    	int i;
    	char full_name[256];
       char iface_part[5];
    	int phy_iface_index;
       int nof_vss_interfaces=0;
		int iface_devid;
#endif


        f = fopen ("/proc/net/dev", "r");

        if (f == NULL) {
                log (LG_FTL, "Can't read /proc/net/dev");
                return 0;
        }

        /* Get configuration for all interfaces */
        if ( (sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) {
                printf("sided_multicast_init: Could not open socket\n");
        }


        /* The first two lines contain only header */
        fgets (buf, sizeof (buf), f); fgets (buf, sizeof (buf), f);

#if (PAED_MSSID == 1)

   	 /* Get interface count. Required to allocate memory for
	  * config array
     	  */
    	nof_vss_interfaces = config_get_vss_interface_count();
        log(LG_INF,"MSSID: Number of configured VSS interfaces is %d", nof_vss_interfaces);

	if (0 == nof_vss_interfaces){
           log(LG_FTL, "MSSID: No VSS Interfaces found!!");
	   return -1;
	}

    	config = (struct config_t *) malloc(nof_vss_interfaces * sizeof(struct config_t));

    	if (NULL == config){
            log (LG_FTL, "MSSID: Unable to allocate memory for config array");
            return 0;
    	}

	while (fgets (buf, sizeof (buf), f)  && (nof_interfaces < (MAX_INTERFACES * MSSID_MAX_VSS_COUNT)))
#else

        while (fgets (buf, sizeof (buf), f)  && (nof_interfaces<MAX_INTERFACES))
#endif
        {
            if (split (buf, ':', part, 2) == 2)
            {
                iface = part[0];
                while(isspace(*iface) || *iface == 0) iface++;
                /* We are only interested in the ethX interfaces */
                if(strncmp(iface, "eth", 3) == 0)
                {
            if( strncmp( iface, IF_CONF_FIRST, 4) < 0 )
            {
                    log(LG_INF, "Skip interface %s", iface);
            }
            else
            {
                    log(LG_INF, "Found interface %s", iface);
                    /* Parse possible config files */


#if (PAED_MSSID == 1)

					/* if this is a non-wireless interface, skip it */
					if (blob_mssid_wireless_interface(iface) < 0)
					{
						log(LG_INF, "MSSID: Found non-wireless interface %s. skipping...", iface);
						continue;
					}
					

					/* Get kernel ifindex for this interface */
                    iface_devid = mssid_name2idx(iface);

					 /* map devid to index value for physical interface in config_index
					  * mapping array 
					  */
					 phy_iface_index = mssid_devid_to_phy_iface_index(iface_devid); 
					 log(LG_INF, "MSSID: Interface index for iface %s is %d", iface, phy_iface_index);

					strcpy(iface_part, iface);

                     /* Parse possible config files */
                     for (i = 0; i < MSSID_MAX_VSS_COUNT; i++) {

						/* Does the VSS configuration file exist? */
						if (0 != config_check_vss_exists(iface_part, i)){
								 /* Vss config file does not exist for this VSSID
								  * Continue to the next VSSID
								  */
								  continue;
						}

                         sprintf(full_name, "%s.%d", iface_part, i);
                         iface = full_name;

                         // DEBUG code
                         log(LG_INF, "MSSID: Calling config_read_simple for vss %s", iface);
#endif /* #if (PAED_MSSID == 1) */


                        if (config_read_simple(iface, &config[nof_interfaces]) == TRUE)
                        {
#if (PAED_MSSID == 1)

                           log(LG_INF,"MSSID: Success return from config_read_simple.
                                       This is %dth VSS with VSSID %d", nof_interfaces, i);

                            /* config->ifname stores only the physical interface name */
                           strcpy(config[nof_interfaces].ifname, iface_part);


                            /* vss interface is of type - vssethX.Y. store the vss name in
                             * config->vss_ifname
                             */
                           sprintf(config[nof_interfaces].vss_ifname, "vss%s.%d", iface_part, i);

                           log(LG_INF,"MSSID: Ifname is %s, and VSS Name is %s",
                                config[nof_interfaces].ifname,
                                config[nof_interfaces].vss_ifname);

                            /* set VSSID */
                            config[nof_interfaces].vssid = i;

                            /* Set appropriate pointer in the
                             * VSSID lookup table - config_index
                             */
                            config_index[phy_iface_index][i] = &config[nof_interfaces];

                            /* No clients as of now. The VSS client list is empty */
                            config[nof_interfaces].client_ptr_list = NULL;

                            /* IOCTL call is for ethX interface only.
                             * Use only the physical wireless interface name.
                             */
                             iface = &iface_part[0];

                             // DEBUG code
                             //log(LG_INF,"MSSID: Setting iface back to %s for ioctl", iface);
#else
                            strncpy(config[nof_interfaces].ifname, iface, IFNAMSIZ);

#endif /*#if (PAED_MSSID == 1)*/

                            memset(&ifr_for_index, 0, sizeof(struct ifreq));
                            strncpy(ifr_for_index.ifr_ifrn.ifrn_name, iface, IFNAMSIZ);
                            if(ioctl(sd, SIOCGIFINDEX, &ifr_for_index))
                            {
                                log(LG_WRN, "Could not find device id (SIOCGIFINDEX)");
                            }

                            // DEBUG code
                            log(LG_INF, "MSSID: IOCTL call for ifindex executed succesfully. Devid is %d",
                                config[nof_interfaces].devid);

                            config[nof_interfaces].devid = ifr_for_index.ifr_ifindex;

#if (PAED_MSSID == 1)
				/* Capture device id of the wireless interfaces in an
				  * interface index to devid mapping table. Used in handle_traps()
				  * to get the index of the physical interface entry in config_index
				  */
				phy_iface_devid[phy_iface_index] = config[nof_interfaces].devid;

#endif /*#if (PAED_MSSID == 1)*/


                            if ( ioctl(sd, SIOCGIFHWADDR, &ifr_for_index) < 0 )
                            {
                                log(LG_WRN, "Could not get mac address (SIOCGIFHWADDR)");
                            }

                            // log(LG_INF, "MSSID: After IOCTL call for macaddress");

                            memcpy(config[nof_interfaces].ifhwaddr, ifr_for_index.ifr_hwaddr.sa_data, 6);
                            config[nof_interfaces].cgtk = &gtk[nof_interfaces];
                            nof_interfaces++;
                        }

#if (PAED_MSSID == 1)
                     } /* for all VSSs */
#endif

                    }
                }
            }
        } // While lines read from /proc/net/dev

        fclose(f);

        log(LG_INF, "Found total number %d WLAN interfaces", nof_interfaces);
        return nof_interfaces;
}

// #endif


// This function is used for reading simple 'standard' configuration
// files with the form 'keyword value'

static int config_read_simple(char *fname, struct config_t *config)
{
        FILE *f;
        char full_name[256];
        char buf[128];
        char *part[2];
        char *key, *value;
        char *p;
        int i;

        struct cfg_opt_t cfg_opt[] = {
            { &config->suppTimeout,             30,     "suppTimeout" },
            { &config->serverTimeout,           15,     "serverTimeout" },
            { &config->txPeriod,                30,     "txPeriod" },
            { &config->reAuthMax,               2,      "reAuthMax" },
            { &config->quietPeriod,             60,     "Mic quietPeriod"},
            { &config->idleTimeout,             120,    "idleTimeout" },

            { &config->dot1XRekeying,           1,      "Dot1XRekeying" },
            { &config->dot1XRekeySeconds,       3600,   "Dot1XRekeyingTime" },
            { &config->dot1XRekeyPackets,       1000,   "Dot1XRekeyingPackets" },

            { &config->ssnRekeying,             1,      "WPAGroupRekeying" },
            { &config->ssnRekeySeconds,         3600,   "WPAGroupRekeyingTime" },
            { &config->ssnRekeyPackets,         1000,   "WPAGroupRekeyingPackets" },

            { &config->reAuthPeriod,            3600,   "ReauthenticationTime" },

            { &config->strictUpdate,            1,      "WPAGroupKeyStrictUpdate" },

            { &config->enableOS,                0,      "dot11PrivacyInvoked" },
            { &config->enableWEP,               0,      "WEPEnabled" },
            { &config->enableACL,               0,      "ACLEnabled" },
            { &config->enable1X,                0,      "Dot1XEnabled" },
            { &config->enableSSN,               0,      "WPAEnabled" },
            { &config->enableSSNPSK,            0,      "WPAUsePreSharedKey" },
            { &config->Dot1XWEPKeySize,         13,     "Dot1XWEPKeySize" },
            { &config->WEPKeySize,              13,     "WEPKeySize" },
            { &config->DefaultKeyId,            0,      "dot11WEPDefaultKeyID" },
            { &config->WEPexunencrypt,          0,      "dot11ExcludeUnencrypted" },
            
            { &config->wpa_cipher,		3,	"WPACipher" },

#if (PAED_MSSID == 1)
            { &config->exclusivelimit,        100,   	"ExclusiveMemberLimit" },
	    { &config->totallimit,            100,   	"TotalMemberLimit" },
            { &config->unicastcipher,   MSSID_CIPHER_TKIP, "dot1XSSNUnicastSuite" },
            { &config->multicastcipher, MSSID_CIPHER_TKIP, "dot1XSSNMcastSuite" },
            { &config->mixedsecurity,          0,       "MixedSecurityEnabled" },
 
            /* vlan 0 implies untagged port */
            { &config->vlan_tag,        0,              "VlanTag" },
#endif

            { NULL,                             0,      NULL}
        };
        /* Initialise the current config pointer */
        i=0;
        memset(config, 0, sizeof(struct config_t));

        while(cfg_opt[i].keyword != NULL) {
                *(int *) (cfg_opt[i].varptr) = cfg_opt[i].dfltval;
                i++;
        }

#if (PAED_MSSID == 1)

    	/* the file name is of format - wss.vssethX.Y.conf */
    	sprintf(full_name, "%s/wss.vss%s.conf", PATH_ETH_CONF, fname);

        log(LG_INF,"MSSID: reading VSS configuration from file %s", full_name);

        /* fname is currently of format ethX.Y
         * Need to change it to ethX.
         */
        fname[4] = '\0';
#else
        sprintf(full_name, "%s/wss.%s.conf", PATH_ETH_CONF, fname);

#endif

        f = fopen (full_name, "r");

        // DEBUG Code
        //log(LG_DBG, "MSSID: Done fopen for %s", full_name);


        if (f == NULL) {
                log (LG_DBG, "Can't read configfile %s", full_name);
                return FALSE;
        }
        log(LG_INF, "Reading configuration file: %s", full_name);
        // Read config file


       /* need to clean the keys table */
       memset(config->keystable, 0, sizeof(config->keystable));


        while (fgets (buf, sizeof (buf), f)) {
                if (skip_white((char *)buf) == 0) continue; // Skip white and #
                p = strchr (buf, '\n');
                if (p)  *p = 0;

                if (split (buf, ' ', part, 2) == 2)
                {
                        key = part[0];
                        value = part[1];

                        i = 0;

                        while (cfg_opt[i].keyword != NULL) {

                                if (strcmp (key, cfg_opt[i].keyword) == 0) {
                                        *(int *) (cfg_opt[i].varptr) =  atoi (value);
                                }

                                i++;
                        }
                        if (strcmp (key, "WPAConfigPSKValue") == 0)
                        {
                            /* The 265-bits SSN-PSK value */
                            config_setssnhexkey(value, config);
                        }
                        if (strcmp (key, "WPAConfigPSKPassPhrase") == 0)
                        {
                            /* The Password Phrase SSN-PSK value */
                            config_setssnpasswordkey(value, config);
                        }

#if (PAED_MSSID == 1)
                        if(strcmp(key, "dot11WEPDefaultKey1") == 0)
                        {
                            strcpy(config->keystable[0], value);
                            log(LG_DBG, "MSSID: Key value is %s", config->keystable[0]);
                        }

                        if(strcmp(key, "dot11WEPDefaultKey2") == 0)
                        {
                            strcpy(config->keystable[1], value);
                            log(LG_DBG, "MSSID: Key value is %s", config->keystable[1]);
                        }
                        if(strcmp(key, "dot11WEPDefaultKey3") == 0)
                        {
                            strcpy(config->keystable[2], value);
                            log(LG_DBG, "MSSID: Key value is %s", config->keystable[2]);
                        }
                        if(strcmp(key, "dot11WEPDefaultKey4") == 0)
                        {
                            strcpy(config->keystable[3], value);
                            log(LG_DBG, "MSSID: Key value is %s", config->keystable[3]);
                        }

#else

                        if(strcmp(key, "dot11WEPDefaultKey1") == 0)
                        {
                            config_sethexkey(value, config, fname, 0);
                        }
                        if(strcmp(key, "dot11WEPDefaultKey2") == 0)
                        {
                            config_sethexkey(value, config, fname, 1);
                        }
                        if(strcmp(key, "dot11WEPDefaultKey3") == 0)
                        {
                            config_sethexkey(value, config, fname, 2);
                        }
                        if(strcmp(key, "dot11WEPDefaultKey4") == 0)
                        {
                            config_sethexkey(value, config, fname, 3);
                        }

#endif  /* #if (PAED_MSSID == 1) */


                        if(strcmp(key, "dot11DesiredSSID") == 0)
                        {
                            config_setssid(value, config);
                        }

                }
        }



        fclose(f);
        return (TRUE);
}



static void config_setssid(char *value,  struct config_t *config)
{
    config->ssidlength = strlen(value);
    if (config->ssidlength > 32)
        config->ssidlength = 32;
    strncpy(config->ssid, value, config->ssidlength);
    log(LG_INF, "config->ssid: %s", config->ssid);
}

static void config_setssnhexkey(char *asckey, struct config_t *config)
{
        char binkey[32];
        int len;
        int t;
        char *pin, *pout;

        pin=strchr(asckey, '\r'); if(pin) *pin=0;
        pin=strchr(asckey, '\n'); if(pin) *pin=0;

        pin=asckey;
        pout=binkey;

        while(*pin == ' ')pin++;


        if(strncmp(pin, "HEX:", 4) != 0)
        {
                return;
        }
        /* Lets move to the start of key */
        pin+=4;

        len = 0;

        while(*pin && (*pin != '\0') ) {

                sscanf(pin, "%2X", &t);
                (char)(*pout) = t;

                pout++;
                pin+=2;
                len++;
        }

        if(len == 32)
        {

            for (t = 0; t < len; t++)
            {
                /* When the key is 0x00 .... 0x00 it's not filled. */
                if (binkey[t] != 0x00)
                {
                    memcpy(config->ssnpsk256key, binkey, 32);
                    config->ssnpsk256keyfiled = 1;
                    dump_bindata("WPAPSK hex key", config->ssnpsk256key, 32);
                    break;
                }
            }
        }

}

static void config_setssnpasswordkey(char *asckey, struct config_t *config)
{
        if (strlen(asckey) > 0)
            memcpy(config->ssnpskASCIIkey, asckey, strlen(asckey));

        log(LG_DBG, "MSSID: Found ascii pasphrase for WPA-PSK key = %s",
             asckey);

}
// Convert the hex-notated key to binary form, and plumb it in the
// blob's mcastkey 0. This function is called for the key defined in the
// dot11smt and for the key in dot11Ext2. If defined, the second
// overrides the first

#if (PAED_MSSID == 1)

void config_sethexkey(char *asckey, struct config_t *config, char *ifname, int keyid)
#else

static void config_sethexkey(char *asckey, struct config_t *config, char *ifname, int keyid)
#endif
{
        char binkey[32];
        int len;
        int t;
        char *pin, *pout;
        int filled = 0;

        pin=strchr(asckey, '\r'); if(pin) *pin=0;
        pin=strchr(asckey, '\n'); if(pin) *pin=0;

        pin=asckey;
        pout=binkey;

        while(*pin == ' ')pin++;

        if(strncmp(pin, "HEX:", 4) != 0)
        {
                return;
        }
        /* Lets move to the start of key */
        pin+=4;

        len = 0;
        while(*pin && (*pin != '\n') && ((pin-asckey) < strlen(asckey))) {

                sscanf(pin, "%2X", &t);
                (char)(*pout) = t;
                if (t != 0x00) filled = 1;
                pout++;
                pin+=2;
                len++;
        }

        if((len == 5 || len == 13) && (filled == 1)) {

#if (PAED_MSSID == 1)

                /* Just set the broadcast key in the service set */
                blob_mssid_set_bcast_key(config, binkey, len, keyid);

#else

                blob_set_bcast_key(binkey, len, keyid, ifname);
                // Always reset the default key id to the configured number.
                blob_set_bcast_keyid(config->DefaultKeyId, ifname);

#endif /* #if (PAED_MSSID == 1) */

                config->WEPKeySize = len;
        }

}


// Read radius servers from configuration file
static int config_read_radius(char *fname)
{
        FILE *f;
        char buf[128];
        char *part[6];
        char *p;
        struct hostent *hp;
        char tmp[MAX_RADIUS_SECRET_LEN];
        int len;

	/*Code added for RADIUS Accounting*/
	/*******Sukanya Bhattacharya*******/
	/********** Code Start ************/
	
#if (RAC_SUPPORT == RAC_TRUE)
	int part_count;
	int acct_port;
#endif

	/*********** Code End *************/
       
      	log (LG_DBG, "Reading servers file %s", fname);

	log (LG_DBG, "Reading servers file " );
	
	f = fopen (fname, "r");

        log (LG_DBG, "Reading servers file %s", fname);

	log (LG_DBG, "READ servers file AS I WANT  %s", fname);

        if (f == NULL) {
		log (LG_DBG, " CANNOT READ servers file %s", fname);
                log (LG_ERR, "Can't read servers file %s", fname);
                return -1;
        }


	log (LG_DBG, "CAN READ servers file ");
	
	 /*Code added for RADIUS Accounting*/
	 /*******Sukanya Bhattacharya*******/
	 /********** Code Start ************/
	
#if (RAC_SUPPORT == RAC_TRUE)

	log(LG_DBG, "RAC_SUPPORT == RAC_TRUE");
	log(LG_INF, "RAC_SUPPORT == RAC_TRUE");
	
	log(LG_DBG, "READING RADIUS FILE");
	log(LG_INF, "READING RADIUS FILE");
	/* The first line of the radius file contains accounting status */ 
	if (fgets (buf, sizeof (buf), f)){
		p = strchr (buf, '\n');
		if (p)
			*p = 0;
		if ('1' == buf[0]){
			rac_status = RAC_STATUS_ENABLE;
			log(LG_DBG, "RAC_STATUS_ENABLE %d", rac_status);
			log(LG_INF, "RAC_STATUS_ENABLE %d", rac_status);
		}
		else {
			rac_status = RAC_STATUS_DISABLE;
			log(LG_DBG, "RAC_STATUS_DISABLE %d", rac_status);
			log(LG_INF, "RAC_STATUS_DISABLE %d", rac_status);
		}
	}

	/* The second line of the radius file contains Interim Interval */ 
	if (fgets (buf, sizeof (buf), f)){
		p = strchr (buf, '\n');
		if (p)
			*p = 0;
		rac_interim_interval = atoi(buf);
		log(LG_DBG, "RAC_INTERIM_INTERVAL %d", rac_interim_interval);
	}

	/* The third line of the radius file contains NAS-Identifier */ 
	if (fgets (buf, sizeof (buf), f)){
		p = strchr (buf, '\n');
		if (p)
			*p = 0;

		strncpy(rac_nas_id, buf, strlen(buf));
		
	}

#endif
	/*********** Code End *************/
	
	radius.radius_serv_list = NULL;
	log(LG_DBG, "\n\nREADING PER SERVER CONF\n\n");
        		
        while (fgets (buf, sizeof (buf), f)) {
                if (skip_white((char *)buf) == 0) continue; // Skip white and #
                p = strchr (buf, '\n');
                if (p)
                        *p = 0;

	/*Code added for RADIUS Accounting*/
	/*******Sukanya Bhattacharya*******/
	/********** Code Start ************/
		

#if (RAC_SUPPORT == RAC_TRUE)

		if (RAC_STATUS_ENABLE == rac_status){
			part_count = 6;
		}
		else {
		/* Accounting is disabled. Don.t read the accounting port */ 
			part_count = 5;
		}

		/* The configuration also includes acct port */
		if (split (buf, ' ', part, part_count) == part_count) {
#else 
		if (split (buf, ' ', part, 5) == 5) {
#endif 
                 /*********** Code End *************/
			
			if (strcmp (part[0], "server") == 0)
                        {

                                hp = gethostbyname (part[2]);

                                if (hp == NULL) {
                                	log (LG_ERR,"Radiusserver %s not found, 
					     skipping",part[1]);
                                        continue;
                                }

                                /* check for old/new configfile */
                                if (strncmp(part[3], "HEX:", 4) == 0) {
                                        len = hex2string(tmp, part[3] + 4, MAX_RADIUS_SECRET_LEN);
                                }
                                else {
                                        len = hex2string(tmp, part[3], MAX_RADIUS_SECRET_LEN);
                                }

				 /*Code added for RADIUS Accounting*/
				 /*******Sukanya Bhattacharya*******/
				 /********** Code Start ************/
				
#if (RAC_SUPPORT == RAC_TRUE)
				if (RAC_STATUS_ENABLE == rac_status){
					acct_port = atoi(part[5]);
				}
				else {
					/* 
		                         * Accounting is disabled. 
		                         * Set accounting port to invalid value 
		                         */ 
					acct_port = RAC_INVALID_PORT;
				}

				config_add_radiusserver(part[2], atoi(part[4]), 
						        tmp, len,acct_port);
#else 

				config_add_radiusserver(part[2], atoi(part[4]), tmp, 
						len);
#endif             
 				/*********** Code End *************/
				
                        }
                }
        }

        if(radius.radius_serv_list == NULL) {
                aplog(EV_EAP_NO_RADIUS_SRV, NULL);
                log(LG_ERR, "No radius servers defined.");
                return -1;
        }


        radius.current_radius_serv = radius.radius_serv_list;

	/*Code added for RADIUS Accounting*/
	/*******Sukanya Bhattacharya*******/
	/********** Code Start ************/
	
#if (RAC_SUPPORT == RAC_TRUE)
	if (RAC_STATUS_ENABLE == rac_status){
		radius.current_rac_serv = radius.radius_serv_list;
	}
#endif
 	/*********** Code End *************/
	
      
	fclose (f);

        return (0);
}


/*
 * Add a radius server to the list of servers.
 */


/*Code added for RADIUS Accounting*/
/*******Sukanya Bhattacharya*******/
/********** Code Start ************/

#if (RAC_SUPPORT == RAC_TRUE)

static int config_add_radiusserver(char *hostname, int port, char *secret,
	       			   int secret_len, int acct_port)
#else

static int config_add_radiusserver(char *hostname, int port, char *secret,
		                   int secret_len)
#endif
/*********** Code End *************/

{
        struct radius_serv_t *new, *tmp;

        new = malloc(sizeof(struct radius_serv_t));

        if(new == NULL) {
                log(LG_FTL, "Can't malloc radius_serv_t");
        }

        if(radius.radius_serv_list == NULL) {
                radius.radius_serv_list = new;
        } else {
                tmp = radius.radius_serv_list;

                while(tmp->next) {
                        tmp = tmp->next;
                }

                tmp->next = new;
        }

        new->address = inet_addr(hostname);
	new->port = port;

	/*Code added for RADIUS Accounting*/
	/*******Sukanya Bhattacharya*******/
	/********** Code Start ************/
	
#if (RAC_SUPPORT == RAC_TRUE)

	/* 
	 * Can perform this regardless of RADIUS status being disabled 
	 */
	new->acct_port = acct_port;
#endif
	/*********** Code End *************/
	
	memcpy(new->secret, secret, secret_len);
        new->secret_len = secret_len;
        new->next = NULL;

        log(LG_INF, "Added radius server %s:%d", hostname, port);
        radius.nofRadiusServers++;
        return(0);
}

/*
 * This function logs all current config settings in config.
 */
static void config_log_config (struct config_t *config)
{
        int i = 0;

        struct cfg_opt_t cfg_opt[] = {
            { &config->suppTimeout,             30,     "suppTimeout" },
            { &config->serverTimeout,           15,     "serverTimeout" },
            { &config->txPeriod,                30,     "txPeriod" },
            { &config->reAuthMax,               2,      "reAuthMax" },
            { &config->quietPeriod,             60,     "Mic quietPeriod"},
            { &config->idleTimeout,             60,     "idleTimeout" },

            { &config->dot1XRekeying,           1,      "Dot1XRekeying" },
            { &config->dot1XRekeySeconds,       3600,   "Dot1XRekeyingTime" },
            { &config->dot1XRekeyPackets,       1000,   "Dot1XRekeyingPackets" },

            { &config->ssnRekeying,             1,      "WPAGroupRekeying" },
            { &config->ssnRekeySeconds,         3600,   "WPAGroupRekeyingTime" },
            { &config->ssnRekeyPackets,         1000,   "WPAGroupRekeyingPackets" },

            { &config->reAuthPeriod,            3600,   "ReauthenticationTime" },

            { &config->strictUpdate,            1,      "WPAGroupKeyStrictUpdate" },

            { &config->enableOS,                0,      "dot11PrivacyInvoked" },
            { &config->WEPexunencrypt,          0,      "dot11ExcludeUnencrypted" },
            { &config->enableWEP,               0,      "WEPEnabled" },
            { &config->enableACL,               0,      "ACLEnabled" },
            { &config->enable1X,                0,      "Dot1XEnabled" },
            { &config->enableSSN,               0,      "WPAEnabled" },
            { &config->enableSSNPSK,            0,      "WPAUsePreSharedKey" },
            { &config->Dot1XWEPKeySize,         13,     "Dot1XWEPKeySize" },
            { &config->WEPKeySize,              13,     "WEPKeySize" },

#if (PAED_MSSID == 1)
            { &config->exclusivelimit,          100,               "ExclusiveMemberLimit" },
            { &config->totallimit,              100,               "TotalMemberLimit" },
            { &config->unicastcipher,           MSSID_CIPHER_TKIP, "dot1XSSNUnicastSuite" },
            { &config->multicastcipher,         MSSID_CIPHER_TKIP, "dot1XSSNMcastSuite" },
            { &config->vlan_tag,                0,      "VlanTag" },
	        { &config->mixedsecurity,          0,       "MixedSecurityEnabled" },

#endif /* #if (PAED_MSSID == 1) */

            { NULL,                             0,      NULL}
        };

        log(LG_DBG, "============================================================");
        while(cfg_opt[i].keyword != NULL) {
                log(LG_DBG, "  %-40s : %d",
                        cfg_opt[i].keyword, *(cfg_opt[i].varptr));
                i++;
        }

    // DEBUG code
    for (i=0; i< 4; i++){
        log(LG_DBG, "MSSID: WEP Key %d = %s", i, config->keystable[i]);
    }

        log(LG_DBG, "============================================================");

}

/* skip all white spaces and return 1 if found something either end of
   line or a comment character */
int skip_white(char *ptr)
{

  if (ptr == NULL) return (0);
  while (*ptr != 0 && isspace(*ptr)) ptr++;
  if (*ptr == 0 || *ptr == '#') return (0);
  return (1);
}



#if (PAED_MSSID == 1)

int config_check_vss_exists(char *iface, int vssid)
{
	char conf_file_name[256];
	FILE *vss_iface_f;

	sprintf(conf_file_name, "%s/wss.vss%s.%d.conf",
		PATH_ETH_CONF, iface, vssid);

       vss_iface_f = fopen (conf_file_name, "r");

        if (NULL != vss_iface_f) {
            log (LG_INF, "MSSID: found vss config file %s", conf_file_name);

            fclose(vss_iface_f);

            return 0;
        }

        return -1;
}


/* Returns the number of VSS interfaces from /proc/net/dev */
static int config_get_vss_interface_count (void)
{
	FILE *phy_iface_f=NULL;
	FILE * vss_iface_f=NULL;
	int nof_interfaces = 0;
	char buf[256];
	char *part[2];
	char *iface;
	char full_name[256];
	int i;
        char iface_part[5];
        int nof_vss_interfaces=0;

	MSSID_LOG_FN_ENTRY("config_get_vss_interface_count");

        /* Read interface list. We get the physical interfaces */
        phy_iface_f = fopen ("/proc/net/dev", "r");

        if (NULL == phy_iface_f) {
                log (LG_FTL, "MSSID: Can't read /proc/net/dev");
                return 0;
        }

        /* The first two lines contain only header */
        fgets (buf, sizeof (buf), phy_iface_f);
        fgets (buf, sizeof (buf), phy_iface_f);

        while (fgets (buf, sizeof (buf), phy_iface_f)  &&
                (nof_interfaces < MAX_INTERFACES)){
            if (split (buf, ':', part, 2) == 2) {
                iface = part[0];
                while(isspace(*iface) || *iface == 0) iface++;

                /* We are only interested in the ethX interfaces */
                if(strncmp(iface, "eth", 3) == 0) {

		    /* if this is a non-wireless interface, skip it */
		     if (blob_mssid_wireless_interface(iface) < 0){
			log(LG_INF, "MSSID: Skip interface %s", iface);
			continue;
		    }

		    /* Add devid of this wireless interface in the 
		     * interface index to devid mapping table.
		     */
		    mssid_add_devid(iface);

                    /* VSS interfaces. The interface name format is
                     * vsseth<ethX>.<vssid>. We add wss as a prefix for the
                     * conf files.
                     */
                    for (i = 0; i < MSSID_MAX_VSS_COUNT; i++){
                        sprintf(full_name, "%s/wss.vss%s.%d.conf",
                                   PATH_ETH_CONF, iface, i);

                        // DEBUG code
                        log(LG_INF, "MSSID: Trying VSS conf file name %s", full_name);

                        vss_iface_f = fopen (full_name, "r");

                        if (NULL != vss_iface_f) {
                           log (LG_INF, "MSSID: found vss config file %s",
                                 full_name);

                            /* Found a vss interface. Increment count */
                            nof_vss_interfaces++;
                            fclose(vss_iface_f);

                            vss_iface_f = NULL;
                        }

                    } /* for */

                    nof_interfaces++;
                } /* if iface eth */
            }
	    } /* While lines read from /proc/net/dev */
		fclose(phy_iface_f);

    log(LG_INF, "MSSID: Found total number %d VSS interfaces",
                                                      nof_vss_interfaces);

    /* store VSS count in global variable */
    vss_count = nof_vss_interfaces;

    MSSID_LOG_FN_EXIT("config_get_vss_interface_count");
    return nof_vss_interfaces;
}

#endif /* #if (PAED_MSSID == 1) */



// eot



// DEBUG CODE

int make_dummy_config(struct config_t *configp)
{
     /* privacy is not invoked */
     configp->enableOS = 0;

     configp->unicastcipher = 3;
     configp->multicastcipher = 3;

     configp->enableWEP = 0;
     configp->enableSSN = 1;
     configp->enableSSNPSK = 0;

     configp->enable1X = 0;
     configp->enableACL = 0;

     configp->ssnRekeying = 0;
     configp->ssnRekeySeconds = 3600;
     configp->ssnRekeyPackets = 1000;

     configp->idleTimeout = 3600;

     return 0;
}


#if 0 // Compiled out

// DEBUG CODE
/*
 * This function will get a interfaces list from the kernel and call config_read_simple
 * to get the configuration for that device.
 * INPUT/OUTPUT = None.
 */
static int config_read_config_interface (void)
{
        int sd;
        FILE *f;
        int nof_interfaces = 0;
        char iface[20];
        struct ifreq ifr_for_index;


    	int i;
    	int phy_iface_index;
       int nof_vss_interfaces=0;

       int config_entries;

       int ret_val;

/*
* use static allocation

        // size of config array
        config_entries = 16;

        log(LG_DBG, "MSSID: Allocating memory for config array");

	config = (struct config_t *) malloc(config_entries * sizeof(struct config_t));

	if (NULL == config) {
		log(LG_FTL, "MSSID: unable to allocate memory for the config table");
		return 0;

	}

        log(LG_DBG, "MSSID: Doing memset on config array of size %d",
             config_entries * sizeof(struct config_t));
        memset(config, 0, config_entries * sizeof(struct config_t));
*/
// IF WE USE STATIC ARRAY
       memset(config, 0, sizeof(config));


        /* Get configuration for all interfaces */
        if ( (sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) {
                printf("sided_multicast_init: Could not open socket\n");
        }

        // DEBUG code
        // log(LG_DBG, "MSSID: Socket %d opened for IOCTL calls", sd);

	// call config read for vss 0 on eth1
//	strcpy(iface, "eth1.0");
//	ret_val = config_read_simple(iface, &config[0]);

//	log(LG_INF, "MSSID: return value from config_read_simple for ETH1 VSS 0 - %d", ret_val);

nof_interfaces = 0;

                           /* create the dummy config structure for vss 0 */
                           make_dummy_config(&config[0]);

                            /* set the ifname */
                           strcpy(config[0].ifname, "eth1");

                            /* vss interface is of type - vssethX.Y */
                           strcpy(config[0].vss_ifname, "vsseth1.0");

                           log(LG_INF,"MSSID: Ifname is %s, and VSS name is %s",
                                config[0].ifname,
                                config[0].vss_ifname);


                            /* set VSSID */
                            config[0].vssid = 0;

                            /* Set appropriate pointer in the
                             * VSSID lookup table - config_index- for eth1. index is 0
                             */
                            config_index[0][0] = &config[0];

                            /* No clients as of now. The VSS client list is empty */
                            config->client_ptr_list = NULL;

                            /* IOCTL call is for ethX interface only.
                             * Use only the physical wireless interface name.
                             */

                            memset(&ifr_for_index, 0, sizeof(struct ifreq));
                            strcpy(ifr_for_index.ifr_ifrn.ifrn_name, "eth1");

                            if(ioctl(sd, SIOCGIFINDEX, &ifr_for_index))
                            {
                                log(LG_WRN, "Could not find device id (SIOCGIFINDEX)");
                            }

                            log(LG_INF, "MSSID: After IOCTL call for SIOCGIFINDEX. Devid is %d",
                                config[0].devid);

                            config[0].devid = ifr_for_index.ifr_ifindex;

				/* Capture device id of the wireless interfaces in an
				  * interface index to devid mapping table. Used in handle_traps()
				  * to get the index of the physical interface entry in config_index
				  */
				phy_iface_devid[0] = config[0].devid;



                            if ( ioctl(sd, SIOCGIFHWADDR, &ifr_for_index) < 0 )
                            {
                                log(LG_WRN, "Could not get mac address (SIOCGIFHWADDR)");
                            }

                            log(LG_INF, "MSSID: After  IOCTL call for macaddress.");

                            memcpy(config[0].ifhwaddr, ifr_for_index.ifr_hwaddr.sa_data, 6);
                            config[0].cgtk = &gtk[0];



                 /* now for the second vss */
                     nof_interfaces= 1;



                           /* create the dummy config structure for vss 1 */
                           make_dummy_config(&config[1]);

                            /* set the ifname */
                           strcpy(config[1].ifname, "eth1");

                            /* vss interface is of type - vssethX.Y */
                           strcpy(config[1].vss_ifname, "vsseth1.1");


                           log(LG_INF,"MSSID: Ifname is %s, and VSS name is %s",
                                config[1].ifname,
                                config[1].vss_ifname);


                            /* set VSSID */
                            config[1].vssid = 1;

                            /* Set appropriate pointer in the
                             * VSSID lookup table - config_index- for eth1. index is 0
                             */
                            config_index[0][1] = &config[1];

                            /* No clients as of now. The VSS client list is empty */
                            config->client_ptr_list = NULL;

                            /* IOCTL call is for ethX interface only.
                             * Use only the physical wireless interface name.
                             */

                            memset(&ifr_for_index, 0, sizeof(struct ifreq));
                            strcpy(ifr_for_index.ifr_ifrn.ifrn_name, "eth1");

                            if(ioctl(sd, SIOCGIFINDEX, &ifr_for_index))
                            {
                                log(LG_WRN, "Could not find device id (SIOCGIFINDEX)");
                            }

                            log(LG_INF, "MSSID: After IOCTL for SIOCGIFINDEX call. Devid is %d",
                                config[1].devid);

                            config[1].devid = ifr_for_index.ifr_ifindex;

				/* Capture device id of the wireless interfaces in an
				  * interface index to devid mapping table. Used in handle_traps()
				  * to get the index of the physical interface entry in config_index
				  */
				phy_iface_devid[0] = config[1].devid;



                            if ( ioctl(sd, SIOCGIFHWADDR, &ifr_for_index) < 0 )
                            {
                                log(LG_WRN, "Could not get mac address (SIOCGIFHWADDR)");
                            }

                            log(LG_INF, "MSSID: IOCTL call for macaddress executed succesfully.");

                            memcpy(config[1].ifhwaddr, ifr_for_index.ifr_hwaddr.sa_data, 6);
                            config[1].cgtk = &gtk[1];


   /* IMPORTANT: set the vss count */
    vss_count = 2;

     /* we have 2 vss interfaces*/
	return 2;
}

#endif // #if 0

