/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * interpreting and creation of EAP packets.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <net/ethernet.h>
#include <string.h>

#include "state_be.h"
#include "state_pae.h"
#include "radius.h"
#include "eapol.h"
#include "eap.h"
#include "md5.h"
#include "log.h"

extern unsigned char frame[1514];

/* we do 16-byte challenges for local MD5 auth */
unsigned char challenge[16];



void eap_process(struct client_t *client)
{
	int l;
	struct eaphdr *eap;

	eap = (struct eaphdr *)client->eap_pkt;

	log(LG_DBG, "eap code=%d id=%d type=%d len=%d", eap->code, eap->id, eap->type, client->eap_pkt_len);
	switch (eap->code) {

		case EAP_CODE_RESPONSE:
			switch (eap->type) {

				case EAP_TYPE_IDENTITY:

					// We only accept type_identity if paestate is connectiong

					if(client->paestate == CONNECTING) {

						l = htons(eap->length) - EAP_HDR_SIZE - 1;
						if(l > MAX_EAP_IDENTITY_LEN) l =MAX_EAP_IDENTITY_LEN;
						memcpy(client->identity, eap->type_data, l);
						client->identity[l] = 0;
						log(LG_DBG, "saving client identity : %s", client->identity);

						pae_authenticating(client);
					} else {
						log(LG_DBG, "Client is not in CONNECTING state, ignoring EAP-identity");
					}

					break;
#ifdef NOTGOOD
				This is not good!!! We need to be able to relay *any* EAP type, not just these,
				we must be transparent to everything. So... I changed this. Discovered this while
				waiting for new SSN code and test setup supported PEAP so thought give PEAP a try

				only EAP-Response/Identity and EAP-Response/Notification are handled specially

				case EAP_TYPE_MD5:
					log(LG_DBG, "relaying EAP-MD5 frame to RADIUS server");
					be_response(client);
					break;

				case EAP_TYPE_TLS:
					log(LG_DBG, "relaying EAP-TLS frame to RADIUS server");
					be_response(client);
					break;

#endif
				case EAP_TYPE_NOTIFICATION:
					log(LG_DBG, "EAP-Response/Notification");

					// This is a special case, the only one we don't send through

					break;
#ifdef NOTGOOD
				case EAP_TYPE_NAK:
					log(LG_DBG, "EAP-Response/Nak");
					break;

#endif
				default:
					log(LG_DBG, "Relaying EAP frame to RADIUS server");
					be_response(client);

					/*log(LG_WRN, "Not a valid eap type '%d'", eap->type);*/
					break;

			}
			break;

		default:
			log(LG_WRN, "not a valid EAP code '%d'", eap->code);
			break;
	}
}



// Create a eap packet in the client structure, and call eapol_tx to send
// it away

int eap_tx(struct client_t *client, unsigned char code, unsigned char type)
{
	struct eaphdr *eap;
	int i;
//	char bufje[256];

//	memcpy(bufje, "This is a display string\x00networkid=eeek,nasid=ap100,portid=22", 119);


	eap = (struct eaphdr *)client->eap_pkt;

	eap->code = code;
	eap->id = client->currentId;
	eap->type = type;

	switch (code) {

	case EAP_CODE_REQUEST:

		switch (type) {

		case EAP_TYPE_IDENTITY:
			log(LG_DBG, "sending EAP-Request/Identity");

//			memcpy(eap->type_data, bufje, 119);

			eap->length = htons(EAP_HDR_SIZE + 1);
			break;

		case EAP_TYPE_NOTIFICATION:
			log(LG_DBG, "sending EAP-Request/Notification");
			eap->length = htons(EAP_HDR_SIZE + 1);
			break;

		case EAP_TYPE_MD5:
			/* only used for local MD5 auth, not when RADIUS is used */
			log(LG_DBG, "sending EAP-Request/MD5-Challenge");

			/* Set value_size (16) and value in packet, we do not set name */
			eap->type_data[0] = 16;

			/* Generate Challenge */
			for (i=1; i<17; i++) {
				eap->type_data[i] = (char) (rand());
				challenge[i-1] = eap->type_data[i];
			}

			/* Length = EAP hdr + type + value_size + value */
			eap->length = htons(EAP_HDR_SIZE + 1 + 1 + 16);
			break;
		}
		break;

	case EAP_CODE_SUCCESS:
		log(LG_DBG, "sending EAP-Success");
		eap->length = htons(EAP_HDR_SIZE);
		break;

	case EAP_CODE_FAILURE:
		log(LG_DBG, "sending EAP-Failure");
		eap->length = htons(EAP_HDR_SIZE);
		break;

	default:
		log(LG_WRN, "unsupported frame");
		return -1;
	}

	client->eap_pkt_len = ntohs(eap->length);
	eapol_tx(client, EAPOL_TYPE_EAP_PACKET);

	return 0;
}



// Fixme, Does XP even show this message ?!

int eap_tx_notification(struct client_t *client, char *msg)
{
	struct eaphdr *eap;

        eap = (struct eaphdr *)client->eap_pkt;

	eap->code = EAP_CODE_REQUEST;
	eap->id = ++client->currentId;
	eap->type = EAP_TYPE_NOTIFICATION;

	strncpy(eap->type_data, msg, strlen(msg));
	eap->length = htons(EAP_HDR_SIZE + strlen(msg) + 1);
	client->eap_pkt_len = EAP_HDR_SIZE + strlen(msg) + 1;

	eapol_tx(client, EAPOL_TYPE_EAP_PACKET);

	log(LG_DBG, "Send eap notification '%s'", msg);

	return(0);
}

// end
