/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * EAPOL packet sending and receiving
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <net/ethernet.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <time.h>

#undef EOVERFLOW

#include "paed.h"
#include "misc.h"
#include "eapol.h"
#include "eap.h"
#include "log.h"
#include "client.h"
#include "state_pae.h"
#include "state_gtk.h"
#include "state_ssn.h"
#include "rc4.h"
#include "hmac_prf.h"
#include "config.h"
#include "keystuff.h"
#include "aes_keywrap.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif

int eapolfd = -1;

#if (PAED_MSSID == 1)

/* Use the global IE variables in MSSID */
extern unsigned char *ie;
unsigned short ie_length;

#endif

//extern unsigned char *ie;
//unsigned short ie_length;

// globals

in_addr_t ifinaddr = -1;
extern char frame[1514];
unsigned long replay_counter_low = 0;           // fixme ?
unsigned long replay_counter_high = 0;          // fixme ?


/*
 * Receive packet from eapol socket, and do some basic sanity checks to see if it
 * is a legal eapol-frame. Match the sender MAC-address with the clientlist. If the
 * packet comes from an unknown client, create a new client struct for it.
 */

int eapol_rx(int sock)
{
        int n;
        struct eapolhdr *eapol;
        struct sockaddr from;
        int fromlen;
        struct client_t *client;
        char src_hwaddr[6];

        log(LG_DBG, "getting pkt from eapol socket");

        fromlen = sizeof(from);
        n = recvfrom(sock, frame, sizeof(frame), 0, &from, &fromlen);
        if (n == -1) {
                log(LG_WRN, "recvfrom");
                return(-1);
        }

        if (n < EAPOL_HDR_SIZE) {
                log(LG_WRN, "too few bytes received for EAPOL");
                return(-1);
        }

        dump_bindata("Incoming EAPOL frame", frame, n);

        /* Point EAPOL frame to data after Ethertype */
        eapol = (struct eapolhdr *)(frame + ETH_HLEN);

        if (eapol->version != EAPOL_VERSION) {
                log(LG_WRN, "bad eapol version");
                return(-1);
        }

        // Get client MAC address from ethernet packet header
        // and find (or create) associated client structure.
        memcpy(src_hwaddr, (char *)frame+6, 6);

        client = client_find_by_hwaddr(src_hwaddr);

        if(client == NULL) {
                log(LG_ERR, "Recv eapol packet from unknown client %s", hwaddr2string(src_hwaddr));
                return(-1);
        }

        /* Everything fine, process frame */
        eapol_process(client, eapol);

        return(0);
}



/*
 * Processess recieved EAPOL frame
 */

int eapol_process(struct client_t *client, struct eapolhdr *eapol)
{
        struct eaphdr *eap;

        log(LG_DBG, "eapol type:%d", eapol->type);

        switch (eapol->type) {

                case EAPOL_TYPE_EAP_PACKET:
                        log(LG_DBG, "EAPOL-Eap");

                        if (ntohs(eapol->length) < EAP_HDR_SIZE) {
                                log(LG_WRN, "packet too short");
                                return(1);
                        }
                        if (ntohs(eapol->length) > 1540)
                        {
                            log(LG_FTL, "eapol->length to big for store.");
                        }

                        // this is a valid EAP packet. Copy it to the eap_pkt in
                        // the client structure, and let eap_process do its work
                        client_malloc_eap_pkt(client);

                        eap = (struct eaphdr *)((unsigned char *)eapol + EAPOL_HDR_SIZE);
                        client->eap_pkt_len = ntohs(eap->length);
                        memcpy(client->eap_pkt, eap, client->eap_pkt_len);

                        eap_process(client);
                        break;

                case EAPOL_TYPE_START:
                        log(LG_DBG, "EAPOL-Start, starting authentication procedure for %s", hwaddr2string(client->hwaddr));
                        if (client->cconfig->enable1X     == 0 &&
                            client->cconfig->enableSSN    == 0 &&
                            client->cconfig->enableSSNPSK == 0)
                        {
                            log(LG_INF, "None of the authentication modes enabled, ignoring 802.1X Eapol-start");
                            break;
                        }

                        /* First EAPOL-Start from WPA-PSK client ignored,
                         * when a EAPOL-Start is received from a already
                         * authenticated (thus secure) client. The client asks for
                         * a re-authentication lets give it to him.
                         */
                        if(client->cconfig->enableSSNPSK == 0 || client->secure) {

                                if(client->paestate != HELD) {
                                        /* Client at least does .1X */
                                        /* But maybe it was not configured !!*/
                                        if (client->does1X)
                                                pae_initialize(client);
                                        if(client->doesSSN) {
                                                ssn_initialize(client);
                                        }

                                } else {
                                        log(LG_DBG, "Client is in HELD state, discarting EAPOL-start");
                                }
                        } else {
                                log(LG_INF, "enableSSNPSK is true : ignoring 802.1X Eapol-start");
                        }

                        break;

                case EAPOL_TYPE_LOGOFF:
                        log(LG_DBG, "EAPOL-Logoff for %s", hwaddr2string(client->hwaddr));

                        pae_aborting(client);

                        break;

                case EAPOL_TYPE_KEY:
                        log(LG_DBG, "EAPOL-Key");

                        // this is an EAPOL-KEY. Copy it to the eap_pkt in
                        // the client structure, and set the EAPOLKeyRecvd flag

                        client_malloc_eap_pkt(client);
                        if (ntohs(eapol->length) > 1540)
                        {
                            log(LG_FTL, "eapol->length to big for store.");
                        }
                        memcpy(client->eap_pkt, eapol, ntohs(eapol->length)+ EAPOL_HDR_SIZE);
                        client->eap_pkt_len = ntohs(eapol->length) + EAPOL_HDR_SIZE;

                        client->EAPOLKeyRecvd = 1;

                        break;

                default:
                        log(LG_WRN, "Recieved unknown eapol type %d from %s ", eapol->type, hwaddr2string(client->hwaddr));
                        break;
        }

        return(0);
}


int eapol_opensocket( void )
{
        // open the socket to get EAP packets
        eapolfd = socket(AF_INET, SOCK_PACKET, htons(ETH_P_EAP));

        if(eapolfd < 0) {
                log(LG_FTL, "Can't open SOCK_PACKET socket: %s");
                return -1;
        }

        return 0;
}

int eapol_bindtosocket( struct config_t *config )
{
        struct ifreq ifr;
        int ifup = 0;
        int tries=0;

#if (PAED_MSSID == 1)
   	char *pstr;
   	char temp_buff[IFNAMSIZ];
#endif

        log(LG_INF, "MSSID: In EAPOL BIND SOCKET");

        /* Bind to the wireless interface */
        memset(&config->eapol_salocal, 0, sizeof(struct sockaddr));
        config->eapol_salocal.sa_family = AF_INET;
        strncpy(config->eapol_salocal.sa_data, config->ifname,
                sizeof(config->eapol_salocal.sa_data));

        // Fetch the hw address of the interface
#if (PAED_MSSID == 1)

// DEBUG
       memset(&ifr, 0, sizeof(struct ifreq));

    	/* the ifname is of type - vssethX.Y. Only copy the Ethernet part 
     	 * in ifr.ifr_name
       */
       pstr = config->ifname;

       // DEBUG
//       pstr = strstr(config->ifname, "eth");

       
       if  (NULL == pstr){
           log(LG_FTL, "MSSID: Invalid Interface name %s", config->ifname);
           return (0);
       }

       /* copy 4 characters in ifr_name - "ethX"*/

// DEBUG Code
      strcpy(ifr.ifr_name, config->ifname);

      // DEBUG
      log(LG_DBG, "MSSID: ifr.ifr_name is %s", ifr.ifr_name);

#else
        memcpy(ifr.ifr_name, config->ifname, sizeof(ifr.ifr_name));

#endif /* #if (PAED_MSSID == 1) */

//        if ( ioctl(eapolfd, SIOCGIFHWADDR, &ifr) < 0 ) {
//                log(LG_FTL, "Couldn't get mac address for interface %s", config->ifname);
//        }


        log(LG_DBG, "local mac address = %02x-%02x-%02x-%02x-%02x-%02x",
                (unsigned char) config->ifhwaddr[0], (unsigned char) config->ifhwaddr[1],
                (unsigned char) config->ifhwaddr[2], (unsigned char) config->ifhwaddr[3],
                (unsigned char) config->ifhwaddr[4], (unsigned char) config->ifhwaddr[5]);

        // Wait for the interface to come up.

        while(!ifup) {
                if ( ioctl(eapolfd, SIOCGIFFLAGS, &ifr) < 0 ) {
                        log(LG_FTL, "Couldn't get mac address for interface %s", config->ifname);
                }

                if(ifr.ifr_flags & IFF_UP) {
                        log(LG_DBG, "Interface %s is up, continuing", config->ifname);
                        ifup = 1;
                } else {
                        tries++;
                        log(LG_DBG, "Waiting for interface %s to come up (%d)", config->ifname, tries);
                        sleep((tries < 10) ? 1 : 5);
                }
        }

        log(LG_INF, "MSSID: EXIT EAPOL BIND SOCKET");
        return(0);
}

/*
 * Wrap the EAP packet from the client struct into an EAPOL packet
 * and send it out
 */

int eapol_tx(struct client_t *client, unsigned char type)
{
        struct eapolhdr *eapol;

        log(LG_DBG, "entered eapol_tx. client->eap_pkg=%p", client->eap_pkt);

        /* Fill in Ethernet header */

        memcpy(frame+0, client->hwaddr, 6);
        memcpy(frame+6, client->cconfig->ifhwaddr, 6);
        frame[12] = 0x88;
        frame[13] = 0x8e;

        /* Fill in EAPOL header */

        eapol = (struct eapolhdr *)(frame + ETH_HLEN);
        eapol->version = EAPOL_VERSION;
        eapol->type = type;
        eapol->length = htons(client->eap_pkt_len);

        /* Copy the EAP packet from the client struct */
        memcpy(eapol->eap_data, client->eap_pkt, client->eap_pkt_len);


        log(LG_DBG, "Sending eapol frame, version=%d type=%d len=%d",
                        eapol->version, eapol->type, client->eap_pkt_len);

        if (sendto(eapolfd, frame, ETH_HLEN + EAPOL_HDR_SIZE + client->eap_pkt_len, 0,
                        &client->cconfig->eapol_salocal, sizeof(struct sockaddr)) < 0) {

                log(LG_ERR, "sendto says '%s'", strerror(errno));
                return(-1);
        }

        dump_bindata("Outgoing EAPOL frame", frame, client->eap_pkt_len + ETH_HLEN + EAPOL_HDR_SIZE);

        return 0;
}


/*
 * Send an old-style eapol key to the client
 */

int eapol_tx_key(struct client_t *client, unsigned char *key_iv, unsigned char *key,
        int key_length, int use_key_length, unsigned char key_index)
{
        struct eapolkey *eapolkey;
        struct eapolhdr *eapol;
        unsigned short length;
        rc4_key rc4key;
        char key_buffer[48];


        // Fill in ethernet header

        memcpy(frame+0, client->hwaddr, 6);
        memcpy(frame+6, client->cconfig->ifhwaddr, 6);
        frame[12] = 0x88;
        frame[13] = 0x8e;

        // Fil in EAPOL header

        eapol = (struct eapolhdr *)(frame + ETH_HLEN);

        eapol->version  = EAPOL_VERSION;
        eapol->type     = EAPOL_TYPE_KEY;
        length = sizeof(struct eapolkey) - 1 + key_length;
        eapol->length   = htons(length);


        // Fil in EAPOLKEY structure

        eapolkey = (struct eapolkey *)eapol->eap_data;

        eapolkey->descriptor_type               = RC4_KEY_DESCRIPTOR;
        eapolkey->length                        = htons(use_key_length);
        eapolkey->replay_counter_low            = htonl(++replay_counter_low);
        if (replay_counter_low == 0x00)
        {
            replay_counter_high++;
        }
        eapolkey->replay_counter_high           = htonl(replay_counter_high);
        memcpy(eapolkey->key_iv, key_iv, 16);
        eapolkey->key_index = key_index;

        memset(eapolkey->signature, 0x0, 16);


        if(key_length > 0) {

                // Copy the encrypted key in the EAPOL-Key                             /* ARNOUD DONT EDIT THIS FUNCTION FOR SSN */
                memcpy(eapolkey->key, key, key_length);                                 /* I KEEP DOING THIS */

                // RC4 encrypt the key with the Tx session key
                memcpy(key_buffer, key_iv, 16);
                memcpy((char *)key_buffer+16, (char *)client->tx_session_key, client->session_key_len);
                rc4_prepare_key((char *)key_buffer, 16 + client->session_key_len, &rc4key);

                rc4(eapolkey->key, key_length, &rc4key);
        }

        // Sign EAPOL-Key with Rx master key

        hmac_md5(       (char *)eapol,
                        length + EAPOL_HDR_SIZE,
                        (char *)client->rx_session_key,
                        client->session_key_len,
                        (char *)eapolkey->signature);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("rx_session key", client->rx_session_key, client->session_key_len);
        dump_bindata("tx_session key", client->tx_session_key, client->session_key_len);
#endif

        log(LG_DBG, "Sending eapol-key frame, version=%d type=%d index=%d",
                        eapol->version, eapol->type, key_index);


        length = ETH_HLEN + EAPOL_HDR_SIZE + sizeof(struct eapolkey) - 1 + key_length;

        if (sendto(eapolfd, frame, length, 0, &client->cconfig->eapol_salocal,
                sizeof(struct sockaddr)) < 0) {

                log(LG_ERR, "sendto says '%s'", strerror(errno));
                return(-1);
        }

        dump_bindata("Outgoing EAPOLKEY frame", frame, length);


        return 0;
}


/*
 * Send an SSN-style eapol key to the client
 *
 * This function constructs and sends eapol_key packets for WPA (former SSN)
 * It is used for the 4-way handshake, as well as the GTK update.
 */

int eapol_tx_key_ssn(   struct client_t *client,
                        u_char secure,
                        u_char mic_present,
                        u_char ack_reqd,
                        u_char tx_install,
                        u_char key_index,
                        u_char key_type,
                        u_char *nonce,
                        u_char *key,
                        u_char key_length,
                        u_char use_key_length)

{
        struct eapolkey_ssn *eapolkey;
        struct eapolhdr *eapol;
        unsigned short length;
        rc4_key rc4key;
        char key_buffer[48];
        char dummybuffer[256];
#ifdef BLOB_V1
        long dummy_stastat[32];
        struct obj_stastat *stat;
        struct sub_obj_seq_counter *counter;
#endif
#ifdef BLOB_V2
    long dummy_stasc[4];
        struct obj_stasc *stat;
#endif

        /*
         * Fill in ethernet header
         */

        memcpy(frame+0, client->hwaddr, 6);
        memcpy(frame+6, client->cconfig->ifhwaddr, 6);
        frame[12] = 0x88;
        frame[13] = 0x8e;

        /*
         * Fill in EAPOL header
         */
         
        eapol = (struct eapolhdr *)(frame + ETH_HLEN);
        
        memset(eapol, 0, sizeof(struct eapolhdr));

        eapol->version  = EAPOL_VERSION;
        eapol->type     = EAPOL_TYPE_KEY;
        length = sizeof(struct eapolkey_ssn) - 1 + key_length;
        eapol->length   = htons(length);

        /*
         * Fill in EAPOLKEY structure
         */
         
        eapolkey = (struct eapolkey_ssn *)eapol->eap_data;

        eapolkey->descriptor_type               = SSN_KEY_DESCRIPTOR;
        eapolkey->length                        = htons(use_key_length);
        eapolkey->replay_counter_low            = client->replay_counter_low            = htonl(++replay_counter_low);
        if (replay_counter_low == 0x00)
        {
            replay_counter_high++;
        }
        eapolkey->replay_counter_high   = client->replay_counter_high   = htonl(replay_counter_high);

        if (nonce)
                memcpy(eapolkey->nonce, nonce, NONCE_LEN);

        /*
         * Fill in key info flags
         */
       
       	eapolkey->key_info = 0;
       
       	if(client->client_wpa_ie.ucast == UCAST_CIPHER_TKIP) {
       		eapolkey->key_info |= (EAPOL_KEY_DESCRIPTOR_HMAC_MD5_RC4 << 0);
       	}
       	
	if(client->client_wpa_ie.ucast == UCAST_CIPHER_AES_VAR_2) {
       		eapolkey->key_info |= (EAPOL_KEY_DESCRIPTOR_AES_128_CBC_MAC << 0);
	}

        
        eapolkey->key_info |= (unsigned short)(key_type << 3);
        eapolkey->key_info |= (unsigned short)(key_index << 4);
        eapolkey->key_info |= (unsigned short)(tx_install << 6);
        eapolkey->key_info |= (unsigned short)(ack_reqd << 7);
        eapolkey->key_info |= (unsigned short)(mic_present << 8);
        eapolkey->key_info |= (unsigned short)(client->secure << 9);
	log(LG_INF, "key_info = %x\n", eapolkey->key_info);

        eapolkey->key_info = htons(eapolkey->key_info);

        memset(eapolkey->MIC, 0x0, 16);

        eapolkey->key_len = htons(key_length);



        /* 
         * If we have keying material, put this in (encrypted) 
         */

        if(key_length > 0) {

                // Note the IV is not necessary anymore. The RC4 engine is initialized by the
                // EAPOL-Key encryption key only and the RC4 is seen as a stream (over multiple
                // packets). If packets (EAPOL-Keys) get lost, we reset everything and disassociate, so this works.
                // This implies we need to maintain RC4 state per client. The EAPOL-Key
                // encryption key therefore doesn't need to be saved, but we still do that.

                // Ok the IV is back in :) state maintained was too complex for retries...
                // IV is now in the previous GNonce field

                gtk_inc_counter(client->cconfig);


		/*
		 * TKIP clients get their key RC4 encoded
		 */

		if(client->client_wpa_ie.ucast == UCAST_CIPHER_TKIP) {
		
			log(LG_DBG, "RC4'ing the key, key length=%d", key_length);
	
	                // Init RC4

	                memcpy(eapolkey->key_iv, (client->cconfig->cgtk->Counter + 15), 16);  // Take the least significant part.

	                memcpy(key_buffer, eapolkey->key_iv, 32);
       	        	memcpy(key_buffer+16, client->eapolkey_enckey, 16);
                	rc4_prepare_key(key_buffer, 32, &rc4key);

                	// Discard the first 256 bytes of RC4 outputa

                	rc4(dummybuffer, 256, &rc4key);

                	// Copy the key into the EAPOL-Key key field

                	memcpy(eapolkey->key, key, key_length);

                	// RC4 encrypt the key with the RC4 state

                	rc4(eapolkey->key, key_length, &rc4key);

                	/* Set address for group key update TSC*/
                	memset(dummybuffer, 0 , 256);
                }
                

                /*
                 * AES clients want aes_keywrap encoding
                 */
                
                if(client->client_wpa_ie.ucast == UCAST_CIPHER_AES_VAR_2) {

                	AESKeyWrapContext *cont;
                	int outlen;
                	char output[64];
                	
                	log(LG_DBG, "AES-keywrapping the key : key_length = %d", key_length);

                	cont = AESKeyWrap_CreateContext (client->eapolkey_enckey, "\xa6\xa6\xa6\xa6\xa6\xa6\xa6\xa6", 1, sizeof(client->eapolkey_enckey));
                	
                	AESKeyWrap_Encrypt(cont, output, &outlen, sizeof(output), key, key_length);

                	memcpy(eapolkey->key, output, outlen);
                	eapolkey->key_len = htons(outlen);
                	
                	/*
                	 * AES-Keywrap output is longer then the input; add the difference of this length
                	 * to the eapol->length too ! 
                	 */
                	
		        length = sizeof(struct eapolkey_ssn) - 1 + key_length + 8;
		        eapol->length   = htons(length);

                	AESKeyWrap_DestroyContext(cont);
                }

#ifdef BLOB_V1
                stat = (struct obj_stastat *)dummy_stastat;
                memset(stat->address, 0xff, 6);
                stat->code = DOT11_STAT_TSC;
                stat->size = sizeof(struct sub_obj_seq_counter);
                counter = (struct sub_obj_seq_counter*)stat->data;

                /*
                 * Add the Sequence Counter of the broadcast key in the group key update
                 * The counter should only be filled in when the key is already active,
                 * in this case the SC is not ZERO.
                 * In the other case when the key is not yet active the SC is ZERO.
                 * The check for gtk.GTKKeyActive == 0 implies the key is not active yet.
                 */
                if (!client->cconfig->cgtk->GTKKeyActive ||
                    blob_get_stastat(stat, client->cconfig) == -1 ||
                    client->cconfig->enable1X || client->cconfig->enableWEP)
                {
                    log(LG_DBG, "Adding zeroed group key sequence counters: sc_low 0 sc_high 0");
                    eapolkey->sc_low = 0;
                    eapolkey->sc_high = 0;
                }
                else
                {
                    /*
                     * Substract 1 from this counter cause this is the counter
                     * of the next packet.
                     */
                    //dump_bindata("RSC", (char*)stat, (sizeof(struct obj_stastat) + sizeof(struct sub_obj_seq_counter) - 1));

                    if( counter->tsc_low != 0 || counter->tsc_high != 0 )
                    {
                        if ( counter->tsc_low == 0 )  // Just rolled over ?
                            counter->tsc_high -= 1;        // Decrement long part
                        counter->tsc_low -= 1;             // Always decrement short part
                    }
                    log(LG_DBG, "Adding current group key sequence counters: sc_low %d sc_high %u",
                        counter->tsc_low, counter->tsc_high );
                    /* Don't set this in network order */
                    eapolkey->sc_low = counter->tsc_low;
                    eapolkey->sc_high = counter->tsc_high;

                }
#endif
#ifdef BLOB_V2
                stat = (struct obj_stasc *)dummy_stasc;
                memset(stat->address, 0xff, 6);
                stat->ext = 0;
                stat->keyid = key_index;
                stat->tx_sc = 1;                // TX sequence counters

                /*
                 * Add the Sequence Counter of the broadcast key in the group key update
                 * The counter should only be filled in when the key is already active,
                 * in this case the SC is not ZERO.
                 * In the other case when the key is not yet active the SC is ZERO.
                 * The check for gtk.GTKKeyActive == 0 implies the key is not active yet.
                 */
                if (!client->cconfig->cgtk->GTKKeyActive ||
                    blob_get_stasc(stat, client->cconfig) == -1 ||
                    client->cconfig->enable1X || client->cconfig->enableWEP)
                {
                    log(LG_DBG, "Adding zeroed group key sequence counters: sc_low 0 sc_high 0");
                    eapolkey->sc_low = 0;
                    eapolkey->sc_high = 0;
                }
                else
                {
                    /*
                     * Substract 1 from this counter cause this is the counter
                     * of the next packet.
                     */
                    //dump_bindata("RSC", (char*)stat, (sizeof(struct obj_stastat) + sizeof(struct sub_obj_seq_counter) - 1));

                    if( stat->sc_low != 0 || stat->sc_high != 0 )
                    {
                        if ( stat->sc_low == 0 )        // Just rolled over ?
                            stat->sc_high -= 1;     // Decrement long part
                        stat->sc_low -= 1;                  // Always decrement short part
                    }
                    log(LG_DBG, "Adding current group key sequence counters: sc_low %d sc_high %u",
                        stat->sc_low, stat->sc_high );
                    /* Don't set this in network order */
                    eapolkey->sc_low = stat->sc_low;
                    eapolkey->sc_high = stat->sc_high;
                }

#endif

                eapolkey->padd = 0;
                memset(eapolkey->key_id, 0x00, 8);
                
//                asm("int $3");

        }
        else
        {
            /* Always clear the SC fields. */
            eapolkey->sc_low = 0;
            eapolkey->sc_high = 0;
            eapolkey->padd = 0;
            memset(eapolkey->key_id, 0x00, 8);
        }

        /* 
         * If this is the 3rd msg of the 4-way handshake, add the SSN IE advertised in the beacon 
         */

        if (key_type == 1 && mic_present) { /* pairwise and mic is present only in 3rd msg */
        
                log(LG_INF, "Adding SSN IE to 3rd EAPOL-Key, length %d", client->cconfig->wpa_ie.raw_ie_length);
                
                dump_bindata("IE", client->cconfig->wpa_ie.raw_ie, client->cconfig->wpa_ie.raw_ie_length + 2);

#if (PAED_MSSID == 1)
				/* Use the global IE variables in MSSID */
                memcpy(eapolkey->key, ie, ie_length);
                eapolkey->key_len = htons(ie_length);
                length = sizeof(struct eapolkey_ssn) - 1 + ie_length;

#else
                memcpy(eapolkey->key, client->cconfig->wpa_ie.raw_ie, client->cconfig->wpa_ie.raw_ie_length + 2);
                eapolkey->key_len = htons(client->cconfig->wpa_ie.raw_ie_length + 2);

                length = sizeof(struct eapolkey_ssn) - 1 + client->cconfig->wpa_ie.raw_ie_length + 2;
#endif /*#if (PAED_MSSID == 1)*/                

                eapol->length   = htons(length);
        }

        if (mic_present) {

	       	if(client->client_wpa_ie.ucast == UCAST_CIPHER_TKIP) {
	       	

	                /*
	                 * For TKIP ucast clients, Sign EAPOL-Key with EAPOL-Key MIC key
	                 */

	       		log(LG_DBG, "Applying MD5 MIC");
       		        hmac_md5(      	(char *)eapol,
       	       				length + EAPOL_HDR_SIZE,
                        	        (char *)client->eapolkey_MICkey,
                               	 	16,
                               	 	(char *)eapolkey->MIC);
       		}
       	
		if(client->client_wpa_ie.ucast == UCAST_CIPHER_AES_VAR_2) {
		
	                /*
	                 * For AES ucast clients, Sign EAPOL-Key with MIC_AES
	                 */
		
	       		log(LG_DBG, "Applying AES MIC");
       		        MIC_AES(      	(char *)eapol,
       	        	           	length + EAPOL_HDR_SIZE,
                        	        (char *)client->eapolkey_MICkey,
                               	 	16,
                               	 	(char *)eapolkey->MIC);
	       	}
        }

#ifdef DEBUG_PRINTKEYS
        dump_bindata("rx_session key", client->rx_session_key, client->session_key_len);
        dump_bindata("tx_session key", client->tx_session_key, client->session_key_len);
#endif

        log(LG_DBG, "Sending eapol-key SSN frame, version=%d type=%d key_index=%d",
                        eapol->version, eapol->type, key_index);


        length = ETH_HLEN + EAPOL_HDR_SIZE + htons(eapol->length);

        if (sendto(eapolfd, frame, length, 0, &client->cconfig->eapol_salocal,
                sizeof(struct sockaddr)) < 0) {

                log(LG_ERR, "sendto says '%s'", strerror(errno));
                return(-1);
        }
        dump_bindata("Outgoing EAPOLKEY frame", frame, length);

        return 0;
}


// eot
