/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
#ifndef _EAPOL_H_
#define _EAPOL_H_

#include "client.h"
#include "config.h"

#define ETH_P_EAP               0x888E

#define EAPOL_VERSION           0x01

/* EAPOL Types */
#define EAPOL_TYPE_EAP_PACKET   0x00
#define EAPOL_TYPE_START        0x01
#define EAPOL_TYPE_LOGOFF       0x02
#define EAPOL_TYPE_KEY          0x03

/* Header sizes */
#define EAPOL_HDR_SIZE          4
#define EAP_HDR_SIZE            4
#define EAPOLKEY_SSN_HDR_SIZE   95

/* EAP Codes */
#define EAP_CODE_REQUEST        0x01
#define EAP_CODE_RESPONSE       0x02
#define EAP_CODE_SUCCESS        0x03
#define EAP_CODE_FAILURE        0x04

/* EAP Types */
#define EAP_TYPE_IDENTITY       0x01
#define EAP_TYPE_NOTIFICATION   0x02
#define EAP_TYPE_NAK            0x03
#define EAP_TYPE_MD5            0x04
#define EAP_TYPE_TLS            0x0d

/* EAPOL-Key definitions */

#define RC4_KEY_DESCRIPTOR      0x01
#define PER_STA_KEY_DESCRIPTOR  0x02
#define SSN_KEY_DESCRIPTOR      0xFE

/* EAPOL-Key Information Field masks
 *    -------------------------------------------------------------------
 *    | b15 b14 b13 b12 b11 b10 b09 b08 b07 b06 b05 b04 b03 b02 b01 b00 |
 *    -------------------------------------------------------------------
 *
 *	The Field is big endian, so we need to use htons(info) to get the
 *      proper orientation on our host system.
 *
 *      b15-12: RESERVED
 *      b11:    REQUEST                 1=Supplicant request 4-way handshake
 *      b10:    ERROR                   1=Integrity failure detected
 *      b09:    SECURE                  1=PTK is installed link is secure 0=other
 *      b08:    MIC                     1=MIC in EAPOL 0=No MIC in EAPOL
 *      b07:    ACK                     Response required.
 *      b06:    USAGE                   PTK 1=plump 0=calc GTK 1=TX/RX 0=RX only
 *      b05-04: KEY INDEX               0,1,2 and 3 for default key index.
 *      b03:    KEY TYPE                1=PTK 0=GTK
 *      b02-00: DESCRIPTOR TYPE         1 .. 7  1=MIC is HMAC-MD5
 *
 */

#define EAPOL_KEY_INFO_MASK_DESCRIPTOR_TYPE     0x0007
#define EAPOL_KEY_INFO_MASK_KEY_TYPE            0x0008
#define EAPOL_KEY_INFO_MASK_KEY_INDEX           0x0030
#define EAPOL_KEY_INFO_MASK_USAGE               0x0040
#define EAPOL_KEY_INFO_MASK_ACK                 0x0080
#define EAPOL_KEY_INFO_MASK_MIC                 0x0100
#define EAPOL_KEY_INFO_MASK_SECURE              0x0200
#define EAPOL_KEY_INFO_MASK_ERROR               0x0400
#define EAPOL_KEY_INFO_MASK_REQUEST             0x0800
#define EAPOL_KEY_INFO_MASK_RESERVE1            0x1000
#define EAPOL_KEY_INFO_MASK_RESERVE2            0x2000
#define EAPOL_KEY_INFO_MASK_RESERVE3            0x4000
#define EAPOL_KEY_INFO_MASK_RESERVE4            0x8000

#define EAPOL_KEY_DESCRIPTOR_HMAC_MD5_RC4       0x0001
#define EAPOL_KEY_DESCRIPTOR_AES_128_CBC_MAC   0x0002

struct eapolhdr {
        unsigned char version;
        unsigned char type;
        unsigned short length;
        unsigned char eap_data[1];
} __attribute__ ((__packed__));

struct eapolkey {
        unsigned char   descriptor_type;
        unsigned short  length;
        unsigned long   replay_counter_high;
        unsigned long   replay_counter_low;
        unsigned char   key_iv[16];
        unsigned char   key_index;
        unsigned char   signature[16];
        unsigned char   key[1];
} __attribute__ ((__packed__));

struct eapolkey_ssn {
        unsigned char   descriptor_type;
        unsigned short  key_info;
        unsigned short  length;
        unsigned long   replay_counter_high;
        unsigned long   replay_counter_low;
        unsigned char   nonce[NONCE_LEN];
        unsigned char   key_iv[16];
        unsigned short  sc_low;
        unsigned long   sc_high;
        unsigned short  padd;
        unsigned char   key_id[8];
        unsigned char   MIC[MIC_LEN];
        unsigned short  key_len;
        unsigned char   key[1];
} __attribute__ ((__packed__));

// proto's

int eapol_rx(int sock);
int eapol_process(struct client_t *client, struct eapolhdr *eapol);
int eapol_opensocket( void );
int eapol_bindtosocket( struct config_t *config );

int eapol_tx(struct client_t *client_t, unsigned char type);
int eapol_tx_key(struct client_t *client, unsigned char *key_iv, unsigned char *key, int key_length, int use_key_length, unsigned char key_index);
int eapol_tx_key_ssn(struct client_t *client, u_char secure, u_char mic_present, u_char ack_reqd, u_char tx_install, u_char key_index, u_char key_type, u_char *nonce, u_char *key, u_char key_length, u_char use_key_length);
int eapol_tx_key_newstyle(struct client_t *client, unsigned char *key_iv, unsigned char *key, int key_length, int use_key_length, unsigned char key_index);

#endif
