/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "md5.h"
#include "sha1.h"


/*
 * hmac_md5
 *
 * data+length is data to encrypt
 * key + key length is key to sign with
 * digest is 16-byte hash - the outcome
 */

void hmac_md5(unsigned char *data, int length, unsigned char *key, int key_length, unsigned char *digest)
{
	unsigned char k_ipad[65];
	unsigned char k_opad[65];
	unsigned char tk[16];
	int i;

	MD5_CTX	context;

	/* If key length > 64, reset it to MD5(key) */
	if (key_length > 64) {
		MD5_CTX tctx;

		MD5Init(&tctx);
		MD5Update(&tctx, key, key_length);
		MD5Final(tk, &tctx);

		key = tk;
		key_length = 16;
	}

	/* HMAC-MD5 transform, start out by storing key in pads */
	bzero(k_ipad, sizeof(k_ipad));
	bzero(k_opad, sizeof(k_opad));
	bcopy(key, k_ipad, key_length);
	bcopy(key, k_opad, key_length);

	/* XOR key with ipad and opad values */
	for (i=0; i<64; i++) {
		k_ipad[i] ^= 0x36;
		k_opad[i] ^= 0x5c;
	}

	/* Perform inner MD5 */
	MD5Init(&context);
	MD5Update(&context, k_ipad, 64);
	MD5Update(&context, data, length);
	MD5Final(digest, &context);

	/* Perform outer MD5 */
	MD5Init(&context);
	MD5Update(&context, k_opad, 64);
	MD5Update(&context, digest, 16);
	MD5Final(digest, &context);
}


/*
 * hmac_sha1
 *
 * data+length is data to encrypt
 * key + key length is key to sign with
 * digest is 20-byte hash - the outcome
 * t is minimal size of output (not necessary for us probably)
 */

#define SHA_BLOCKSIZE		64
#define SHA_DIGESTSIZE		20

void hmac_sha1(unsigned char *data, int length, unsigned char *key, int key_length, char *digest)
{
	SHA1_CTX ictx, octx;
	unsigned char tk[SHA_DIGESTSIZE];
	unsigned char buf[SHA_BLOCKSIZE];
	int i;

	if (key_length > SHA_BLOCKSIZE)
	{
		SHA1_CTX	tctx;

		SHA1Init(&tctx);
		SHA1Update(&tctx, key, key_length);
		SHA1Final(tk, &tctx);

		key = tk;
		key_length = SHA_DIGESTSIZE;
	}

	/*** Inner digest ***/

	SHA1Init(&ictx);

	/* Pad the key for inner digest */
	for (i=0; i<key_length; ++i) buf[i] = key[i] ^ 0x36;
	for (i=key_length; i<SHA_BLOCKSIZE; ++i) buf[i] = 0x36;

	SHA1Update(&ictx, buf, SHA_BLOCKSIZE);
	SHA1Update(&ictx, data, length);

	SHA1Final(digest, &ictx);

	/*** Outer digest ***/

	SHA1Init(&octx);

	/* Pad the key for outer digest */
	for (i=0; i<key_length; ++i) buf[i] = key[i] ^ 0x5c;
	for (i=key_length; i<SHA_BLOCKSIZE; ++i) buf[i] = 0x5c;

	SHA1Update(&octx, buf, SHA_BLOCKSIZE);
	SHA1Update(&octx, digest, SHA_DIGESTSIZE);

	SHA1Final(digest, &octx);
}
