/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
/*
 * Functions for working with 802.11 Information Elements
 */

#include <string.h>
#include <stdio.h>
#include <net/if.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <signal.h>
#include <net/if.h>

#include "config.h"
#include "log.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "client.h"
#include "misc.h"
#include "config.h"
#include "state_pae.h"
#include "state_ssn.h"
#include "ie.h"


#if (PAED_MSSID == 1)

/* the config array pointer */
extern struct config_t *config;
#else

extern struct config_t config[MAX_INTERFACES];
#endif  /* #if (PAED_MSSID == 1) */



#if (PAED_MSSID == 1)

/* Use the global IE variables for MSSID */
long oabuf[16];
struct obj_attachment *oa = (struct obj_attachment *)oabuf;
unsigned char *ie;
unsigned short ie_length;

#endif  /* #if (PAED_MSSID == 1) */


//unsigned char *ie;

const unsigned char OUI[] = { 0x00, 0x50, 0xf2 };



/*
 * ie_set creates the information element according the the settings
 * in the config structure, and passes it to the MVC for inclusion in
 * the beacons
 */
int ie_set(struct config_t *config)
{
    int version = WPA_IE_VERSION;
    unsigned short ie_length;
    long oabuf[16];
    struct obj_attachment *oa = (struct obj_attachment *)oabuf;
    char *ie;
#ifdef SMAC
    unsigned char *p = (unsigned char *) oa + sizeof(struct obj_attachment);
    ie = (unsigned char *) oa + sizeof(struct obj_attachment);
#else
    unsigned char *p = oa->data;
    ie = oa->data;
#endif
    memset(oa, 0x00, sizeof(struct obj_attachment));
    memset(ie, 0x00, sizeof(oabuf)-sizeof(struct obj_attachment)+1);
	
    // Element ID
    *p++ = WPA_IE_TAG;

    ie_length = WPA_IE_LENGTH;
    if(config->wpa_cipher & 1) ie_length += 4;	/* Add length of TKIP suite */
    if(config->wpa_cipher & 2) ie_length += 4;	/* Add length of AES suite */

#ifdef SMAC
    /*
     * The SoftMAC supports only add/remove functionality 
     * First do a remove before the add
     */
#else
    /* Check if the current configuration needs a WPA IE */
    if(!config->enableSSN && !config->enableSSNPSK)
    {
#endif
	// Remove any attachments from previous sessions beacons
	oa->id =	-1;
	oa->size =	0;
	oa->type = 	IEEE_802_11_FRM_TYPE_BEACON;
	blob_set_attachment(oa, (sizeof(struct obj_attachment)), config);

	// Remove any attachments from previous sessions probe responses
	oa->type =	IEEE_802_11_FRM_TYPE_PROBE_RES;
	blob_set_attachment(oa, (sizeof(struct obj_attachment)), config);
#ifdef SMAC
    /* Check if the current configuration needs a WPA IE */
    if(!config->enableSSN && !config->enableSSNPSK)
		return(0);	
#else
	return (0);
    }
#endif

    // Length
    *p++ = ie_length;

    // OUI
    *p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];

    // OUI Type
    *p++ = WPA_OUI_TYPE;

    // Version
    *p++ = (version >> 0) && 0xFF;
    *p++ = (version >> 8) && 0xFF;

    // Multicast suite
    if(config->enable1X || config->enableWEP)
    {
	*p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];
	if (config->Dot1XWEPKeySize == CONF_WEP_40_SIZE)
	{
	    config->wpa_ie.mcast = MCAST_CIPHER_WEP_40;
	    *p++ = MCAST_CIPHER_WEP_40;
	}
	else
	{
	    config->wpa_ie.mcast = MCAST_CIPHER_WEP_104;
	    *p++ = MCAST_CIPHER_WEP_104;
	}
    }
    else
    {
    	/*
    	 * Check what mcast suite should be advertized. If only AES
    	 * is enabeld, we use AES, otherwise TKIP gets preference
    	 */
    	 
	*p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];
	if (config->wpa_cipher & 1) {				/* TKIP is the 'lowest' */
		config->wpa_ie.mcast = MCAST_CIPHER_TKIP;
		*p++ = MCAST_CIPHER_TKIP;
	} else {
		config->wpa_ie.mcast = MCAST_CIPHER_AES_VAR_2;	/* AES otherwise */
		*p++ = MCAST_CIPHER_AES_VAR_2;
	}
    }

    // Unicast suite count

    *p = 0;
    if(config->wpa_cipher & 1) (*p)++;
    if(config->wpa_cipher & 2) (*p)++;
    p++;
    *p++ = 0x00;
    config->wpa_ie.ucast_count = 0;

    if(config->wpa_cipher & 1) {
        *p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];
        config->wpa_ie.ucast[config->wpa_ie.ucast_count] = UCAST_CIPHER_TKIP;
        *p++ = UCAST_CIPHER_TKIP;
	config->wpa_ie.ucast_count ++;
        log(LG_DBG, "Advertizing TKIP unicast suite in IE");
    }

    if(config->wpa_cipher & 2) {
        *p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];
        config->wpa_ie.ucast[config->wpa_ie.ucast_count] = UCAST_CIPHER_AES_VAR_2;
        *p++ = UCAST_CIPHER_AES_VAR_2;
	config->wpa_ie.ucast_count ++;
        log(LG_DBG, "Advertizing AES unicast suite in IE");
    }

    // Authentication suite count
    *p++ = 0x01; *p++ = 0x00;
    config->wpa_ie.auth_count = 1;

    // Authentication algoritm
    if(config->enableSSNPSK) {
	*p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];
	config->wpa_ie.auth[0] = AUTH_WPA_PSK;
	*p++ = AUTH_WPA_PSK;
    } else {
	*p++ = OUI[0]; *p++ = OUI[1]; *p++ = OUI[2];
	config->wpa_ie.auth[0] = AUTH_UNSPECIFIED_OVER_802_1X;
	*p++ = AUTH_UNSPECIFIED_OVER_802_1X;
    }

    /* Add capability field */
    *p++ = WPA_CAP_1_REPLAY_COUNTER;
    *p++ = 0x00;

    dump_bindata("Information element", ie, ie_length);

#if defined(SMAC) || defined(MVC2_1)
    oa->ssnr = 0;
#else   
    oa->reserved = 0;
#endif


    oa->id =		-1;
    oa->size = 		ie_length + 2;

    // Set attachment for beacon
    oa->type = 		IEEE_802_11_FRM_TYPE_BEACON;
    blob_set_attachment(oa, (oa->size + sizeof(struct obj_attachment)), config);

    // set attachment for probe response
    oa->type = 		IEEE_802_11_FRM_TYPE_PROBE_RES;
    blob_set_attachment(oa, (oa->size + sizeof(struct obj_attachment)), config);

    /*
     * Copy the raw IE to the config struct, for later usage at eapol traffic
     */
     
    memcpy(config->wpa_ie.raw_ie, ie, ie_length);
    config->wpa_ie.raw_ie_length = ie_length;

    return(0);
}

/*
 * ie_parse_assocreq:
 * 	Parse a association frame in order to find WPA IE, and determine if client should
 *	be allowed access.
 * IN/OUT:  struct obj_mlmeex *mlmeex	Pointer to mlme structure received from MVC
 *	    struct client_t *client	Pointer to client struture where to store information
 * RETURN:  0 if client should be allowed access
 *	    -1 if a client should be denied access
 */
#ifdef SMAC
int ie_parse_assocreq(struct obj_mlme *mlmeex, struct client_t *client, int reassoc)
#else
int ie_parse_assocreq(struct obj_mlmeex *mlmeex, struct client_t *client, int reassoc)
#endif
{
    int i;
    int len;
    int tag;
    unsigned char *p;
    int ret = 0;
    
    /* The mlmeex->data contains the plaintext MSDU association request frame
     * this frame possibly contains a mac address in case of a REASSOCIATION frame.
     * Setup the pointer p to the beginning of the information ellements.
     */
    if (reassoc)
#ifdef SMAC
    {
        //reassociation contains extra the address of the current AP
	p = (unsigned char *) mlmeex + sizeof(struct obj_mlme) + 10;  
    }
    else
    {
        p = (unsigned char *) mlmeex + sizeof(struct obj_mlme) + 4;
    }
#else
    {
	p = mlmeex->data + 10;  //reassociation contains extra the address of the current AP
    }
    else
    {
        p = mlmeex->data + 4;
    }
#endif

    /* If WPA is on try to find WPA IE */
    if (client->cconfig->enableSSN || client->cconfig->enableSSNPSK)
    {
	for( i = 0 ; i<mlmeex->size-4; i += p[i+1]+2)
	{
	    tag = p[i];
	    len = p[i+1];

	    /* WPA IE found ? */
	    if(tag == WPA_IE_TAG )
	    {



             // DEBUG code
             log(LG_INF, "MSSID: Client sent WPA IE");

//#if (PAED_MSSID == 1)
#if 0
             /* Found WPA IE. No need to validate the IE as it is
              * validated in SoftMAC. Just set the doesSSN flag in client
              * structure. The does1X flag is set in client_init()
              */
              client->doesSSN = 1;

              /* Copy the IE in the client struct */
	    	   if (len+2 > WPA_IE_MAX_LEN)
		   {
                 len = WPA_IE_MAX_LEN - 2;
		   }
    		   memcpy(client->client_wpa_ie.assoc_ie, (p+i), len+2);
  	         client->client_wpa_ie.assoc_ie_len = len;

              /* Just had to check for the wpa ie
               * Return now 
               */
              return 0;
//#else
#endif


            ret = ie_check_ellement((p+i), len, client);
            if (ret == -2)
            {
    		    log(LG_ERR, "Found IE not the correct OUI type");
                continue;
            }
            if (ret == -1)
            {
    		    log(LG_ERR, "The client WPA IE didn't match the APs WPA IE");
#ifdef SMAC
	    	    dump_bindata("Received association data", ((unsigned char *) mlmeex 
		    	+ sizeof(struct obj_mlme)), mlmeex->size);
#else
	    	    dump_bindata("Received association data", mlmeex->data, mlmeex->size);
#endif
                continue;
            }

    		/* Copy the WPA IE in client struture */
	    	if (len+2 > WPA_IE_MAX_LEN)
		    {
                len = WPA_IE_MAX_LEN - 2;
		    }
    		memcpy(client->client_wpa_ie.assoc_ie, (p+i), len+2);
	    	client->client_wpa_ie.assoc_ie_len = len;

    		client->doesSSN = 1;

//#endif /* #if (PAED_MSSID == 1) */

	    	return (0);
	    }
	}
    }

    /* if WEP or 1X or OpenSystem is on allow client */
    if (client->cconfig->enableWEP || client->cconfig->enable1X || client->cconfig->enableOS)
    {
        // DEBUG Code
        log(LG_INF, "MSSID: We support WEP/ OS/ 1X and client did not send a WPA IE");


	return (0);
    }
    /* else don't allow client cause only WPA or WPA-PSK is on and client failed IE check */
    else
    {
	log(LG_INF, "No WP IE found while only WPA or WPA-PSK is on, disassoc client");
	return (-1);
    }
}



int ie_check_ellement(unsigned char* p, unsigned int len, struct client_t *client)
{
    int ret = 0;
    int version;
    int count;
    int i;

    /* First check for correct OUI */
    if (p[2] != OUI[0] || p[3] != OUI[1] || p[4] != OUI[2])
    {
	log(LG_INF, "The OUI didn't match %2.2x %2.2x %2.2x received %2.2x %2.2x %2.2x", OUI[0], OUI[1], OUI[2], p[2], p[3], p[4]);
	ret = -2;
    }

    /* OUI Type */
    if (p[5] != WPA_OUI_TYPE)
    {
	log(LG_INF, "The OUI Type didn't match %2.2x received %2.2x", WPA_OUI_TYPE, p[5]);
	ret = -1;
    }

    /* Check Version */
    version = p[6] + (p[7] << 8);
    if (version > WPA_IE_VERSION)
    {
	log(LG_INF, "The SSN IE version didn't match %2.2x received %2.2x", WPA_IE_VERSION, version);
	ret = -1;
    }

    if (len != 6)
    {
	/* They not decided to default the ciphers and suites, so check the rest */
	if (memcmp(OUI, p+8, 3) != 0)
	{
	    log(LG_INF, "The WPA IE multicast OUI didn't match %2.2x %2.2x %2.2x received %2.2x %2.2x %2.2x", OUI[0], OUI[1], OUI[2], p[8], p[9], p[10]);
	    ret = -1;
	}
	if (p[11] != client->cconfig->wpa_ie.mcast)
	{
	    log(LG_INF, "The WPA IE multicast cipher didn't match %2.2x received %2.2x", client->cconfig->wpa_ie.mcast, p[11]);
	    ret = -1;
	}
	else
	{
	    client->client_wpa_ie.mcast = client->cconfig->wpa_ie.mcast;
    	}
	if (len != 10)
	{
	    /* The client decided not to default unicast and authentication suites selection */
	    count = p[12] + (p[13] << 8);
	    if (count != 1)
	    {
		log(LG_INF, "Selecting more than one unicast cipher not allowed count = %d", count);
		ret = -1;
	    }
	    else
	    {
		if (memcmp(OUI, p+14, 3) != 0)
		{
		    log(LG_INF, "The WPA IE unicast cipher didn't match %2.2x %2.2x %2.2x received %2.2x %2.2x %2.2x", OUI[0], OUI[1], OUI[2], p[14], p[15], p[16]);
		    ret = -1;
		}
		for (i = 0; i < client->cconfig->wpa_ie.ucast_count; i++)
		{
		    if (client->cconfig->wpa_ie.ucast[i] == p[17])
		    {
			client->client_wpa_ie.ucast = client->cconfig->wpa_ie.ucast[i];
			break;
		    }
		}
		if (client->client_wpa_ie.ucast == 0)
		{
		    log(LG_INF, "The WPA IE unicast cipher didn't match one of the advertised received %2.2x", p[17]);
		    ret = -1;
		}
	    }

	    if (len != 16)
	    {
		/* There is an authentication suite in there */
		count = p[18] + (p[19] << 8);
		if (count != 1)
		{
		    log(LG_INF, "Selecting more than one authentication suite not allowedcount = %d", count);
		    ret = -1;
		}
		else
		{
		    if (memcmp(OUI, p+20, 3) != 0)
		    {
		        log(LG_INF, "The WPA IE authentication suite didn't match %2.2x %2.2x %2.2x received %2.2x %2.2x %2.2x", OUI[0], OUI[1], OUI[2], p[20], p[21], p[22]);
		        ret = -1;
		    }
		    for (i = 0; i < client->cconfig->wpa_ie.auth_count; i++)
		    {
			if (client->cconfig->wpa_ie.auth[i] == p[23])
			{
			    client->client_wpa_ie.auth = client->cconfig->wpa_ie.auth[i];
			    break;
			}
		    }
		    if (client->client_wpa_ie.auth == 0)
		    {
			log(LG_INF, "The WPA IE authentication suite didn't match one of the advertised received %2.2x", p[23]);
			ret = -1;
		    }
		}
		if (len != 22)
		{
		    /* Client added capability information */
		    if (p[24] & WAP_CAP_UCAST_USING_GROUP)
		    {
			client->client_wpa_ie.groupkey_only = 1;
			log(LG_INF, "Client doesn't supports pairwise keys");
		    }
		    client->client_wpa_ie.nof_seq_counters = ((p[24] & 0x0c) >> 2);
		    switch (client->client_wpa_ie.nof_seq_counters)
		    {
			case 0:
			    log(LG_INF, "Client max nof of replay indices per key 1");
			break;
			case 1:
			    log(LG_INF, "Client max nof replay indices per key 2");
			break;
			case 2:
			    log(LG_INF, "Client max nof replay indices per key 4");
			break;
			case 4:
			    log(LG_INF, "Client max nof replay indices per key 16");
			break;

		    }
		}
	    }
	    else //(len != 16)
	    {
    	        /* Client defaults IE from auth suite and on.*/
		if (client->cconfig->wpa_ie.auth_count > 1)
		{
		    log(LG_INF, "Cannot default WPA IE when unicast cipher or authentication suites isn't 1");
		    ret = -1;
		}
		else if (client->cconfig->wpa_ie.auth[0] == AUTH_UNSPECIFIED_OVER_802_1X)
		{
		    /* So default all not set*/
		    client->client_wpa_ie.auth = AUTH_UNSPECIFIED_OVER_802_1X;
		}
		else
		{
		    log(LG_INF, "Cannot default WPA IE when advertised settings are not all default");
		    ret = -1;
		}
	    }
    	}
    	else // (len != 10)
    	{
	    /* Client defaults IE from ucast cipher and on.*/
	    if (client->cconfig->wpa_ie.ucast_count > 1 || client->cconfig->wpa_ie.auth_count > 1)
	    {
		log(LG_INF, "Cannot default WPA IE when unicast cipher or authentication suites isn't 1");
		ret = -1;
	    }
	    else if (client->cconfig->wpa_ie.ucast[0] == UCAST_CIPHER_TKIP &&
	    	     client->cconfig->wpa_ie.auth[0] == AUTH_UNSPECIFIED_OVER_802_1X)
	    {
		/* So default all */
		client->client_wpa_ie.ucast = UCAST_CIPHER_TKIP;
		client->client_wpa_ie.auth = AUTH_UNSPECIFIED_OVER_802_1X;
	    }
	    else
	    {
		log(LG_INF, "Cannot default WPA IE when advertised settings are not all default");
		ret = -1;
	    }
	}
    } // (len != 6)
    else
    {
	/* Client defaults IE from mcast cipher and on.*/
	if (client->cconfig->wpa_ie.ucast_count > 1 || client->cconfig->wpa_ie.auth_count > 1)
	{
	    log(LG_INF, "Cannot default WPA IE when unicast cipher or authentication suites isn't 1");
	    ret = -1;
	}
	else if (client->cconfig->wpa_ie.mcast == MCAST_CIPHER_TKIP &&
		 client->cconfig->wpa_ie.ucast[0] == UCAST_CIPHER_TKIP &&
		 client->cconfig->wpa_ie.auth[0] == AUTH_UNSPECIFIED_OVER_802_1X)
	{
	    /* So default all */
	    client->client_wpa_ie.mcast = MCAST_CIPHER_TKIP;
	    client->client_wpa_ie.ucast = UCAST_CIPHER_TKIP;
	    client->client_wpa_ie.auth = AUTH_UNSPECIFIED_OVER_802_1X;
	}
	else
	{
	    log(LG_INF, "Cannot default WPA IE when advertised settings are not all default");
	    ret = -1;
	}
    }
    return ret;
}
