/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * Function for calculation of keys and key derivation
 */

#include <string.h>
#include <stdio.h>
#include <stdint.h>

#include "eapol.h"
#include "paed.h"
#include "misc.h"
#include "client.h"
#include "log.h"
#include "md5.h"
#include "hmac_prf.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "keystuff.h"
#include "config.h"
#include "state_gtk.h"

struct mppe_t {
        char vendor_string[4];
        char vendor_type;
        char vendor_length;
        char salt[2];
        char key[1];
} __attribute__ ((__packed__));


char bcast_key[13] = "1234567890123";
int bcast_key_len = 13;


/*
 * Generate a random multicast key
 */

void init_bcast_key(struct config_t *config)
{

        if (!config->enableWEP)
        {
            // Just take the first Dot1XWEPKeySize bytes since it's used for 1X
            // and set length to correct values.
            memcpy(bcast_key, config->cgtk->GTK[config->cgtk->GN], config->Dot1XWEPKeySize);
            bcast_key_len = config->Dot1XWEPKeySize;
        }
        else
        {
            bcast_key_len = blob_get_bcast_key(BCAST_KEYID_STATIC, bcast_key, bcast_key_len, config);
            log(LG_DBG, "Got bcast key length %d", bcast_key_len);
        }
}


/*
 * This function is called from radius_rx when a vendor specific
 * radius attribute is found. The received keys are decoded using the MS
 * provided algorightm, and stored in the client struct
 */

int decode_mppe_keys(struct client_t *client, char *p, int len)
{
        MD5_CTX MD5c;
        char *keyvalue;
        int numblocks;
        int block;
        int index;
        char digest[16];
        char ciphertext[16];
        struct mppe_t *mppe = (struct mppe_t *)p;
        int keylen;

#ifdef DEBUG_PRINTKEYS
        dump_bindata("Recevied vendor specific", p, len);
#endif

        // Do some checks if this is the expected vendor-specific thing for MPPE keys

        if(memcmp(mppe->vendor_string, MS_VENDOR_STRING, sizeof(mppe->vendor_string)) != 0) {
                log(LG_DBG, "Got unknown vendor-string (not Microsoft). ignoring");
                return(0);
        }

        switch(mppe->vendor_type) {
                case 0x10 :
                        log(LG_DBG, "Received MS MPPE Tx Key");
                        break;
                case 0x11 :
                        log(LG_DBG, "Recieved MS MPPE Rx Key");
                        break;
                default :
                        log(LG_DBG, "Unknown MS MPPE key received (%d), ignoring", mppe->vendor_type);
                        return(0);
        }

        // Decode the received key

        keyvalue = mppe->key;

        numblocks = (len - 8) / 16;

        for(block=0; block<numblocks; block++) {

                        MD5Init(&MD5c);
                        MD5Update(&MD5c, client->radius_secret, client->radius_secret_len);

                        if(block == 0) {
                                MD5Update(&MD5c, client->radius_vector, 16);
                                MD5Update(&MD5c, mppe->salt, 2);
                        } else {
                                MD5Update(&MD5c, ciphertext, 16);
                        }

                        MD5Final(digest, &MD5c);

                        memcpy(ciphertext, keyvalue, 16);

                        for(index=0; index<16; index++) {
                                *keyvalue ^= digest[index];
                                keyvalue++;
                        }
        }

#ifdef DEBUG_PRINTKEYS
        dump_bindata("Decrypted vendor specific", p, len);
#endif

        keyvalue = (char *)p+9;
        keylen = (int)*(p+8);

        // Okay, due to microsoft confusion, the Rx key is stored in the Tx, and
        // the Tx is stored in the Rx thingy

        switch(mppe->vendor_type) {
                case 0x10 :
                        log(LG_DBG, "Storing decoded MS MPPE Tx Key");
                        memcpy(client->rx_session_key, keyvalue, keylen);
                        client->session_key_len = keylen;
#ifdef DEBUG_PRINTKEYS
                        dump_bindata("TX session key", keyvalue, keylen);
#endif
                        break;
                case 0x11 :
                        log(LG_DBG, "Storing decoded MS MPPE Rx Key");
                        memcpy(client->tx_session_key, keyvalue, keylen);
                        client->session_key_len = keylen;
#ifdef DEBUG_PRINTKEYS
                        dump_bindata("RX session key", keyvalue, keylen);
#endif
                        break;
                default :
                        log(LG_FTL, "Shouldn't get here !!");
                        return(0);
        }


        return(0);
}




/*
 * Key setting and sending the 'old' way.
 */

int keying_do_client_oldstyle(struct client_t *client)
{
        char iv[16];

        random_vector(iv, 16);

        // Tell the client to set the per-station key, and use
        // it as the default key, and set the mcast key.
        // Some clients seem to have problems with mixed key lengths
        // so broadcast 40 bits and unicast 104 bits should be avoided.

        if (blob_get_privacy_invoked(client->cconfig)) {

                if(client->cconfig->enableWEP) {
                        /* Set pair-wise key in index PERSTA_KEYID_STATIC */
                        eapol_tx_key(client,
                                     iv,
                                     client->tx_session_key,
                                     0, // No key is send in the frame
                                     client->cconfig->WEPKeySize,
                                     (PERSTA_KEYID_STATIC | 0x80));

                        /* Set group key in index BCAST_KEYID_STATIC */
#ifdef DEBUG_PRINTKEYS
                        dump_bindata("Sending broadcast key", bcast_key, client->cconfig->WEPKeySize);
#endif
                        eapol_tx_key(client,
                                     iv,
                                     bcast_key,
                                     client->cconfig->WEPKeySize,
                                     client->cconfig->WEPKeySize,
                                     (BCAST_KEYID_STATIC  | 0x00));


                        // Set the per-station key for this client in #1 in the MVC, and use
                        // it as the default key for this station

                        client->session_key_len = bcast_key_len;

                        blob_set_persta_key(client, PERSTA_KEYID_STATIC);

                } else {
                        /* Set pair-wise key in index PERSTA_KEYID */
                        eapol_tx_key(client,
                                     iv,
                                     client->tx_session_key,
                                     0, // No key is send in the frame
                                     client->cconfig->Dot1XWEPKeySize,
                                     (PERSTA_KEYID | 0x80));

#ifdef DEBUG_PRINTKEYS
                        dump_bindata("Sending broadcast key", client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], client->cconfig->Dot1XWEPKeySize);
#endif
                        /* Set group key in index gtk.GN (current group key index */
                        eapol_tx_key(client,
                                     iv,
                                     client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN],
                                     client->cconfig->Dot1XWEPKeySize,
                                     client->cconfig->Dot1XWEPKeySize,
                                     (client->cconfig->cgtk->GN  | 0x00));


                        // Set the per-station key for this client in #1 in the MVC, and use
                        // it as the default key for this station

                        client->session_key_len = client->cconfig->Dot1XWEPKeySize;

                        blob_set_persta_key(client, PERSTA_KEYID);
                }
        }

        return(0);
}


int update_dot1x_bcast_key(struct client_t *client)
{
        char iv[16];

        log(LG_INF, "Updating 802.1x bcast key");

        if(!client->cconfig->enableWEP) {
                memcpy(bcast_key, client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], client->cconfig->Dot1XWEPKeySize);
                random_vector(iv, 16);
                eapol_tx_key(client,
                             iv,
                             bcast_key,
                             client->cconfig->Dot1XWEPKeySize,
                             client->cconfig->Dot1XWEPKeySize,
                             (client->cconfig->cgtk->GN  | 0x00));
        }

        client->cconfig->cgtk->GKeyDoneStations--;

        return(0);
}





void MIC(u_char *data, int data_len, u_char *key, int key_len, u_char *out)
{
	hmac_md5(data, data_len, key, key_len, out);
}


void MIC_AES(u_char * data, int data_len, u_char * key, int key_len, u_char * out)
{
	u_char buf[20];		// SHA-1 output

	hmac_sha1(data, data_len, key, key_len, buf);
	memcpy(out, buf, 16);
}


void calc_PTK(struct client_t *client, char *ANonce, char *SNonce, char *out, int out_len)
{
        u_char label[2 * NONCE_LEN + 12];
        int key_length;

        key_length = PMK_LEN;

        if (memcmp(client->hwaddr, client->cconfig->ifhwaddr, 6) < 0) {
                memcpy(label, client->hwaddr, 6);
                memcpy(label+6, client->cconfig->ifhwaddr, 6);
        }
        else {
                memcpy(label, client->cconfig->ifhwaddr, 6);
                memcpy(label+6, client->hwaddr, 6);
        }

        if (memcmp(ANonce, SNonce, NONCE_LEN) < 0) {
                memcpy(label+12, ANonce, NONCE_LEN);
                memcpy(label+12+NONCE_LEN, SNonce, NONCE_LEN);
        }
        else {
                memcpy(label+12, SNonce, NONCE_LEN);
                memcpy(label+12+NONCE_LEN, ANonce, NONCE_LEN);
        }

        dump_bindata("PMK", client->PMK, key_length);
        dump_bindata("Label", label, 76);

        prf(client->PMK, key_length, "Pairwise key expansion", 22, label, 12 + 2 * NONCE_LEN, out, out_len);
}

void calc_GTK(struct config_t *configp, char *GMK, int GMK_len, char *GNonce, char *out, int out_len)
{
        u_char label[NONCE_LEN + 6];

        memcpy(label, configp->ifhwaddr, 6);
        memcpy(label+6, GNonce, NONCE_LEN);

        prf(GMK, GMK_len, "Group key expansion", 19, label, 6 + NONCE_LEN, out, out_len);
}

// Note out_len for the prf is in bytes

void prf(char *secret, int secret_len, char *label, int label_len, char *seed, int seed_len, char *out, int out_len)
{
        int i, offset, copylen, input_len;
        u_char input[128];
        u_char buf[20];  // SHA-1 output

        offset = 0;

        // Concatenate A || '0' || B || i (which is label || '0' || seed || i)

        memcpy(input, label, label_len);
        input[label_len] = 0;
        memcpy(input + label_len + 1, seed, seed_len);
        input[label_len + 1 + seed_len] = 0;  // for first iteration
        input_len = label_len + 1 + seed_len + 1;

        for (i = 0; i < (out_len + 19) / 20; i++) {

                // Change variable in input

                input[input_len - 1] = i;

                // Do SHA-1 (160-bit output)

                hmac_sha1(input, input_len, secret, secret_len, buf);

                // Concatenate intermediate output to final output until out_len reached

                copylen = (offset + 20 < out_len) ? 20 : out_len - offset;
                memcpy(out + offset, buf, copylen);

                offset += 20;
        }
}

void keystuff_setup_key_material(struct config_t *config)
{
        char pass_phrase_key[40];

        if (config->ssnpsk256keyfiled == 1)
        {
            printf("Using key in config.ssnpsk256key\n");
                return;
        }



#ifdef DEBUG_PRINTKEYS
        log (LG_INF, "Input PasswordHash: %s", config->ssnpskASCIIkey);
        dump_bindata("SSID: ", config->ssid, config->ssidlength);
#endif

        if (PasswordHash(config->ssnpskASCIIkey, config->ssid, config->ssidlength, pass_phrase_key))
        {
            memcpy(config->ssnpsk256key, pass_phrase_key, PMK_LEN);
#ifdef DEBUG_PRINTKEYS
            dump_bindata("Output PasswordHash: ", config->ssnpsk256key, PMK_LEN);
#endif

        }
        else
        {
            log (LG_ERR, "Error creating PSK key from password  %s", config->ssnpskASCIIkey);
        }

}



// F(P, S, c, i) = U1 xor U2 xor ... Uc
// U1 = PRF(P, S || Int(i)
// U2 = PRF(P, U1)
// Uc = PRF(P, Uc-1)
//
#define A_SHA_DIGEST_LEN 20

void HashGenerator(char *password, unsigned char *ssid, int ssidlength, int iterations, int count, unsigned char *output)
{
    unsigned char digest[36], digest1[A_SHA_DIGEST_LEN];
    int i, j;

    // U1 = PRF(P, S || int(i))
    memcpy(digest, ssid, ssidlength);
    digest[ssidlength] = (unsigned char)((count>>24) & 0xff);
    digest[ssidlength+1] = (unsigned char)((count>>16) & 0xff);
    digest[ssidlength+2] = (unsigned char)((count>>8) & 0xff);
    digest[ssidlength+3] = (unsigned char)(count & 0xff);
    hmac_sha1(digest, ssidlength+4,
              (unsigned char *)password, (int)strlen(password),
              digest1);

    // output = U1
    memcpy(output, digest1, A_SHA_DIGEST_LEN);

    for(i = 1; i < iterations; i++) {
            // Un = PRF(P, Un-1)
            hmac_sha1(digest1, A_SHA_DIGEST_LEN,
                      (unsigned char *)password, (int)strlen(password),
                      digest);
            memcpy(digest1, digest, A_SHA_DIGEST_LEN);

            // output = output xor Un
            for(j = 0; j < A_SHA_DIGEST_LEN; j++) {
                    output[j] ^= digest[j];
            }
    }
}

// password - ascii string up to 63 characters in length
// ssid - octet string up to 32 octets
// ssidlength - length of ssid
// output must be 40 octets in length and outputs 256 bits of key
int PasswordHash (char *password, unsigned char *ssid, int ssidlength, unsigned char *output)
{
        if((strlen(password) < 8) ||
           (strlen(password) > 63) ||
           (ssidlength > 32)) return 0;

        HashGenerator(password, ssid, ssidlength, 4096, 1, output);
        HashGenerator(password, ssid, ssidlength, 4096, 2, &output[A_SHA_DIGEST_LEN]);
        return 1;
}


// eof...

