/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * Misc small helper functions
 */


#include <stdio.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <net/ethernet.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <errno.h>


#include "log.h"
#include "client.h"
#include "misc.h"


extern int eapolfd;		// comes from eapol.c


int dump_bindata_enabled = 0;


/*
 * Print a hex-dump of the given block data
 */

int dump_bindata(char *txt, char *data, int len)
{
#if (!RAC_UT)
	int i,j;

	if((dump_bindata_enabled) || (txt[0] == '!')){

		fprintf(stderr, "\n    ---- <%s> ----\n", txt);
		for(i=0; i<len; i+=16) {
			fprintf(stderr, "    %04X : ", i);
			for(j=i; (j<i+16) && (j<len);  j++) {
				fprintf(stderr, "%02X ", (unsigned char)*(data+j));
			}
			fprintf(stderr, "  ");
			for(j=i; (j<i+16) && (j<len);  j++) {
				if((*(data+j) >= 32) && (*(data+j)<=127)) {
					fprintf(stderr, "%c", *(data+j));
				} else {
					fprintf(stderr, ".");
				}
			}
			fprintf(stderr, "\n");
		}
		fprintf(stderr, "    ----- <end %s> ----\n\n", txt);
	}
#endif /*(!RAC_UT)*/
	return(0);
}


/*
 * Returns a pointer to the human-readable hardware address of
 * the client. Buffer is static from function and overwritten
 * with each call
 */

char *hwaddr2string(unsigned char *a)
{
	static char buf[18];

	snprintf(buf, 18, "%02x:%02x:%02x:%02x:%02x:%02x",
		a[0], a[1], a[2], a[3], a[4], a[5]);

	return(buf);
}

/*
 * Split a strign into 0-terminated parts and return them
 */

int split (char *str, char sep, char **part, int maxparts)
{
	int parts = 0;
	char *p1 = str;
	char *p2;

	part[parts++] = p1;

	while ((p2 = strchr (p1, sep)) && (parts < maxparts)) {
		*p2 = 0;
		p1 = p2 + 1;
		part[parts++] = p1;
	}

	return parts;
}



/*
 * Retrieves the IP address of the given interface, or 0
 * if error.
 */

in_addr_t if_get_ipaddr(char *ifname)
{
	struct ifreq ifr;
	struct sockaddr_in *p;

	if(eapolfd == -1) {
		log(LG_WRN, "Can't get IP address for interface, eapolfd = -1");
		return(0);
	}

	// Fetch the IP address of the interface

	strncpy(ifr.ifr_name, IF_NAS, sizeof(ifr.ifr_name));

	if ( ioctl(eapolfd, SIOCGIFADDR, &ifr) < 0) {
		log(LG_WRN, "Can't get IP address for interface %s : %s", ifname, strerror(errno));
		return(0);
	} else {
	        p = (struct sockaddr_in *)&(ifr.ifr_addr);
		log(LG_DBG, "IP address for interface %s : %s", ifname, inet_ntoa(p->sin_addr.s_addr));
       		return(p->sin_addr.s_addr);
       	}

	return(0);
}





/*
 * A nice hex to string function from Marco's hand...
 *
 * Thanks Ico... I have made some modification to make it even nicer.
 *
 * returns the length of the converted string (target)
 *
 */

int hex2string (char *target, char *input, size_t maxlen)
{
	char *t;
	char *i;
	int l = 0;

	i = input;
	t = target;

	while ((*i) && (l < maxlen)) {

		switch (*i) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			*t = (char) (16 * (*i - '0'));
			break;
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
			*t = (char) (16 * ((*i - 'A') + 10));
			break;
		}

		/* Second nibble */
		i++;

		switch (*i) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			*t = (char) *t + (*i - '0');
			break;
		case 'A':
		case 'B':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
			*t = (char) *t + ((*i - 'A') + 10);
			break;
		}

		i++;
		t++;

		l++;

	}

	return (l);
}


/*
 * Generate a random vector of data...
 */

int random_vector (unsigned char *vector, int len)
{
	int fd;
	int r;

	fd = open(PATH_URANDOM, O_RDONLY);

	if (fd == -1) {
		return(-1);
	}

	r = read(fd, vector, len);

	if (r < len) {
		log(LG_FTL, "Can't open %s", PATH_URANDOM);
		return(-1);
	}

	close(fd);

	return(0);
}



// eof
