
/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
/* MSSID functionality related functions */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>


#include "paed.h"
#include "misc.h"
#include "eapol.h"
#include "eap.h"
#include "log.h"
#include "client.h"
#include "config.h"
#include "log.h"
#include "netlink_func.h"

#include <fcntl.h>
#include <sys/fcntl.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <linux/if_bridge.h>
#include <linux/if_vlan.h>
#include <linux/sockios.h>



#if (PAED_MSSID == 1)

/* list of physical wireless interfaces */
char mssid_wireless_iface_list[MAX_INTERFACES][128];

/* number of wireless interfaces */
int mssid_wireless_iface_count; 

/* externs */
extern int vss_count;
extern int phy_iface_devid[MAX_INTERFACES];
extern struct config_t *config;


/* externs from ie.c */
long oabuf[16];
extern struct obj_attachment *oa;
extern unsigned char *ie;
extern unsigned short ie_length;



/*
 * Return the count of virtual service sets configured
*/
int mssid_get_vss_count()
{
    return vss_count;
}


/*
 * Make a service set object from given config structure
 *
*/
int mssid_make_service_set(struct config_t *configp, obj_serviceset *service_setp)
{
     unsigned int policies;
     unsigned int mixed_sec_policy;
     
     /* clear the service set */
     memset(service_setp, 0, sizeof(obj_serviceset));

// DEBUG code
// #if 0

     log(LG_INF, "MSSID: Creating service set def for vss %s", configp->vss_ifname);
     
     /* set the SSID */
     strncpy(service_setp->ssid.octets, configp->ssid, configp->ssidlength);
     service_setp->ssid.length = configp->ssidlength;

     log(LG_INF, "MSSID: SSID = %s", service_setp->ssid.octets);

     /* the member limits */
     service_setp->exclusive = configp->exclusivelimit;
     service_setp->total = configp->totallimit;


     /* Security is enabled if enableOS is 0 */
     if (0 == configp->enableOS) {


        log(LG_INF, "MSSID: Privacy Invoked = TRUE");

        // DEBUG code
        log(LG_INF, "MSSID: Security is enabled");
        
        /* set the policies */
        service_setp->policies |= DOT11_MLMEPOLICY_PRIVACYREQUIRED;
        service_setp->policies |= DOT11_MLMEPOLICY_PRIVACYCAPABILITY;

        /* set privacy invoked */  
        service_setp->privinvoked = DOT11_PRIV_INV_ENABLE;

        /* set exclude unencrypted */
        service_setp->exunencrypted = 1;
     }

     /* mixed security */
     if (1 == configp->mixedsecurity) {
	
	/* need to reset privacy required */
	mixed_sec_policy = DOT11_MLMEPOLICY_PRIVACYREQUIRED; 
        mixed_sec_policy = ~mixed_sec_policy;

        log(LG_INF, "MSSID: Mixed Security is enabled. Current policy = %x. Mixed Security policy is %x",
	       service_setp->policies, mixed_sec_policy);
        service_setp->policies &= mixed_sec_policy;

        service_setp->policies |= DOT11_MLMEPOLICY_UNENCRYPTEDBCAST;

	log(LG_INF, "MSSID: New policy = %x", service_setp->policies);
     }

     /* AuthEnable is ALWAYS OPEN SYSTEM */
     service_setp->authenable = DOT11_AUTH_OS;

     /* If WPA is enabled, configure WPA settings. WPA shall be enabled
      * even if the service set is using only WPA-PSK 
      */

      if (1 == configp->enableSSN){ 
          /* The unicast cipher.  Can be either of type TKIP or AES */
          switch(configp->unicastcipher){
          case MSSID_CIPHER_TKIP:
               service_setp->wpa.unicast |= DOT11_PRIVTYPE_TKIP;

               log(LG_INF, "MSSID: Unicast Cipher = TKIP");
            
               break;

          case MSSID_CIPHER_AES:
               service_setp->wpa.unicast |= DOT11_PRIVTYPE_AES_CCMP;

               log(LG_INF, "MSSID: Unicast Cipher = AES");

               break;
    
          default:
               log(LG_WRN,"MSSID: Invalid Unicast cipher type '%d' in 
                 VSS '%d' definition", configp->unicastcipher, configp->vssid);
               break;
          }

          /* If we have WEP on, also add WEP to the unicast cipher */
          if  (1 == configp->enableWEP){


              log(LG_INF, "MSSID: WEP = TRUE. Unicast cipher supports WEP");

              if (CONF_WEP_40_SIZE == configp->Dot1XWEPKeySize){
                  service_setp->wpa.unicast |= DOT11_PRIVTYPE_WEP;
              }
              else{
                 service_setp->wpa.unicast |= DOT11_PRIVTYPE_WEP_104;
              }
          }

          /* the multicast cipher */
          switch(configp->multicastcipher){

          case MSSID_CIPHER_WEP40:
               log(LG_INF, "MSSID: Multicast Cipher = WEP40");

               service_setp->wpa.multicast = DOT11_PRIVTYPE_WEP;
               break;

          case MSSID_CIPHER_WEP104:
               log(LG_INF, "MSSID: Multicast Cipher = WEP104");

               service_setp->wpa.multicast = DOT11_PRIVTYPE_WEP_104;
               break;

          case MSSID_CIPHER_TKIP:
               log(LG_INF, "MSSID: Multicast Cipher = TKIP");

               service_setp->wpa.multicast = DOT11_PRIVTYPE_TKIP;
               break;

          case MSSID_CIPHER_AES:
               log(LG_INF, "MSSID: Multicast Cipher = AES");

               service_setp->wpa.multicast = DOT11_PRIVTYPE_AES_CCMP;
               break;

          default:
               log(LG_WRN,"MSSID: Invalid Multicast cipher type '%d' in VSS '%d' definition",
                   	    configp->multicastcipher, configp->vssid);
             	  break;
          }

          /* Authentication type. Can be multiple types */
          log(LG_INF, "MSSID: Auth Type = WPA");

          service_setp->wpa.authtype |= DOT11_AUTH_WPA;
          
          if (1 == configp->enableSSNPSK){ 
              log(LG_INF, "MSSID: Auth Type = WPAPSK Also supported");
              service_setp->wpa.authtype |= DOT11_AUTH_WPAPSK;
          }
      } /* if enableSSN */

//#endif


// DEBUG code
/* just make the service set */

#if 0 // compiled out as of now

     /* NOTE: the config structure needs to have same values!! */

     sprintf(service_setp->ssid.octets, "ssid_vss%d", configp->vssid);
     service_setp->ssid.length = strlen(service_setp->ssid.octets);

     service_setp->exclusive = 100;
     service_setp->total = 100;

     /* set the policies */
     service_setp->policies |= DOT11_MLMEPOLICY_PRIVACYREQUIRED;
     service_setp->policies |= DOT11_MLMEPOLICY_PRIVACYCAPABILITY;

     /* No privacy. the policies hold no meaning now  */
     service_setp->privinvoked = DOT11_PRIV_INV_NONE;

     /* set exclude unencrypted to 0. NOTE: Verify this setting*/
     service_setp->exunencrypted = 0;

     /* AuthEnable is ALWAYS OPEN SYSTEM */
     service_setp->authenable = DOT11_AUTH_OS;

     service_setp->wpa.unicast = DOT11_PRIVTYPE_TKIP;
     service_setp->wpa.multicast = DOT11_PRIVTYPE_TKIP;

     service_setp->wpa.authtype = DOT11_AUTH_WPA;

#endif

     return 0;
}


/* Return the index value corresponding to the given device Id. 
  * The index value is the wireless ethernet interface index in config_index array
  */
int mssid_devid_to_phy_iface_index(int devid)
{
	int i;

	for (i=0; i < MAX_INTERFACES; i++){
		if (phy_iface_devid[i] == devid){
			return i;
		}
	}
	
	log(LG_WRN, "MSSID: unable to find given device id %d in the physical interface-devid mapping table",
		devid);
	return -1;
}


/*
 * Copy the WPA IE from mlme object to all VSS config structures 
 * This copy is used in the EAPOL key handshake
 */ 
int mssid_get_beacon_ie(struct obj_mlme *mlme, struct config_t *configp)
{
    int i;
    int k;
    int len;
    int tag;
    unsigned char *p;
    int nof_vss_interfaces;

    /* The beacon frame data portion -
     *
     * |<-- TimeStamp 8 bytes -->| <-- Beacon Interval 2 bytes -->|<-- Cap Info 2 bytes -->| IEs... 
     *
     * We need to check for the WPA IE among the available IEs. Therefore the offset is 12.
     */
    p = (unsigned char *) mlme + sizeof(struct obj_mlme) + 12;

    for ( i = 0 ; i < mlme->size-4; i += p[i+1]+2) {
	   tag = p[i];

           /* add 2 for the IE header - tag + length fields */
	   len = p[i+1] + 2;

	   /* WPA IE found ? */
	   if (   tag    == WPA_IE_TAG 
               && p[i+2] == 0x00 
               && p[i+3] == 0x50
               && p[i+4] == 0xF2 ) {

               log(LG_INF,"MSSID: Got WPA IE in the beacon. Capturing the IE in config entries");
           
               /* set the WPA IE length in the global variable - ie_len*/
               ie_length = len;

               /* copy the received WPA IE in the global - oabuf. 
                * Also need to set ptr ie to oabuf 
                */
               ie = (unsigned char *) oa + sizeof(struct obj_attachment);
               memcpy(ie, &p[i], len);
  
               /* copy the IE in all config structures */
               nof_vss_interfaces = mssid_get_vss_count();

               for (k = 0; k < nof_vss_interfaces; k++){
                   // DEBUG code
    	           // memcpy(&(config[k].wpa_ie), (p+i), len+2);   
                         
                  /* Copied wpa ie to all exisiting VSSs. 
                   * Parse the IE to set config values in the config struct
                   */
                  log(LG_INF, "MSSID: --- WPA IE Settings in Beacon ---");
                  mssid_parse_beacon_ie(&p[i], &config[k]);
                  log(LG_INF, "MSSID: ---------------------------------");
               }

               break;
           }
     }
    return 0;    
}

/*
 * Searches for the given vlan tag in the created VSS config
 * structures and returns the vssid of the VSS containing the vlan tag
 * in *vssidp.
 * Returns 0 in case of success and -1 in case of failure.
 */
int mssid_vlan_to_vssid(int vlan_tag_num , int *vssidp)
{
    int i;
    int nof_vss_interfaces;

    nof_vss_interfaces = mssid_get_vss_count();

    for (i=0; i< nof_vss_interfaces; i++){
        if (config[i].vlan_tag == vlan_tag_num){
            *vssidp = config[i].vssid;
            return 0;
        }
    }

    /* error case */
    log(LG_WRN,"MSSID: No vssid found for vlan tag %d", vlan_tag_num);
    return -1;
}


int mssid_name2idx(char *ifname)
{
	struct ifreq ifr;
	int fd;

	memset(&ifr,'\0',sizeof(struct ifreq));
	
	fd = socket (AF_INET, SOCK_DGRAM, 0);
	
	if (fd < 0)
		return 0;
	
	strncpy(ifr.ifr_name,ifname,IFNAMSIZ); 
	
   // DEBUG
    log(LG_DBG,"MSSID: before ioctl for name2idx for ifname %s", ifname);
	ioctl (fd, SIOCGIFINDEX, &ifr);
   // DEBUG
    log(LG_DBG,"MSSID: after ioctl for name2idx for ifname %s", ifname);
	close (fd);
	
	return ifr.ifr_ifindex;
	
}


/* configure VSSs in the bridge */
int mssid_configure_bridge(struct config_t *config)
{
   int vlan_fd;
   int i;
   int vss_ifindex;
   struct ifreq ifr;
   unsigned long args[4];
   int nof_vss_interfaces=0;

   // DEBUG
   //log(LG_DBG,"MSSID: configure_bridge starg. The config array ptr is %p", config);

   vlan_fd = socket(AF_INET, SOCK_STREAM, 0);
   if (vlan_fd < 0){ 
      log(LG_FTL, "MSSID: socket call for vlan_fd failed with returned value %d", vlan_fd);
   }

   // DEBUG
   // log(LG_DBG,"MSSID: socket created");

   nof_vss_interfaces = mssid_get_vss_count();

// DEBUG
 //  log(LG_DBG,"MSSID: before memset. no of interfaces is %d", nof_vss_interfaces);

   /* start with clean configuration */
   memset(&ifr, 0, sizeof(struct ifreq));


  // DEBUG
 //  log(LG_DBG,"MSSID: after memset");
 //  log(LG_DBG,"MSSID: config[0] ifname is %s", config[0].ifname);

   /* For all created VSSs, find the kernel IfIndex and configure in bridge */
   for (i = 0; i < nof_vss_interfaces ; i++) {

     

//       log(LG_INF, "MSSID: Configuring bridge for VSS %s", config[i].vss_ifname);
       vss_ifindex = mssid_name2idx(config[i].vss_ifname);

       log(LG_INF, "MSSID: kernel ifindex for VSS %s is %d", 
                    config[i].vss_ifname, vss_ifindex);

       args[0] = BRCTL_ADD_IF;
       args[1] = vss_ifindex;
       args[2] = 0;
       args[3] = 0;

       sprintf(ifr.ifr_name, "br%d", config[i].vlan_tag);

       log(LG_INF, "MSSID: Configuring VSS %s with bridge %s", 
             config[i].vss_ifname, ifr.ifr_name);

       ifr.ifr_data = (void *)args;
			
       if (ioctl(vlan_fd, SIOCDEVPRIVATE, &ifr) < 0){
          log(LG_WRN, "MSSID: bridge ioctl call for bridge %s and vss %s failed!! ", 
              ifr.ifr_name, config[i].vss_ifname);
       }
   } /* for */ 


// DEBUG
//        log(LG_DBG,"MSSID: Bridge configuration over");
}


int mssid_configure_service_set_keys(struct config_t *configp)
{
    int i;
   
    for (i=0; i< 4; i++){
        log(LG_DBG, "MSSID: WEP Key %d = %s", i, configp->keystable[i]);
    }

    /* set the broadcast key. Only Key id 0 can be used as Broadcast key! */
    if (strlen(configp->keystable[0]) == 0){
        log(LG_FTL, "MSSID: KeyId 0 not available for vss %s", configp->vss_ifname);
    }

    log(LG_INF, "MSSID: Configuring broadcast WEP key for vss %s", configp->vss_ifname); 
    config_sethexkey(configp->keystable[0], configp, configp->ifname, 0);
}

/* Parse the beacon WPA IE and set corresponding fields in the 
 * config structure.
 */
int mssid_parse_beacon_ie(unsigned char *ap_wpa_ie, struct config_t *configp)
{
    unsigned char *p;
    int i;
    unsigned char suite_value;
    int ie_len;
    p = ap_wpa_ie;

    ie_len = p[1];

    log(LG_INF, "MSSID: IE tag = %2x, IE length = %d", p[0], ie_len);

    for(i =0; i<ie_len; i++){
       printf("%2x\t", p[i]);
    }

    printf("\n\n");

    /* skip the IE tag(1), Length(1), Version(2) and the OUI(4) fields. */
    p += 8;

    /* the first 3 bytes are the OUI value. Get the 4th byte */
    suite_value = (unsigned char)p[3];
    configp->wpa_ie.mcast = suite_value;

    log(LG_INF, "MSSID: Multicast cipher suite value: %d", 
          configp->wpa_ie.mcast);

    /* the unicast cipher suite count. this is a 2 byte value */
    p += 4;
    configp->wpa_ie.ucast_count = (unsigned short int) (*p);
   
    log(LG_INF, "MSSID: Number of Unicast cipher suites: %d", 
         configp->wpa_ie.ucast_count);

    /* read the unicast ciphers */
    p += 2; 
    for (i=0; i< configp->wpa_ie.ucast_count; i++){
        /* the first 3 bytes are the OUI value. Get the 4th byte */
        suite_value = (unsigned char)p[3];
        configp->wpa_ie.ucast[i] = suite_value;
        p += 4;
    }

    /* the authenticated key management suite count. this is a 2 byte value */
    configp->wpa_ie.auth_count = (unsigned short int) (*p);

    log(LG_INF, "MSSID: Number of auth suites: %d",
         configp->wpa_ie.auth_count);

    /* read the auth suites */
    p += 2;
    for (i=0; i< configp->wpa_ie.auth_count; i++){
        /* the first 3 bytes are the OUI value. Get the 4th byte */
        suite_value = (unsigned char)p[3];
        configp->wpa_ie.auth[i] = suite_value;
        p += 4;
    }
}

/* locate config struct for given vlan_tag and devid  */
struct config_t *mssid_vlan_to_vss_config(int vlan_tag, int devid)
{
	int vsscount;
	int i;

	vsscount = mssid_get_vss_count();

	for (i=0; i < vsscount; i++){
		if ((config[i].devid == devid) && (config[i].vlan_tag == vlan_tag)){
			/* found vss config */
			log(LG_INF, "MSSID: Found config struct for devid = %x, VlanTag = %d. VSSID is = %d", 
					devid, vlan_tag, config[i].vssid);
			return &config[i];
		}
	} /* for */
	
	/* no vss config found */
	return NULL;
}

/* Migrate given client to the new VSS */
int mssid_change_sta_vss(struct client_t *clientp, int vlan_tag)
{
	struct config_t *configp;
	int vssid;
	int devid;

	/* the physical interface on which the client is associated */
	devid = clientp->cconfig->devid;

	/* Get config struct corresponding to the new vlan tag and physical interface id */
	configp = mssid_vlan_to_vss_config(vlan_tag, devid);

	if (NULL == configp)
	{
		/* 
		 * Unable to find the VSS config struct corresponding to 
		 * the given vlan tag and physical interface. Return error. 
		 */
		 log(LG_WRN, "MSSID: Unable to find VSS for VLAN tag %d for client %s", 
				vlan_tag, hwaddr2string(clientp->hwaddr));

		 return -1;
	}
	
	/* the new vssid */
	vssid = configp->vssid;

	/* is this the same vssid as client's current vssid? */
	if (clientp->cconfig->vssid == vssid)
	{	
		/* nothing to do! */
		 log(LG_INF, "MSSID: The RADIUS assigned VSS %d for client is the same as the original VSS", 
				vlan_tag);
		return 0;
	}
	
	/* This is a new vssid. Each config struct, which corresponds to a VSS, 
	 * has a list of associated clients. Since we are moving this cient to a new VSS, 
	 * need to delete it from the client list in old VSS and add it to the new one.
	 */
	client_delete_from_config(clientp);
	client_add_to_config(configp, clientp);

	/* update config ptr in client struct */
	clientp->cconfig = configp;

	/* GTK stuff! */

	
	/* set new vss for the client in softmac */
	log(LG_INF, "MSSID: Configuring client %s with vss = %s", 
			hwaddr2string(clientp->hwaddr), configp->vss_ifname);

	blob_mssid_set_sta_vss(clientp->cconfig, clientp->hwaddr, vssid);

	return 0;
}

/* Enumerates the list of wireless interfaces and  
 * adds interface names to a global list  
 */
int mssid_make_wireless_iface_list()

{
	int i;
	FILE *phy_iface_f=NULL;
	char buf[256];
	char *part[2];
	char *iface;

	int index=0;

	log(LG_INF, "MSSID: Cleaning mssid_wireless_iface_list");

	/* The interface list is empty */
	memset(mssid_wireless_iface_list, 0, sizeof(mssid_wireless_iface_list));

	log(LG_INF, "MSSID: Opening /proc/net/dev");

    /* Read interface list. We get the physical interfaces */
    phy_iface_f = fopen ("/proc/net/dev", "r");

	if (NULL == phy_iface_f) {
		log (LG_FTL, "MSSID: Can't read /proc/net/dev");
		return 0;
	}

	/* The first two lines contain only header */
	fgets (buf, sizeof (buf), phy_iface_f);
	fgets (buf, sizeof (buf), phy_iface_f);

	log(LG_INF, "MSSID: Reading /proc/net/dev");


	while (fgets (buf, sizeof (buf), phy_iface_f)){
		if (split (buf, ':', part, 2) == 2) {

			iface = part[0];

			while(isspace(*iface) || *iface == 0) iface++;

			/* if this is a non-wireless interface, skip it */
			log(LG_INF, "MSSID: Checking whether %s is a wireless interface by calling BLOB function", iface);
			log(LG_INF, "MSSID: Checking whether %s is a wireless interface by calling BLOB function", iface);
			if (blob_mssid_wireless_interface(iface) < 0){
				log(LG_INF, "MSSID: Skip non-wireless interface %s", iface);
				continue;
			}

			log(LG_INF, "MSSID: Add %s to the list of wireless interfaces", iface);
			strcpy(&mssid_wireless_iface_list[index], iface);
			index++;
		} /* if */
	} /* while */

	mssid_wireless_iface_count = index;
	log(LG_INF, "MSSID: Total %d wireless interfaces found", mssid_wireless_iface_count);

	return mssid_wireless_iface_count;
}


/* Enumerates list of wireless interfaces and sets SoftMAC in Reset/ Commit mode 
 * for each interface in the reset state  
 */
int mssid_modify_softmac_state(int oid)
{
	int i;
	FILE *phy_iface_f=NULL;
	char buf[256];
	char *part[2];
	char *iface;


	/* Read interface list. We get the physical interfaces */

	for (i=0 ; i < mssid_wireless_iface_count; i++) {
	
		/* Set the given OID on the interface */
		switch(oid){
			case GEN_OID_RESET:
				log(LG_INF, "MSSID: Setting wireless interface %s in RESET state", &mssid_wireless_iface_list[i]);
				blob_mssid_reset_softmac(&mssid_wireless_iface_list[i]);
				break;

			case GEN_OID_COMMIT:
				log(LG_INF, "MSSID: Setting wireless interface %s in COMMIT state", &mssid_wireless_iface_list[i]);
				blob_mssid_commit_settings(&mssid_wireless_iface_list[i]);
				break;

			default:
				log(LG_WRN, "MSSID: Invalid OID %x", oid);
				break;
		}
	} /* for */

	return 0;
}

int mssid_init_all_service_sets()
{
	int i, k, r;
	struct obj_serviceset serviceset;

	memset(&serviceset, 0, sizeof(obj_serviceset));

	for (k=0; k < mssid_wireless_iface_count; k++) {

		for (i = 0 ; i < MSSID_MAX_VSS_COUNT; i++ ){
			r = blob_mssid_reset_service_set(&mssid_wireless_iface_list[k], i, &serviceset);
			if( r < 0 ) {
			   log(LG_WRN,"MSSID: Failed to clean configuration for VSS %d on %s", i, &mssid_wireless_iface_list[k]);
			}

		}
	}
	return 0;
}

/* Set the SoftMAC for all wireless interfaces in RESET state */
int mssid_reset_wireless_interfaces()
{
	return mssid_modify_softmac_state(GEN_OID_RESET);
}

/* Set the SoftMAC for all wireless interfaces in COMMIT state */
int mssid_commit_wireless_interfaces()
{
	return mssid_modify_softmac_state(GEN_OID_COMMIT);
}

/* Finds the devid of the given interface name and adds 
 * it to the interface index to devid mapping table.
 */
int mssid_add_devid(char *iface)
{
	static int interface_count=0;
	int iface_devid;
	int index;

	log(LG_WRN,"MSSID: Add entry for interface %s in interface index to devid mapping table", iface);

	/* Get kernel ifindex for this interface */
	iface_devid = mssid_name2idx(iface);

	/* Capture device id of the wireless interfaces in an
	 * interface index to devid mapping table. Used in handle_traps()
	 * to get the index of the physical interface entry in config_index
	 */
	index = interface_count;
	phy_iface_devid[interface_count] = iface_devid;

	interface_count++;
	return index;
}

#endif /* #if (MSSID_PAED == 1) */


