#ifndef _MSSID_H_
#define _MSSID_H_

#include "config.h"


#define MSSID_IFNAME_ETH1					"eth1"
#define MSSID_IFNAME_ETH3					"eth3"

/* Range of legal VSSIDs */
#define 	MSSID_VSSID_MIN				0
#define 	MSSID_VSSID_MAX					15



/* Physical interface number for the two wireless interfaces array */
#define 	MSSID_PHY_IFACE_ETH1            		'1'
#define 	MSSID_PHY_IFACE_ETH3            		'3'


/* Maximum number of VSSs per wireless interface */
#define MSSID_MAX_VSS_COUNT				16


/* */
#define MSSID_IF_CONF_FIRST				"vsseth1.1"


/* Prefix for VSS interfaces */
#define MSSID_VSS_IFACE					"vss.eth"

/* Unicast and Multicast ciphers */
#define MSSID_CIPHER_WEP40				1
#define MSSID_CIPHER_WEP104				2
#define MSSID_CIPHER_TKIP					3
#define MSSID_CIPHER_AES					4


/* The physical interface index values for the wireles interfaces in the
 * config_index array. This array indexed on physical index and on vssid.
 * The macros below define the index values for eth1 and eth3.
 */
#define MSSID_PHY_IFACE_INDEX_ETH1                             0
#define MSSID_PHY_IFACE_INDEX_ETH3                             1




/* Logging macros for function entry and exit  */
#define MSSID_LOG_ENABLED				1

#if (MSSID_LOG_ENABLED == 1)

#define MSSID_LOG_FN_ENTRY(log_str)    	log(LG_INF, "MSSID: Fn Entry - %s", (log_str))

#define MSSID_LOG_FN_EXIT(log_str)    	log(LG_INF, "MSSID: Fn Exit - %s", (log_str))

#else

#define MSSID_LOG_FN_ENTRY(log_str)
#define MSSID_LOG_FN_EXIT(log_str)


#define MSSID_DEVID_TO_PHY_IFACE

#endif /*#if (MSSID_LOG_ENABLED == 1)*/

int mssid_get_vss_count();
int mssid_make_service_set(struct config_t *configp,
	                   obj_serviceset *service_setp);

int mssid_devid_to_phy_iface_index(int devid);

int mssid_get_beacon_ie(struct obj_mlme *mlme, struct config_t *configp);

int mssid_vlan_to_vssid(int vlan_tag_num , int *vssidp);

int mssid_configure_bridge(struct config_t *config);

int mssid_parse_beacon_ie(unsigned char *ap_wpa_ie, struct config_t *configp);

struct config_t *mssid_vlan_to_vss_config(int vlan_tag, int devid);

int mssid_change_sta_vss(struct client_t *clientp, int vlan_tag);

int mssid_modify_softmac_state(int oid);
int mssid_reset_wireless_interfaces();
int mssid_commit_wireless_interfaces();
int mssid_add_devid(char *iface);
int mssid_init_all_service_sets();
int mssid_name2idx(char *ifname);


#endif /*#ifndef _MSSID_H_*/

