/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * Generic functions for communicating with the blob through prismeth IOCTLS
 */

#include <string.h>
#include <stdio.h>
#include <net/if.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <signal.h>
#include <net/if.h>
#include <errno.h>
#ifndef SMAC
#include "linux_mvc_oids.h"
#endif
#include "prismoids.h"
#include "log.h"
#include "netlink_func.h"
#include "client.h"
#include "misc.h"
#include "config.h"
#include "state_pae.h"
#include "state_ssn.h"

#if (PAED_MSSID == 1)
#include "assert.h"
#endif


extern int eapolfd;


#if (PAED_MSSID == 1)

/* the config array pointer */
extern struct config_t *config;

extern struct config_t *config_index[MAX_INTERFACES][MSSID_MAX_VSS_COUNT];
#else

extern struct config_t config[MAX_INTERFACES];
#endif  /* #if (PAED_MSSID == 1) */



#define MLMEEX_BUF_SIZE 256
long mlmeexbuf[MLMEEX_BUF_SIZE/4];
#ifdef SMAC
struct obj_mlme* mlmeex = (struct obj_mlme *)mlmeexbuf;
#if (RAC_SUPPORT == RAC_TRUE)
int association_id; 
#endif
#else
struct obj_mlmeex* mlmeex = (struct obj_mlmeex *)mlmeexbuf;
#endif

struct obj_sta sta;


#if (PAED_MSSID == 1) 

/* flag to note whether the IE from beacon has already been 
 * captured or not.
 */
int beacon_ie_set;

#endif /*  #if (PAED_MSSID == 1)  */

static int blob_find_sta(struct client_t *client, struct obj_sta *sta);

/*
 * Open NETLINK interface for traps
 */
int open_netlink( struct netlink_handle *blobtrap_h )
{
        // Open NETLINK for get/set OIDs
        if (prismoid_open() < 0)
        {
            log(LG_FTL, "Couldn't open NETLINK interface for prismoids!");
            return 0;
        }

        // Open NETLINK for subscription to traps

        if (prismtraps_open(blobtrap_h, TRAPGRP_WLAN_TRAPS) < 0)
        {
            log(LG_FTL, "Couldn't open NETLINK interface for traps!");
            return 0;
        }

        log(LG_INF, "Successfully opened NETLINK interface");
        return 1;
}


/*
 * Close NETLINK interface for traps
 */
void close_netlink( struct netlink_handle *blobtrap_h )
{
        // Close NETLINK for get/set OIDs

        prismoid_close();

        // Close NETLINK for subscription to traps

        prismtraps_close(blobtrap_h);
        log(LG_INF, "Closed NETLINK interface");
}
/* MV>> Defining macro to remove compilation error */
// #define MSSID_DEVID_TO_PHY_IFACE(devid) 1
/*
 * Handle traps that are received through the NETLINK interface
 */
int handle_traps( struct netlink_handle *blobtrap_h )
{
    char *trapdata = 0;
    unsigned int oid;
    unsigned int len;
    unsigned int devid;
    int i;
    struct config_t *configp = NULL;

	struct client_t *clientp=NULL;
	int reassoc=0;

#if (PAED_MSSID == 1)
    int service_set_no;
    int phy_interface_index;
#endif

    //DEBUG
    if (prismtraps_decode(blobtrap_h, &oid, &devid, &trapdata, &len) < 0)
    {
        log(LG_WRN, "Couldn't handle received trap!");
        return -1;
    }


#if (PAED_MSSID == 1)

     /* map devid to index value for physical interface in config_index
      * mapping array 
      */
     phy_interface_index = mssid_devid_to_phy_iface_index(devid); 

	 if (phy_interface_index < 0){
        log(LG_WRN, "MSSID: Invalid Devid value %d for OID %x", devid, oid);
        return -1;
	 }

     /* Find VSSID of the client and locate config pointer to the client */
     service_set_no = ((struct obj_mlme*)trapdata)->ssnr;

    //configp = config_index[phy_interface_index][service_set_no];

#else


    for(i = 0 ; i < paed_get_nof_interfaces() ; i++)
    {
        if (config[i].devid == devid)
        {
            configp = &config[i];
            break;
        }
    }

#endif /* #if (PAED_MSSID == 1) */


    switch (oid)
    {
#ifdef SMAC
    case DOT11_OID_AUTHENTICATE:
#else
    case DOT11_OID_AUTHENTICATEEX:
#endif

    // DEBUG code
   log(LG_DBG, "MSSID: Trap Type is - Authenticate, for Service set No. %d", service_set_no);
   

#if (PAED_MSSID == 1)

     /* The service set number needs to be between MSSID_VSSID_MIN 
      * and MSSID_VSSID_MAX. In particular, SoftMAC cannot send the invalid VSSID
      * value of 0xFF
      */
     if((service_set_no < MSSID_VSSID_MIN) || (service_set_no > MSSID_VSSID_MAX)){

		log(LG_WRN, "MSSID: received INVALID Service Set Number = %d for (Authenticate) OID = %x from UMAC", 
			service_set_no, oid);
		return 0;
	 }

    /* Get the config pointer for the concerned interface */
    configp = config_index[phy_interface_index][service_set_no];

#endif /* #if (PAED_MSSID == 1) */

    	if (len > MLMEEX_BUF_SIZE) len = MLMEEX_BUF_SIZE;
	memcpy(mlmeex, trapdata, len);
	/* Don't reset the code MVC is checking auth algoritms
	 * and will set the error code is necessary
	 * paed has no interest in changing the code.
	 */
	//mlmeex->code = DOT11_SC_SUCCESSFUL;
        free(trapdata);
	mlmeex->size = 0;

        log(LG_INF,"TRAP: Authenticate [value = %x], state %d", oid, mlmeex->state );

// DEBUG CODE - Look at this closely
// Looks like PAED needs to send the Auth Success msg also
// #if (PAED_MSSID == 1)

//      	/* Do not send authenticate response in case of MSSID */

// #else

	if (mlmeex->state == 1) {// Only respond on state AUTHING

                log(LG_DBG, "MSSID: AUTH state is 1. sending auth response");

		/* Respond to the UMAC without modifying mleex->code */
		blob_authenticate_client(mlmeex, configp);
        }
// #endif /* #if (PAED_MSSID == 1) */

    	break;



#ifdef SMAC
    case DOT11_OID_ASSOCIATE:
    case DOT11_OID_REASSOCIATE:   // Fall through
#else
    case DOT11_OID_ASSOCIATEEX:
    case DOT11_OID_REASSOCIATEEX:   // Fall through

#endif

    if (len > MLMEEX_BUF_SIZE) len = MLMEEX_BUF_SIZE;
    memcpy(mlmeex, trapdata, len);
    free(trapdata);

    // DEBUG code
     log(LG_DBG, "MSSID: Trap Type is - (Re)Associate [value = %x], for service set no. %d", oid, service_set_no);

#if (PAED_MSSID == 1)

     /* The service set number needs to be between MSSID_VSSID_MIN 
      * and MSSID_VSSID_MAX. In particular, SoftMAC cannot send the invalid VSSID
      * value of 0xFF
      */
     if((service_set_no < MSSID_VSSID_MIN) || (service_set_no > MSSID_VSSID_MAX)){

		log(LG_WRN, "MSSID: received INVALID Service Set Number = %d for ((Re)Associate) OID = %x from UMAC", 
			service_set_no, oid);
		return 0;
	 }

    /* Get the config pointer for the concerned interface */
    configp = config_index[phy_interface_index][service_set_no];

#endif /* #if (PAED_MSSID == 1) */

	/* If code is != SUCCESSFUL, respond back to UMAC with the same mlmeex */
	if (DOT11_SC_SUCCESSFUL != mlmeex->code)
	{	
#ifdef SMAC
		if (DOT11_OID_REASSOCIATE == oid){
			reassoc = 1;
		}
		else {
			reassoc = 0;
		}
#else
#ifdef BLOB_V1
    if (DOT11_STATE_ASSOCING == mlmeex->state){
			reassoc = 1
		}
		else {
			reassoc = 0;
		}
#endif
#ifdef BLOB_V2
		if (DOT11_OID_REASSOCIATEEX == oid){
			reassoc = 1
		}
		else {
			reassoc = 0;
		}
#endif
#endif /* SMAC */

	    log(LG_WRN, "MSSID: Received (Re)Assoc Trap with error code value = %x. Reassoc = %d", 
				mlmeex->code, reassoc);

            blob_associate_client(mlmeex, configp, reassoc );
            return 0;
	}

#ifdef SMAC

        log(LG_INF,"TRAP: (Re)associate, state %d", mlmeex->state );

        handle_new_assoc(configp, mlmeex, (oid == DOT11_OID_REASSOCIATE? 1 : 0) );
#if (RAC_SUPPORT == RAC_TRUE)
	association_id  =  mlmeex->id ;
#endif	

	
#else
#ifdef BLOB_V1
        handle_new_assoc(configp, mlmeex);
#endif
#ifdef BLOB_V2
        handle_new_assoc(configp, mlmeex, (oid == DOT11_OID_REASSOCIATEEX? 1 : 0) );
#endif
#endif /* SMAC */
        break;
#ifdef SMAC
    case DOT11_OID_DISASSOCIATE:
    case DOT11_OID_DEAUTHENTICATE: // Fall through
#else
    case DOT11_OID_DISASSOCIATEEX:
    case DOT11_OID_DEAUTHENTICATEEX: // Fall through
#endif


    log(LG_DBG, "MSSID: Trap Type is - Disassociate/ Deauthenticate [value = %x], for service set no. %d",
              oid, service_set_no);
    
#if (PAED_MSSID == 1)

     /* The service set number needs to be between MSSID_VSSID_MIN 
      * and MSSID_VSSID_MAX. In particular, SoftMAC cannot send the invalid VSSID
      * value of 0xFF
      */
     if((service_set_no < MSSID_VSSID_MIN) || (service_set_no > MSSID_VSSID_MAX)){

		log(LG_WRN, "MSSID: received INVALID Service Set Number = %d for (Deassociate/ Dauthenticate) OID = %x from UMAC", 
			service_set_no, oid);
		return 0;
	 }

    /* Get the config pointer for the concerned interface */
    configp = config_index[phy_interface_index][service_set_no];

#endif /* #if (PAED_MSSID == 1) */


        memcpy(mlmeex, trapdata, len);
        free(trapdata);
        handle_disassoc(mlmeex);
        break;
    case DOT11_OID_MICFAILURE:


    // DEBUG code
    log(LG_DBG, "MSSID: Trap Type is - MIC Failure, for service set no. %d", service_set_no);
    
        memcpy(&sta, trapdata, sizeof(struct obj_sta));
        free(trapdata);
        handle_mic_failure();
        break;

#if (PAED_MSSID == 1)

    case DOT11_OID_BEACONTX:


    
         /* Trap Beacon message to capture the WPA IE. The IE is applicable
          * over all created service sets. Therefore, we ignore the SSNR in 
          * the mlme object. Copy the IE in all config structures.
          * We need a way to deregister for the Beacon Trap from now on
          * otherwise, it'll lead to high processing load.
          */

          memcpy(mlmeex, trapdata, len);
          free(trapdata);
          if (0 == beacon_ie_set) {

    
              log(LG_INF, "MSSID: Trap Type is - BeaconTx. Capturing Beacon and setting IE flag");

              beacon_ie_set = 1; 
              mssid_get_beacon_ie(mlmeex, config);
          }
          break;

#endif /*#if (PAED_MSSID == 1) */


    default:
        //log(LG_DBG, "Received trap not for us.. oid: 0x%X", oid);
        free(trapdata);
        return -1;
    }

    return 0;
}

/*
 * Set default-policy global portstats of all ports/clients
 */

int blob_set_mlme_mode(long mode, struct config_t *config)
{
        int r;

#ifdef SMAC
	/* The SoftMAC architecture does not support any auto mode. Check the driver */
#else
	r = set_prismoid( config->ifname, DOT11_OID_MLMEAUTOLEVEL, (void *)&mode, sizeof(mode) );

	if(r<0) {
		log( LG_ERR, "set_prismoid( DOT11_OID_MLMEAUTOLEVEL ) on %s returned %d\n", 
                    config->ifname, r );
		return( -1 );
	}
#endif

        return(0);
}


/*
 * Set default-policy global portstats of all ports/clients
 */

int blob_portstatus_set_global(enum portstatus_t state, struct config_t *config)
{
        int r;
        char stat;

        switch(state) {
                case AUTHORIZED:
                        stat = 0;
                        break;
                case UNAUTHORIZED:
                        stat = 1;
                        break;
        }

        log( LG_INF, "set_prismoid( DOT11_OID_DOT1XENABLE ) on %s set to %d\n", 
            config->ifname, stat );

        r = set_prismoid( config->ifname, DOT11_OID_DOT1XENABLE, (void *)&stat, sizeof(stat) );

	if(r<0) {
		log( LG_ERR, "set_prismoid( DOT11_OID_DOT1XENABLE ) on %s returned %d\n",
                    config->ifname, r );
		return( -1 );
	}

        return(0);
}

/*
 * Set portstatus of specific client to authorized or unauthorized
 */

int blob_portstatus_set(struct client_t *client, enum portstatus_t state)
{
        int r;
        unsigned int oid;

        switch(state) {
                case AUTHORIZED:
                        oid = DOT11_OID_EAPAUTHSTA;
                        break;
                case UNAUTHORIZED:
                        oid = DOT11_OID_EAPUNAUTHSTA;
                        break;
                default:
                        return -1;
        }

        log( LG_INF, "set_prismoid( %08lx ) on %s mac %02x:%02x:%02x:%02x:%02x:%02x\n", 
            oid, config->ifname,
            client->hwaddr[0], client->hwaddr[1], client->hwaddr[2],
            client->hwaddr[3], client->hwaddr[4], client->hwaddr[5] );

        r = set_prismoid( client->cconfig->ifname, oid, (void *)client->hwaddr, 6 );

	if(r<0) {
		log( LG_ERR, "set_prismoid( %08x ) on %s returned %d\n", 
                    oid, config->ifname, r );
		return( r );
	}

        client->portStatus = state;

        return(0);
}

/*
 * Get information about this client from the blob. Returns the age of
 * the client, or -1 if client is not known in the blob.
 */

int blob_get_client_by_aid(struct config_t *config, int aid, struct obj_sta* sta)
{
        int r;
        unsigned long oid = (DOT11_OID_CLIENT1 - 1) + aid;

        r = get_prismoid( config->ifname, oid, (void *)sta, sizeof(struct obj_sta) );

	if(r<0) {
		log(LG_DBG, "get_prismoid( DOT11_OID_CLIENT%d ) on %s returned %d",
                    aid, config->ifname, r);
		return(r);
	}

        return(r);
}

/*
 * Get information about this client from the blob. Returns the obj_sta of
 * the client, or -1 if client is not known in the blob.
 */
static int blob_find_sta(struct client_t *client, struct obj_sta *sta)
{
        int r;

        memcpy(sta->address, client->hwaddr, 6);

        r = get_prismoid( client->cconfig->ifname, DOT11_OID_CLIENTFIND, (void *)sta, sizeof(struct obj_sta) );

	if(r<0) {
		log(LG_DBG, "get_prismoid( DOT11_OID_CLIENTFIND ) on %s returned %d\n",
                    config->ifname, r);
	}

        return (r);
}

/*
 * Get information about this client from the blob. Returns the age of
 * the client, or -1 if client is not known in the blob.
 */

int blob_get_client_age(struct client_t *client)
{
        int r;
        struct obj_sta sta;

        r = blob_find_sta (client, &sta);

        if (r >= 0)
            return(sta.age);

        return (r);
}


/*
 * Get information about this client from the blob. Returns the 'state'
 * value of the client obj_sta, or -1 if client is not known in the blob.
 */

int blob_get_client_state(struct client_t *client)
{
        int r;
        struct obj_sta sta;

	r = blob_find_sta (client, &sta);
        log(LG_DBG, "State: blob_find_sta returned %d state = %d\n", r, sta.state);

        if (r >= 0)
            return(sta.state );

        return (r);
}

/*
 * This function sends a authenticate confirm to the MVC.
 */
#ifdef SMAC
int blob_authenticate_client(struct obj_mlme *mlmeex, struct config_t *config)
{
	int r;

	r = set_prismoid( config->ifname, DOT11_OID_AUTHENTICATE, (void *)mlmeex, sizeof(struct obj_mlme) );

	if(r<0) {
		log(LG_DBG, "set_prismoid( DOT11_OID_AUTHENTICATE ) on %s returned %d\n",
                    config->ifname, r);
		return( r );
	}

	return(0);
}
#else
int blob_authenticate_client(struct obj_mlmeex *mlmeex, struct config_t *config)
{
        int r;

        r = set_prismoid( config->ifname, DOT11_OID_AUTHENTICATEEX, (void *)mlmeex, sizeof(struct obj_mlmeex) );

	if(r<0) {
		log(LG_DBG, "set_prismoid( DOT11_OID_AUTHENTICATEEX ) on %s returned %d\n",
                    config->ifname, r);
		return( r );
	}

        return(0);
}
#endif

/*
 * This function sends a associate confirm to the MVC.
 */
#ifdef SMAC
int blob_associate_client(struct obj_mlme *mlmeex, struct config_t *config, int reassoc)
{
	int r;

	if (reassoc == 0)
		r = set_prismoid( config->ifname, DOT11_OID_ASSOCIATE, (void *)mlmeex, sizeof(struct obj_mlme) );
	else
		r = set_prismoid( config->ifname, DOT11_OID_REASSOCIATE, (void *)mlmeex, sizeof(struct obj_mlme) );

	if(r<0) {
	        if (reassoc == 0)
	        {
		    log(LG_DBG, "set_prismoid( DOT11_OID_ASSOCIATE ) on %s returned %d\n",
                        config->ifname, r);
		}
		else
		{
		    log(LG_DBG, "set_prismoid( DOT11_OID_REASSOCIATE ) on %s returned %d\n",
                        config->ifname, r);
		}
		return( r );
	}

	return(0);
}
#else
int blob_associate_client(struct obj_mlmeex *mlmeex, struct config_t *config, int reassoc)
{
        int r;

        if (reassoc == 0)
                r = set_prismoid( config->ifname, DOT11_OID_ASSOCIATEEX, (void *)mlmeex, sizeof(struct obj_mlmeex) );
        else
                r = set_prismoid( config->ifname, DOT11_OID_REASSOCIATEEX, (void *)mlmeex, sizeof(struct obj_mlmeex) );

	if(r<0) {
	        if (reassoc == 0)
	        {
		    log(LG_DBG, "set_prismoid( DOT11_OID_ASSOCIATEEX ) on %s returned %d\n",
                        config->ifname, r);
		}
		else
		{
		    log(LG_DBG, "set_prismoid( DOT11_OID_REASSOCIATEEX ) on %s returned %d\n",
                        config->ifname, r);
		}
		return( r );
	}

	return(0);
}
#endif


/*
 * Deauthenticate all clients
 */
int blob_deauthenticate_all(struct config_t *config)
{
        int r,i,j;
        struct obj_mlme mlme;
        struct obj_sta deauth_sta;

	r = get_prismoid( config->ifname, DOT11_OID_CLIENTS, (void *)&j, 4 );
	if(r<0) 
        {
		log(LG_DBG, "get_prismoid( DOT11_OID_CLIENTS ) on %s returned %d\n", 
                    config->ifname, r );
		return( r );
	}

        log(LG_INF, "blob_deauthenticate all: %d clients\n", j );


        for (i = 0; i < j; i++)
        {
                memset (&mlme, 0x00, sizeof(mlme));
                memset (&deauth_sta, 0x00, sizeof(deauth_sta));

                r = get_prismoid( config->ifname, (DOT11_OID_CLIENT1 + i), (void *)&deauth_sta, sizeof(deauth_sta));
                
                if (r == 0 && deauth_sta.state == 5)
                {

                        mlme.code = 1;
                        mlme.id = (i + 1);

                        r = set_prismoid( config->ifname, DOT11_OID_DEAUTHENTICATE, (void *)&mlme, sizeof(mlme) );
                        
 			if(r<0) {
 				log(LG_DBG, "set_prismoid( DOT11_OID_DEAUTHENTICATE ) on %s for AID: %d returned %d\n",
                                    config->ifname, mlme.id,r);
 				return( r );
 			}

                        log(LG_INF, "blob_deauthenticate client AID: %d OK\n", mlme.id);

                }
        }

        log(LG_INF, "blob_deauthenticate_all OK\n");

        return(0);
}


/*
 * Deauthenticate the given client hwaddr
 */
int blob_deauthenticate_client(struct client_t *client)
{
        int r;
        struct obj_mlme mlme;

	log( LG_DBG, "deauth_client client %p config %p ifname %s\n", client, client->cconfig,
        	client->cconfig->ifname );

 	memset (&mlme, 0x00, sizeof(mlme));
	memcpy( mlme.address, client->hwaddr, 6 );
	mlme.code = 1;
	mlme.id   = -1;

        r = set_prismoid( client->cconfig->ifname, DOT11_OID_DEAUTHENTICATE, (void *)&mlme, sizeof(mlme) );

	if(r<0) {
		log( LG_DBG, "set_prismoid( DOT11_OID_DEAUTHENTICATE ) on %s returned %d\n", 
                    client->cconfig->ifname, r );
		return( r );
	}

        log( LG_INF, "blob_deauthenticate_client(%s)\n", hwaddr2string(client->hwaddr) );

        return( 0 );
}

/*
 * Get the privacy_invoked setting from the blob
 */
int blob_get_privacy_invoked(struct config_t *config)
{
        long pi = 0;
        int r;

        r = get_prismoid( config->ifname, DOT11_OID_PRIVACYINVOKED, (void *)&pi, sizeof(pi) );

	if( r < 0 ) {
		log(LG_FTL, "blob_get_privacy_invoked(DOT11_OID_PRIVACYINVOKED) on %s returned %d\n",
                    config->ifname, r );
		return r;
	}
	return pi;
}

/*
 * Get the exclude_unencrypted setting from the blob
 */
int blob_get_exclude_unencrypted(struct config_t *config)
{
        long pi = 0;
        int r;

        r = get_prismoid( config->ifname, DOT11_OID_EXUNENCRYPTED, (void *)&pi, sizeof(pi) );

	if( r < 0 ) {
		log(LG_FTL, "blob_get_exclude_unencrypted(DOT11_OID_EXUNENCRYPTED) on %s returned %d\n",
                    config->ifname, r );
		return r;
	}
	return pi;
}

/*
 * Get the number of sent broadcast packets
 */
int blob_get_mcast_packets(struct config_t *config)
{
        long pkts = 0;
        int r;

        r = get_prismoid( config->ifname, GEN_OID_MSDUTXMULTICAST, (void *)&pkts, sizeof(pkts) );

	if( r < 0 ) {
	    log(LG_FTL, "blob_get_mcast_packets(GEN_OID_NSDUTXMULTICAST) on %s returned %d\n",
                    config->ifname, r );
            return r;
        }
        return pkts;
}

/*
 * Set the privacy_invoked setting in the blob
 */
int blob_set_privacy_invoked(long value, struct config_t *config)
{
        int r;

        r = set_prismoid( config->ifname, DOT11_OID_PRIVACYINVOKED, (void *)&value, sizeof(value) );

	if( r < 0 ) {
		log(LG_FTL, "blob_set_privacy_invoked(DOT11_OID_PRIVACYINVOKED) on %s returned %d\n",
                    config->ifname, r );
        return r ;
        }

    return(0);
}

/*
 * Set the exclude_unencrypted setting in the blob
 */
int blob_set_exclude_unencrypted(long value, struct config_t *config)
{
        int r;

        r = set_prismoid( config->ifname, DOT11_OID_EXUNENCRYPTED, (void *)&value, sizeof(value) );

	if( r < 0 ) {
		log(LG_FTL, "blob_set_exclude_unencrypted(DOT11_OID_EXUNENCRYPTED) on %s returned %d\n",
                    config->ifname, r );
        return r ;
        }

    return(0);
}

/*
 * set per-station key for the given client
 */
int blob_set_persta_key(struct client_t *client, int keyid)
{
        int r;
        struct obj_stakey stakey;
	
	memset( &stakey, 0, sizeof(struct obj_stakey));
	
        log(LG_DBG, "blob_set_persta_key(%d, %d)\n", keyid, client->session_key_len);

        memcpy(stakey.address, client->hwaddr, 6);
        stakey.keyid = keyid;
        stakey.length = client->session_key_len  /*13*/;
        if (stakey.length == PTK_LEN)
        {
            stakey.type = DOT11_PRIV_TKIP;
        }
        else
        {
            stakey.type = DOT11_PRIV_WEP;
        }
        
        if(client->client_wpa_ie.ucast == UCAST_CIPHER_AES_VAR_2) {
        	stakey.type = DOT11_PRIV_AES_CCMP;
        }

#ifdef BLOB_V1
        stakey.operation = DOT11_STAKEY_SET_KEY;
#endif

        memcpy(stakey.key, client->tx_session_key, stakey.length);

#ifdef BLOB_V1
        r = set_prismoid( client->cconfig->ifname, DOT11_OID_STAKEY, (void *)&stakey, sizeof(stakey) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_STAKEY ) on %s returned %d\n",
                    client->cconfig->ifname, r );
        	return r ;
	}

        // Set the keyid to use for this client

        stakey.keyid = (keyid << 4);
        stakey.operation = DOT11_STAKEY_SET_KEYID;
#endif

        r = set_prismoid( client->cconfig->ifname, DOT11_OID_STAKEY, (void *)&stakey, sizeof(stakey) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_STAKEY ) on %s returned %d\n",
                    client->cconfig->ifname, r );
        	return r ;
	}

        return(0);
}

#if (PAED_MSSID == 1)

/*
 * set multicast/broadcast key for the service set
 */

int blob_mssid_set_bcast_key(struct config_t *configp, char *keybuf, int keylen, int keyid)
{
        int r;
        struct obj_ssmckey key;

        memset( &key, 0, sizeof(struct obj_ssmckey));

        log( LG_DBG, "MSSID: blob_mssid_set_bcast_key (length %d)\n", keylen );
 
        /* set the service set number */
        key.ssnr = configp->vssid;

        memset(key.address, 0xff, 6);
        if (keyid < 0 || keyid > 3){
                return -1;
        }

        key.keyid = keyid;
        key.length = keylen;

        if (key.length == PTK_LEN)
        {
            key.type  = DOT11_PRIV_TKIP;
        }
        else
        {
            key.type  = DOT11_PRIV_WEP;
        }

        memcpy(key.key, keybuf, keylen);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("MSSID: Setting BCAST key", key.key, key.length);
#endif


        r = set_prismoid( configp->ifname, DOT11_OID_SSMCKEY, (void *)&key, sizeof(key) );

        if( r < 0 ) {
                log(LG_FTL, "MSSID: set_prismoid( DOT11_OID_SSMCKEY ) on %s returned %d\n",
                    configp->ifname, keyid, r );
                return r ;
        }
        return (0);
}

#endif  /* #if (PAED_MSSID == 1) */

// bcast key for mssid









/*
 * set multicast/broadcast key
 */

int blob_set_bcast_key(char *keybuf, int keylen, int keyid, char *ifname)
{
#ifdef BLOB_V1
        int r;
        struct obj_key key;

        log( LG_DBG, "blob_set_bcast_key (length %d)\n", keylen );

        memcpy(key.key, keybuf, keylen);
        key.length = keylen;
        if (key.length == PTK_LEN)
        {
            key.type  = DOT11_PRIV_TKIP;
        }
        else
        {
            key.type  = DOT11_PRIV_WEP;
        }
        key.domain = 0;

#ifdef DEBUG_PRINTKEYS
        dump_bindata("Setting BCAST key", key.key, key.length);
#endif

        r = set_prismoid( ifname, (DOT11_OID_DEFKEY1 + keyid), (void *)&key, sizeof(key) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_DEFKEY1 + %d ) on %s returned %d\n", 
                    ifname, keyid, r );
	        return r ;
	}
	return (0);
#else /* BLOB_V2 / SMAC */
        int r;
        struct obj_stakey key;

	memset( &key, 0, sizeof(struct obj_stakey));

        log( LG_DBG, "blob_set_bcast_key (length %d)\n", keylen );

        memset(key.address, 0xff, 6);
        if (keyid < 0 || keyid > 3)
                return -1;
        key.keyid = keyid;
        key.length = keylen;

        if (key.length == PTK_LEN)
        {
            key.type  = DOT11_PRIV_TKIP;
        }
        else
        {
            key.type  = DOT11_PRIV_WEP;
        }

        memcpy(key.key, keybuf, keylen);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("Setting BCAST key", key.key, key.length);
#endif

        r = set_prismoid( ifname, DOT11_OID_STAKEY, (void *)&key, sizeof(key) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_STAKEY ) on %s returned %d\n", 
                    ifname, keyid, r );
	        return r ;
	}
	return (0);

#endif
}

int blob_set_bcast_keyid(int keyid, char *ifname)
{
        int r;
        unsigned long key_index = keyid;

        log( LG_DBG, "blob_set_bcast_keyid (keyid %d)\n", keyid );

        r = set_prismoid( ifname, DOT11_OID_DEFKEYID, (void *)&key_index, sizeof(key_index) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_DEFKEYID ) on %s returned %d\n",
                    ifname, r );
	        return r ;
	}

        return(0);
}


/*
 * Retrieve key number <keyid> from the blob and place it into <keybuf>
 * with max size <keylen>. Return's actual key length.
 */

int blob_get_bcast_key(int keyid, char *keybuf, int keylen, struct config_t *config)
{
        int r;
        struct obj_key key;

        log( LG_DBG, "blob_get_bcast_key: keyid %d keylen %d ifname %s\n", keyid, keylen, config->ifname );

        r = get_prismoid( config->ifname, (DOT11_OID_DEFKEY1 + keyid), (void *)&key, sizeof(struct obj_key) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_DEFKEY1 + %d ) on %s returned %d\n", 
                    config->ifname, keyid, r );
        	return r ;
	}

        memcpy(keybuf, key.key, (keylen < key.length) ? keylen : key.length);

        return(key.length);
}


int blob_set_attachment(struct obj_attachment *oa, int len, struct config_t *config)
{
         int r;
         /* len passed must be length of obj_attachment struct + ie element in data field of struct */
         r = set_prismoid( config->ifname, DOT11_OID_ATTACHMENT, (void *)oa, len );

	if( r < 0 ) {
		log(LG_DBG, "set_prismoid( DOT11_OID_ATTACHMENT ) on %s returned %d\n", 
                    config->ifname, r );
                return r ;
        }

        return(0);
}


#ifdef BLOB_V1
/*
 * Retrieve station statistics from the blob and place it into stastat struct
 */
int blob_get_stastat(struct obj_stastat *stat, struct config_t *config)
{
        int r;

        /* Make sure the correct size is set in stat.size cause this is used to determine
         * the amount of data after the obj_stastat structure.
         */
        r = get_prismoid( config->ifname, DOT11_OID_STASTAT, (void *)stat, sizeof(struct obj_stastat) + stat->size - 1 );

	if( r < 0 ) {
		log(LG_FTL, "get_prismoid( DOT11_OID_STASTAT ) on %s returned %d\n",
                    config->ifname, r );
	        return r ;
	}

        return(0);
}

#else	// BLOB_V2 & SMAC

/*
 * Retrieve station statistics from the blob and place it into stastat struct
 */
int blob_get_stasc(struct obj_stasc *stat, struct config_t *config)
{
        int r;

        /* Make sure the correct size is set in stat.size cause this is used to determine
         * the amount of data after the obj_stastat structure.
         */
        r = get_prismoid( config->ifname, DOT11_OID_STASC, (void *)stat, sizeof(struct obj_stasc));

	if( r < 0 ) {
		log(LG_FTL, "get_prismoid( DOT11_OID_STASC ) on %s returned %d\n",
                    config->ifname, r );
	        return r ;
	}

	return(0);
}
#endif

#ifdef SMAC
/*
 * Set MLME policies in the capability field 
 */
int blob_set_mlme_policies(int policies, char *ifname)
{
	int r;
        unsigned long mask = policies;

	log( LG_DBG, "blob_set_mlme_policies (mask %x)\n", mask );

	r = set_prismoid( ifname, DOT11_OID_MLMEPOLICIES, (void *)&mask, sizeof(mask) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_MLMEPOLICIES ) on %s returned %d\n",
                    ifname, r );
	        return r ;
	}

	return(0);
}

/*
 * Get MLME policies from the capability field 
 */
int blob_get_mlme_policies(int *policies, char *ifname)
{
	int r;
        unsigned long mask;
        
	r = get_prismoid( ifname, DOT11_OID_MLMEPOLICIES, (void *)&mask, sizeof(mask) );

	if( r < 0 ) {
		log(LG_FTL, "get_prismoid( DOT11_OID_MLMEPOLICIES ) on %s returned %d\n",
                    ifname, r );
	        return r ;
	}

	log( LG_DBG, "blob_get_mlme_policies (mask %x)\n", mask );

        *policies = mask;
	return(0);
}

/*
 * Commit settings to make them operational
 */
int blob_commit_settings(struct config_t *config)
{
    int r;
    unsigned long dummy = 0;

    // DEBUG code
    log(LG_DBG, "MSSID: Doing COMMIT for ifname %s", config->ifname);

    
    r = set_prismoid( config->ifname, GEN_OID_COMMIT, (void *)&dummy, sizeof(dummy) );

    if( r < 0 ) {
        log(LG_FTL, "set_prismoid( GEN_OID_COMMIT ) on %s returned %d\n", 
            config->ifname, r );
        return r ;
    }
    
    return (0);
}


#ifdef LINUX
/*
 * Set SSID for standard linux environment 
 */
int blob_set_ssid(struct config_t *config)
{
	int r;
        struct obj_ssid ssid_obj;
	
	strncpy( ssid_obj.octets, config->ssid, config->ssidlength );
	ssid_obj.length = config->ssidlength;

	r = set_prismoid( config->ifname, DOT11_OID_SSID, (void *)&ssid_obj, sizeof(struct obj_ssid) );

	if( r < 0 ) {
		log(LG_FTL, "set_prismoid( DOT11_OID_SSID ) on %s returned %d\n",
                    config->ifname, r );
	        return r ;
	}

	log( LG_DBG, "blob_set_ssid (ssid %s)\n", ssid_obj.octets );

	return(0);
}



#endif	// LINUX
#endif	// SMAC


#if (PAED_MSSID == 1)

/*
 * Configure the given service set in SoftMAC
*/
int blob_mssid_configure_service_set(struct config_t *configp, 
                               obj_serviceset *servicesetp)
{
    int r;


    log(LG_INF, "MSSID: Configuring service set for interface %s and vss %s", 
         configp->ifname, configp->vss_ifname);

    r = set_prismoid( configp->ifname, (DOT11_OID_SERVICESET0 + configp->vssid), 
                      (void *) servicesetp, sizeof(obj_serviceset));

    if ( r < 0 ) {
       log(LG_FTL, "MSSID: set_prismoid(DOT11_OID_SERVISET0 + %d) on %s returned %d\n", 
         	 configp->vssid, configp->ifname, r );
       return r ;
    }
    return(0);
}

/*
 * Set the Virtual Service Set associated with the client
*/

int blob_mssid_set_sta_vss(struct config_t *configp, char *sta_addr, int vssid)
{
    int r;
    obj_stasetss sta_sets;

    /* clean the object */
    memset(&sta_sets, 0, sizeof(obj_stasetss));
    
    /* Copy the client address in the structure */
    memcpy(sta_sets.address, sta_addr, 6);
    sta_sets.ssnr = vssid;

    log(LG_DBG, "MSSID: PRISMOID for DOT11_OID_STASETSS - Ifname = %s, client = %s, vss = %d",
		  configp->ifname, hwaddr2string(sta_addr), vssid);
    r = set_prismoid( configp->ifname, DOT11_OID_STASETSS, 
                      (void *) &sta_sets, sizeof(obj_stasetss));

	if ( r < 0 ) {
         log(LG_FTL, "MSSID: set_prismoid(DOT11_OID_STASETSS) on %s 
             returned %d\n", configp->ifname, r );
         return r ;
     }

     return(0);
}


/*
 * Deauthenticate all the clients in the VSS
 */
int blob_mssid_deauth_vss(struct config_t *configp, int vssid)
{   
    int r;
    obj_mlme mlme;
    mlme.ssnr = vssid;

    /* set address as broadcast address */
    mlme.address[0]= 0xFF;
    mlme.address[1]= 0xFF;
    mlme.address[2]= 0xFF;
    mlme.address[3]= 0xFF;
    mlme.address[4]= 0xFF;
    mlme.address[5]= 0xFF;

    r = set_prismoid( configp->ifname, DOT11_OID_DEAUTHENTICATE, 
                      (void *) &mlme, sizeof(obj_mlme));

    if ( r < 0 ) {
        log(LG_FTL, "MSSID: set_prismoid(DOT11_OID_STASETSS) on %s 
             returned %d\n", configp->ifname, r );
        return r ;
     }
     return(0);
}

/*
 * reset the softmac
 */
int blob_mssid_reset_softmac(char *ifname)
{
    int r;
    unsigned long dummy = 0;

    r = set_prismoid( ifname, GEN_OID_RESET, (void *)&dummy, sizeof(dummy) );

    if( r < 0 ) {

// DEBUG code. Verify if this should actyually be treated as Fatal

        log(LG_WRN, "set_prismoid( GEN_OID_RESET ) on %s returned %d\n",
            ifname, r );
        return r ;
    }

    return (0);
}

int blob_mssid_reset_service_set(char *ifname, int vssid, obj_serviceset *servicesetp)
{
    int r;

    r = set_prismoid( ifname, (DOT11_OID_SERVICESET0 + vssid), 
                      (void *) servicesetp, sizeof(obj_serviceset));
    if ( r < 0 ) {
       log(LG_WRN, "MSSID: set_prismoid for resetting VSS (DOT11_OID_SERVISET0 + %d) on %s returned %d\n", 
         	 vssid, ifname, r );
       return r ;
    }
    return(0);
}

int blob_mssid_commit_settings(char *ifname)
{
    int r;
    unsigned long dummy = 0;
    
    r = set_prismoid(ifname, GEN_OID_COMMIT, (void *)&dummy, sizeof(dummy) );
    if( r < 0 ) {
        log(LG_WRN, "set_prismoid( GEN_OID_COMMIT ) on %s returned %d\n",
            ifname, r );
        return r ;
    }

    return(0);	
}

/* 
 * Check for a PRISM OID value on the given interface. If the call fails, the
 * given interface is not a Wireless interface.
 */
int blob_mssid_wireless_interface(char *ifname)
{
	int r;
	long data = 0;

#if (PAED_ON_UAP == 1)

	r = get_prismoid(ifname, DOT11_OID_FREQUENCY, &data, sizeof(data));

	if (0 != r) {
	   log(LG_INF, "MSSID: PRISMOID call for DOT11_OID_FREQUENCY failed on interface %s", ifname);
	   return -1;
	}
	return 0;
#else

	if (!strcmp("eth1", ifname) || !strcmp("eth3", ifname)){
	   log(LG_INF,"MSSID: Found wireless interface %s", ifname); 	
	   return 0;
	}
	return -1;

#endif	
}


#endif /* #if (PAED_MSSID == 1) */

#if (RAC_SUPPORT == RAC_TRUE)

#ifdef SMAC
/*
 * Retrieve station statistics from the blob and place it into stastat struct
 */
int blob_get_stastats(struct client_t *client, struct obj_stastats *stastats, struct config_t *config)
{
	int r;
	memcpy(stastats->address, client->hwaddr, 6);
	r = get_prismoid( config->ifname, DOT11_OID_STASTATS, (void *)stastats, sizeof(struct obj_stastats)  );

	if( r < 0 ) {
		log(LG_FTL, "get_prismoid( DOT11_OID_STASTATS ) on %s returned %d\n",config->ifname, r );
		return r ;
	}

	return(0);
}

/*Retrieve the number of octets that have been received by the AP from the client*/
long blob_get_input_octet_count(struct client_t *client)
{
	int r;
	struct obj_stastats stastats;

	r = blob_get_stastats( client, &stastats, client->cconfig);

	if (r >= 0)
		return(stastats.octets_rx);

	return (-1);
}

/*Retrieve the number of octets that have been sent by the AP to the client.*/
long blob_get_output_octet_count(struct client_t *client)
{
	int  r;
	struct obj_stastats stastats;

	r = blob_get_stastats( client, &stastats, client->cconfig);

	if (r >= 0)
		return(stastats.octets_tx);

	return (-1);
}


/*Retrieve the the number of MSDU packets that have been received by the AP from the client */
long blob_get_input_acct_packet_count(struct client_t *client)
{
	int r;
	struct obj_stastats stastats;

	r = blob_get_stastats( client, &stastats, client->cconfig);

	if (r >= 0)
		return(stastats.msdu_rx);

	return (-1);
}

/*Retrieve the number of MSDU packets that have been sent by the AP to the client*/
long blob_get_output_acct_packet_count(struct client_t *client)
{
	int r;
	struct obj_stastats stastats;

	r = blob_get_stastats( client, &stastats, client->cconfig);

	if (r >= 0)
		return(stastats.msdu_tx);

	return (-1);
}

int blob_send_null_data_request(struct obj_mlme *mlmeex, struct config_t *config)
{
	int r;
	r = set_prismoid( config->ifname,  DOT11_OID_NULL,
		          (void *)mlmeex, sizeof(struct obj_mlme) );
	if(r<0){
		log(LG_DBG, "set_prismoid( DOT11_OID_NULL ) on %s returned %d\n",
		    config->ifname, r);
		return( r );
	}
	return(0);
}

#endif	//  SMAC

#endif//RAC_SUPPORT


