/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

/*
 * Main program
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <string.h>
#include <time.h>
#include <getopt.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#ifdef LINUX
#include "linux.h"
#else
#include <aplog.h>
#endif

#include "cmd.h"
#include "paed.h"
#include "misc.h"
#include "log.h"
#include "acl.h"
#include "radius.h"
#include "eapol.h"
#include "eap.h"
#include "client.h"
#include "config.h"
#include "state_be.h"
#include "state_pae.h"
#include "state_ssn.h"
#include "state_gtk.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "keystuff.h"
#include "ie.h"
#include "rac.h"
#if (PAED_MSSID == 1)
#include "netlink_func.h"
#endif

#define max(a,b) ((a>b)?a:b)


// extern vars
#if (RAC_SUPPORT == RAC_TRUE)
extern int rac_rx(struct client_t *client); 
extern int rac_status;
#endif
extern int eapolfd;                     // Needed here for select on incoming EAPOl packets
extern struct client_t *clientlist;             // Everybody uses this
extern int dump_bindata_enabled;        // This can be set for extremely verbose debugging

extern struct obj_sta sta;                  // obj_sta struct for prismeth ioctls
extern volatile int micfailure;         // set to 1 by micfailure_sighandler when a micfailure occures
struct netlink_handle blobtrap_h; // handle for reception of the netlink traps.

#if (PAED_MSSID == 1)
/* beacon IE flag initialized here */
extern int beacon_ie_set;

/* the config array pointer */
extern struct config_t *config;
#else

extern struct config_t config[MAX_INTERFACES];  // And everybody uses this too
#endif  /* #if (PAED_MSSID == 1) */

// globals

char frame[1514];       // This is a buffer used for all actions where a packet or frame is needed
char hostname[32]="";   // This is used as NAS-identifier in radius...
int nof_interfaces=0;


struct gtkmachine_t gtk[MAX_INTERFACES];
int no_mic_errors = 0;


// proto's

int paed_lock(char *fname);
void paed_touch(char *fname);
int mainselectloop(void);
int check_states(struct client_t *client);
void usage(char **argv);

//extern protos


int main(int argc, char **argv)
{
        int c;
        int loglevel = LG_WRN;
//        int logto = LG_SYSLOG;
	int logto = LG_STDERR;
        int logcolor = 0;
        struct config_t *configp = NULL;
        int i;
#if (PAED_MSSID == 1)
    int r=0;
    obj_serviceset serviceset;

    /* IE not captured from the beacon */
    beacon_ie_set = 0;

#endif 


#ifndef LINUX
        set_malloc_mmap_size(16000);
#endif


        gethostname(hostname, 32);

        while((c = getopt(argc, argv, "acl:dn:su")) != EOF) {
                switch(c) {
                        case 'c':
                                logcolor=1;
                                break;
                        case 'l':
                                loglevel = atoi(optarg);
                                break;
                        case 'd':
                                dump_bindata_enabled = 1;
                                break;
                        case 'n':
                                strncpy(hostname, optarg, sizeof(hostname));
                                break;
                        case 's':
                                logto = LG_STDERR;
                                break;
                        case 'm':
                                no_mic_errors=1;
                break;
                        default :
                                usage(argv);
                                break;
                }
        }

        log_init(logto, loglevel, logcolor);
        paed_lock(PATH_LOCKFILE);
#ifndef LINUX
        paed_touch(PATH_RESPAWND_FILE);
#endif
        log(LG_INF, "%s %s started on %s", PROGNAME, VERSION, hostname);
	
        // open a socket to listen for EAP packets
        eapol_opensocket();

        /* Open netlink for prismoids and prismtraps */
        if (open_netlink(&blobtrap_h) == 0)
        {
            log(LG_FTL, "Could not open netlink interface");
        }
#if (RAC_SUPPORT == RAC_TRUE)
	        log(LG_DBG, "\n\n\nRAC_SUPPORT == RAC_TRUE\n\n\n");
		        log(LG_INF, "LG_INF:RAC_SUPPORT == RAC_TRUE");
#endif/*(RAC_SUPPORT == RAC_TRUE)*/


#if (PAED_MSSID == 1)

		/* make list of names of all physical wireless interfaces */
		mssid_make_wireless_iface_list();

        fprintf(stderr, "mssid_print: opened netlink\n");

        log(LG_INF, "MSSID: Before doing config_read");
#endif

        /* Read config for all possible interfaces and return amount of configurable interfaces */
        nof_interfaces = config_read();

        log(LG_INF, "MSSID: After doing config_read");

        /* Read the ACL once */
        acl_read_file(PATH_ACL_CONF);

#if (PAED_MSSID == 1)

		/* Reset SoftMAC for all wireless interfaces */
	    mssid_reset_wireless_interfaces();

        /* Clean softmac configuration. 
         * Configure service sets with NULL for all VSSIDs 
         */
		mssid_init_all_service_sets();

#endif /* #if (PAED_MSSID == 1) */

        for ( i = 0 ; i < nof_interfaces; i++ )
        {
            	/* Set config pointer */
            	configp = &config[i];

#if (PAED_MSSID == 1)

        	/* Create the VSS in SoftMAC. Config struct contains the vssid */
        	mssid_make_service_set(configp, &serviceset); 
        	blob_mssid_configure_service_set(configp, &serviceset);

                /* If we have WEP, need to configure the WEP key
                 * in the service set for Brodcast
                 */
                if (1 == configp->enableWEP){
                   mssid_configure_service_set_keys(configp);
                }
#endif

            // bind the interface to the EAP socket
            eapol_bindtosocket( configp );

            /* unauthorize the port by default */
            blob_portstatus_set_global(UNAUTHORIZED, configp);

#if (PAED_MSSID == 1)
	/* No global configuration setting for MSSID */
#else


#ifdef LINUX
#ifdef SMAC
	    /* set SSID from the config file setting */
	    blob_set_ssid( config );
#endif
#endif

            if(configp->enableOS == 0)
            {
#ifdef SMAC
                int policies;

                blob_get_mlme_policies( &policies, configp->ifname );                
                policies |= DOT11_MLMEPOLICY_PRIVACYCAPABILITY;
                blob_set_mlme_policies( policies, configp->ifname );
#endif                
		blob_set_privacy_invoked(1, configp);
	    } 
            else 
            {
#ifdef SMAC
                int policies;

                blob_get_mlme_policies( &policies, configp->ifname );                
                policies &= ~DOT11_MLMEPOLICY_PRIVACYCAPABILITY;
                blob_set_mlme_policies( policies, configp->ifname );
#endif                
		blob_set_privacy_invoked(0, configp);
		log(LG_WRN, "Privacy invoked is NOT set because open system enabled");
	    }

            if( configp->WEPexunencrypt )
                blob_set_exclude_unencrypted(1, configp);
            else
                blob_set_exclude_unencrypted(0, configp);

#endif /* PAED_MSSID */


            if (configp->enableSSNPSK)
            {
                keystuff_setup_key_material(configp);
            }

#if (PAED_MSSID == 1)
		/* No IE processing for MSSID */
#else
            ie_set(configp);
#endif

            if ( (configp->enableSSN || configp->enableSSNPSK || configp->enable1X) &&
                 !configp->enableWEP ) {
                    gtk_initialize(configp);
                    log(LG_DBG, "terug van gtk_init\n");
                    gtk_check_states(configp);
            }

#ifdef SMAC
            /* For SoftMAC architecture MLME is always in extended mode */
#else
#ifdef BLOB_V1
            if (blob_set_mlme_mode(DOT11_MLME_ADVANCED, configp) == -1)
#endif
#ifdef BLOB_V2
            if (blob_set_mlme_mode(DOT11_MLME_EXTENDED, configp) == -1)
#endif
	    {
		log(LG_FTL, "Could not set MVC in MLME advanced mode");
	    }
#endif /* SMAC */	    
	    blob_deauthenticate_all(configp);

            init_bcast_key(configp);

#ifdef SMAC

#if (PAED_MSSID == 1)
	/* In case of MSSID, dont do a commit per VSS interface */
#else
            blob_commit_settings(configp);

#endif /*#if (PAED_MSSID == 1)*/

#endif

      } /* end for */
   
#if (PAED_MSSID == 1)

	 /* Commit settings for physical wireless interfaces */
	mssid_commit_wireless_interfaces();

#endif /* #if (PAED_MSSID == 1) */

            /* Service sets created. Now configure the bridge */
          log(LG_INF, "MSSID: Service sets created. Configure Bridge") 
          mssid_configure_bridge(config);

          log(LG_INF, "MSSID: Bridge configured") 
#if (RAC_SUPPORT == RAC_TRUE)
	log(LG_DBG, "RAC_SUPPORT == RAC_TRUE");
	log(LG_INF, "LG_INF:RAC_SUPPORT == RAC_TRUE");
#endif/*(RAC_SUPPORT == RAC_TRUE)*/
        mainselectloop();

    close_netlink(&blobtrap_h);

        return(0);
}


/*
 * Check if the lockfile exists and ignore if stale. Create a
 * new lock
 */
void paed_touch(char *filename)
{
        pid_t pid, pid2;
        char *cmd[4];
        char buffer[128];

        /* The file to start */
        sprintf(buffer, "%s", filename);
        cmd[0] = "/bin/touch";
        cmd[1] = buffer;
        cmd[2] = NULL;
        log(LG_INF, "touch-ing %s", filename);

        /* Do a double vfork to make the respawned process a child of init */
        if ( (pid = vfork()) == 0 )
        {
                if ( ( pid2 = vfork()) == 0)
                {
                        execv(cmd[0], cmd);
                        /* Something went wrong */
                        exit(-1);
                }
                exit(0);
        } else {
                /* wait until child 1 ends */
                waitpid(pid, NULL, 0);
        }
}


int paed_lock(char *fname)
{
        FILE *f;
        int pid;
        char buf[32];

        // Check if lock exists

        f = fopen(fname, "r");

        if(f != NULL) {

                // if exists, get pid from file and check if this pid
                // is currently running.

                fscanf(f, "%d", &pid);

                fclose(f);

                log(LG_DBG, "lockfile %s found, pid=%d", fname, pid);

                sprintf(buf, "/proc/%d/status", pid);

                f = fopen(buf, "r");

                if(f != NULL) {
                        log(LG_FTL, "lockfile suggest %s is already running, aborting", PROGNAME);
                } else {
                        log(LG_INF, "lockfile is stale, overriding");
                }
        }

        // Create lockfile

        f = fopen(fname, "w");

        if(f == NULL) {
                log(LG_FTL, "Unable to create lockfile %s : %s", fname, strerror(errno));
        }

        fprintf(f, "%d\n", getpid());
        fclose(f);

        log(LG_DBG, "Created lockfile %s", fname);

        return(0);
}




/*
 * basic help
 */

void usage(char **argv)
{       char *p = argv[0];

        while(strchr(p, '/')) {
                p=strchr(p, '/')+1;
        }

        printf("\n");
        printf("\n");
        printf("Usage: %s [-l level] [-d]\n", p);
        printf("\n");
        printf("%s %s is the Intersil 802.1x EAP (Port Access Entity) daemon\n", PROGNAME, VERSION);
#ifdef LINUX
#ifdef SMAC
	printf("prismoids library version %s\n", prismoids_version() );
#endif
#endif
        printf("\n");
        printf("Options:\n");
        printf("\n");
        printf("  -l        : Log facilitie mask. If omitted everything is shown\n");
        printf("  -l level  : Set logging level\n");
        printf("  -d        : Dump all received and generated packets\n");
        printf("  -m        : Disable MIC countermeasures\n");
        printf("  -n name   : Use name as radius NAS-identifier\n");
        printf("  -s        : Log to stderr instead of syslog\n");
        printf("\n");

        exit(0);
}




/*
 * Do a select on the eapol socket and all active radius sockets. Call the
 * appropriate function for handling the incoming data
 */

int mainselectloop(void)
{
        struct client_t *client, *nextclient;
        fd_set fdset;
        struct timeval tv;
        int maxfd;
        int r, i;
        struct config_t *configp = NULL;

        while(43) {
                tv.tv_sec = 1;
                tv.tv_usec = 0;
                maxfd = 0;
#if (RAC_IT)
		// log(LG_DBG, "I am in mainselectloop");
#endif
	    	FD_ZERO(&fdset);

                FD_SET(eapolfd, &fdset);
                maxfd = max(maxfd, eapolfd);
                /* Incoming blob traps */
                FD_SET(blobtrap_h.fd, &fdset);
                maxfd = max(maxfd, blobtrap_h.fd);

                client = clientlist;

                while(client) {
                        if(client->radiusfd != -1) {
                                FD_SET(client->radiusfd, &fdset);
                                maxfd = max(maxfd, client->radiusfd);
                        }

#if (RAC_SUPPORT == RAC_TRUE)

			
			//log(LG_DBG, "setting  RAC_FD in fdset");
			if (client->rac.rac_fd != -1){
				FD_SET(client->rac.rac_fd, &fdset);
			        maxfd = max(maxfd, client->rac.rac_fd);
				//log(LG_DBG, "COULD SET RAC FD");
			}
			

#endif
							
                        client = client->next;
                }

                r = select(maxfd+1, &fdset, NULL, NULL, &tv);

                if(r>=0) {
//                        printf( "r=%08x blob_fd=%d eapol_fd=%d\n", r, blobtrap_h.fd, eapolfd );
//                        printf( "fdset.fds_bits[blob_fd]=%08x fdset.fds_bits[eapol_fd]=%08x\n",
//                                fdset.__fds_bits[blobtrap_h.fd >> 5], fdset.__fds_bits[eapolfd >> 5] );

                        // Check for MVC traps.
                        if (FD_ISSET(blobtrap_h.fd, &fdset))
                        {
                            handle_traps(&blobtrap_h);
                        }

                        // Check for incoming EAP data
                        if(FD_ISSET(eapolfd, &fdset)) {
                                eapol_rx(eapolfd);
                        }

                        // check for various possible timeouts in the
                        // pae- and backend-statemachines.

                        client = clientlist;

                        while(client) {

                                client_chksan(client);

                                if(client->radiusfd != -1) {
                                        if(FD_ISSET(client->radiusfd, &fdset)) {
                                                radius_rx(client);
                                        }
                                }

#if (RAC_SUPPORT == RAC_TRUE)
				if (1 == rac_status){
					/* check for any response from the radius 
					 * accounting server   
					 */

					// log(LG_DBG, "CHECKING RAC_FD");
			
							
					if (client->rac.rac_fd != -1) {
						// log(LG_DBG, "client->rac.rac_fd != -1");
						if (FD_ISSET(client-> rac.rac_fd,&fdset)){
							log(LG_DBG, "calling rac_rx");
							
							rac_rx(client);
						}
					}
				}
#endif
				
				// log(LG_DBG, "going to check states");

                                check_states(client);

                                client_chksan(client);

                                nextclient = client->next;

                                /* Remove client if necessary */
                                client_toberemoved(client);

                                client = nextclient;

                        }

                        for ( i = 0 ; i < nof_interfaces; i++ )
                        {
                            /* Set config pointer */
                            configp = &config[i];

                            gtk_check_states(configp);
                        }
                }

        }

        return(0);

}


/*
 * Check state transitions for this client, including timeouts etc.
 * mostly conform the IEEE draft, so look at fig 8-8 and fig-8-12 for
 * explanation
 */

int check_states(struct client_t *client)
{
#if (RAC_SUPPORT == RAC_TRUE)
	if (RAC_STATUS_ENABLE == rac_status){

		/* 
		 * If radius accounting state is idle and no NULL response  
		 * received in timeout period, set terminate 
		 * cause to  connection loss and set state to stopped
		 */
		if (RAC_STATE_IDLE == client->rac.acct_state)
		{
			if (time(NULL) > client->rac.idle_null_resp_timeout)
			{
				client->rac.acct_event = RAC_EVENT_SERVICE_STOP;                                      
				if(blob_get_client_age(client)> client->cconfig->idleTimeout )
				{
					client->rac.terminate_cause = RAC_TERMINATE_LOST_CARRIER;
				}
				else
				{
					client->rac.terminate_cause =RAC_TERMINATE_IDLE_TIMEOUT;
				}
			}

			/* 
			 * If we are waiting for resp to NULL req, no need to
			 *   check further states.
			 */

			return 0;
		}
	}
#endif

        // If this client is idle too long, mark it for removal

        client->idleTime = blob_get_client_age(client);

        if((client->idleTime == -1) || (client->idleTime > client->cconfig->idleTimeout)) {
                log(LG_INF, "Client %s to be removed from list (idleTime=%d)",
                        hwaddr2string(client->hwaddr), client->idleTime);
#if (RAC_SUPPORT == RAC_TRUE)
		if (RAC_STATUS_ENABLE == rac_status)
		{
			client->rac.acct_event = RAC_EVENT_IDLE_TIMEOUT;

		}
#endif     
	
		client->toBeRemoved = 1;
                return(0);
        }

        if(client->does1X) {
                pae_check_states(client);
        }

        if(client->doesSSN) {
                ssn_check_states(client);
	}
#if (RAC_SUPPORT == RAC_TRUE)
	if (RAC_STATUS_ENABLE == rac_status){
		/* Check for RADIUS Accounting states and events */
		rac_check_states(client); 
	}
#endif     


        return(0);
}

inline int paed_get_nof_interfaces(void)
{
     return nof_interfaces;
}


