/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 * Intersil ® is a registered trademark of Intersil Americas Inc.
 * Copyright © 2002 Intersil Americas Inc. All Rights Reserved.
 *
 */


 /*
  ** <FILE_NAME> rac.c
  ** <FILE_PURPOSE> File contains the functions (including State Machine)
  **  required to develope RADIUS Accounting (RAC) feature in WISOC
  */


/*
 ** System Include Files 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>
#include <time.h>

/*
 **   Component/module Include Files
 */

#include "paed.h"
#include "misc.h"
#include "client.h"
#include "radius.h"
#include "radius_pkt.h"
#include "config.h"
#include "hmac_prf.h"
#include "md5.h"
#include "keystuff.h"
#include "log.h"
#if (!RAC_UT)

#include "netlink_func.h"

#endif /*(!RAC_UT)*/

#include "rac.h"


/*
 * global variables
 */

char frame_rac[RAC_MSG_LEN+MAX_RADIUS_SECRET_LEN];
char frame_rac_rx[RAC_RX_MSG_LEN+MAX_RADIUS_SECRET_LEN];

int acct_status_type = 0;
/*
 * exernal variables
 */
extern struct radius_t radius;
#if (!RAC_UT)
extern in_addr_t ifinaddr;
#else
extern int ifinaddr_rac_ut;
#endif(!RAC_UT)

extern char rac_nas_id[32];
extern char hostname[32];
#if (RAC_UT)
extern int rac_interim_timeout;
extern int rac_resp_timeout;
#endif /*(RAC_UT)*/

/*
 * * extern functions declaration
 */
extern int association_id ;
extern int rac_pkt_start(char *pdu, int code, int ident, char *vector);
//extern int rac_pkt_finish(char *pdu);
extern int rac_pkt_finish(char *pdu, char *secret, int secret_len, char *vector);
extern int rac_pkt_verify(char *pdu, char *secret, int secret_len, char *vector);
#if (!RAC_UT)

extern long blob_get_input_octet_count(struct client_t *client);
extern long blob_get_output_octet_count(struct client_t *client);
extern long blob_get_output_acct_packet_count(struct client_t *client);
extern long blob_get_input_acct_packet_count(struct client_t *client);
extern int blob_send_null_data_request(struct obj_mlme *mlmeex, struct config_t *config);

#endif /*(!RAC_UT)*/

/* 
 * function declaration for rac.c
 */
enum rac_ret_t rac_open(struct client_t *client);
int rac_rx(struct client_t *client);
int rac_check_states(struct client_t *client);
enum rac_ret_t rac_tx(struct client_t *client);
int rac_retransmit(struct client_t *client);
void rac_set_idle( struct client_t *client);
int rac_close(struct client_t *client);
char *make_session_id(unsigned char *a, unsigned char *b);
char *make_called_station_id(unsigned char *a, struct client_t *client);
char *make_calling_station_id(unsigned char *a);
/*
 ** Global Function Definitions
 */

/**
 ** <FN_NAME> rac_open
 ** <BRIEF_DESC> to open a new socket for RAC to send a acct pkt for a client
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN 
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>enum rac_ret_t
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from rac_tx function in rac.c file 
 ** </CONTEXT>
 */




enum rac_ret_t rac_open(struct client_t *client)
	{
		struct sockaddr_in *sin;
		int len;
		int ret_val;

		log(LG_DBG, "RADIUS ACCT: Opening radius socket");


#if (!RAC_UT)
		client->rac.rac_fd = socket(AF_INET, SOCK_DGRAM, 0);

		if(client->rac.rac_fd < 0) {
			log(LG_ERR, " RADIUS ACCT: error while opening radius accounting 
                	socket  '%s'", strerror(errno));
			client->rac.rac_fd = RAC_INVALID_SOCK ;
			return RAC_RET_FAIL;
		}

		len = sizeof(client->rac.rac_salocal);

		sin = (struct sockaddr_in *)&client->rac.rac_salocal;
		memset((char *)sin, 0, len);
		sin->sin_family = AF_INET;
		sin->sin_addr.s_addr = htonl(INADDR_ANY);
		sin->sin_port = htons(0);
		ret_val = bind(client->rac.rac_fd, (struct sockaddr *)sin, 
                      len);

		if(ret_val < 0) {
			log(LG_ERR, "RADIUS ACCT: bind error '%s'", strerror(errno));
			client->rac.rac_fd = RAC_INVALID_SOCK;
			return RAC_RET_FAIL;
		}

		log(LG_DBG, "RADIUS ACCT: Current radius accounting server : %s:%d", (
			inet_ntoa(radius.current_rac_serv->address)),
			radius.current_rac_serv->acct_port);

		sin = (struct sockaddr_in *)&client->rac.rac_saremote;
		memset((char *)sin, 0, len);
		sin->sin_family = AF_INET;
		sin->sin_addr.s_addr = radius.current_rac_serv->address;
		sin->sin_port = htons(radius.current_rac_serv->acct_port);

		len = radius.current_rac_serv->secret_len;
		if (len > MAX_RADIUS_SECRET_LEN){ 
         		len=MAX_RADIUS_SECRET_LEN;
     		}
		memcpy(client->rac.rac_secret,radius.current_rac_serv->secret, len);

		client->rac.rac_secret_len = len;
#else
		client->rac.rac_fd =6 ;
#endif /*(!RAC_UT)*/
		return RAC_RET_SUCCESS;
	}
/**
 *  ** <FN_NAME> rac_rx
 ** <BRIEF_DESC> to raceive response packet from RADIUS server for RAC 
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>int 
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from mainselectloop function in file paed.c
 ** </CONTEXT>
 */


int rac_rx(struct client_t *client)
{

	int n;
        struct sockaddr from;
        int fromlen;
        struct radius_hdr_t *hdr = (struct radius_hdr_t *)frame_rac_rx;
      
      
        fromlen = sizeof(from);
        n = recvfrom(client->rac.rac_fd , frame_rac_rx, sizeof(frame_rac_rx), 0, &from, &fromlen);
        log(LG_DBG, "Received packet in function rac_rx: ");
	
	if (n == -1) {
                log(LG_DBG, "recvfrom");
                return(-1);
        }

        /*have to check*/
        if (ntohs(hdr->len) > RAC_MSG_LEN ){
                log(LG_DBG, "Received packet too big (%d)", ntohs(hdr->len));
                return(-1);
        }
        
        log(LG_DBG, "RADIUS ID              %d", hdr->ident);

	if(hdr->ident == client->rac.rac_id){
		log(LG_DBG, "RADIUS ACCT: expected radius accounting reply, 
                                           increasing rac_id");
		client->rac.rac_id = (client->rac.rac_id + 1) % 256;
	}
   
 	if (RAC_MSG_TYPE_RESPONSE != hdr->code) 
 		return -1;

	/*
	// For verification of the packet, copy the original client's
        // radius vector in place of the received radius vector. Verify the
        // packet and if it fails, drop it.

        have to check

	*/
        /*memcpy(hdr->auth, client->rac.rac_vector, 16);*/
 	if (0 != rac_pkt_verify(frame_rac_rx, client->rac.rac_secret,
			       	   client->rac.rac_secret_len, client->rac.rac_vector))
 		return -1;

  	switch(client->rac.acct_state){
 		case RAC_STATE_START_SENT:  
			client->rac.acct_event =RAC_EVENT_START_RESPONSE_RECVD ;
			log(LG_DBG, "Got Acct Resp:RAC_EVENT_START_RESPONSE_RECVD");		
			break;
  
 		case RAC_STATE_INTERIM_SENT:        
             		client->rac.acct_event =RAC_EVENT_INTERIM_RESPONSE_RECVD;
			log(LG_DBG, "Got Acct Resp:RAC_EVENT_INTERIM_RESPONSE_RECVD");
             		break;

 		case RAC_STATE_STOP_SENT:        
             		client->rac.acct_event = RAC_EVENT_STOP_RESPONSE_RECVD;
			log(LG_DBG, "Got Acct Resp:RAC_EVENT_STOP_RESPONSE_RECVD");
             		break;

 		default:
			log(LG_DBG, "RADIUS ACCT: Got Acct Resp while in invalid state 
                                      %d", client->rac.acct_state);
           		break;
  	}
  	return RAC_RET_SUCCESS;
}



/**
 ** <FN_NAME> rac_tx
 ** <BRIEF_DESC> to send accounting packet to RADIUS server for a client in RAC
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>enum rac_ret_t
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from rac_check_states function in rac.c file
 ** </CONTEXT>
 */


enum rac_ret_t rac_tx(struct client_t *client)
{

	long input_octet_count=0;
	long output_octet_count=0;
	long input_acct_packet=0; 
	long output_acct_packet=0 ; 
	int session_time=0; 
	int ret_val;
	int len;	
	char *called_station_id;
	int  called_station_id_len ;
	char p[16];
#if (RAC_IT)
	char s[]="1148";
#endif

#if (!RAC_UT)
	if (ifinaddr == -1) ifinaddr = if_get_ipaddr(IF_NAS);
#else
	int i; /*LOOP INDEX*/
#endif /*(!RAC_UT)*/

	if (RAC_EVENT_SERVICE_START == client->rac.acct_event)
	{
     		
		/*
		 * Check whether NAS IP address has been assigned or not
		 */
#if (!RAC_UT)
   		if (ifinaddr == -1) 
#else
		if(ifinaddr_rac_ut == -1)
#endif /*(!RAC_UT)*/		
          		return RAC_RET_START_PENDING;
      		
              	
   	}
	
#if (RAC_UT)
	printf("\n TRANSMITTING PACKET \n");
#endif /*(RAC_UT)*/



	client->rac.rac_id = (client->rac.rac_id+1) % 256;
#if (RAC_UT)
        printf("\n rac_id %d \n", client->rac.rac_id );
#endif /*(RAC_UT)*/

	//random_vector(client->rac.rac_vector, 16);
	
#if (RAC_UT)
        printf("\n random_vector %s \n", client->rac.rac_vector );
#endif /*(RAC_UT)*/
	
	bzero(p, 16);

	/*rac_pkt_start(frame_rac, RAC_MSG_TYPE_REQUEST,
		      client->rac.rac_id,client->rac.rac_vector);*/
	rac_pkt_start(frame_rac, RAC_MSG_TYPE_REQUEST,
		client->rac.rac_id,p);
	
	/*
	 *  Add attributes to radius packet
 	 */

	
 	/*
	 *  The username of user who is logging  on
	 */
        radius_pkt_add_str(frame_rac, RA_USERNAME, client->identity, strlen(client->identity));
        log(LG_DBG, "RA_USERNAME            %s", client->identity);

	/*
	 *  The NAS IP address
	 */
	
//#if (!RAC_IT)
	 radius_pkt_add_addr(frame_rac, RA_NAS_IP_ADDRESS, ifinaddr);
	 log(LG_DBG, "RA_NAS_IP_ADDRESS      %s", inet_ntoa(ifinaddr));

	/*
         *  The NAS PORT
         */

       	radius_pkt_add_int(frame_rac, RA_NAS_PORT, association_id);
        log(LG_DBG, "RA_NAS_PORT            %d", association_id);
		 
		

		
	/*
	 * Add Framed-MTU attribute with value RA_ATTRIB_FRAMED_MTU
	 * Framed mtu. Fixed to 1488 
	 */
	
	radius_pkt_add_int(frame_rac, RA_FRAMED_MTU, 1488);
        log(LG_DBG, "RA_FRAMED_MTU          1488");


	/*
	 * Add Called station ID : MAC address of Acces point 
	 */
	called_station_id_len = (17 + client->cconfig->ssidlength+1);
	called_station_id = make_called_station_id(client->cconfig->ifhwaddr,client);
        
	log(LG_DBG, "called_station_id          %s",called_station_id );
	radius_pkt_add_str(frame_rac, RA_CALLED_STATION_ID,
	               called_station_id,  called_station_id_len);
	
	
	
	log(LG_DBG, "RA_FRAMED_MTU          %s",called_station_id );
	
	/*
	radius_pkt_add_str(frame_rac, RA_CALLED_STATION_ID,
		           hwaddr2string(client->cconfig->ifhwaddr), 17); 

	*/
#if (RAC_UT)
        printf("\n RA_CALLED_STATION_ID %s \n", called_station_id );
#endif /*(RAC_UT)*/



	/*
	 * Add Calling station ID : MAC address of client
	 */
	radius_pkt_add_str(frame_rac, RA_CALLING_STATION_ID,
			   make_calling_station_id(client->hwaddr), 17);
	


	log(LG_DBG, "RA_FRAMED_MTU          %s",make_calling_station_id(client->hwaddr) );
#if (RAC_UT)
        printf("\n RA_CALLING_STATION_ID %s \n", make_calling_station_id(client->hwaddr) );
#endif /*(RAC_UT)*/


	/* 
	  * Add NAS port type : 19 = 820.11 wireless
	  */
	radius_pkt_add_int(frame_rac, RA_NAS_PORT_TYPE,RA_WIRELESS_PORT_TYPE);

#if (RAC_UT)
        printf("\n RA_NAS_PORT_TYPE %d \n", RA_WIRELESS_PORT_TYPE);
#endif /*(RAC_UT)*/


//#endif /*(!RAC_IT)*/

	/*
	 * Add session id from client->rac.session_id
	 */
	
	
	//radius_pkt_add_str(frame_rac, RA_ACCT_SESSION_ID, s, 4);


	 radius_pkt_add_str(frame_rac, RA_ACCT_SESSION_ID, client->rac.session_id, 59);
         printf("\n IN FUNCTION RAC _TX :RA_ACCT_SESSION_ID \n" );
        printf("%s\n",client->rac.session_id);
	
	
	/*
	 * Add class Attribute if class length is not zero
	 */
	if (0 != client->rac.class_len)
    		radius_pkt_add_str(frame_rac, RA_CLASS, client->rac.class,
				   client->rac.class_len);

#if (RAC_UT)
        printf("\n RA_CLASS %s \n",client->rac.class );
#endif /*(RAC_UT)*/


	/*
	 * Add SESSION TIMEOUT Attribute from client->cconfig->reAuthPeriod
	 */
	radius_pkt_add_int(frame_rac, RA_SESSION_TIMEOUT, client->cconfig->reAuthPeriod);

	/*
	 * Add IDLE TIMEOUT Attribute from client->cconfig->idleTimeout
	 * */
	radius_pkt_add_int(frame_rac, RA_IDLE_TIMEOUT, client->cconfig->idleTimeout);

	/*
	 * Add Acct-Status-Type Attribute from client->rac.acct_event for 
	 * START, STOP, INTERIM
	 */


	radius_pkt_add_int(frame_rac, RA_ACCT_STATUS_TYPE, acct_status_type );
	log(LG_DBG, "client->rac.acct_event          %d", client->rac.acct_event);
	/*
	 * Add NAS Identifier attribute from nas-identifier.
	 * If nas-identifier configured then copy attribute 
	 * from rac_nas_id else from hostname.
	 */

	if(strlen(rac_nas_id)>0){
		radius_pkt_add_str (frame_rac, RA_NAS_IDENTIFIER, 
				    rac_nas_id, strlen(rac_nas_id));
		log(LG_DBG, "RA_NAS_IDENTIFIER      %s", rac_nas_id);
	}
	else {
	    if(strlen(hostname)>0) {
		radius_pkt_add_str (frame_rac, RA_NAS_IDENTIFIER, hostname, strlen(hostname));
		log(LG_DBG, "RA_NAS_IDENTIFIER      %s", hostname);
	    }
	}

	/* Add INTERIM_INTERVAL  attribute from client->rac.interim_interval 
	 * If configured value not equal to zero and there is INTERIM_INTERVAL
	 * attribute field in the access accept 
	 */
	if (0 != client->rac.interim_interval) 
    	    radius_pkt_add_int(frame_rac, RA_INTERIM_INTERVAL, client->rac.interim_interval);
	
	/*
	 * Add authentication :whether local or RADIUS
	 */
	radius_pkt_add_int(frame_rac, RA_ACCT_AUTH, client->rac.acct_auth);

	/*
	 *  Add current timestamp
	 */
	radius_pkt_add_int(frame_rac, RA_EVENT_TIMESTAMP, time(NULL));

	 if ((RAC_EVENT_SERVICE_STOP == client->rac.acct_event)||
             (RAC_EVENT_INTERIM == client->rac.acct_event)){

    		/*
		 * Get Input octet count for the client and add RA_ACCT_INPUT_OCTET
		 */
	
#if (!RAC_UT)


		input_octet_count = blob_get_input_octet_count( client);
    		radius_pkt_add_int(frame_rac, RA_ACCT_INPUT_OCTET, input_octet_count);

    		/*
		 * Get Output octet count for the client and add RA_ACCT_OUTPUT_OCTET
		 */
		output_octet_count =blob_get_output_octet_count( client);
    		radius_pkt_add_int(frame_rac, RA_ACCT_OUTPUT_OCTET, output_octet_count);


    		/*
		 * Get Input packet count for the client and add RA_ACCT_INPUT_PACKET
		 */
    		input_acct_packet = blob_get_input_acct_packet_count(client);
    		radius_pkt_add_int(frame_rac, RA_ACCT_INPUT_PACKET, input_acct_packet);



    		/*
		 * Get Output packet count for the client and add RA_ACCT_OUTPUT_PACKET
		 */
    		output_acct_packet  =blob_get_output_acct_packet_count(client);
    		radius_pkt_add_int (frame_rac, RA_ACCT_OUTPUT_PACKET, output_acct_packet );


#endif /*(!RAC_UT)*/
    		
    		/*
		 * session time = time(NULL) - client->rac.start_time
		 * Add RA_SESSION_TIME attribute with value session time
		 */
		session_time = time(NULL) - client->rac.start_time;
    		radius_pkt_add_int(frame_rac,RA_ACCT_SESSION_TIME,session_time);
#if (RAC_UT)
        printf("\n RAC_TX :RA_ACCT_SESSION_TIME %d \n", session_time );
#endif /*(RAC_UT)*/



		if (RAC_EVENT_SERVICE_STOP == client->rac.acct_event){
    		    /*If event type is stop then add attribute RA_TERMINATE_CAUSE 
		     * with value client->rac.acc_attrib.terminate_cause
		     */
		    radius_pkt_add_int(frame_rac, RA_TERMINATE_CAUSE,
				       client->rac.terminate_cause);
		
#if (RAC_UT)
        printf("\n RAC_TX :RA_TERMINATE_CAUSE %d \n", client->rac.terminate_cause );
#endif /*(RAC_UT)*/
		}


	}

	
	
	/*
	 * TO CHECK 
	len = radius_pkt_finish(frame_rac, client->rac.rac_secret, 
			    	client->rac.rac_secret_len);
	*/
	len = rac_pkt_finish(frame_rac, radius.current_rac_serv->secret,
		              radius.current_rac_serv->secret_len, client->rac.rac_vector);	
	client->rac.tx_count++ ;
#if (RAC_UT)
	printf("\nFRAME LENGTH %d\n ", len);
	printf("\nFRAME SENT\n ");
	for(i=0 ; i <len ;i++){
		printf("%c ", frame_rac[i]);
	}
	printf("\nFRAME END\n ");

#endif /*(RAC_UT)*/
	 if(client->rac.rac_fd == -1){
		rac_open(client);
	}
	printf("\nclient->rac.rac_fd %d\n ",client->rac.rac_fd );
	
	ret_val = sendto(client->rac.rac_fd, frame_rac, len, 0,
		         (struct sockaddr *) &client->rac.rac_saremote,
		         sizeof (struct sockaddr_in));

	 return RAC_RET_SUCCESS;

}


/**
 ** <FN_NAME> rac_retransmit
 ** <BRIEF_DESC> to retransmit  accounting packet to RADIUS server for a client in RAC
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>int
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from rac_check_states function in rac.c file
 ** </CONTEXT>
 */



int rac_retransmit(struct client_t *client)
{
   	if (client->rac.tx_count >RAC_MAX_RETRIES) {

   		rac_close(client);       
#if (!RAC_UT)
		radius.current_rac_serv = radius.current_rac_serv->next;
	      	if (radius.current_rac_serv == NULL){
            		radius.current_rac_serv = radius.radius_serv_list;
           		log(LG_DBG, 	"RADIUS ACCT: Timeout connecting Radius accounting, 
                 			servers. Using first again");
         	}
#endif /*(!RAC_UT)*/

      		client->rac.tx_count= 0;
      }

   	rac_tx(client);
  	client->rac.resp_timeout = time(NULL) + RAC_RETRY_TIMEOUT;
   	return RAC_RET_SUCCESS;

}


/**
 ** <FN_NAME> rac_set_idle
 ** <BRIEF_DESC> when  Client is idle. Send null data request to verify
             * whether this is a case of connection loss. Set state to idle
	     * 
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>void
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from rac_check_states function in  file rac.c
 ** </CONTEXT>
 */
void rac_set_idle( struct client_t *client)
{
        
#if (!RAC_UT)
#ifdef SMAC
	struct obj_mlme* mlmeex_rac;
#endif
#endif /* (!RAC_UT) */
	client->rac.acct_state= RAC_STATE_IDLE;
#if (!RAC_UT)
	/*
	 * *Send NULL data request
	 */
#ifdef SMAC	
	memset (&mlmeex_rac, 0x00, sizeof(mlmeex_rac));
	memcpy( mlmeex_rac->address, client->hwaddr, 6 );
	//blob_send_null_data_request( mlmeex_rac, client->cconfig);
#endif//SMAC
#endif /* (!RAC_UT) */
        client->rac.idle_null_resp_timeout =  time(NULL) + RAC_IDLE_NULL_RESP_TIMEOUT;
}


/**
 ** <FN_NAME> rac_close
 ** <BRIEF_DESC> closes the socket after use
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>int
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from rac_retransmit function  in file rac.c
 ** </CONTEXT>
 */


int rac_close(struct client_t *client)
{
        if(client->rac.rac_fd != -1) {
                log(LG_DBG, "Closing radius socket");
                close(client->rac.rac_fd);
                client->rac.rac_fd = -1;
		printf("\nclient->rac.rac_fd=%d\n",client->rac.rac_fd);

        }

        return(0);
}


/**
 ** <FN_NAME> rac_check_states
 ** <BRIEF_DESC> RAC State machine
 **
 ** <ARGUMENTS>
 ** <ARG_NAME> struct client_t *client
 ** <ARG_TYPE> IN
 ** </ARGUMENTS>
 **
 ** <RETURN_VALUES>
 ** <RET_VAL>int
 ** </RETURN_VALUES>
 **
 ** <CONTEXT>
 ** invoked from check_states sunction in paed.c
 ** </CONTEXT>
 */



int rac_check_states(struct client_t *client)
{
	enum rac_ret_t return_val ;
	char *p;
//	log(LG_DBG, "ENTERING FUNCTION CHECK STATES ");
	/* 
	 * *Check for RADIUS Accounting states, and events and timeouts 
	 */
	switch (client->rac.acct_state){
		case RAC_STATE_STOPPED:

			log(LG_DBG, "RAC_STATE_STOPPED");
			if (client->rac.acct_event == RAC_EVENT_SERVICE_START){

				client->rac.start_time = time(NULL);

				/*
				 *  Create Accounting session id and store in 
				 *  Client->rac.session_id
				 */
				p = make_session_id(client->cconfig->ifhwaddr ,client->hwaddr);
				memcpy(client->rac.session_id, p ,128);
				acct_status_type = 1;
				return_val = rac_tx(client);
				if (return_val ==  RAC_RET_START_PENDING )
					client->rac.acct_state= RAC_STATE_START_PENDING;
				else {
					client->rac.acct_state= RAC_STATE_START_SENT;
					client->rac.tx_count = 1;
					client->rac.acct_event = RAC_EVENT_NONE;
				}
			}
			break;

		case RAC_STATE_START_PENDING:

			log(LG_DBG, "RAC_STATE_START_PENDING");
			if (client->rac.acct_event == RAC_EVENT_SERVICE_STOP){    
				client->rac.acct_state= RAC_STATE_STOPPED;
				client->rac.acct_event = RAC_EVENT_NONE;
			
			}
			else if (client->rac.acct_event == RAC_EVENT_IDLE_TIMEOUT){                                		
				client->rac.acct_state = RAC_STATE_STOPPED;
				client->rac.acct_event = RAC_EVENT_NONE;
			}

#if (!RAC_UT)
			else if (ifinaddr == -1) 
				ifinaddr = if_get_ipaddr(IF_NAS);

			if (ifinaddr != -1){
#else
			if (ifinaddr_rac_ut != -1){

#endif /*(!RAC_UT)*/
				/*
				 * sent pending start
				 */
				acct_status_type = 1;			
				rac_tx(client);
				
				client->rac.tx_count= 1;
				client->rac.acct_state= RAC_STATE_START_SENT;
				client->rac.acct_event = RAC_EVENT_NONE;
			}          

			break;


		case RAC_STATE_START_SENT:

			log(LG_DBG, "RAC_STATE_START_SENT");
			if (client->rac.acct_event == RAC_EVENT_SERVICE_STOP ){
				client->rac.acct_state =  RAC_STATE_STOP_SENT;
				/*
				 * create and send accounting stop message
				 */
				acct_status_type = 2;
				rac_tx(client);
				client->rac.tx_count= 1;
				client->rac.acct_event = RAC_EVENT_NONE;
			}
			else if (client->rac.acct_event == RAC_EVENT_START_RESPONSE_RECVD){
                                client->rac.acct_state = RAC_STATE_RUNNING;
                                client->rac.acct_event = RAC_EVENT_NONE;
                        }

			else if (client->rac.acct_event == RAC_EVENT_IDLE_TIMEOUT){
				rac_set_idle(client);
				client->rac.acct_event = RAC_EVENT_NONE;
			}
			/*
			 *  in case of timeout, retransmit 
			 */

#if (!RAC_UT)
			else if  (time(NULL) > client->rac.resp_timeout){
				acct_status_type = 1;
				rac_retransmit(client);
#else 
			 else if  (rac_resp_timeout){
				 acct_status_type = 1;
	                         rac_retransmit(client);
#endif /*(!RAC_UT)*/				
			
				log(LG_DBG, "RAC_STATE_START_SENT");	
			}
			log(LG_DBG, "client->rac.tx_count=%d",client->rac.tx_count);
		       	break;


		case RAC_STATE_INTERIM_SENT:

			log(LG_DBG, "RAC_STATE_INTERIM_SENT");
			if (client->rac.acct_event == RAC_EVENT_SERVICE_STOP ){
				client->rac.acct_state = RAC_STATE_STOP_SENT;
				/*
				 * create and send accounting stop message
				 */
				acct_status_type = 2;
				rac_tx(client);
				client->rac.tx_count= 1;
				client->rac.acct_event = RAC_EVENT_NONE;
			}
			 else if (client->rac.acct_event == RAC_EVENT_INTERIM_RESPONSE_RECVD){
                                client->rac.acct_state = RAC_STATE_RUNNING;
                                client->rac.acct_event = RAC_EVENT_NONE;
                        }
			else if (client->rac.acct_event == RAC_EVENT_IDLE_TIMEOUT){
				rac_set_idle(client);
				client->rac.acct_event = RAC_EVENT_NONE;
			}
			/* 
			 * in case of timeout, retransmit
			 */
#if (!RAC_UT)
			else if (time(NULL) > client->rac.resp_timeout){
				
				client->rac.acct_event = RAC_EVENT_INTERIM;	
				acct_status_type = 3;
				rac_retransmit(client);
#else
			else if  (rac_resp_timeout){
				client->rac.acct_event = RAC_EVENT_INTERIM;
                                acct_status_type = 3; 
				rac_retransmit(client);

#endif /*(!RAC_UT)*/
			

			}
			break;

		case RAC_STATE_RUNNING:
			/*
			 * If interim accounting enabled, we only need to check
			 * for next  interim update timeout
			 */
			// log(LG_DBG, "RAC_STATE_RUNNING");
			 
#if (RAC_UT)
		        printf("\n  RAC_STATE_RUNNING: \n" );
			printf("\n  interim_interval:%d \n", client->rac.interim_interval );
			printf("\n  rac_interim_timeout:%d \n", rac_interim_timeout );


#endif /*(RAC_UT)*/

			if (client->rac.acct_event == RAC_EVENT_SERVICE_STOP ){
                                client->rac.acct_state = RAC_STATE_STOP_SENT;
                                /*
                                 * create and send accounting stop message
                                 */
				acct_status_type = 2;
                                rac_tx(client);
                                client->rac.tx_count= 1;
                                client->rac.acct_event = RAC_EVENT_NONE;
                        }
                        else if (client->rac.acct_event == RAC_EVENT_IDLE_TIMEOUT)
                        {
                                rac_set_idle(client);
                                client->rac.acct_event = RAC_EVENT_NONE;
                        }

      
#if (!RAC_UT)
			else if ((client->rac.interim_interval )&&(time(NULL) > client->rac.interim_timeout)){
#else
			else if ((client->rac.interim_interval )&&(rac_interim_timeout)){
#endif /*(!RAC_UT)*/			
			        client->rac.acct_event = RAC_EVENT_INTERIM;
				/*
				 * prepare and send interim acct message by calling rac_tx()
				 */
				acct_status_type = 3;
				rac_tx(client);
				client->rac.tx_count= 1;
				client->rac.interim_timeout = time(NULL)
				       			    + client->rac.interim_interval
							    + 1
							    + (int)(5.0*rand()/(RAND_MAX+1.0));
				/*
				 * set state to interim sent
				 */
				client->rac.acct_state = RAC_STATE_INTERIM_SENT;
				client->rac.acct_event = RAC_EVENT_NONE;
			}
			break;


		case RAC_STATE_STOP_SENT:

			log(LG_DBG, "RAC_STATE_STOP_SENT");
			log(LG_DBG, "client->rac.acct_event %d ",client->rac.acct_event);
			if (client->rac.acct_event == RAC_EVENT_STOP_RESPONSE_RECVD){
	
				
				log(LG_DBG, "RAC_EVENT_STOP_RESPONSE_RECVD");
				client->rac.acct_state = RAC_STATE_STOPPED;
				client->rac.acct_event = RAC_EVENT_NONE;
#if (!RAC_UT)
		                rac_close(client);
#endif /*(!RAC_UT)*/

			}
			else if (client->rac.acct_event == RAC_EVENT_SERVICE_START){


				client->rac.start_time = time(NULL);
				/*
				 * Create Accounting session id and store in
				 * Client->rac.session_id
				 */
				p = make_session_id(client->cconfig->ifhwaddr ,client->hwaddr);
                                memcpy(client->rac.session_id, p ,128);

			      printf("\n client->rac.session_id = %s \n",client->rac.session_id);	
				/*
				 * create and send accounting start message
				 */
				acct_status_type = 1;
				rac_tx(client);
				client->rac.acct_state = RAC_STATE_START_SENT;
				client->rac.tx_count= 1;
				client->rac.acct_event = RAC_EVENT_NONE;
			}

			       /*
				* in case of timeout, retransmit
                                */
#if (!RAC_UT)
			else if  (time(NULL) > client->rac.resp_timeout){
				client->rac.acct_event = RAC_EVENT_SERVICE_STOP;
				acct_status_type = 2;
				rac_retransmit(client);
#else
			else if  (rac_resp_timeout){
				//client->rac.acct_event = RAC_EVENT_SERVICE_STOP;
				acct_status_type = 2;
				rac_retransmit(client);
#endif /*(!RAC_UT)*/

			}
			break;
		case RAC_STATE_IDLE:
			log(LG_DBG, "RAC_STATE_IDLE");
			if (client->rac.acct_event == RAC_EVENT_SERVICE_STOP)
			{
				client->rac.acct_state = RAC_STATE_STOP_SENT;
				/*
				 * create and send accounting stop message
				 */
				acct_status_type = 2;
				rac_tx(client);
				client->rac.tx_count= 1;
				client->rac.acct_event = RAC_EVENT_NONE;
			}
			break;
		default:
			log(LG_DBG, "INVALID STATE");
			/* 
			 * invalid state 
			 */
		break;
			
	}
	return RAC_RET_SUCCESS;
}


char *make_session_id(unsigned char *a, unsigned char *b)
{
	static char buf[128];
	int i;
	unsigned char *p = (unsigned char *)&i ;

	
	i = time(NULL);
printf("\n%x\n", i);
	i = htonl(i);
printf("\n%x\n", i);
	sprintf(buf, "%02X-%02X-%02X-%02X-%02X-%02X-%02X-%02X-%02X-%02X-%02X-%02X-00-00-00-00-%02X-%02X-%02X-%02X",
	a[0], a[1], a[2], a[3], a[4], a[5], 
	b[0], b[1], b[2], b[3], b[4], b[5],
	*p,*(p+1),*(p+2),*(p+3) );
	printf("\nSESSION ID %s\n",buf);

	return(buf);
}







char *make_called_station_id(unsigned char *a, struct client_t *client)
{
	static char buf1[51];
	
	log(LG_DBG, "entering function to make called_station_id " );
	sprintf(buf1,  "%02X-%02X-%02X-%02X-%02X-%02X:%s",
			a[0], a[1], a[2], a[3], a[4], a[5],client->cconfig->ssid);

	return(buf1);
}


char *make_calling_station_id(unsigned char *a)
{
	static char buf2[18];

	snprintf(buf2, 18, "%02X-%02X-%02X-%02X-%02X-%02X",
			a[0], a[1], a[2], a[3], a[4], a[5]);

	return(buf2);
}







