/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 * Intersil ® is a registered trademark of Intersil Americas Inc.
 * Copyright © 2002 Intersil Americas Inc. All Rights Reserved.
 *
 *                              
*/


#ifndef _RAC_H
  #define _RAC_H 

#include <netinet/in.h>
#include "client.h"

/*Macros defined for RADIUS Accounting */

#define RAC_TRUE                        1
#define RAC_FALSE                       0

/*
 *  * RADIUS Accounting support flag. Set to RAC_TRUE
 *  * to compile in RADIUS accounting facility.
 *  */
#define RAC_SUPPORT                     RAC_TRUE
#define RAC_UT 				0
#define RAC_IT				0


#if(RAC_SUPPORT == RAC_TRUE)


/*
 *  * RADIUS Accounting Re-Transmission support flag. Set to RAC_TRUE
 *  * to enable retransmission facility
 *
 * */
#define RAC_RETRANSMIT          	RAC_TRUE
#if (RAC_RETRANSMIT == RAC_TRUE)
/*
 *  * Maximum number of retries for receiving the accounting response message
 *   * corresponding to a sent Accounting request message. Default value is 3.
 *    */
 #define RAC_MAX_RETRIES                3
#else
/*
 *  * Number of retries set to 0 in case Re-transmission is disabled.
 *   */
#define RAC_MAX_RETRIES         0
#endif//RAC_RETRANSMIT

/*
 *  * RADIUS Accounting Retry timeout in seconds. Default value is 15.
 *  */
#define RAC_RETRY_TIMEOUT       15

/*
 *  * Timeout value for NULL data response. If no response received to
 *  * NULL request within this time, set terminate cause to lost carrier
 *  * and acct state to stopped. Else if response received, set terminate
 *  * cause to Idle and set acct state to stopped.
 *  */
#define RAC_IDLE_NULL_RESP_TIMEOUT         15

/*
 *  * RADIUS Accounting message length.
 *  */
#define RAC_MSG_LEN            		4095 //TO CHECK  
#define RAC_RX_MSG_LEN                  400 //TO CHECK



/*
 *  * RADIUS Accounting invalid port.
 *  */
#define RAC_INVALID_PORT                -1

/*
 *  * RADIUS Accounting invalid socket.
 *  */
#define RAC_INVALID_SOCK                -1
/*
 *  * RADIUS Accounting Message Types
 *  */
#define RAC_MSG_TYPE_REQUEST            4
#define  RAC_MSG_TYPE_RESPONSE          5


/*
 *  * Added RADIUS Attribute Ids
 *   */
#define RA_CLASS                                        25
#define RA_NAS_PORT                                     5
#define RA_INTERIM_INTERVAL                             85
#define RA_ACCT_AUTH                                    45
#define RA_EVENT_TIMESTAMP                              55
#define RA_TERMINATION_ACTION                   	29
#define RA_ACCT_STATUS_TYPE                             40
#define RA_ACCT_INPUT_OCTET                             42
#define RA_ACCT_OUTPUT_OCTET                    	43
#define RA_ACCT_INPUT_PACKET                    	47
#define RA_ACCT_OUTPUT_PACKET                   	48
#define RA_TERMINATE_CAUSE                              49
#define RA_ACCT_SESSION_TIME                    	46
#define RA_ACCT_SESSION_ID                              44
#define RA_ACCT_INPUT_GIGAWORDS                 	52
#define RA_ACCT_OUTPUT_GIGAWORDS                        53


/* RADIUS Attribute  CLASS length*/
#define MAX_RADIUS_CLASS_LEN           			32 // -To Do

/*
 *   Termination Action value
 * */
#define  TERM_ACTION_TERMINATE          0
#define  TERM_ACTION_REAUTH             1


# define RA_WIRELESS_PORT_TYPE		19 
/*
 *  * Enumerations for RADIUS Accounting status <96> enable or disable
 *   */
enum rac_status_t{
	RAC_STATUS_DISABLE=0,
	RAC_STATUS_ENABLE
};

/*
 *  * Enumerations for RADIUS function return values
 *   */
enum rac_ret_t{
	RAC_RET_SUCCESS=0,
	RAC_RET_START_PENDING=1,
	RAC_RET_FAIL=2
};


/*
 *  * RADIUS Accounting states
 *  */
enum rac_state_t{
	RAC_STATE_STOPPED=0,
	RAC_STATE_START_PENDING, /* Accounting Start message kept
				  * pending as AP IP address not
				  * configured. */
	RAC_STATE_START_SENT,    /* Accounting Start message sent */
	RAC_STATE_RUNNING,       /* Accounting Start or Interim
				  * response received.*/
	RAC_STATE_INTERIM_SENT,  /* Interim Accounting message sent */
	RAC_STATE_STOP_SENT,     /* Accounting Stop message sent */
	RAC_STATE_IDLE,          /* Idle timeout occurred for the
				  * client */
};

/*
 *  * Enumerations for events triggering RADIUS accounting state machine
 *   */
enum rac_event_t{
	RAC_EVENT_NONE=0,                 /* no event */
	RAC_EVENT_SERVICE_START=1,        /* service started for client */
	RAC_EVENT_SERVICE_STOP =2,        /* service stopped for client */
	RAC_EVENT_INTERIM =3,		 /*Interim Interval timeout for client*/
	RAC_EVENT_IDLE_TIMEOUT =4,	   /*Idle Timeout for client*/
	RAC_EVENT_START_RESPONSE_RECVD =5, /*Start Response recived from RAC                      			     Server  for client*/
	RAC_EVENT_STOP_RESPONSE_RECVD =6,  /*Stop Response recived from RAC     					     Server  for client*/
	RAC_EVENT_INTERIM_RESPONSE_RECVD =7/*Interim Response recived from RAC  					     Server  for client*/
};
/*
 *  * Enumerations for RADIUS Acct-Authentic attribute
 *   */
enum rac_auth_type_t{
	RAC_AUTH_RADIUS=1, /* Authentication done by RADIUS server */
	RAC_AUTH_LOCAL=2   /* Authentication done by local policy */
};
/*
 *  * Enumerations for RADIUS Acct-Terminate-Cause attribute. The assigned
 *   * values are the actual values as per RADIUS standard.
 *    */
enum rac_terminate_cause_t{
	RAC_TERMINATE_USR_REQ=1,          /*Session Terminated by user request*/
	RAC_TERMINATE_LOST_CARRIER=2, /*Session Terminated for lost carrier*/
	RAC_TERMINATE_IDLE_TIMEOUT=4, /*Session Terminated for idle timeout*/
	RAC_TERMINATE_SESSION_TIMEOUT=5, /*Session timeout*/
	RAC_TERMINATE_NAS_ERROR=9, /*Session Terminated for NAS error*/
	RAC_TERMINATE_REAUTH_FAIL=20  /*Session Terminated due to
					reauthentication failure*/
};

/*
 *  * Definition for RADIUS accounting structure
 *   */
struct rac_t{

	int start_time;                         /* accounting start time*/
	char *acct_msg;
	enum rac_event_t acct_event;    /* Next event triggering accounting
					 * state machine.*/
	enum rac_state_t acct_state;    /* Current accounting state */
	int tx_count;                   /* Number of times the message has
				 	 * been transmitted*/
	int resp_timeout;               /* Timeout for response messages */
	int rac_fd ;
	int rac_id;                     /* Id for Radius message*/
	int interim_timeout;            /* Next interim timeout */
	
	int idle_null_resp_timeout;     /* Next null response timeout */
	char rac_vector[16];            /*the descriptor value in the */
       					/*accounting packet to verify packet*/
	int rac_secret_len;
	char rac_secret[MAX_RADIUS_SECRET_LEN]; /*this is saved per client to
						 * receive accounting packet
						 * even if  the current rac
						 * server is different from
						 * the server for which the
						 *  packet was sent*/
	enum rac_auth_type_t acct_auth;          /* auth type <96> radius or local */
	enum rac_terminate_cause_t terminate_cause; /* cause of session termination
						  for the client */
	int interim_interval;           /* The interim interval value for the
					 * Client. */
	char class[MAX_RADIUS_CLASS_LEN];
	int class_len;

	struct sockaddr rac_salocal;
	struct sockaddr rac_saremote;
	char session_id[128];
	
};
	
	

#endif//RAC_SUPPORT

#endif
