/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
/*
 * Minimalistic radius client implementation for building,
 * sending and receiving radius packets
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>

#include "paed.h"
#include "state_be.h"
#include "misc.h"
#include "client.h"
#include "eap.h"
#include "radius.h"
#include "radius_pkt.h"
#include "config.h"
#include "hmac_prf.h"
#include "md5.h"
#include "keystuff.h"
#include "log.h"


// globals

char frame[2514];
extern in_addr_t ifinaddr;      // from eapol.c
extern struct radius_t radius;
extern char hostname[];         // from paed.c
#if (RAC_SUPPORT == RAC_TRUE)
extern int rac_status;
extern int rac_interim_interval;
#endif



/*
 * radius_rx :  Recieve a radius PDU from the client's radius socket
 */

int radius_rx(struct client_t *client)
{
        int n;
        struct sockaddr from;
        int fromlen;
        struct radius_hdr_t *hdr = (struct radius_hdr_t *)frame;
        int r;
        char *p;
        struct eaphdr *eap;
        unsigned long sto;

#if (PAED_MSSID == 1)

    	int vlan_tag_num;
	char vlan_tag_str[5]; /* vlan id as string */
	int vssid; 

#endif

        fromlen = sizeof(from);
        n = recvfrom(client->radiusfd, frame, sizeof(frame), 0, &from, &fromlen);
        if (n == -1) {
                log(LG_WRN, "recvfrom");
                return(-1);
        }

        if (ntohs(hdr->len) > 1564)
        {
                log(LG_WRN, "Received packet too big (%d)", ntohs(hdr->len));
                return(-1);
        }

        log(LG_DBG, "received radius pkt. code:%d id:%d", hdr->code, hdr->ident);

        log(LG_DBG, "RADIUS ID              %d", hdr->ident);

        if(hdr->ident == client->radius_id) {
                log(LG_DBG, "expected reply, increasing radius_id");
                client->radius_id = (client->radius_id + 1) % 256;
        }

        radius_pkt_dump(frame);


        // For verification of the packet, copy the original client's
        // radius vector in place of the received radius vector. Verify the
        // packet and if it fails, drop it.

        memcpy(hdr->auth, client->radius_vector, 16);
        r = radius_pkt_verify(frame, client->radius_secret, client->radius_secret_len);
        if(r != 0) return(-1);

        switch(hdr->code) {
                case RC_ACCESS_CHALLENGE:

                        // Save the state attribute for later communcations with the radius server

                        r = radius_pkt_get_str(frame, RA_STATE, &p);

                        if(r > 0) {
                                log(LG_DBG, "Saving radius state, %d bytes", r);
                                if (r>MAX_RADIUS_STATE_LEN) r=MAX_RADIUS_STATE_LEN;
                                memcpy(client->radius_state, p, r);
                                client->radius_state_len = r;
                        }


                        // Copy the EAP packet(s) to the client->eap_pkt buffer

                        r = radius_pkt_get_str(frame, RA_EAP_MESSAGE, &p);

                        if(r>0) {
                                client->eap_pkt_len = 0;
                        }

                        while(r>0) {
                                memcpy(client->eap_pkt + client->eap_pkt_len, p, r);
                                client->eap_pkt_len += r;
                                r = radius_pkt_get_str(frame, RA_EAP_MESSAGE, &p);
                        }

                        if(client->eap_pkt_len > 0) {
                                eap = (struct eaphdr *)client->eap_pkt;

                                client->idFromServer = eap->id;
                                client->aReq = 1;

                                log(LG_DBG, "Got EAP pkt in radius pkt. id=%d len=%d",
                                        eap->id, client->eap_pkt_len);

                        }

                        // Save session-timeout in config.suppTimout if available

                        r = radius_pkt_get_int(frame, RA_SESSION_TIMEOUT, &sto);

                        if((r == 0) && (sto != 0)){
                                client->cconfig->suppTimeout = sto;
                                log(LG_DBG, "Received session timeout from radius, set suppTimeout to %d", sto);
                        }

                        break;

                case RC_ACCESS_REJECT:
                        log(LG_DBG, "Received radius access-reject");



                        be_fail(client);
                        break;


                case RC_ACCESS_ACCEPT:
                        log(LG_DBG, "Received radius access-accept");

                        r = radius_pkt_get_int(frame, RA_SESSION_TIMEOUT, &sto);

                        if((r == 0) && (sto != 0)){
                                client->cconfig->reAuthPeriod = sto;
                                log(LG_DBG, "Received session timeout from radius, set reAuthPeriod to %d", sto);
                        }

                        r = radius_pkt_get_int(frame, RA_IDLE_TIMEOUT, &sto);

                        if((r == 0) && (sto != 0)){
                                client->cconfig->idleTimeout = sto;
                                log(LG_DBG, "Received idle timeout from radius, set idleTimeout to %d", sto);
                        }


#if (PAED_MSSID == 1)

		           /* Check for the VLAN tag sent by the RADIUS server */
		           r = radius_pkt_get_tunnel_int(frame, RA_TUNNEL_TYPE, &sto);

		           if (0 == r){   

		                /* Expecting a tunnel type of VLAN */
		                if (RADIUS_TUNNEL_TYPE_VLAN == sto) {

                                    log(LG_INF, "MSSID: Received VLAN Tunnel attribute");

		                    r = radius_pkt_get_tunnel_int(frame, 
		                                              RA_TUNNEL_MEDIUM_TYPE, &sto);

		                    /* for VLAN assignment, medium type should be 802. i.e.
                                     * the tunnel medium type value should be 
                                     * RA_TUNNEL_MEDIUM_TYPE_VAL_802 (6) - section 3.2, rfc2868
                                     */ 
		                    if (RA_TUNNEL_MEDIUM_TYPE_VAL_802 == sto) {

					memset(vlan_tag_str, 0, sizeof(vlan_tag_str));  

		                        r = radius_pkt_get_tunnel_str(frame, 
		                                           RA_TUNNEL_PRIVATE_GROUP_ID, &p);

		                        /* this contains the VLAN tag assignment. 
		                         * The VLAN-ID value is between 1-4094 but is sent 
		                         * as a string. Therefore, maximum length for 
		                         * the string can be 4. Also, returned length cannot be 0.
		                         */
		                        if ((r > 0) && (r <=4 )){

		                            memcpy(vlan_tag_str,  p, r);
					    log(LG_DBG, "MSSID: VLAN TAG String is = '%s'", vlan_tag_str);
		                            vlan_tag_num = atoi(vlan_tag_str);
                                    
                                    log(LG_INF, "MSSID: Received VLAN tag %d for client %s",
                                                 vlan_tag_num, hwaddr2string(client->hwaddr));

		                            /* Shift the client to the new VSS */
									r = mssid_change_sta_vss(client, vlan_tag_num);

									/* Deauth client if unable to set new vss for the client */
		                            if (r < 0){
		                               	log(LG_WRN, "MSSID: Unable to set new VSS for the client");

		                               	client->toBeRemoved = 1;
		                               	return -1;
		                            }
		                        } 
		                        else {
		                            char tmp_buf[64];
		                            memset(tmp_buf, 0, sizeof(tmp_buf));

		                            if (r < sizeof(tmp_buf)){
		                               	strncpy(tmp_buf, p, r);
		                            }
		                            log(LG_WRN, 
		                            	"MSSID: RADIUS Server sent invalid VLAN Tag in Tunnel Private Group Id attribute = %s", 
		                            	tmp_buf);
		                        }
		                    }
		                 }
		           } /* if attribute tunnel type present */

#endif /* #if (PAED_MSSID == 1) */


#if (RAC_SUPPORT == RAC_TRUE)
			if (RAC_STATUS_ENABLE == rac_status){

				/* Check whether an Interim-Interval attribute has been sent
				 * or not. The locally configured value is used unless the
				 * server disables Interim-Updates by not sending any Interim
				 * interval attribute
				 */
				r = radius_pkt_get_int(frame, RA_INTERIM_INTERVAL, &sto); 
				client->rac.interim_interval = 0;

				if (r == 0){
					client->rac.interim_interval = rac_interim_interval;
				}

				/* If Class attribute is present, gets its value */
				r = radius_pkt_get_str(frame, RA_CLASS, &p);
				if (0 != r){
					memcpy(client->rac.class,p ,r);
				}
				client->rac.class_len =r;

			}


			/* If TERMINATION ACTION attribute is present, gets its value */
			r = radius_pkt_get_int(frame, RA_TERMINATION_ACTION, &sto);

			if(r == 0) {
				client->terminationAction = sto;
				log(LG_DBG, "Received termination Action from radius, 
						set terminationAction to %d", sto);
			}

#endif /*(RAC_SUPPORT == RAC_TRUE)*/

			// Get MPPE keys if available

                        while((r = radius_pkt_get_str(frame, RA_VENDOR_SPECIFIC, &p)) > 0) {
                                decode_mppe_keys(client, p, r);
                        }

                        be_success(client);
                        break;

                default:
                        log(LG_WRN, "Unexpected radius code : %d", hdr->code);
                        break;
        }

        return(0);
}



/*
 * radius_tx :  Encapsulate the latest EAP packet in a radius packet,
 *              and send to server
 */

int radius_tx(struct client_t *client)
{
        int r;
        int len;
        int offset;

#if (PAED_MSSID == 1)
         char ssid_str[33];
         /* length of called station id string - 
          * - 17 bytes for MAC address 
          * - 1 bytes for separator ":"
          * - 32 bytes for SSID 
          * Plus string terminator 
          */
         char called_station_id_str[51]; 
#endif



        // Start radius packet building

        client->radius_id = (client->radius_id+1) % 256;

        random_vector(client->radius_vector, 16);

        radius_pkt_start(frame, RC_ACCESS_REQUEST, client->radius_id, client->radius_vector);

        log(LG_DBG, "RADIUS ID              %d", client->radius_id);




        // Add attributes to radius packet

        // The username trying to log on

        radius_pkt_add_str (frame, RA_USERNAME, client->identity, strlen(client->identity));
        log(LG_DBG, "RA_USERNAME            %s", client->identity);

        // NAS Ip address : This is the AP address of the access point

        // Fetch the IP address from our ethernet interface for use in the
        // NAS_IP_ADDRESS attribute. If we can't get it (yet), just don't
        // include this attribute in the packet, we'll try next time. This is
        // done because paed is started when there still might be no valid
        // IP address on the interface (dhcp,autoip)

        if(ifinaddr == -1) ifinaddr = if_get_ipaddr(IF_NAS);

        if(ifinaddr != -1) {
                radius_pkt_add_addr(frame, RA_NAS_IP_ADDRESS, ifinaddr);
                log(LG_DBG, "RA_NAS_IP_ADDRESS      %s", inet_ntoa(ifinaddr));
        }

        // Framed mtu. Fixed to 1488

        radius_pkt_add_int (frame, RA_FRAMED_MTU, 1488);
        log(LG_DBG, "RA_FRAMED_MTU          1488");


#if (PAED_MSSID == 1)

        /* Add SSID of the VSS to the CALLED STATION ID Attribute */ 
        strncpy(ssid_str, client->cconfig->ssid, 
                               client->cconfig->ssidlength);

        sprintf(called_station_id_str, "%s:%s",
               hwaddr2string(client->cconfig->ifhwaddr), ssid_str);

        // Called station ID : MAC address of Acces point
        radius_pkt_add_str (frame, RA_CALLED_STATION_ID, 
                called_station_id_str, strlen(called_station_id_str));

        log(LG_DBG, "MSSID: RA_CALLED_STATION_ID   %s", 
                          called_station_id_str);
#else

        // Called station ID : MAC address of Acces point

        radius_pkt_add_str (frame, RA_CALLED_STATION_ID, hwaddr2string(client->cconfig->ifhwaddr), 17);
        log(LG_DBG, "RA_CALLED_STATION_ID   %s", hwaddr2string(client->cconfig->ifhwaddr), 17);

#endif /* #if (PAED_MSSID == 1) */

        // Calling station ID : MAC address of client

        radius_pkt_add_str (frame, RA_CALLING_STATION_ID, hwaddr2string(client->hwaddr), 17);
        log(LG_DBG, "RA_CALLING_STATION_ID  %s", hwaddr2string(client->hwaddr), 17);

        // NAS port type : 19 = 820.11 wireless

        radius_pkt_add_int (frame, RA_NAS_PORT_TYPE, 19);
        log(LG_DBG, "RA_NAS_PORT_TYPE       19");

        // NAS identifier. We use the hostname

        if(strlen(hostname)>0) {
                radius_pkt_add_str (frame, RA_NAS_IDENTIFIER, hostname, strlen(hostname));
                log(LG_DBG, "RA_NAS_IDENTIFIER      %s", hostname);
        }

        // Connect-info : Fixed text "CONNECT 11Mbps 802.11b"

        radius_pkt_add_str (frame, RA_CONNECT_INFO, "CONNECT 11Mbps 802.11b", 22);
        log(LG_DBG, "RA_CONNECT_INFO        CONNECT 11Mbps 802.11b");

        // State, only include if there's a previous state received

        if(client->radius_state_len > 0) {
                radius_pkt_add_str (frame, RA_STATE, client->radius_state, client->radius_state_len);
                log(LG_DBG, "RA_STATE               %d bytes", client->radius_state_len);
        }

        // EAP message(s), if defined

        if(client->eap_pkt_len > 0) {
                offset = 0;

                while(offset < client->eap_pkt_len) {
                        if (client->eap_pkt_len - offset < 253) {
                                len = client->eap_pkt_len - offset;
                        } else {
                                len = 253;
                        }

                        radius_pkt_add_str (frame, RA_EAP_MESSAGE, client->eap_pkt + offset, len);
                        log(LG_DBG, "RA_EAP_MESSAGE         %d bytes", len);
                        offset += len;
                }
        }


        // Finish the packet

        len = radius_pkt_finish  (frame, client->radius_secret, client->radius_secret_len);


        log(LG_DBG, "sending radius packet to server");

        radius_pkt_dump(frame);

        r = sendto (client->radiusfd, frame, len, 0,
                (struct sockaddr *) &client->radius_saremote,
                sizeof (struct sockaddr_in));

        return(0);
}



/*
 * radius_open : opens a socket to radius server
 */

int radius_open(struct client_t *client)
{
        struct sockaddr_in *sin;
//      struct hostent *hp;
        int len;
        int r;

        log(LG_DBG, "Opening radius socket");

        client->radiusfd = socket(AF_INET, SOCK_DGRAM, 0);

        if(client->radiusfd < 0) {
                log(LG_ERR, "socket says '%s'", strerror(errno));
                client->radiusfd = -1;
                return(-1);
        }

        len = sizeof(client->radius_salocal);

        sin = (struct sockaddr_in *)&client->radius_salocal;
        memset((char *)sin, 0, len);
        sin->sin_family = AF_INET;
        sin->sin_addr.s_addr = htonl(INADDR_ANY);
        sin->sin_port = htons(0);

        r = bind(client->radiusfd, (struct sockaddr *)sin, len);

        if(r<0) {
                log(LG_ERR, "bind says '%s'", strerror(errno));
                client->radiusfd = -1;
                return(-1);
        }

        log(LG_DBG, "Current radius server : %s:%d", (
                inet_ntoa(radius.current_radius_serv->address)),
                radius.current_radius_serv->port);


        sin = (struct sockaddr_in *)&client->radius_saremote;
        memset((char *)sin, 0, len);
        sin->sin_family = AF_INET;
        sin->sin_addr.s_addr = radius.current_radius_serv->address;
        sin->sin_port = htons(radius.current_radius_serv->port);

        len = radius.current_radius_serv->secret_len;
        if(len>MAX_RADIUS_SECRET_LEN) len=MAX_RADIUS_SECRET_LEN;
        memcpy(client->radius_secret, radius.current_radius_serv->secret, len);
        client->radius_secret_len = len;


        return(0);
}


int radius_close(struct client_t *client)
{
        if(client->radiusfd != -1) {
                log(LG_DBG, "Closing radius socket");
                close(client->radiusfd);
                client->radiusfd = -1;
        }

        return(0);
}





// eof

