/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
/*
 * Minimalistic radius client implementation for buidling radius packets
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>

#include "misc.h"
#include "md5.h"
#include "hmac_prf.h"
#include "radius.h"
#include "radius_pkt.h"
#include "log.h"


//////////////////////////////////////////////////////////////////////
/////// Functions for adding AV-paris to radius packet ///////////////
//////////////////////////////////////////////////////////////////////


int radius_pkt_start(char *pdu, int code, int ident, char *vector)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;

	hdr->code  = code;
	hdr->ident = ident;
	hdr->len   = sizeof(struct radius_hdr_t);

	switch(code) {
		case RC_ACCESS_REQUEST:
			memcpy(hdr->auth, vector, 16);
			break;
		default:
			printf("nonvalid code %d\n", code);
			break;
	}


	return(0);
}

int radius_pkt_add_str(char *pdu, int attr, char *string, int len)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char *p = pdu + hdr->len;

	(char)*(p+0) = attr;
	(char)*(p+1) = len + 2;
	memcpy(p+2, string, len);

	hdr->len += len+2;

	return(0);
}


int radius_pkt_add_int(char *pdu, int attr, unsigned long val)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char *p = pdu + hdr->len;
	unsigned long tmpval;

	tmpval = htonl(val);

	(char)*(p+0) = attr;
	(char)*(p+1) = sizeof(unsigned long) + 2;
	memcpy(p+2, (char *)&tmpval, sizeof(tmpval));

	hdr->len += 6;

	return(0);
}

int radius_pkt_add_addr(char *pdu, int attr, in_addr_t addr)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char *p = pdu + hdr->len;

	(char)*(p+0) = attr;
	(char)*(p+1) = sizeof(unsigned long) + 2;
	memcpy(p+2, &addr, sizeof addr);

	hdr->len += 6;

	return(0);
}



/*
 * This 'finishes' the newly created pack by converting the len
 * to netork byte order, and calculating the MD5 checksum over
 * the packet and adding it into the RA_MESSAGE_AUTHENTICATOR
 * thingy
 */

int radius_pkt_finish(char *pdu, char *secret, int secret_len)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char tmp[16];
	char *p = pdu + hdr->len;
	char *authptr = p+2;
	int len;

	// Add a message authenticator attribute

	(char)*(p+0) = RA_MESSAGE_AUTHENTICATOR;
	(char)*(p+1) = 16 + 2;
	bzero(authptr, 16);

	hdr->len += 18;

	len=hdr->len;

	hdr->len = htons(len);

	// Now calculate the MD5 over the complete packet, and fill
	// in the value for this attribute

	bzero(tmp, 16);
	hmac_md5(pdu, len, secret, secret_len, tmp);
	memcpy(authptr, tmp, 16);


	return(len);
}



/*
 * This verifies a radius packet by checking the message-authenticator.
 * Returns 0 if the packet is ok, !0 otherwise.
 */

int radius_pkt_verify(char *pdu, char *secret, int secret_len)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char *auth;
	char authcopy[16];
	char tmp[16];
	int authlen;

	authlen = radius_pkt_get_str(pdu, RA_MESSAGE_AUTHENTICATOR, &auth);

	if(authlen != 16) {
		log(LG_WRN, "Message-authenticator not found or not 16 bytes long");
		return(-1);
	}

	// Copy the original message authenticator, zero the location
	// of the old, calculate hmac_md5 over the packet and compare
	// the values.

	memcpy(authcopy, auth, authlen);

	bzero(auth, authlen);

	hmac_md5(pdu, ntohs(hdr->len), secret, secret_len, tmp);


	if(memcmp(authcopy, tmp, 16) != 0) {
		log(LG_WRN, "Message-authenticator not valid");
		return(-1);
	}

	log(LG_DBG, "Radius message-authenticator is OK");

	return(0);
}




int radius_pkt_dump(char *pdu)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	int len = ntohs(hdr->len);

	dump_bindata("Radius packet", pdu, len);

	return(0);
}




//////////////////////////////////////////////////////////////////////
///// Functions for retrieving AV-pairs from radius packet ///////////
//////////////////////////////////////////////////////////////////////

/*
 * Get the next string attribute 'attr' from the packet, and copy it
 * to *dst. (no more than *dstlen bytes), and place pointer to this
 * value in *dst. returns the lenght of data, or 0 if not found.
 *
 * The first occurence of 'attr' is returned the first time. On subsequent
 * calls where pdu and attr are the same, it returns the
 * next values. A bit like strtok
 */

int radius_pkt_get_str(unsigned char *pdu, int attr, char **dst)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	int pdulen = ntohs(hdr->len);
	int len;
	int i=0;
	int id;

	static int previd = 0;
	static unsigned char *p;
	static int prevattr = -1;

	id = (unsigned char)*(pdu+1);

	if((id != previd) || (attr != prevattr)) {
		p = pdu + sizeof(struct radius_hdr_t);
		previd = id;
		prevattr = attr;
	}

	while( (p-pdu) < pdulen) {
		if(*p == attr) {
			len = *(p+1)-2;
			*dst = p+2;
			p += *(p+1);

			return(len);
		}

		p += *(p+1);

		// prevent endless loop
		if(*(p+1) == 0) break;
		if(i++>32) break;
	}

	return(0);
}


/*
 * Get integer from radius packet. Returns 0 on success, -1
 * on failure
 */

int radius_pkt_get_int(unsigned char *pdu, int attr, unsigned long *dst)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	unsigned char *p = pdu + sizeof(struct radius_hdr_t);
	int pdulen = ntohs(hdr->len);
	unsigned long tmp;

	while( (p-pdu) < pdulen) {
		if(*p == attr) {
			memcpy((char *)&tmp, p+2, sizeof(tmp));
			*dst = ntohl(tmp);

			return(0);
		}

		p += *(p+1);

		// prevent endless loop
		if(*(p+1) == 0) break;
	}

	return(-1);
}

/*Code added for RADIUS Accounting*/
/*******Sukanya Bhattacharya*******/
/********** Code Start ************/

int rac_pkt_start(char *pdu, int code, int ident, char *vector)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu; 
	hdr->code  = code;
	hdr->ident = ident;
	hdr->len   = sizeof(struct radius_hdr_t);
#if (RAC_UT)
                        printf("I M IN rac_pkt_start");
#endif /*(RAC_UT)*/

	switch(code) {
		case RAC_MSG_TYPE_REQUEST:
			memcpy(hdr->auth, vector, 16);
#if (RAC_UT)
			printf("I M IN rac_pkt_start");
#endif /*(RAC_UT)*/	
			break;
			
		default:
			printf("nonvalid code %d\n", code);
			break;
	}
	return(0);
}


int rac_pkt_finish(char *pdu, char *secret, int secret_len, char *vector)
{

	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char tmp[16];
	int len;
	MD5_CTX	context;

	len=hdr->len;
	hdr->len = htons(len);
	memcpy(pdu+len, secret, secret_len); 
	
	MD5Init(&context);
        MD5Update(&context, pdu, len+secret_len);
        MD5Final(hdr->auth, &context);	
	memcpy(vector, hdr->auth, 16);
	return(len);
}


int rac_pkt_verify(char *pdu, char *secret, int secret_len, char *vector)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char *auth;
	char authcopy[16];
	char tmp[16];
	int authlen;
	int len;
	MD5_CTX context;

	len=ntohs(hdr->len);
	memcpy(authcopy, hdr->auth, 16);
	memcpy(hdr->auth, vector, 16);
		
	memcpy(pdu+len, secret, secret_len);
	MD5Init(&context);
	MD5Update(&context, pdu, len+secret_len);
	MD5Final(tmp, &context);

	if(memcmp(authcopy, tmp, 16) != 0) {
		log(LG_DBG, "Message-authenticator not valid");
		return(-1);
	}

	log(LG_DBG, "Radius message-authenticator is OK");

	return(0);
}



/*********** Code End *************/



#if (PAED_MSSID == 1)

/*
 * Get integer from a Tunnel attribute in the radius packet. Tunnel 
 * attributes have a 1 byte TAG field, just after the LENGTH field.
 * Returns 0 on success, -1
 * on failure
 */

int radius_pkt_get_tunnel_int(char *pdu, int attr, unsigned long *dst)
{
     char tmp_int_buff[4];
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	char *p = pdu + sizeof(struct radius_hdr_t);
	int pdulen = ntohs(hdr->len);
	unsigned long tmp;

/*
 * Tunnel Integer Attributes structure is shown below:
 *  0                   1                   2                   3
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |     Type      |    Length     |     Tag       |     Value
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *             Value (cont)        |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
 * 
 * Value is a 3 byte field. Equivalent to a 4 byte integer 0
 * with first byte = 0
 */
	while( (p - pdu) < pdulen) {
		if(*p == attr) {
                /* skip the Type, Length and the TAG fields - 3 bytes */
                tmp_int_buff[0] = 0;
			memcpy(&tmp_int_buff[1], p+3, 3);

                /* copy 4 bytes */  
			memcpy((char *)&tmp, tmp_int_buff, sizeof(tmp));
			*dst = ntohl(tmp);

			return(0);
		}

		p += *(p+1);
	}

	return(-1);
}


/*
 * Get string from a Tunnel attribute in the radius packet. Tunnel 
 * attributes have a 1 byte TAG field, just after the LENGTH field.
 * Returns 0 on success, -1
 * on failure
 */

int radius_pkt_get_tunnel_str(unsigned char *pdu, int attr, char **dst)
{
	struct radius_hdr_t *hdr = (struct radius_hdr_t *)pdu;
	int pdulen = ntohs(hdr->len);
	unsigned char len;
	int i=0;
	int id;
	int tag=0;

        unsigned char *p = pdu + sizeof(struct radius_hdr_t);

/*
 * Tunnel String Attributes structure is shown below:
 *  0                   1                   2                   3
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |      Type     |    Length     |      Tag      |   String ...
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+  
*/

	/* Special handling for RA_TUNNEL_PRIVATE_GROUP_ID. If tag > 0x1F, 
	 * the tag value is the first byte of the string following it. 
	 */
	if (RA_TUNNEL_PRIVATE_GROUP_ID == attr)
	{
		while( (p - pdu) < pdulen) {
			if(*p == attr) {

				/* Check the tag value */
				tag = (unsigned char) (*(p + 2));

				if (tag > 0x1F)
				{
					/* The length field includes total bytes in the pkt. 
					 * The string starts from Tag and the string length will be length - 2
					 */
					len = (unsigned char) (*(p+1)-2);

					/* Skip the type and length fields */
					*dst = p+2;
					return len;
				}
				else
				{
					/* The length field includes total bytes in the pkt.
					 * The string starts After Tag and the string length will be length - 3
					 */
					len = (unsigned char) (*(p+1)-3);

					/* Skip the type, length and tag fields */
					*dst = p+3;

					return(len);
				}
			}
				   
					/* skip to the next attribute */
			p += *(p+1);

			// prevent endless loop

			if(*(p+1) == 0) break;
			if(i++>32) break;
		} /* while */

	} /* if (RA_TUNNEL_PRIVATE_GROUP_ID == attr)*/

	return(0);
}


#endif /* #if (PAED_MSSID == 1) */

// eof

