/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
/*
 * Backend statemachine functions
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifdef LINUX
#include "linux.h"
#else
#include <aplog.h>
#endif

#include "radius.h"
#include "log.h"
#include "client.h"
#include "state_be.h"
#include "state_pae.h"
#include "config.h"
#include "eap.h"
#include "eapol.h"
#include "keystuff.h"

extern struct radius_t radius;

static int triedRadiusServers = 0;

int be_check_states(struct client_t *client)
{

	switch(client->bestate) {

		case REQUEST :

			if(time(NULL) >= client->aWhile) {
				log(LG_INF,"aWhile timeout connecting client");
				be_request(client);
			}

			if(client->rxResp) {
				be_response(client);
			}

			break;


		case RESPONSE :

			if(time(NULL) >= client->aWhile) {
				aplog(EV_EAP_RADIUS_TIMEOUT, NULL);

				radius.current_radius_serv = radius.current_radius_serv->next;

				if(radius.current_radius_serv == NULL) {
					radius.current_radius_serv = radius.radius_serv_list;
				}
				/* Count the amount of radius servers used
				 * which didn't respond to determine
				 * if we should send EAPOL failure to client
				 */
				triedRadiusServers++;

				if (triedRadiusServers == radius.nofRadiusServers)
				{
   				    log(LG_WRN, "aWhile timeout connecting Radius, using first again");
				    triedRadiusServers = 0;
				    be_timeout(client);
				    break;
				}
				log(LG_WRN, "aWhile timeout connecting Radius, using next");
				be_idle(client);
				pae_connecting(client);
			}

			if(client->aReq == 1) {
			        triedRadiusServers = 0;

				be_request(client);
			}

			break;

		default:


	}


	return(0);
}


int be_request(struct client_t *client)
{
	log(LG_DBG, "be state set to REQUEST");
	client->bestate = REQUEST;

	client->currentId = client->idFromServer;

	eapol_tx(client, EAPOL_TYPE_EAP_PACKET);

	client->aWhile = time(NULL) + client->cconfig->suppTimeout;
	client->reqCount++;

	return(0);
}


int be_response(struct client_t *client)
{
	log(LG_DBG, "be state set to RESPONSE");
	client->bestate = RESPONSE;

	// Open the radius socket when necassery

	if(client->radiusfd == -1) {
		radius_open(client);
	}

	client->aReq = 0;
	client->rxResp = 0;
	client->aWhile = time(NULL) + client->cconfig->serverTimeout;
	radius_tx(client);

	return(0);
}


int be_success(struct client_t *client)
{
	log(LG_DBG, "be state set to SUCCESS");
	client->bestate = SUCCESS;


	client->currentId = client->idFromServer;

	eap_tx(client, EAP_CODE_SUCCESS, 0);

	if(!client->doesSSN) {
		keying_do_client_oldstyle(client);
	}


	be_idle(client);
	pae_authenticated(client);

	return(0);
}


int be_fail(struct client_t *client)
{
	log(LG_DBG, "be state set to FAIL");
	client->bestate = FAIL;

	client->currentId = client->idFromServer;
	eap_tx(client, EAP_CODE_FAILURE, 0);

	be_idle(client);
	pae_held(client);

	return(0);
}


int be_timeout(struct client_t *client)
{
	log(LG_DBG, "be state set to TIMEOUT");
	client->bestate = TIMEOUT;

        eap_tx(client, EAP_CODE_FAILURE, 0);

        be_idle(client);

	return(0);
}

int be_idle(struct client_t *client)
{
	log(LG_DBG, "be state set to IDLE");
	client->bestate = IDLE;

	// Radius socket no longer needed, close it

	radius_close(client);

	// EAP buffer can be freed here...

	client_free_eap_pkt(client);

	return(0);
}
