/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
#include <stdint.h>
#include <string.h>
#include <time.h>

#ifdef LINUX
#include "linux.h"
#else
#include <aplog.h>
#endif

#include "paed.h"
#include "log.h"
#include "keystuff.h"
#include "misc.h"
#include "config.h"
#include "client.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif

#include "state_gtk.h"

extern char ifhwaddr[6];
extern struct client_t *clientlist;
extern int SSN_GlobalMicFailWhile;


void gtk_check_states(struct config_t *config)
{
	int a,b;

	switch(config->cgtk->state) {
		case GTK_SETKEYS:

			if(config->cgtk->GKeyDoneStations==0) gtk_setkeysdone(config);

		default:
	}

	if(config->cgtk->reKeyTime) {

		a = config->cgtk->reKeyTime;

		a += config->cgtk->lastRekeySeconds;
		b = time(NULL);

		if(b > a) {
			log(LG_INF, "Rekeying due to time limit");
			gtk_setkeys(config);
			config->cgtk->lastRekeySeconds = time(NULL);
		}
	}
        if(config->cgtk->reKeyPackets) {

		a = config->cgtk->reKeyPackets;

		a += config->cgtk->lastRekeyPackets;
		b = blob_get_mcast_packets(config);


		if(b > a) {
			log(LG_INF, "Rekey due to mcast packets limit");
			gtk_setkeys(config);
			config->cgtk->lastRekeyPackets = blob_get_mcast_packets(config);
		}
	}

}



// This state is added because i think it's "missing" in the SSN draft. We
// need to init the statemachine somewhere ?!

int gtk_initialize(struct config_t *config)
{
	char tmprnd[32];
	int a = 0;

	log(LG_INF, "gtk state set to GTK_INITIALIZE");
	config->cgtk->state = GTK_INITIALIZE;

	config->cgtk->GInitDone		= 0;
	config->cgtk->GKeyReady		= 0;

	// Initialize the gtk.Counter with a prf_256
	random_vector(tmprnd, 32);
	prf(tmprnd, 32, "Init Counter", 12, config->ifhwaddr, 6, config->cgtk->Counter, 32);

	// Initialize the gtk.GMK with a random vector.
	random_vector(config->cgtk->GMK, GMK_LEN);

	// Debugprint

#ifdef DEBUG_PRINTKEYS
	dump_bindata("Initialized GTK Counter", config->cgtk->Counter, 32);
	dump_bindata("Initialized GMK", config->cgtk->GMK, GMK_LEN);
#endif

	gtk_setkeys(config);

	config->cgtk->reKeyTime    = 0;
	config->cgtk->reKeyPackets = 0;

	if(((config->enableSSN || config->enableSSNPSK)  && config->ssnRekeying == 2) ||
	   (config->enable1X && config->dot1XRekeying == 2))
	{

	    	// So are we in mixed mode?
		if ((config->enableSSN || config->enableSSNPSK) && (config->enable1X))
		{
		    a = config->ssnRekeySeconds;

		    if ((config->enable1X) && (config->dot1XRekeying == 2))
		    {
			// If the dot1X value is smaller or the ssn rekey time is off
			if(config->dot1XRekeySeconds < a || a == 0)
			    a = config->dot1XRekeySeconds;
		    }
		    // Set value to GTK structure.
		    config->cgtk->reKeyTime = a;
		}
		else if ((config->enableSSN || config->enableSSNPSK) && (config->ssnRekeying == 2))
			config->cgtk->reKeyTime = config->ssnRekeySeconds;
		else if ((config->enable1X) && (config->dot1XRekeying == 2))
		    	config->cgtk->reKeyTime = config->dot1XRekeySeconds;

	}
        if(((config->enableSSN || config->enableSSNPSK)  && config->ssnRekeying == 3) ||
           (config->enable1X && config->dot1XRekeying == 3)) {

	    	// So are we in mixed mode?
		if ((config->enableSSN || config->enableSSNPSK) && (config->enable1X))
		{
		    a =  config->ssnRekeyPackets;

		    if((config->enable1X) && (config->dot1XRekeying == 3)) {
			if(config->dot1XRekeyPackets < a || a == 0)
			    a=config->dot1XRekeyPackets;
		    }
		    // Set value to GTK structure.
		    config->cgtk->reKeyPackets = a;
		}
		else if ((config->enableSSN || config->enableSSNPSK) && (config->ssnRekeying == 3))
			config->cgtk->reKeyPackets = config->ssnRekeyPackets;
		else if ((config->enable1X) && (config->dot1XRekeying == 3))
		    	config->cgtk->reKeyPackets = config->dot1XRekeyPackets;

	}
	log(LG_INF, "gtk reKeyTime set to %d", config->cgtk->reKeyTime);
	log(LG_INF, "gtk reKeyPackets set to %d", config->cgtk->reKeyPackets);


	return(0);
}


int gtk_setkeys(struct config_t *config)
{
	int tmp;
	struct client_t *client;

#if (PAED_MSSID == 1)
        client_ptr_node_t *client_ptr_nodep=NULL;
#endif


	log(LG_DBG, "gtk state set to GTK_SETKEYS");
	config->cgtk->state = GTK_SETKEYS;


	if(config->cgtk->GInitDone) {
		tmp			= config->cgtk->GM;
		config->cgtk->GM	= config->cgtk->GN;
		config->cgtk->GN	= tmp;
	} else {
		config->cgtk->GN  	= 2;
		config->cgtk->GM	= 3;
	}

	config->cgtk->GTKKeyActive	= 0;
	config->cgtk->GInitDone 	= 1;

	gtk_inc_counter(config);
	memcpy(config->cgtk->GNonce, config->cgtk->Counter, NONCE_LEN);
	
	/*
	 * The length of the GTK depends on the group cipher used
	 */
	 
	if(config->enableSSN) {
		if(config->wpa_cipher & 2) config->cgtk->gtk_len = 16;	/* 16 bytes for CCMP */
		if(config->wpa_cipher & 1) config->cgtk->gtk_len = 32;	/* 32 bytes for TKIP */
	}
	if(config->enableWEP) {
		config->cgtk->gtk_len = config->Dot1XWEPKeySize;		/* For WEP, use wep key size */
	}

	calc_GTK(config, config->cgtk->GMK, GMK_LEN, config->cgtk->GNonce, config->cgtk->GTK[config->cgtk->GN], GTK_LEN);

	// Tell all client SSN-statemachines to send the Global Transent Key to the
	// stations.

#if (PAED_MSSID == 1)

       /* Perform key distribution to clients in this VSS only. */ 

       client_ptr_nodep = config->client_ptr_list;
       if (NULL != client_ptr_nodep){

     	  client = client_ptr_nodep->client_ptr;
       }
       else{
          client = NULL;
       }

       config->cgtk->GKeyDoneStations = 0;

       while(client_ptr_nodep != NULL) {

           client = client_ptr_nodep->client_ptr;

	   config->cgtk->GKeyDoneStations++;
           log(LG_DBG, "Client %s GTK should be updated", 
                hwaddr2string(client->hwaddr));

	   client->update_gtk = 1;
   	   client_ptr_nodep = client_ptr_nodep->next;
       }

#else

	client=clientlist;

	config->cgtk->GKeyDoneStations = 0;

	while(client != NULL) {
		config->cgtk->GKeyDoneStations++;
        	log(LG_DBG, "Client %s GTK should be updated", hwaddr2string(client->hwaddr));

		client->update_gtk = 1;
		client = client->next;
	}

#endif /* #if (PAED_MSSID == 1) */

	return(0);
}


int gtk_setkeysdone(struct config_t *config)
{
	log(LG_INF, "gtk state set to GTK_SETKEYSDONE");
	config->cgtk->state = GTK_SETKEYSDONE;

	if ( config->enable1X )
	{
#if (PAED_MSSID == 1)
            /* Just set the broadcast key in the service set */
    	    blob_mssid_set_bcast_key(config, config->cgtk->GTK[config->cgtk->GN], 
                                     config->Dot1XWEPKeySize, config->cgtk->GN);
#else

	    /* If plain dot1x enabled use 13 bytes (104 bits) keys WEP thus */
    	    blob_set_bcast_key(config->cgtk->GTK[config->cgtk->GN], config->Dot1XWEPKeySize, config->cgtk->GN, config->ifname);
    	    blob_set_bcast_keyid(config->cgtk->GN, config->ifname);
#endif
        }
	else
	{
#if (PAED_MSSID == 1)
            /* Just set the broadcast key in the service set */
	    blob_mssid_set_bcast_key(config, config->cgtk->GTK[config->cgtk->GN], 
                                     GTK_LEN, config->cgtk->GN);

#else
        blob_set_bcast_key(config->cgtk->GTK[config->cgtk->GN], config->cgtk->gtk_len, config->cgtk->GN, config->ifname);
	    blob_set_bcast_keyid(config->cgtk->GN, config->ifname);
#endif
	}
	config->cgtk->GTKKeyActive	= 1;   // The key is set at MVC and active.
	config->cgtk->GKeyReady = 1;

	config->cgtk->lastRekeySeconds = time(NULL);
	config->cgtk->lastRekeyPackets = blob_get_mcast_packets(config);

	return(0);
}


//////////////////////////////////// Helper functions /////////////////////

/*
 * Increase the 256 bit Global Key Counter and return the pointer to it.
 */

char *gtk_inc_counter(struct config_t *config)
{
	int i;
	unsigned char a = 0;
	unsigned char b = 0;
	unsigned char c = 0;

	a=1;
	for(i=31; i>=0; i--) {
		b = config->cgtk->Counter[i];
		c = (b==255) ? 1 : 0;
		config->cgtk->Counter[i] = b+a;
		a = c;
	}

	return(config->cgtk->Counter);
}


int gtk_integrityfailure(struct config_t *config)
{

    // When EAPOLKeyRecv with error bit set from the STA. This does
    // indicate a Michael check failure and then the WaitUpto60() applies before sending new keys.
    // We must also hear from our MVC if it detects an integrity failure for Michael...

    if ( (config->cgtk->intFailWhile) < time(NULL) )
    {
	log(LG_WRN, "MIC Integrity failure detected, but NO extra action necessary !");

        gtk_setkeys(config);

	config->cgtk->intFailWhile = time(NULL) + 60;
    }
    else
    {
	log(LG_WRN, "MIC Integrity failure detected, but EXTRA action should be taken !");
	aplog(EV_EAP_MICFAIL_BROAD, NULL);

	/* Block broadcast traffic */
	log(LG_INF, "All clients not allowed, portstate=UNAUTHORIZED");
	blob_portstatus_set_global(UNAUTHORIZED, config);

	/* Block total SSN handshake */
	SSN_GlobalMicFailWhile = time(NULL) + 60;
        gtk_setkeys(config);

    }
    return(0);

}

// eof
