/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */
/*
 * Pae statemachine functions
 */

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>

#ifdef LINUX
#include "linux.h"
#else
#include <aplog.h>
#endif

#include "log.h"
#include "eap.h"
#include "client.h"
#include "misc.h"
#include "state_pae.h"
#include "state_be.h"
#include "state_ssn.h"
#include "config.h"
#include "acl.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "keystuff.h"
#if (RAC_SUPPORT == RAC_TRUE)
extern int rac_status;
#endif


int pae_check_states(struct client_t *client)
{
	int post_state;

	switch(client->paestate) {

		case CONNECTING :

			if(time(NULL) >= client->txWhen) {
				log(LG_INF, "txWhen timeout while CONNECTION.");

				if(client->cconfig->enableOS || client->cconfig->enableWEP)
				{
					/* Set client port status to Authorized only if it uses 1x only */
					if (client->does1X == 1 && client->doesSSN == 0)
					{
					    log(LG_INF, "Assuming client does not do 802.1x, falling back");
					    post_state = acl_list_check(client, 1);
					    if (post_state == UNAUTHORIZED)
					    {
						client->toBeRemoved = 1;
					    }

#if (RAC_SUPPORT == RAC_TRUE)
					    else 
					    {
						    /*WEP client does not support dot1X. 
						     *Passed in ACL checking. We consider 
						     *this as point of service start. Send
						     *accounting start if accounting enabled. 
						     */    
						    if (RAC_STATUS_ENABLE == rac_status){
							    client->rac.acct_event = 
								    RAC_EVENT_SERVICE_START;   

							    /* Authentication type is local */ 
							    client->rac.acct_auth = RAC_AUTH_LOCAL;
						   	log(LG_DBG, "RAC_EVENT_SERVICE_START");
						    } 
					    }
#endif
				
					}
					client->does1X = 0;
				}
			       	else {
					pae_connecting(client);
				}
			}

			if(client->aReq) {
				pae_authenticating(client);
			}

			if(client->reAuthCount > client->cconfig->reAuthMax) {
				log(LG_INF, "reAuthCount > reAuthMax, going to DISCONNECTED");

				// To avoid clients to keep 'alive' in the paed forever,
				// they are removed when reAuthCound is exceeded

				client->toBeRemoved = 1;

#if (RAC_SUPPORT == RAC_TRUE)
				/*
				 * Client exceeded maximum number of reauth attempts
				 * Need to send deauth message to the client.
				 * Send Accounting stop message only if the client
				 * was already getting service and failed during
				 * re-auth. I.e. if accounting state is not STOPPED.
				 */    
				if ((RAC_STATUS_ENABLE == rac_status) &&
				    (RAC_STATE_STOPPED != client->rac.acct_state)&&
				    (RAC_STATE_STOP_SENT != client->rac.acct_state))
				{
					client->rac.acct_event = RAC_EVENT_SERVICE_STOP;

					/* Setting Terminate cause to REAUTH FAIL */ 

					client->rac.terminate_cause = RAC_TERMINATE_REAUTH_FAIL;

				} 

#endif
				
//				pae_disconnected(client);
			}

			break;

		case AUTHENTICATING :

			be_check_states(client);

			break;


		case AUTHENTICATED :

			if( (client->reAuthWhen != 0) && (time(NULL) >= client->reAuthWhen)) {
#if (RAC_SUPPORT == RAC_TRUE)	
	
				/* valu of termination Action in access accept*/ 
				if (client->terminationAction !=TERM_ACTION_TERMINATE)
				{
#endif	
					

				log(LG_INF, "reAuthWhen timeout, doing reauthentication");
				pae_connecting(client);
				//oldy pae_disconnected(client);
				ssn_disconnected(client);


#if (RAC_SUPPORT == RAC_TRUE)
				}
				else  
				{
					client->toBeRemoved = 1;

					/*
					 * Termination action = Terminate scenario.
					 */    
					if ((RAC_STATUS_ENABLE == rac_status) &&
					    (RAC_STATE_STOPPED != client->rac.acct_state)&&
					    (RAC_STATE_STOP_SENT != client->rac.acct_state))
					    
					{

						client->rac.acct_event = RAC_EVENT_SERVICE_STOP;

						/* Setting Terminate cause to NAS ERROR */ 
						client->rac.terminate_cause=RAC_TERMINATE_SESSION_TIMEOUT;

					}

				}
#endif /*#if (RAC_SUPPORT == RAC_TRUE)*/	


				
			}

			if (!client->doesSSN && client->update_gtk) {
				update_dot1x_bcast_key(client);
				client->update_gtk = 0;
			}

			break;

		case HELD :

			if(time(NULL) >= client->quietWhile) {
				pae_connecting(client);
			}

			break;

		default:
	}

	return(0);
}


int pae_initialize(struct client_t *client)
{
	log(LG_INF, "pae state set to INITIALIZE");
	client->paestate = INITIALIZE;

	client->currentId = 0;

	pae_disconnected(client);

	return(0);
}


int pae_disconnected(struct client_t *client)
{
	log(LG_INF, "pae state set to DISCONNECTED");
	client->paestate = DISCONNECTED;

	blob_portstatus_set(client, UNAUTHORIZED);
	client->reAuthCount = 0;

	pae_connecting(client);

	return(0);
}



int pae_connecting(struct client_t *client)
{
	log(LG_INF, "pae state set to CONNECTING");
	client->paestate = CONNECTING;

	// Grabbed from pae_disconnected */
	client->currentId++;
	if(client->currentId > 255) client->currentId = 0;

	// Malloc EAP buffer if necassery

	client_malloc_eap_pkt(client);

	// fixme. The client->txWhen for initial dot1x should be
	// configurable, but must be short ( +/- seconds ??)

	client->txWhen = time(NULL) + 5;

	client->radius_state_len = 0;
	eap_tx(client, EAP_CODE_REQUEST, EAP_TYPE_IDENTITY);
	client->reAuthCount ++;

	return(0);
}


int pae_authenticating(struct client_t *client)
{
	log(LG_INF, "pae state set to AUTHENTICATING");
	client->paestate = AUTHENTICATING;

	client->bestate = REQUEST;
	client->rxResp = 1;
	client->aWhile = time(NULL) + client->cconfig->suppTimeout;

	aplog(EV_EAP_AUTH_START, hwaddr2string(client->hwaddr), client->identity, NULL);

	return(0);
}


int pae_authenticated(struct client_t *client)
{
	log(LG_INF, "pae state set to AUTHENTICATED");
	client->paestate = AUTHENTICATED;

	if(client->cconfig->reAuthPeriod > 0) {
		client->reAuthWhen = time(NULL) + client->cconfig->reAuthPeriod;
	} else {
		client->reAuthWhen = 0;
	}
	/* Set client port status to Authorized only if it uses 1x only */
	if (client->does1X == 1 && client->doesSSN == 0)
	{
	     blob_portstatus_set(client, AUTHORIZED);

#if (RAC_SUPPORT == RAC_TRUE)
	     if (RAC_STATUS_ENABLE == rac_status){
		     /*
		      * * Succesful RADIUS authentication of WEP client supporting dot1X.
		      * * We consider this as point of service start. However, we should
		      * * not send account start if this is a case of successful re-auth
		      * */    
		     if ((RAC_STATE_STOPPED == client->rac.acct_state)||
				     (RAC_STATE_STOP_SENT == client->rac.acct_state))
		     {
			     client->rac.acct_event = RAC_EVENT_SERVICE_START ;
			     client->rac.acct_auth = RAC_AUTH_RADIUS;  
	 		     log(LG_DBG, "RAC_EVENT_SERVICE_START");		     
		     }  
	     }  
#endif
        }
	client->currentId ++;
	// After AUTHENTICATION was succes reset the counter.
	client->reAuthCount = 0;

	aplog(EV_EAP_AUTH_SUCCESS, hwaddr2string(client->hwaddr), client->identity, NULL);

	return(0);
}

int pae_aborting(struct client_t *client)
{
	log(LG_INF, "pae state set to ABORTING");
	client->paestate = ABORTING;

	client->currentId ++;

	aplog(EV_EAP_AUTH_ABORTED, hwaddr2string(client->hwaddr), client->identity, NULL);

	// HOW DO WE GET OUT OF HERE ?!!

	return(0);
}


int pae_held(struct client_t *client)
{
	log(LG_INF, "pae state set to HELD");
	client->paestate = HELD;

	blob_portstatus_set(client, UNAUTHORIZED);
        client->quietWhile = time(NULL) + client->cconfig->quietPeriod;
	client->currentId++;

	aplog(EV_EAP_AUTH_FAIL, hwaddr2string(client->hwaddr), client->identity, NULL);

	return(0);
}

