/*
 *
 *
 * IMPORTANT--READ CAREFULLY:
 * Right, including without limitation, use, reproduction, modification, and
 * creation of derivative works, pertaining to this Source Code ("Document") are
 * governed by a legal Agreement ("Agreement") between you (either an individual or
 * a single entity) and Intersil Americas Inc ("Intersil") and/or its licensee
 * (both hereinafter called Licensor).
 * By reproducing, modifying, creating derivative works, accepting, installing,
 * viewing, accessing, transforming or otherwise using this Document, you agree
 * to be bound by the terms of the Agreement. If you do not agree to the terms of
 * the Agreement, Licensor is unwilling to license rights to this Document to you.
 * In such event, you may not use, reproduce, modify, create derivative works, install,
 * view, access, transport, transform or circulate this Document, and you should
 * promptly contact Licensor from which you acquired this Document for instructions
 * on returning or destroying this Document.
 *
 * CONFIDENTIAL, PROPRIETARY AND TRADE SECRET INFORMATION:
 * This Document contains confidential, proprietary and trade secret information of
 * Intersil Americas Inc. and should be treated in a confidential manner.
 * Furthermore, this Document is protected by national copyright laws and
 * international treaties.
 * Unauthorized use, reproduction, modification, creation of derivative works or
 * circulation is strictly prohibited.
 *
 * Intersil  is a registered trademark of Intersil Americas Inc.
 *
 * Copyright  2002 Intersil Americas Inc. All Rights Reserved.
 *
 *
 */

/*
 * Revision Control System Keywords:
 * $Header   : $
 * $DateTime : $
 * $Change   : $
 * $Author   : $
 */

#include <stdint.h>
#include <string.h>
#include <time.h>

#ifdef LINUX
#include "linux.h"
#else
#include <aplog.h>
#endif

#include "client.h"
#include "log.h"
#include "paed.h"
#include "config.h"
#include "eap.h"
#include "eapol.h"
#ifdef DBAP
#include "netlink_func.h"
#else
#include "blobfuncs.h"
#endif
#include "misc.h"
#include "state_ssn.h"
#include "state_gtk.h"
#include "keystuff.h"


#if (PAED_MSSID == 1)

/* the config array pointer */
extern struct config_t *config;
#else

extern struct config_t config[MAX_INTERFACES];  
#endif  /* #if (PAED_MSSID == 1) */


// globals

int SSN_GlobalMicFailWhile = 0;

// externals
extern struct client_t *clientlist;
extern int no_mic_errors;
#if (RAC_SUPPORT == RAC_TRUE)
extern int rac_status;
#endif

int ssn_check_states(struct client_t *client)
{
        char bcast_key[13];
        int r;

        if (SSN_GlobalMicFailWhile >= time(NULL))
        {
            log(LG_WRN , "Still waiting on MIC failure time to expire!");
            return (0);
        }

        switch(client->ssnstate) {

                case SSN_AUTHENTICATION :

                        if(client->paestate == AUTHENTICATED) {
                                ssn_initpmk(client);
                        }

                        break;

                case SSN_PTKSTART :

                        if(client->EAPOLKeyRecvd) {

                                ssn_ptkinitnegotiating(client);
                                client->EAPOLKeyRecvd = 0;
                        }
                        else if (time(NULL) > client->timeOutEvt) {

                                // CHeck if we need to retry and max limit
				if (++client->timeOutCtr > SSN_MAX_RETRIES) {

#if (RAC_SUPPORT == RAC_TRUE)
					/*
					 * Timeout event during key handshake.
					 * Send Accounting stop message only if the client
					 * was already getting service and timed-out during
					 * re-keying. I.e. if accounting state is not STOPPED.
					 */    
					if ((RAC_STATUS_ENABLE == rac_status) &&
					    (RAC_STATE_STOPPED != client->rac.acct_state)&&
					    (RAC_STATE_STOP_SENT != client->rac.acct_state))
					{
						client->rac.acct_event = RAC_EVENT_SERVICE_STOP;

						/* Setting Terminate cause to LOST CARRIER */ 
						client->rac.terminate_cause = RAC_TERMINATE_LOST_CARRIER;

					} 

#endif
					log(LG_DBG, "MSSID: Time out whil doing eapol key handshake with client");                         

					ssn_disconnect(client);
    					break;
                                }

                                // Retry first EAPOL-Key of the handshake
                                eapol_tx_key_ssn(client, 0, 0, 1, 0, PERSTA_KEYID, 1, client->ANonce, NULL, 0, PTK_LEN /*13*/);

                                // Set timeout event to at least 1 second
                                client->timeOutEvt = time(NULL) + 2;
                        }

                        break;

                case SSN_PTKINITNEGOTIATING :

                        if(client->EAPOLKeyRecvd) {
                                ssn_ptkinitdone(client);
                                client->EAPOLKeyRecvd = 0;
                        }
                        else if (time(NULL) > client->timeOutEvt) {

                                // Check if we need to retry and max limit
                                if (++client->timeOutCtr > SSN_MAX_RETRIES) {

                                        // Now this is nice, we don't bail out but actually continue with the group key!
                                        // (If 4th message is lost in transit or in AP rx buffer (very rare chance)
                                        // we think the key may been plumbed on the client correctly after all and
                                        // we check this by sending the group key (encrypted). If reply, things are ok.
                                        if (client->client_wpa_ie.groupkey_only && client->cconfig->enableWEP)
                                        {
                                            // Don't set pair-wise key for stations that doesn't
                                            // support that.
                                        }
                                        else
                                        {
                                        // Plumb down key - data gets encrypted from now on
                                        blob_set_persta_key(client, PERSTA_KEYID);
                    }

                                        /*
                                         * This is the most logical place to put this all negociated key material will be
                                         * complete with the next EAPOL-key message so the secure bit in EAPOL-key should
                                         * be set.
                                         */
                                client->secure = 1;

                                        if (client->cconfig->enableSSNPSK)
                                            aplog(EV_EAP_SSNPSKAUTH_SUCC, hwaddr2string(client->hwaddr), NULL);
                                        else
                                            aplog(EV_EAP_SSNAUTH_SUCCESS, hwaddr2string(client->hwaddr), client->identity, NULL);

                                        // Directly fall into PTKINITDONE
                                        client->ssnstate = SSN_PTKINITDONE;
                                        break;
                                }

                                // Retry 3rd message of the handshake
                                if (client->client_wpa_ie.groupkey_only && client->cconfig->enableWEP)
                                {
                                    /*
                                     * Client capability field indicated client doesn't support 'real' pair-wise keys
                                     * in that case the pair-wise key should not be installed for this client.
                                     */
                                    log(LG_INF, "PTK without the install bit");
                                    eapol_tx_key_ssn(client, 0, 1, 1, 0, PERSTA_KEYID, 1, client->ANonce, NULL, 0, PTK_LEN);
                                }
                                else
                                {
                                    eapol_tx_key_ssn(client, 0, 1, 1, 1, PERSTA_KEYID, 1, client->ANonce, NULL, 0, PTK_LEN);
                                }

                                // Set timeout event to at least 1 second
                                client->timeOutEvt = time(NULL) + 2;
                        }
                        break;


                case SSN_PTKINITDONE :

                        if(client->EAPOLKeyRecvd) {

                                ssn_updatekeys(client);
                                client->EAPOLKeyRecvd = 0;

                        }

                        else if(client->update_gtk) {
                                ssn_update_gtk(client);
                        }

                        break;

                case SSN_UPDATE_GTK :

                        if(client->EAPOLKeyRecvd) {
                                ssn_update_gtk_done(client);
                                client->EAPOLKeyRecvd = 0;
                        }
                        else if (time(NULL) > client->timeOutEvt) {

                                // Check if we need to retry and max limit
                                if (++client->timeOutCtr > SSN_MAX_RETRIES) {
                                        aplog(EV_EAP_GROUP_KEY_FAIL, hwaddr2string(client->hwaddr), NULL);
#if (RAC_SUPPORT == RAC_TRUE)
					/*
					 * Timeout event during key handshake.
					 * Send Accounting stop message only if the client
					 * was already getting service and timed-out during
					 * re-keying. I.e. if accounting state is not STOPPED.
					 */    
					if ((RAC_STATUS_ENABLE == rac_status) &&
					    (RAC_STATE_STOPPED != client->rac.acct_state)&&
					    (RAC_STATE_STOP_SENT != client->rac.acct_state))
					{

                       			      	client->rac.acct_event = RAC_EVENT_SERVICE_STOP;
						/* Setting Terminate cause to LOST CARRIER */ 
					 	client->rac.terminate_cause =RAC_TERMINATE_LOST_CARRIER;
					}

#endif					
					ssn_disconnect(client);
                                        break;
                                }
                                if( client->cconfig->enableWEP)
                                {
                                        r = blob_get_bcast_key(0, bcast_key, client->cconfig->WEPKeySize, client->cconfig);
                                        if (client->client_wpa_ie.groupkey_only)
                                        {
                                            // Install broadcast key as TX and RX key on client
                                            log(LG_INF,"GTK update for TX and RX");
                                            eapol_tx_key_ssn(client, 1, 1, 1, 1, BCAST_KEYID_STATIC, 0, client->cconfig->cgtk->GNonce, bcast_key, r, r);
                                        }
                                        else
                                            eapol_tx_key_ssn(client, 1, 1, 1, 0, BCAST_KEYID_STATIC, 0, client->cconfig->cgtk->GNonce, bcast_key, r, r);
                                }
                                else if ( client->cconfig->enable1X )
                                {
                                        eapol_tx_key_ssn(client, 1, 1, 1, 0, client->cconfig->cgtk->GN, 0, client->cconfig->cgtk->GNonce, client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], client->cconfig->Dot1XWEPKeySize, client->cconfig->Dot1XWEPKeySize);
                                }
                                else
                                {
                                        eapol_tx_key_ssn(client, 1, 1, 1, 0, client->cconfig->cgtk->GN, 0, client->cconfig->cgtk->GNonce, client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], GTK_LEN, GTK_LEN);
                                }
                                // Set timeout event to at least 1 second
                                client->timeOutEvt = time(NULL) + 2;
                        }
                        break;


                case SSN_INTEGRITYFAILURE :

                        if(client->intFailWhile > time(NULL)) {
                                client->intFailWhile = 0;
                                ssn_keyupdate(client);
                        }

                        break;

                default :



        }


        return(0);
}


int ssn_disconnect(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_DISCONNECT");
        client->ssnstate = SSN_DISCONNECT;
    blob_portstatus_set(client, UNAUTHORIZED);

        if (client->cconfig->enableSSNPSK)
            aplog(EV_EAP_SSNPSKAUTH_FAIL, hwaddr2string(client->hwaddr), NULL);
        else
        aplog(EV_EAP_SSNAUTH_FAIL, hwaddr2string(client->hwaddr), client->identity, NULL);

        blob_deauthenticate_client(client);

        client->toBeRemoved = 1;

        return(0);
}


int ssn_disconnected(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_DISCONNECTED");
        client->ssnstate = SSN_DISCONNECTED;

        ssn_initialize(client);         // UCT

        return(0);
}


int ssn_initialize(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_INITIALIZE");
        client->ssnstate = SSN_INITIALIZE;

//      RemovePTK(client);
//      RemoveGTK(client, 0..N);

        ssn_authentication(client);

        // Indicate we need to send a group key
        client->update_gtk = 1;

        return(0);
}


int ssn_authentication(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_AUTHENTICATION");
        client->ssnstate = SSN_AUTHENTICATION;

#ifdef DEBUG_PRINTKEYS
        dump_bindata("ANonce", client->ANonce, NONCE_LEN);
#endif
        client->timeOutCtr = 0;

        // If pre-shared key enabled, start 4-way handshake right away,
        // for .1X auth, wait till .1X PAE state is authenticated

        if(client->cconfig->enableSSNPSK) {
                ssn_initpsk(client);
        }

        return(0);
}


int ssn_initpmk(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_INITPMK");
        client->ssnstate = SSN_INITPMK;

        memcpy(client->PMK, client->tx_session_key, PMK_LEN);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("PMK", client->PMK, PMK_LEN);
#endif

        ssn_ptkstart(client);   // UCT

        return(0);
}


int ssn_initpsk(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_INITPSK");
        client->ssnstate = SSN_INITPSK;

        // Get Key #0 from the blob and use this as PMK
        memcpy(client->PMK, client->cconfig->ssnpsk256key, PMK_LEN);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("PMK", client->PMK, PMK_LEN);
#endif

        ssn_ptkstart(client);   // UCT;

        return(0);
}


int ssn_ptkstart(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_PTKSTART");
        client->ssnstate = SSN_PTKSTART;

        gtk_inc_counter(client->cconfig);
        memcpy(client->ANonce, client->cconfig->cgtk->Counter, NONCE_LEN);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("ANonce", client->ANonce, NONCE_LEN);
#endif

        eapol_tx_key_ssn(client, 0, 0, 1, 0, PERSTA_KEYID, 1, client->ANonce, NULL, 0, PTK_LEN /*13*/);  /* Add this to doc ! */

        // Set timeout event to at least 1 second

        client->timeOutEvt = time(NULL) + 2;

        return(0);
}


int ssn_ptkinitnegotiating(struct client_t *client)
{
        unsigned char micbuf1[MIC_LEN];
        unsigned char micbuf2[MIC_LEN];
        struct eapolhdr *eh;
        struct eapolkey_ssn *ek;
        char tmp_ptk[64];

        log(LG_DBG , "ssn state set to SSN_PTKINITNEGOTIATING");
        client->ssnstate = SSN_PTKINITNEGOTIATING;

        /* Setup the EAPOL-key structures */
        eh = (struct eapolhdr *) client->eap_pkt;
        ek = (struct eapolkey_ssn *) eh->eap_data;

        /*
         * Check for the correct key descriptor 1
         * Currently we only support HMAC MD5 with RC4 en/de-cryption
         */
        if ((ntohs(ek->key_info) & EAPOL_KEY_DESCRIPTOR_HMAC_MD5_RC4) !=
                EAPOL_KEY_DESCRIPTOR_HMAC_MD5_RC4)
        {
                log(LG_WRN, "EAPOL-key discriptor type not hmac/md5 and rc4, discarding EAPOL-Key");
                client->ssnstate = SSN_PTKSTART;
                return(0);
        }

        // Copy the SNonce from the received EAPOL-KEY to the client struct
        memcpy(client->SNonce, ek->nonce, NONCE_LEN);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("SNonce", client->SNonce, NONCE_LEN);
#endif

        calc_PTK(client, client->ANonce, client->SNonce, tmp_ptk, 64);
        memcpy(client->eapolkey_MICkey, tmp_ptk +  0, 16);
        memcpy(client->eapolkey_enckey, tmp_ptk + 16, 16);
        memcpy(client->tx_session_key, tmp_ptk + 32, 32);
        client->session_key_len = 32;

#ifdef DEBUG_PRINTKEYS
        dump_bindata("PTK", tmp_ptk, 64);
        dump_bindata("EAPOL-Key MIC key", client->eapolkey_MICkey, 16);
        dump_bindata("EAPOL-Key encryption key", client->eapolkey_enckey, 16);
        dump_bindata("Hardware encryption key", client->tx_session_key, 32);
#endif

        // Save the MIC from the EAPOL packet (MIC check needed to wait for PTK)

        memcpy(micbuf1, ek->MIC, MIC_LEN);

        // Calculate the MIC over the EAPOL packet with the EAPOL-Key MIC key

        memset(ek->MIC, 0, MIC_LEN);
        MIC((char *)eh, client->eap_pkt_len, client->eapolkey_MICkey, 16, micbuf2);

#ifdef DEBUG_PRINTKEYS
        dump_bindata("MIC in packet", micbuf1, 16);
        dump_bindata("MIC calculated", micbuf2, 16);
#endif

        // Check if our calculated MIC(ek, PTK) is equal to the received MIC

        if (memcmp(micbuf1, micbuf2, MIC_LEN) != 0) {
                log(LG_WRN, "EAPOL-key mic failure, discarding EAPOL-Key");
                client->ssnstate = SSN_PTKSTART;
                return(0);
        }

        // Check whether replay_counter matched that in EAPOL-Key sent

        if (ek->replay_counter_low  != client->replay_counter_low ||
            ek->replay_counter_high != client->replay_counter_high) {
                log(LG_WRN, "Replay counter mismatch, discarding EAPOL-Key");

                client->ssnstate = SSN_PTKSTART;

                return(0);
        }
        else {
                log(LG_INF, "Replay counter matched request. Ok");
        }

        /* Check 2nd msg of the handshake for the SSN IE in the Associate Request */
        /* don't forget to copy the tag and len fields (+2) */

        dump_bindata("IE in packet", ek->key, client->client_wpa_ie.assoc_ie_len+2);
        dump_bindata("IE from Assoc", client->client_wpa_ie.assoc_ie, client->client_wpa_ie.assoc_ie_len+2);

        if (memcmp(ek->key, client->client_wpa_ie.assoc_ie, client->client_wpa_ie.assoc_ie_len+2)) {
                log(LG_ERR, "2nd msg of the handshake didn't contain same SSN IE as in Assoc.Req");

                blob_deauthenticate_client(client);
                client->ssnstate = SSN_PTKSTART;
                return(0);
        }

        // Send 3rd message of the handshake
        if (client->client_wpa_ie.groupkey_only && client->cconfig->enableWEP)
        {
            /*
             * Client capability field indicated client doesn't support 'real' pair-wise keys
             * in that case the pair-wise key should not be installed for this client.
             */
            log(LG_INF, "PTK without the install bit");
            eapol_tx_key_ssn(client, 0, 1, 1, 0, PERSTA_KEYID, 1, client->ANonce, NULL, 0, PTK_LEN);
        }
        else
        {
            eapol_tx_key_ssn(client, 0, 1, 1, 1, PERSTA_KEYID, 1, client->ANonce, NULL, 0, PTK_LEN);
        }

        // Set timeout event to at least 1 second
        client->timeOutEvt = time(NULL) + 2;


        return(0);
}


int ssn_ptkinitdone(struct client_t *client)
{
        unsigned char micbuf1[MIC_LEN];
        unsigned char micbuf2[MIC_LEN];
        struct eapolhdr *eh;
        struct eapolkey_ssn *ek;

        log(LG_DBG , "ssn state set to SSN_PTKINITDONE");
        client->ssnstate = SSN_PTKINITDONE;

        /* Setup the EAPOL-key structures */
        eh = (struct eapolhdr *) client->eap_pkt;
        ek = (struct eapolkey_ssn *) eh->eap_data;

        /*
         * Check for the correct key descriptor 1
         * Currecntly we only support HMAC MD5 with RC4 en/de-cryption
         */
        if ((ntohs(ek->key_info) & EAPOL_KEY_DESCRIPTOR_HMAC_MD5_RC4) !=
                EAPOL_KEY_DESCRIPTOR_HMAC_MD5_RC4)
        {
                log(LG_WRN, "EAPOL-key discriptor type not hmac/md5 and rc4, discarding EAPOL-Key");
                client->ssnstate = SSN_PTKSTART;
                return(0);
        }

        // Save the MIC from the EAPOL packet
        memcpy(micbuf1, ek->MIC, MIC_LEN);

        // Calculate the MIC over the EAPOL packet with the PTK
        memset(ek->MIC, 0, MIC_LEN);
        MIC((char *)eh, client->eap_pkt_len, client->eapolkey_MICkey, 16, micbuf2);


        // Check if our calculated MIC(ek, PTK) is equal to the received MIC

        if (memcmp(micbuf1, micbuf2, MIC_LEN) != 0) {
                log(LG_WRN, "EAPOL-key mic failure, discarding EAPOL-Key");
                client->ssnstate = SSN_PTKINITNEGOTIATING;

                return(0);
        }

        // Check whether replay_counter matched that in EAPOL-Key sent

        if (ek->replay_counter_low  != client->replay_counter_low ||
            ek->replay_counter_high != client->replay_counter_high) {
                log(LG_WRN, "Replay counter mismatch, discarding EAPOL-Key");

                client->ssnstate = SSN_PTKINITNEGOTIATING;

                return(0);
        }

        else {
                log(LG_INF, "Replay counter matched request. Ok");
        }


        if (client->client_wpa_ie.groupkey_only && client->cconfig->enableWEP)
        {
            /*
             * Client capability field indicated client doesn't support 'real' pair-wise keys
             * in that case the pair-wise key should not be installed for this client.
             */
            // Everything good, plumb down tx_session_key (= 3d 128 bits of PTK) to MVC
            log(LG_INF, "*** NOT plumb-ing down PTK client grouponly ***");

        }
        else
        {
            // Everything good, plumb down tx_session_key (= 3d 128 bits of PTK) to MVC
            log(LG_INF, "*** Ready to plumb down pairwise key material ***");

            blob_set_persta_key(client, PERSTA_KEYID );
        }

        /*
         * This is the most logical place to put this all negociated key material will be
         * complete with the next EAPOL-key message so the secure bit in EAPOL-key should
         * be set.
         */
       client->secure = 1;

        if (client->cconfig->enableSSNPSK)
            aplog(EV_EAP_SSNPSKAUTH_SUCC, hwaddr2string(client->hwaddr), NULL);
        else
            aplog(EV_EAP_SSNAUTH_SUCCESS, hwaddr2string(client->hwaddr), client->identity, NULL);

        // Data gets encrypted from now on
    if( client->update_gtk)
    {
        ssn_update_gtk(client);
    }

        return(0);
}


/*
 * Newly added state to let the per-sta SSN-statemachine update the GTK.
 * Does something like the ReKeyNegotiating state from the SSN docs.
 */

int ssn_update_gtk(struct client_t *client)
{
        char key0[13];
        int r;

        log(LG_DBG , "ssn state set to SSN_UPDATE_GTK");
        client->ssnstate = SSN_UPDATE_GTK;

        if(client->update_gtk) {

                // Set timeout event to at least 1 second

                client->timeOutEvt = time(NULL) + 2;

                if(client->cconfig->enableWEP) {

                        r=blob_get_bcast_key(0, key0, client->cconfig->WEPKeySize, client->cconfig);

#ifdef DEBUG_PRINTKEYS
                        dump_bindata("1-GTK", key0, r);
#endif
                        if (client->client_wpa_ie.groupkey_only)
                        {
                            // Install broadcast key as TX and RX key on client
                            log(LG_INF,"GTK update for TX and RX");
                            eapol_tx_key_ssn(client, 1, 1, 1, 1, BCAST_KEYID_STATIC, 0, client->cconfig->cgtk->GNonce, key0, r, r);
                        }
                        else
                            eapol_tx_key_ssn(client, 1, 1, 1, 0, BCAST_KEYID_STATIC, 0, client->cconfig->cgtk->GNonce, key0, r, r);

                }
                else if (client->cconfig->enable1X)
                {
#ifdef DEBUG_PRINTKEYS
                        dump_bindata("2-GTK", client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], GTK_LEN);
#endif
                        eapol_tx_key_ssn(client, 1, 1, 1, 0, client->cconfig->cgtk->GN, 0, client->cconfig->cgtk->GNonce, client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], client->cconfig->Dot1XWEPKeySize, client->cconfig->Dot1XWEPKeySize);

                }
                else {

#ifdef DEBUG_PRINTKEYS
                        dump_bindata("3-GTK", client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], GTK_LEN);
#endif
                        eapol_tx_key_ssn(client, 1, 1, 1, 0, client->cconfig->cgtk->GN, 0, client->cconfig->cgtk->GNonce, client->cconfig->cgtk->GTK[client->cconfig->cgtk->GN], GTK_LEN, GTK_LEN);

                }



                client->update_gtk = 0;
        }

        return(0);
}

/*
 * Newly added state to let the per-sta SSN-statemachine update the GTK. Replacement
 * for the REKEYESTABLISHED state from the SSN docs.
 */

int ssn_update_gtk_done(struct client_t *client)
{
        unsigned char micbuf1[MIC_LEN];
        unsigned char micbuf2[MIC_LEN];
        struct eapolhdr *eh;
        struct eapolkey_ssn *ek;

        log(LG_DBG , "ssn state set to SSN_UPDATE_GTK");
        client->ssnstate = SSN_UPDATE_GTK;

        // Save the MIC from the EAPOL packet

        eh = (struct eapolhdr *) client->eap_pkt;
        ek = (struct eapolkey_ssn *) eh->eap_data;
        memcpy(micbuf1, ek->MIC, MIC_LEN);

        // Calculate the MIC over the EAPOL packet with the PTK

        memset(ek->MIC, 0, MIC_LEN);
        MIC((char *)eh, client->eap_pkt_len, client->eapolkey_MICkey, 16, micbuf2);


#ifdef DEBUG_PRINTKEYS
        dump_bindata("MIC in packet", micbuf1, 16);
        dump_bindata("MIC calculated", micbuf2, 16);
#endif

        // Check if our calculated MIC(ek, PTK) is equal to the received MIC

        if (memcmp(micbuf1, micbuf2, MIC_LEN) != 0) {
                log(LG_WRN, "EAPOL-key mic failure, discarding EAPOL-Key");
                client->ssnstate = SSN_PTKINITNEGOTIATING;
                client->update_gtk = 1;
                return(0);
        }

        // Check whether replay_counter matched that in EAPOL-Key sent

        if (ek->replay_counter_low  != client->replay_counter_low ||
            ek->replay_counter_high != client->replay_counter_high) {
                log(LG_WRN, "Replay counter mismatch, discarding EAPOL-Key");

                client->ssnstate = SSN_PTKINITNEGOTIATING;
                client->update_gtk = 1;
                return(0);
        }
        else {
                log(LG_INF, "Replay counter matched request. Ok");
        }

        /* Check if secure bit is set */
        if (client->secure && ((ntohs(ek->key_info) & EAPOL_KEY_INFO_MASK_SECURE) != EAPOL_KEY_INFO_MASK_SECURE))
        {
                log(LG_WRN, "Secure bit in key_info field not set while client is, discarding EAPOL-Key.");

                client->ssnstate = SSN_PTKINITNEGOTIATING;
                client->update_gtk = 1;
                return(0);
        }

        // This is a bit tricky: From this state we go back to the PTIKINITDONE state,
        // but we *do not* execute the ptkinitdone function again, just set the state-variable
        // back to SSN_PTKINITDONE

        /* Set client port status to Authorized */
        log(LG_INF, "Client %s is allowed, portstate=AUTHORIZED", hwaddr2string(client->hwaddr));
        blob_portstatus_set(client, AUTHORIZED);

#if (RAC_SUPPORT == RAC_TRUE)

	/*
	 * SSN or SSN-PSK client granted access.
	 * We consider this as point of service start.
	 * Send accounting start if accounting enabled.
	 */    
	if (RAC_STATUS_ENABLE == rac_status){
		if (RAC_STATE_STOPPED == client->rac.acct_state){

			client->rac.acct_event = RAC_EVENT_SERVICE_START;
			log(LG_DBG, "RAC_EVENT_SERVICE_START");

			/* Need to identify the auth type */ 
			if (client->cconfig->enableSSNPSK)
			{
				/* SSN-PSK is configured. No RADIUS! */            
				client->rac.acct_auth = RAC_AUTH_LOCAL;
			}
			else 
			{
				/* Case of SSN. Auth type is RADIUS! */ 
				client->rac.acct_auth =RAC_AUTH_RADIUS;
			}
		}    
	}
#endif


	
        aplog(EV_EAP_GROUP_KEY_SUCC, hwaddr2string(client->hwaddr), NULL);

        client->ssnstate = SSN_PTKINITDONE;

        client->cconfig->cgtk->GKeyDoneStations--;


//      eap_tx_notification(client, "SSN Authenticated Dusssss");
//      eap_tx_notification(client, "All EAP's, EAPOL's, Radius, MPPE's, MIC's, 802.1X, SHA1's, ACL's, MD5's, RC4's, PSK's, PTK's, PMK's, GTK's, PRF's, MIC', OPSET's, OPGETS, PRISMIOCTL's succeeded!!!");

        return(0);
}


int ssn_updatekeys(struct client_t *client)
{
        unsigned char micbuf1[MIC_LEN];
        unsigned char micbuf2[MIC_LEN];
        struct eapolhdr *eh;
        struct eapolkey_ssn *ek;

        log(LG_DBG , "ssn state set to SSN_UPDATEKEYS");
        client->ssnstate = SSN_UPDATEKEYS;

        // Save the MIC from the EAPOL packet

        eh = (struct eapolhdr *) client->eap_pkt;
        ek = (struct eapolkey_ssn *) eh->eap_data;
        memcpy(micbuf1, ek->MIC, MIC_LEN);

        // Calculate the MIC over the EAPOL packet with the PTK

        memset(ek->MIC, 0, MIC_LEN);
        MIC((char *)eh, client->eap_pkt_len, client->eapolkey_MICkey, 16, micbuf2);


        // Check if our calculated MIC(ek, PTK) is equal to the received MIC

        if (memcmp(micbuf1, micbuf2, MIC_LEN) != 0) {
                log(LG_WRN, "EAPOL-key mic failure, discarding EAPOL-Key");
                client->ssnstate = SSN_PTKINITDONE;
                return(0);
        }

        // Check replay counter in this case

        if(ek->replay_counter_low  < client->andere_replay_counter_low ||
           ek->replay_counter_high < client->andere_replay_counter_high ) {
                log(LG_ERR, "Client andere_replay_counter mismatch");
                client->ssnstate = SSN_PTKINITDONE;
                return(0);
        } else {
                client->andere_replay_counter_low = ek->replay_counter_low;
                client->andere_replay_counter_high = ek->replay_counter_high;
        }


        // Only react to EAPOL-Key from client in this state that has request bit
        if ((ntohs(ek->key_info) & EAPOL_KEY_INFO_MASK_REQUEST) == EAPOL_KEY_INFO_MASK_REQUEST)
        {
            if ((ntohs(ek->key_info) & EAPOL_KEY_INFO_MASK_ERROR) == EAPOL_KEY_INFO_MASK_ERROR &&
                !no_mic_errors)
            {
                /* Pair wise key? */
                if ((ntohs(ek->key_info) & EAPOL_KEY_INFO_MASK_KEY_TYPE) == EAPOL_KEY_INFO_MASK_KEY_TYPE)
                {
                    log(LG_WRN, "EAPOL-Key received with Pair-wise key mic error!");
                    ssn_integrityfailure(client);
                }
                else
                {
                    log(LG_WRN, "EAPOL-Key received with Group key mic error!");
                    /* Update group key on group key MIC error */
                    gtk_setkeys(client->cconfig);
                    ssn_integrityfailure(client);
                }
            }
            else
            {
                ssn_ptkstart(client);
            }
        }
        else
        {
            log(LG_WRN, "EAPOL-Key received without request bit set, discarding");
            client->ssnstate = SSN_PTKINITDONE;
        }

        return(0);
}


int ssn_integrityfailure(struct client_t *client)
{

#if (PAED_MSSID == 1)
    	int i;
    	int nof_vss;
#endif

        struct client_t *tmp_client = clientlist;

        log(LG_DBG , "ssn state set to SSN_INTEGRITYFAILURE");
        client->ssnstate = SSN_INTEGRITYFAILURE;

        // Ok, this is not really what it should be, I discussed this. WaitUpto60() is only when there's
        // Michael MIC failure... for EAPOL-Key MIC failure we just disassoc and deauth.

        // Case we must consider is when EAPOLKeyRecv with error bit set from the STA. This does
        // indicate a Michael check failure and then the WaitUpto60() applies before sending new keys.
        // We must also hear from our MVC if it detects an integrity failure for Michael...

        /* So this is changed in the last draft WiFi WPA spec 1.7
         * There is no difference between GTK and PTK Mic errors also
         * the countermeasures are applied not on a client basis but globaly
         * If STA1 reports an error and in a minute STA2 reports an error
         * all clients should be removed and blocked in the 4-way handshake for one
         * minute
         */
        if ( (client->cconfig->cgtk->intFailWhile) < time(NULL) )
        {
            log(LG_WRN, "First MIC Integrity failure detected, NO countermeasures necessary !");
            aplog(EV_EAP_MICFAIL_STA, hwaddr2string(client->hwaddr), client->identity, NULL);
            ssn_ptkstart(client);

        /* Set the global MIC timer to NOW + 60 sec */
            client->cconfig->cgtk->intFailWhile = time(NULL) + 60;
        }
        else
        {
            /* So the last Mic error was closer than a minute ago */
            log(LG_ERR, "Second MIC Integrity failure detected, apply countermeasures!");
            aplog(EV_EAP_MICFAIL_STA, hwaddr2string(client->hwaddr), client->identity, NULL);

            /* Close EAP ports for all clients */
        blob_portstatus_set_global(UNAUTHORIZED, client->cconfig);

#if (PAED_MSSID == 1)

       /* Send Broadcast deauthenticate message to all the clients in all VSSs. */
       nof_vss = mssid_get_vss_count();

       for (i = 0; i < nof_vss; i++) {
           blob_mssid_deauth_vss(&config[i], config[i].vssid); 
       }
#endif


        while(tmp_client != NULL) {
            log(LG_DBG, "Client %s should be removed", hwaddr2string(tmp_client->hwaddr));
	    tmp_client->toBeRemoved = 1;
#if (RAC_SUPPORT == RAC_TRUE)
	    /*
	     * TKIP Integrity failure scenario.
	     * There has been a MIC failure for the second time. 
	     * Send Accounting stop message for all clients.
	     */  

	    if ((RAC_STATUS_ENABLE == rac_status) &&
		(RAC_STATE_STOPPED != client->rac.acct_state)&&
	        (RAC_STATE_STOP_SENT != client->rac.acct_state)) 
	    {
		    tmp_client->rac.acct_event = RAC_EVENT_SERVICE_STOP;

		    /* Setting Terminate cause to NAS ERROR */ 
		    tmp_client->rac.terminate_cause=RAC_TERMINATE_NAS_ERROR;
	    } 
#endif
	    tmp_client = tmp_client->next;
        }

            /* Block total SSN handshake for NOW plus 60 secs.*/
            SSN_GlobalMicFailWhile = time(NULL) + 60;
        }
        return(0);
}


int ssn_keyupdate(struct client_t *client)
{
        log(LG_DBG , "ssn state set to SSN_KEYUPDATE");
        client->ssnstate = SSN_KEYUPDATE;

        gtk_inc_counter(client->cconfig);
        memcpy(client->ANonce, client->cconfig->cgtk->Counter, NONCE_LEN);

        return(0);
}

// eof
