/*  $Header$
 *
 *  Copyright (C) 2003 Intersil Americas Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include "version_info.h"

/* ioctl interface */
#define SOFTMACIOCTL                    SIOCDEVPRIVATE

struct s_softmac_ioctl {
    unsigned int cmd;
    unsigned int oid;
    unsigned int len;
    char *data;
};


static void usage(char *name)
{
    printf("smconf version %s\n", VERSIONID );
    printf("Usage: %s devname <cmd number> [data (dec, for hex use 0x)]\n", name);
    exit(1);
}

static void invalid_args(int argc, char *argv[])
{
    fprintf(stderr, "Invalid number of arguments (%d) for command %c\n", argc, argv[2][0]);
    exit(1);
}

static int do_cmd(int fd, struct ifreq *ifr, int cmd, char *data, int len)
{
    struct s_softmac_ioctl smioctl;
    int err;

    smioctl.cmd = cmd;
    smioctl.oid = 0;
    smioctl.len = len;
    smioctl.data = data;

    ifr->ifr_data = (caddr_t)&smioctl;
    err = ioctl(fd, SOFTMACIOCTL, ifr);
    if (err < 0) {
        fprintf(stderr, "SoftMAC Command %d on interface %s failed (%d)",
                cmd, ifr->ifr_name, err);
        perror(" ");
        return -1;
    }

    return 0;
}

int main(int argc, char *argv[])
{
    struct ifreq ifr;
    int fd;
    int cmd;
    long data = 0;
    int len = 0;

    if (argc < 3)
        usage(argv[0]);

    /* Setup our control structures. */
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, argv[1], IFNAMSIZ);

    /* Open control socket. */
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0) {
        perror("Cannot open socket");
        exit(1);
    }

    if (argc != 3 && argc != 4)
        invalid_args(argc, argv);

    cmd = atoi(argv[2]);
    if(argc == 4) {
        data = strtol(argv[3], NULL, 0);
        len = sizeof(long);
    }

    do_cmd(fd, &ifr, cmd, (char *)&data, len);

    exit(0);
}

