/*  $Header$
 *
 *  Copyright (C) 2003 Intersil Americas Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include "prismioctl.h"
#include "version_info.h"


static struct ifreq ifr;
static int fd = 0;
 
int prismioctl_open(void)
{
    /* Setup our control structures. */
    memset(&ifr, 0, sizeof(ifr));

    /* Open control socket. */
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0) {
        perror("prismioctl_open: Cannot open socket");
        return -1;
    }
    return 0;
}

int prismioctl_close(void)
{
    if(fd > 0)
        close(fd);
}

int get_prismioctl(char *ifname, unsigned int oid, void *data, int data_len)
{
    struct s_softmac_ioctl smioctl;
    int err;

    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);

    smioctl.cmd = SMIOC_GETOID;
    smioctl.oid = oid;
    smioctl.len = data_len;
    smioctl.data = data;

    ifr.ifr_data = (caddr_t)&smioctl;
    err = ioctl(fd, SOFTMACIOCTL, &ifr);
    if (err < 0) {
        fprintf(stderr, "get_prismioctl OID 0x%x on interface %s failed (%d)",
                oid, ifr.ifr_name, err);
        perror(" ");
        return -1;
    }
    return 0;
}

int set_prismioctl(char *ifname, unsigned int oid, void *data, int data_len)
{
    struct s_softmac_ioctl smioctl;
    int err;

    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);

    smioctl.cmd = SMIOC_SETOID;
    smioctl.oid = oid;
    smioctl.len = data_len;
    smioctl.data = data;

    ifr.ifr_data = (caddr_t)&smioctl;
    err = ioctl(fd, SOFTMACIOCTL, &ifr);
    if (err < 0) {
        fprintf(stderr, "set_prismioctl OID 0x%x on interface %s failed (%d)",
                oid, ifr.ifr_name, err);
        perror(" ");
        return -1;
    }
    return 0;
}

char *prismioctl_version( void ) 
{ 
    return VERSIONID;
}
